﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;


namespace Ice.Net.Ecommerce.BusinessObjects.Common
{
	public class FrontParameters
	{
		public enum KindEnum
		{
			CategoryShortcut,
			RedirectToFile,
			MixedParameters
		}

		KindEnum mFpKind;

		public KindEnum Kind
		{
			get { return mFpKind; }
		}

		string mLinkName = ""; // example: www.domain.com/LinkName-d43-c23-q1-g.aspx?q=search expression
		int mManufacturerID = 0; // m
		int mProductID = 0;		// d
		int mCategoryID = 0;	// c
		int mPagerPageIndex = 0;	// n
		int mSortBy = 0;			// o
		int mWebPageID = 0; // h
		string mRedirectToFile = "";
		int mUseSearch = 0; // q
		string mSearchExpression = "";
		string mCategoryShortcutName = ""; // p
		int mOrderID = 0; // oid

		#region Public properties

		/// <summary>
		/// Link name, example: www.domain.com/LinkName-d43-c23-g.aspx
		/// </summary>
		public string LinkName
		{
			get { return mLinkName; }
			set { mLinkName = value; }
		}

		public int ProductID
		{
			get { return mProductID; }
			set { mProductID = value; }
		}

		public int CategoryID
		{
			get { return mCategoryID; }
			set { mCategoryID = value; }
		}

		public int PagerPageIndex
		{
			get { return mPagerPageIndex; }
			set { mPagerPageIndex = value; }
		}

		public int SortBy
		{
			get { return mSortBy; }
			set { mSortBy = value; }
		}

		public int WebPageID
		{
			get { return mWebPageID; }
			set { mWebPageID = value; }
		}

		public string RedirectToFile
		{
			get { return mRedirectToFile; }
			set { mRedirectToFile = value; }
		}

		public string CategoryShortcutName
		{
			get { return mCategoryShortcutName; }
			set { mCategoryShortcutName = value; }
		}

		public string SearchExpression
		{
			get { return mSearchExpression; }
			set { mSearchExpression = value; }
		}

		public int OrderID
		{
			get { return mOrderID; }
		}

		public int ManufacturerID
		{
			get { return mManufacturerID; }
			set { mManufacturerID = value; }
		}


		/// <summary>
		/// Get or set ParametersString
		/// </summary>
		public string RelativeLink
		{
			get { return this.GetParametersString(); }
			//set { this.ParseParameters(value); }
		}

		public string AbsoluteLink
		{
			get 
			{
				if (VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.SSLMode == SSLModeUtility.SSLModeEnum.CreateSSLLinks || 
				    VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.SSLMode == SSLModeUtility.SSLModeEnum.ForceAlways)
				{
				    return ApplicationUtility.ApplicationHttpRoot.Replace("http://", "https://") + RelativeLink;
				}
				else
				{
					return ApplicationUtility.ApplicationHttpRoot + RelativeLink; 
				}
			}
		}

		public string RelativeLinkWithSearch
		{
			get { return this.RelativeLink + GetSearch(); }
		}

		public string AbsoluteLinkWithSearch
		{
			get { return this.AbsoluteLink + GetSearch(); }
		}

		#endregion

		#region Constructors
		public FrontParameters()
		{
		}

		public FrontParameters(System.Web.HttpContext context) : this(context.Request)
		{
		}

		public FrontParameters(System.Web.HttpRequest request)
		{
			ParseParameters(request);
		}

		#endregion

		#region Private methods

		static Regex re =new Regex(
				"(" +
					"((\\/)|(\\-))" +
					"(h(?<h>[\\d]+)\\-){0,1}" +
					"(m(?<m>[\\d]+)\\-){0,1}" +
					"(d(?<d>[\\d]+)\\-){0,1}" +
					"(c(?<c>[\\d]+)\\-){0,1}" +
					"(n(?<n>[\\d]+)\\-){0,1}" +
					"(o(?<o>[\\d]+)\\-){0,1}" +
					"(q(?<q>[\\d]+)\\-){0,1}" +
					"g\\.aspx" +
					"(\\?q=(?<search>[^\\=\\&]+)){0,1}" +
				")" +
				"|" +
				    "(\\/g-(?<file>[^\\.\\/]+\\.aspx)$)" + // file
				"|" + 
				"(" +
				    "(p\\=(?<p>[^\\?\\=\\#\\&\\:]+))" +
					"(\\&oid\\=(?<oid>[\\d]+)){0,1}" +
				")"
				);

		private void ParseParameters(System.Web.HttpRequest request)
		{
			string p = request.QueryString["p"];
			string oid = request.QueryString["oid"];
			if (!String.IsNullOrEmpty(p))
			{
				mCategoryShortcutName = p;
				mFpKind = KindEnum.CategoryShortcut;

				if (!string.IsNullOrEmpty(oid))
				{
					int.TryParse(oid, out mOrderID);
				}

				return;
			}

			string parametersString = System.Web.HttpUtility.UrlDecode(request.RawUrl);

			// string parametersString = "http://www.mydomain.com/category-name-d32-c12-p12-o12-g.aspx"; 
			// must end with .....-d32-c13-g.aspx  or .../d32-c13-g.aspx or .../g-url.aspx (example: ..../g-customer.aspx )

			Match m = re.Match(parametersString);

			if (m.Groups["p"].Success)
			{
				mFpKind = KindEnum.CategoryShortcut;
				DecodeValue(m, "oid", ref mOrderID);
				mCategoryShortcutName = m.Groups["p"].Value;
			}
			else if (m.Groups["file"].Success)
			{
				mFpKind = KindEnum.RedirectToFile;
				
				mRedirectToFile = m.Groups["file"].Value;
			}
			else
			{
				mFpKind = KindEnum.MixedParameters;

				DecodeValue(m, "h", ref mWebPageID);
				DecodeValue(m, "m", ref mManufacturerID);
				DecodeValue(m, "d", ref mProductID);
				DecodeValue(m, "c", ref mCategoryID);
				DecodeValue(m, "n", ref mPagerPageIndex);
				DecodeValue(m, "o", ref mSortBy);

				if (m.Groups["search"].Success)
				{
					mSearchExpression = m.Groups["search"].Value;
					if (!string.IsNullOrEmpty(mSearchExpression))
					{
						DecodeValue(m, "q", ref mUseSearch);
						if (mUseSearch <= 0)
						{
							mSearchExpression = "";
						}
					}
				}
			}
		}

		private void DecodeValue(Match m, string parName, ref int field)
		{
			Group group = m.Groups[parName];
			if (group.Success)
			{
				int.TryParse(group.Value, out field);
			}
		}

		StringBuilder sb = null;

		private string GetParametersString()
		{
			sb = new StringBuilder();
			if (!String.IsNullOrEmpty(mLinkName))
			{
				sb.Append(System.Web.HttpUtility.UrlEncode(mLinkName.Replace(" ", "-") + "-"));
				//sb.Append(mLinkName.Replace(" ", "-") + "-");
			}
			AddParameter("h", mWebPageID);
			AddParameter("m", mManufacturerID);
			AddParameter("d", mProductID);
			AddParameter("c", mCategoryID);
			AddParameter("n", mPagerPageIndex);
			AddParameter("o", mSortBy);
			if (!string.IsNullOrEmpty(mSearchExpression))
			{
				AddParameter("q", mUseSearch);
			}
			sb.Append("g.aspx");
			return sb.ToString();
		}

		private void AddParameter(string name, int value)
		{
			if (value > 0)
			{
				sb.Append(name + value + "-");
			}
		}

		private string GetSearch()
		{
			if (!string.IsNullOrEmpty(this.mSearchExpression))
			{
				return "?q=" + System.Web.HttpUtility.UrlEncode(this.mSearchExpression);
			}
			else
			{
				return "";
			}
		}

		#endregion

		public override string ToString()
		{
			return GetParametersString();
		}

	}
}
