﻿using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Collections.Generic;
using Ice.Net.Ecommerce.BusinessObjects.Database.VISTACART;

namespace Ice.Net.Ecommerce.BusinessObjects.Common
{
	internal sealed class LicenseManager
	{
		private BusinessObjects.Database.VISTACART.LicenseInfo mLicenseInfo = null;

		private LicenseManager()
		{
			mLicenseInfo = BusinessObjects.Common.LicenseHelper.LoadLicenseInfoFromWebConfig();
		}

		private static LicenseManager mLicenseManager = null;

		internal static LicenseManager Current
		{
			get
			{
				if (mLicenseManager == null)
				{
					mLicenseManager = new LicenseManager();

				}
				return mLicenseManager;
			}
		}

		private string CurrentSite
		{
			get { return HttpContext.Current.Request.ServerVariables["SERVER_NAME"].Replace("www.", ""); }
		}

		private bool IsCurrentSiteLicensed()
		{
			return TestIsSiteLicensed(CurrentSite);
		}

		private bool TestIsSiteLicensed(string site)
		{
			string url = site.ToLower();

			if (url == "localhost")
			{
				return true;
			}

			if (url.StartsWith("www."))
			{
				url = url.Substring(4);
			}

			foreach (LicensedSite licSite in Current.mLicenseInfo.GetLicensedSiteCollection())
			{
				if (licSite.SiteAddress.ToLower().Trim() == url)
				{
					return true;
				}
			}
			return false;
		}

		private bool? mIsLocalhost;
		private bool IsLocalhost
		{
			get
			{
				if (mIsLocalhost.HasValue != true)
				{
					mIsLocalhost = CurrentSite.ToLower() == "localhost";
				}
				return mIsLocalhost.Value;
			}
		}

		private bool? mIsSiteLicensed;
		private bool IsSiteLicensed
		{
			get
			{
				if (mIsSiteLicensed.HasValue != true)
				{
					mIsSiteLicensed = IsCurrentSiteLicensed();
				}
				return mIsSiteLicensed.Value;
			}
		}

		private bool IsLicenseKeyFound
		{
			get{ return Current != null && Current.mLicenseInfo != null;}
		}

		private bool IsTrialLicense
		{
			get {return Current.mLicenseInfo.IsTrial;	}
		}

		private int TrialDaysLeft
		{
			get{return (int)(Current.mLicenseInfo.EndDate.GetValueOrDefault() - DateTime.Now).TotalDays;}
		}

		private static DateTime mStartWarning = DateTime.Now.AddSeconds(30);
		private bool ShowWarning
		{
			get { return mStartWarning < DateTime.Now; }
		}

		private string SiteToBeLicensed()
		{
			if (IsLocalhost)
			{
				return "";
			}
			else
			{
				return "<br/>Site to be licensed: <b>" + CurrentSite + "</b><br/>http(s)://(www.)<input type='text' value='" + CurrentSite + "'/>/";
			}
		}

		public void TestMe(bool showLeft)
		{
			TestLicenseInternal(showLeft);
		}

		private void TestLicenseInternal(bool showDaysLeft)
		{

			return;// license test is disabled

			if (IsLocalhost)
			{
				return; // always works with localhost
			}

			if (IsLicenseKeyFound != true)
			{
				ShowInfoBox("License key not found or invalid.<br/>Trial license key or full version license key may be ordered <a href='http://www.iceasp.net/?p=licensekey'>HERE</a>" + SiteToBeLicensed(), true);
			}
			else if (LicenseManager.Current.IsTrialLicense)
			{
				if (LicenseManager.Current.TrialDaysLeft < 1 || LicenseManager.Current.TrialDaysLeft > 17)
				{
					ShowInfoBox("Trial license key expired. Expired trial license works on localhost only.<br/>Order full version license key <a href='http://www.iceasp.net/?p=licensekey'>HERE</a>" + SiteToBeLicensed(), false);
					if (IsLocalhost != true)
					{
						System.Web.HttpContext.Current.Response.End();
					}
				}
				else if (showDaysLeft)
				{
					ShowInfoBox(String.Format("Trial license, {0} day(s) left.<br/>Order full version license key <a href='http://www.iceasp.net/?p=licensekey'>HERE</a>" + SiteToBeLicensed(), LicenseManager.Current.TrialDaysLeft), false);
				}
			}
			else if (IsSiteLicensed != true)
			{
				// not trial key, current site is not licensed
				ShowInfoBox(String.Format("Current web site ({0}) is not licensed.<br/>Order full version license key <a href='http://www.iceasp.net/?p=licensekey'>HERE</a>" + SiteToBeLicensed(), CurrentSite), true);
			}
			else
			{
				// full license key

			}
		}

		private void ShowInfoBox(string message, bool endResponse)
		{
			System.Web.HttpContext.Current.Response.Write("<div style='background:white;z-index:10000;font-size:14px; position:absolute;top:2px; left:2px; border:solid 2px gray; padding:2px;'>" + message + "</div>");
			if (endResponse)
			{
				System.Web.HttpContext.Current.Response.End();
			}

		}

	
	}
}