﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Ice.Net.Ecommerce.BusinessObjects.Common
{
	public class SSLModeUtility
	{
		public enum SSLModeEnum
		{
			/// <summary>
			/// SSL is not used
			/// </summary>
			NoSSL = 0,

			/// <summary>
			/// <para>If specific page is accessed without SSL, it will be redirected to SSL</para>
			/// </summary>
			IfRequired = 1,

			/// <summary>
			/// <para>Access homepage without SSL, but create all links with SSL (https:// prefix)</para>
			/// </summary>
			CreateSSLLinks = 2,

			/// <summary>
			/// Always use SSL, if page is accessed without SSL http:// , it will be redirected to SSL https://
			/// </summary>
			ForceAlways = 3
		}

		public static bool UseSSL
		{
			get {return BusinessObjects.Common.VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.SSLMode != SSLModeEnum.NoSSL;		}
		}

		public static bool IsRedirectToHttpsRequired
		{
			get
			{
				string url = System.Web.HttpContext.Current.Request.Url.ToString().ToLower();

				if (SSLModeUtility.UseSSL && url.StartsWith("https://") == false)
				{
					return true;
				}

				return false;
			}
		}

		/// <summary>
		/// Returns SSL link to redirect to if required, else return null
		/// </summary>
		/// <returns></returns>
		public static string GetRedirectUrlIfRequired()
		{
			string url = System.Web.HttpContext.Current.Request.Url.ToString().ToLower();

			if (SSLModeUtility.UseSSL && url.StartsWith("https://") == false)
			{
				return "https://" + url.Substring(7);
			}

			return null;

		}

		public static void CheckForRedirect()
		{
			string url = System.Web.HttpContext.Current.Request.Url.ToString();

			if (!string.IsNullOrEmpty(VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.WebSite) && url.Contains(VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.WebSite) == false)
			{
				// redirect to web site
				System.Web.HttpContext.Current.Response.Redirect("http://" + VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.WebSite);
			}
			if (VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.SSLMode == SSLModeUtility.SSLModeEnum.NoSSL && url.StartsWith("https://"))
			{
				System.Web.HttpContext.Current.Response.Redirect("http://" + System.Web.HttpContext.Current.Request.Url.ToString().Substring(8));
			}
			if (VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.SSLMode == SSLModeUtility.SSLModeEnum.ForceAlways && url.StartsWith("http://"))
			{
				// redirect to SSL always
				System.Web.HttpContext.Current.Response.Redirect("https://" + System.Web.HttpContext.Current.Request.Url.ToString().Substring(7));
			}

		}

		public static void RedirectIfRequired()
		{
			string url = System.Web.HttpContext.Current.Request.Url.ToString().ToLower();

			if (SSLModeUtility.UseSSL && url.StartsWith("https://") == false)
			{
				// redirect to SSL
				System.Web.HttpContext.Current.Response.Redirect(System.Web.HttpContext.Current.Request.Url.ToString().Replace("http://", "https://"));
			}
		}

	}
}
