﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Data.SqlClient;
using Ice.Orm.Core.Entity;
using Ice.Orm.Core.Criteria;
using Ice.Orm.Core.DataStore;
using System.Data;
using System.Linq;

namespace Ice.Net.Ecommerce.BusinessObjects.Database.VISTACART
{
	public class CampaignHelper
	{
		private Campaign mCampaign = new Campaign();
		private ProductInCampaign mProductInCampaign = new ProductInCampaign();
		private Product mProduct = null;

		public Campaign Campaign
		{
			get { return mCampaign; }
		}

		public Product Product
		{
			get { return mProduct; }
		}

		public CampaignHelper(Product product)
		{
			mProduct = product;
			this.LoadCurrentCampaignByProduct(product);
		}

		/// <summary>
		/// <para>Load currently active campaign</para>
		/// <para>if there is more than 1 campaign active, the top one is used (sorted by Ordinal DESC)</para>
		/// </summary>
		/// <param name="product"></param>
		/// <returns></returns>
		private void LoadCurrentCampaignByProduct(Product product)
		{
			Campaign camp = new Query()
			.Select(new SqlName(db.Campaign + ".*"))
			.From(db.Campaign)
			.InnerJoin(db.ProductInCampaign, db.Campaign.CampaignID, db.ProductInCampaign.CampaignID)
			.Where(db.ProductInCampaign.ProductID, product.ProductID)
			.Where(db.Campaign.IsActive, true)
				// date interval
			.Where(db.Campaign.StartDate, null, DBWhereKeyOperator.IsNULL, DBWhereKeyLogic.And, DBWhereBrackets.OpenDouble)
			.Where(db.Campaign.StartDate, DateTime.Now, DBWhereKeyOperator.LessThanOrEquals, DBWhereKeyLogic.Or, DBWhereBrackets.Close)
			.Where(db.Campaign.EndDate, null, DBWhereKeyOperator.IsNULL, DBWhereKeyLogic.Or, DBWhereBrackets.Open)
			.Where(db.Campaign.EndDate, DateTime.Now, DBWhereKeyOperator.GreaterThanOrEquals, DBWhereKeyLogic.Or, DBWhereBrackets.CloseDouble)
				// order by
			.OrderBy(db.Campaign.Ordinal, true)
			.Take(1)
			.ToList<Campaign>()
			.FirstOrDefault();

			if (camp != null)
			{
				mCampaign = camp;

				if (mCampaign.IsDiscountPriceUsed)
				{
					mIsCampaignValid = true;
				}
				else if (mCampaign.DiscountPercentage > 0)
				{
					mIsCampaignValid = true;
				}
			}
		}

		private bool mIsCampaignValid = false;
		/// <summary>
		/// If true, valid active campaign found for current product
		/// </summary>
		public bool IsCampaignValid
		{
			get { return mIsCampaignValid; }
		}

		public bool ApplyToCurrentProduct
		{
			get{ return (mCampaign.ApplyToNextNProducts > 0) != true;}
		}

		private decimal GetDiscountPrice(decimal price, decimal discountPercent)
		{
			return (decimal) Math.Round((price - (price * discountPercent / 100)), 2);
		}

		/// <summary>
		/// Tax amount based on SalePrice (based on current campaign)
		/// </summary>
		public decimal GetTaxAmount()
		{
			return Math.Round(((this.GetSalePrice() * BusinessObjects.Common.VISTACARTHelper.AppDefaults.Tax.RatePercent / 100)), 2);
		}

		/// <summary>
		/// SalePrice based on campaign or product if campaign is not valid. TaxAmount is not included!
		/// </summary>
		public decimal GetSalePrice()
		{
			if (this.IsCampaignValid && this.ApplyToCurrentProduct)
			{
				if (mCampaign.IsDiscountPriceUsed)
				{
					return Math.Round(mCampaign.DiscountPrice, 2);
				}
				else
				{
					return (decimal)Math.Round(GetDiscountPrice(mProduct.SalePrice, mCampaign.DiscountPercentage), 2);
				}
			}
			else
			{
				// no active campaign for current product
				return Math.Round( mProduct.SalePrice, 2);
			}
		}

		/// <summary>
		/// Get price for specific product, but using currently loaded campaign
		/// </summary>
		/// <param name="product"></param>
		/// <returns></returns>
		public decimal GetSalePrice(Product product)
		{
			if(this.IsCampaignValid)
			{
				if(mCampaign.IsDiscountPriceUsed)
				{
					return Math.Round(mCampaign.DiscountPrice, 2);
				}
				else
				{
					return (decimal)Math.Round(GetDiscountPrice(product.SalePrice, mCampaign.DiscountPercentage), 2);
				}
			}
			else
			{
				// no active campaign for current product
				return Math.Round(product.SalePrice, 2);
			}
		}

		/// <summary>
		/// Tax amount for specific product, based on SalePrice (based on current campaign)
		/// </summary>
		public decimal GetTaxAmount(Product product)
		{
			return Math.Round(((this.GetSalePrice(product) * BusinessObjects.Common.VISTACARTHelper.AppDefaults.Tax.RatePercent / 100)), 2);
		}

	}


}
