﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Data.SqlClient;
using Ice.Orm.Core.Entity; using Ice.Orm.Core.Criteria;
using System.Data;
using System.Linq;


namespace Ice.Net.Ecommerce.BusinessObjects.Database.VISTACART
{
	public partial class Campaign 
	{
		#region Public properties

		private ProductInCampaignCollection mProductInCampaignCollection = null;

		public ProductInCampaignCollection GetProductInCampaignCollection()
		{
			if (mProductInCampaignCollection == null)
			{
				mProductInCampaignCollection = new ProductInCampaignCollection();
				mProductInCampaignCollection.AddRange(ProductInCampaignCollection.LoadByCampaignID(this.CampaignID));
			}
			return mProductInCampaignCollection;
		}

		private ProductCollection mProductCollection = null;
		/// <summary>
		/// Products included into campaign
		/// </summary>
		public ProductCollection GetProductCollection()
		{
			if (mProductCollection == null)
			{
				mProductCollection = new ProductCollection();
				mProductCollection.AddRange(ProductCollection.LoadByCampaignID(this.CampaignID));
			}
			return mProductCollection;
		}

		private WebPageCollection mWebPageCollection = null;
		/// <summary>
		/// Load all web pages from db and keep it in local variable.
		/// </summary>
		/// <returns></returns>
		public WebPageCollection GetWebPageCollection()
		{
			if (mWebPageCollection == null)
			{
				mWebPageCollection = new WebPageCollection();
				mWebPageCollection.AddRange(WebPageCollection.LoadCollectionByCampaignID(this.CampaignID, false));
			}
			return mWebPageCollection;
		}

		public bool IsPublished
		{
			get
			{
				if (StartDate == null && (EndDate == null || EndDate > DateTime.Now))
				{
					return true;
				}
				else if (StartDate < DateTime.Now && (EndDate == null || EndDate > DateTime.Now))
				{
					return true;
				}
				else
				{
					return false;
				}
			}
		}

		#endregion

		#region Public methods

		/// <summary>
		/// Add product to this campaign.
		/// </summary>
		public bool AddProduct(Product product)
		{
			if (product.IsInCampaign(this) == false)
			{
				// force reloading
				mProductInCampaignCollection = null;
				mProductCollection = null;

				ProductInCampaign productInCampaign = new ProductInCampaign();
				productInCampaign.CampaignID = this.CampaignID;
				productInCampaign.ProductID = product.ProductID;
				productInCampaign.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
				return true;
			}
			return false;
		}

		/// <summary>
		/// Remove product from this campaign
		/// </summary>
		public bool RemoveProduct(Product product)
		{
			mProductCollection = null;
			ProductInCampaign.DeleteByProductIDAndCampaignID(product.ProductID, this.CampaignID);
			return true;
		}

		public bool AddWebPage(WebPage webPage)
		{
			if (webPage.IsInCampaign(this) == false)
			{
				WebPageInCampaign webPageInCampaign = new WebPageInCampaign();
				webPageInCampaign.CampaignID = this.CampaignID;
				webPageInCampaign.WebPageID = webPage.WebPageID;
				webPageInCampaign.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
				return true;
			}
			return false;
		}

		public bool RemoveWebPage(WebPage webPage)
		{
			return WebPageInCampaign.DeleteByWebPageIDAndCampaignID(webPage.WebPageID, this.CampaignID);
		}

		#endregion

		#region Database methods

		public override bool DeleteFromDB()
		{
			GetDataStore().UseTransaction(DeleteCallback);
			return true;
		}

		private void DeleteCallback()
		{
			if (this.GetProductCollection().Count > 0 || this.GetWebPageCollection().Count > 0)
			{
				throw new Exception("Please remove products and web pages from campaign first!");
			}

			base.DeleteFromDB();
			return;
		}

		/// <summary>
		/// Load currently active and published campaign.
		/// </summary>
		/// <returns></returns>
		public static Campaign LoadCurrentCampaign()
		{
			return new Query().CreateQuery(@"SELECT * FROM {5} WHERE {0} = {1} AND (({2} IS NULL OR {2} <= {3}) AND ({4} IS NULL OR {4} >= {3})) ORDER BY {6} DESC",
					db.Campaign.IsActive,
					true,
					db.Campaign.StartDate,
					DateTime.Now,
					db.Campaign.EndDate,
					db.Campaign,
					db.Campaign.Ordinal)
					.Take(1)
					.ToList<Campaign>()
					.FirstOrDefault();
		}

		#endregion

	}

}
