﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Data.SqlClient;
using Ice.Orm.Core.Entity; using Ice.Orm.Core.Criteria;

namespace Ice.Net.Ecommerce.BusinessObjects.Database.VISTACART
{
	public partial class CategoryCollection : List<Category>
	{
		#region Database methods
		// Database methods

		public static List<Category> LoadByProductID(int productID)
		{
			return new Query()
			.Select(new SqlName(db.Category + ".*"))
			.From(db.Category)
			.InnerJoin(db.ProductInCategory, db.Category.CategoryID, db.ProductInCategory.CategoryID)
			.Where(db.ProductInCategory.ProductID, productID)
			.ToList<Category>();
		}

		/// <summary>
		/// <para>Load ordered. Active only if activeOnly == true and  Order by Ordinal ASC than by CategoryID ASC if sortByOrdinalAsc == true</para> 
		/// <para>The most fresh category is on top.</para>
		/// </summary>
		/// <param name="parentCategoryID"></param>
		/// <returns></returns>
		public static List<Category> LoadByParentCategoryID(int parentCategoryID, bool activeOnly, bool sortByOrdinalAsc)
		{
			return new Query()
			.Case(parentCategoryID > 0, q => q.Where(db.Category.ParentCategoryID, parentCategoryID))
			.Case(parentCategoryID <= 0, q => q.Where(db.Category.ParentCategoryID, null))
			.Case(activeOnly, q => q.Where(db.Category.IsActive, true))
			.OrderBy(db.Category.Ordinal, sortByOrdinalAsc ? false : true)
			.Take(1000)
			.ToList<Category>();
		}

		/// <summary>
		/// Load categories chain. if sortRootCategoryFirst == true, First element is root category, last element is currentCategoryID.
		/// </summary>
		public int LoadCategoriesChain(int? currentCategoryID, int? rootCategoryID, bool rootCategoryFirst)
		{
			int? categoryIdToLoad = currentCategoryID;

			while (true)
			{
				Category category = new Category();
				if (category.LoadFromDB(categoryIdToLoad))
				{
					this.Add(category);
					categoryIdToLoad = category.ParentCategoryID;
				}
				else
				{
					break;
				}
				if (category.CategoryID == rootCategoryID)
				{
					break;
				}
			}
			if (rootCategoryFirst)
			{
				this.Reverse();
			}
			return this.Count;
		}

		public static List<Category> LoadByShortcutName(string shortcutName, bool activeOnly)
		{
			return new Query()
			.Where(db.Category.ShortcutName, shortcutName)
			.Case(activeOnly, q => q.Where(db.Category.IsActive, activeOnly))
			.Take(100)
			.ToList<Category>();
		}

		#endregion

		#region public methods

		public Category FindByCategoryID(int categoryId)
		{
			foreach (Category cat in this)
			{
				if (cat.CategoryID == categoryId)
				{
					return cat;
				}
			}
			return null;
		}

		public bool Contains(int categoryId)
		{
			return FindByCategoryID(categoryId) != null;
		}


		#endregion
	}
}
