﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Data.SqlClient;
using Ice.Orm.Core.Entity; using Ice.Orm.Core.Criteria;
using System.Linq;

namespace Ice.Net.Ecommerce.BusinessObjects.Database.VISTACART
{
	public partial class Category 
	{
		public enum Shortcut
		{
			home,
			newproducts,
			specials,
			myaccount,
			personaldata,
			login,
			newaccount,
			passwordrecovery,
			orderslist,
			company,
			privacy,
			aboutus,
			terms,
			contactus,

			showcart,
			checkoutaddress,
			checkoutpayment,
			ordercomplete,

			wishlist,
			sitemap,

			search

		}

		#region Public properties

		private ProductCollection mProductCollection = null;
		/// <summary>
		/// Load all products on-demand from db and keep it in local variable.
		/// </summary>
		public ProductCollection GetProductCollection()
		{
			if (mProductCollection == null)
			{
				mProductCollection = new ProductCollection();
				mProductCollection.AddRange(ProductCollection.LoadByCategoryID(this.CategoryID));
			}
			return mProductCollection;
		}

		private WebPageCollection mWebPageCollection = null;
		/// <summary>
		/// Load all web pages from db and keep it in local variable.
		/// </summary>
		/// <returns></returns>
		public WebPageCollection GetWebPageCollection()
		{
			if (mWebPageCollection == null)
			{
				mWebPageCollection = new WebPageCollection();
				mWebPageCollection.AddRange(WebPageCollection.LoadCollectionByCategoryID(this.CategoryID, false));
			}
			return mWebPageCollection;
		}

		private Category mParentCategory = null;
		/// <summary>
		/// Load on-demand and keep in local variable.
		/// </summary>
		public Category GetParentCategory()
		{
				if (mParentCategory == null)
				{
					mParentCategory = new Category();
					mParentCategory.LoadFromDB(this.ParentCategoryID);
				}
				return mParentCategory;
		}

		public bool IsPublished
		{
			get
			{
				if(PublishDate == null && (CloseDate == null || CloseDate > DateTime.Now))
				{
					return true;
				}
				else if (PublishDate < DateTime.Now && (CloseDate == null || CloseDate > DateTime.Now))
				{
					return true;
				}
				else
				{
					return false;
				}
			}
		}

		#endregion

		#region Database methods overrides

		/// <summary>
		/// Delete this and it's CategoryInLanguageItems
		/// </summary>
		/// <returns></returns>
		public override bool DeleteFromDB()
		{
			GetDataStore().UseTransaction(this.DeleteCallback);
			return true;
		}

		private void DeleteCallback()
		{
			if (this.GetProductCollection().Count > 0 || this.GetWebPageCollection().Count > 0)
			{
				throw new Exception("Please remove products and web pages from category first!");
			}

			// delete this
			base.DeleteFromDB();
		}

		#endregion

		#region Public methods

		/// <summary>
		/// Add product to current category.
		/// </summary>
		/// <param name="product"></param>
		/// <returns></returns>
		public bool AddProduct(Product product)
		{
			// force reloading
			this.mProductCollection = null;
			return product.AddCategory(this);
		}
		/// <summary>
		/// Remove product from current category.
		/// </summary>
		/// <param name="product"></param>
		/// <returns></returns>
		public bool RemoveProduct(Product product)
		{
			// force reloading
			this.mProductCollection = null;
			return product.RemoveCategory(this);
		}

		public bool AddWebPage(WebPage webPage)
		{
			if (webPage.IsLoadedFromDB() && webPage.IsInCategory(this) == false)
			{
				// add new 
				WebPageInCategory dcInCategory = new WebPageInCategory();
				dcInCategory.CategoryID = this.CategoryID;
				dcInCategory.WebPageID = webPage.WebPageID;
				dcInCategory.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
				return true;
			}
			return false;
		}

		public bool RemoveWebPage(WebPage webPage)
		{
			return WebPageInCategory.DeleteByWebPageIDAndCategoryID(webPage.WebPageID, this.CategoryID);
		}

		private List<Category>mChildrenCategoryCollection = null;
		/// <summary>
		/// Load on-demand and keep in local variable.
		/// </summary>
		public List<Category> GetChildrenCategoryCollection(bool activeOnly, bool sortByOrdinalAsc)
		{
			if (mChildrenCategoryCollection == null)
			{
				mChildrenCategoryCollection = CategoryCollection.LoadByParentCategoryID(this.CategoryID, activeOnly, sortByOrdinalAsc);
			}
			return mChildrenCategoryCollection;
		}

		public static Category LoadByShortcutName(string shortcutName)
		{
			if (BusinessObjects.Common.VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.RootCategoryID > 0)
			{
				// load all Category objects and use only first one where root node is current root node
				List<Category> collection = CategoryCollection.LoadByShortcutName(shortcutName, true);

				if (collection.Count > 0)
				{
					foreach (Category category in collection)
					{
						CategoryCollection chain = new CategoryCollection();
						chain.LoadCategoriesChain(category.CategoryID, BusinessObjects.Common.VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.RootCategoryID, true);
						if (chain.Contains(BusinessObjects.Common.VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.RootCategoryID.GetValueOrDefault()))
						{
							// ok, Category found by webActionName belongs to the currently active tree
							return category;
						}
					}
				}
			}
			else
			{
				return new Query().Where(db.Category.ShortcutName, shortcutName).ToList<Category>().FirstOrDefault();
			}
			return null;
		}

		#endregion

	}

}
