﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Data.SqlClient;
using Ice.Orm.Core.Entity; using Ice.Orm.Core.Criteria;

namespace Ice.Net.Ecommerce.BusinessObjects.Database.VISTACART
{
	public partial class OrderDetailCollection : List<OrderDetail>
	{
		/// <summary>
		/// OrderDetail collection to be removed from database at save time
		/// </summary>
		List<OrderDetail> mRemoveFromDB = new List<OrderDetail>();

		/// <summary>
		/// OrderDetail object to be removed from database at saveToDB() - if Quantity ==0, for example
		/// </summary>
		/// <param name="detail"></param>
		public void ToBeRemovedFromDB(OrderDetail detail)
		{
			mRemoveFromDB.Add(detail);
		}

		#region Database methods
		// Database methods

		public int CountByProductID(int productId)
		{
			return new Query().Count(db.OrderDetail)
				.Where(db.OrderDetail.ProductID, productId)
				.ExecuteScalar<OrderDetail, int>();
		}

		public int SaveToDB()
		{
			int saved = 0;
			foreach (OrderDetail detail in this)
			{
				if (detail.Quantity < 1)
				{
					mRemoveFromDB.Add(detail);
				}
				else
				{
					detail.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
					saved++;
				}
			}
			foreach (OrderDetail detail in mRemoveFromDB)
			{
				detail.DeleteFromDB();
			}
			return saved;
		}

		public static List<OrderDetail> LoadByOrderID(int orderID)
		{
			return new Query()
			.Where(db.OrderDetail.OrderID, orderID)
			.Take(1000)
			.ToList<OrderDetail>();
		}

		#endregion

		private OrderDetail FindByProductIDAndAttributes(int? productID, string attributes)
		{
			foreach (OrderDetail detail in this)
			{
				if (detail.ProductID == productID && detail.Attributes == attributes)
				{
					return detail;
				}
			}
			return null;
		}

		private void RemoveByOrderDetailID(int orderDetailID)
		{
			for (int n = 0; n < this.Count; n++)
			{
				if (this[n].OrderDetailID == orderDetailID)
				{
					mRemoveFromDB.Add(this[n]);
					this.RemoveAt(n);

					break;
				}
			}
		}

		/// <summary>
		/// Set product quantity to an existing order detail object.
		/// </summary>
		/// <param name="productID">ProductID to adjust</param>
		/// <param name="quantity">Quantity to set</param>
		/// <returns></returns>
		public int SetQuantity(int? detailIndex, int? quantity)
		{
			if (this.Count > detailIndex)
			{
				if(quantity.GetValueOrDefault() < 1)
				{
					RemoveAtDetailIndex(detailIndex);
					return 0;
				}
				this[detailIndex.GetValueOrDefault()].Quantity = quantity.GetValueOrDefault();
			}
			return quantity.GetValueOrDefault();
		}

		public int AddProduct(int? productID, int? quantity, string attributes)
		{
			if (Ecommerce.BusinessObjects.Common.VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.SingleCartItem.GetValueOrDefault())
			{
				return AddProduct2(productID, quantity, attributes);
			}
			else
			{
				return AddProduct3(productID, quantity, attributes);
			}
		}

		/// <summary>
		/// Pozitive quantity to add, negative quantity to remove quantity from collection.
		/// </summary>
		/// <param name="productID"></param>
		/// <param name="quantity"></param>
		/// <param name="attributes"></param>
		/// <returns></returns>
		public int AddProduct2(int? productID, int? quantity, string attributes)
		{
			OrderDetail detail = FindByProductIDAndAttributes(productID, attributes);
			////if (detail != null)
			////{
			////    // product already exist in cart, change it's quantity only
			////    detail.Quantity += quantity;
			////    if (detail.Quantity < 1)
			////    {
			////        // remove OrderDetail from collection
			////        RemoveByOrderDetailID(detail.OrderDetailID);
			////        return 0;
			////    }
			////    else
			////    {
			////        return detail.Quantity;
			////    }
			////}

			if (detail != null)
			{
				// product already exist in cart
				int newQuantity = detail.Quantity + quantity.GetValueOrDefault();
				if (newQuantity < 1)
				{
					// remove OrderDetail from collection
					RemoveByOrderDetailID(detail.OrderDetailID);
					return 0;
				}
			}
			
			if(quantity > 0)
			{
				// add new OrderDetail, recalculate redundant values, than merge with previous OrderDetail if values are the same
				for (int nn = 0; nn < quantity; nn++)
				{
					OrderDetail det = new OrderDetail(productID, 1, attributes);
					if (det.ProductID > 0)
					{
						// product with ProductID loaded successfuly
						this.Add(det);
					}
				}
				return quantity.GetValueOrDefault();
			}
			return 0;
		}

		/// <summary>
		/// Pozitive quantity to add, negative quantity to remove quantity from collection.
		/// </summary>
		/// <param name="productID"></param>
		/// <param name="quantity"></param>
		/// <param name="attributes"></param>
		/// <returns></returns>
		public int AddProduct3(int? productID, int? quantity, string attributes)
		{
			OrderDetail detail = FindByProductIDAndAttributes(productID, attributes);
			if (detail != null)
			{
				detail.Quantity += quantity.GetValueOrDefault();
				if (detail.Quantity < 1)
				{
					// remove OrderDetail from collection
					RemoveByOrderDetailID(detail.OrderDetailID);
					return 0;
				}
				else
				{
					return detail.Quantity;
				}
			}
			else if (quantity > 0)
			{
				detail = new OrderDetail(productID, quantity, attributes);
				if (detail.ProductID > 0)
				{
					// product with ProductID loaded successfuly
					this.Add(detail);
				}
				return detail.Quantity;
			}
			return 0;
		}


		public int RemoveAtDetailIndex(int? detailIndex)
		{
			if(this.Count > detailIndex)
			{
				OrderDetail detail = this[detailIndex.GetValueOrDefault()];
				this.RemoveAt(detailIndex.GetValueOrDefault());
				mRemoveFromDB.Add(detail);
			}
			return 0;
		}
	}
}
