﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Collections;
using System.Data.SqlClient;
using Ice.Orm.Core.Entity;
using Ice.Orm.Core.Criteria;
using Ice.Orm.Core.DataStore;
using System.Linq;

namespace Ice.Net.Ecommerce.BusinessObjects.Database.VISTACART
{
	public partial class UserCollection
	{
		#region Database methods
		// Database methods

		public int CountTotalUsers()
		{
			return new Query().Count(db.User).ExecuteScalar<User, int>();
		}

		public static int CountByRoleID(int roleID)
		{
			Query query = new Query()
			.Count(db.User)
			.InnerJoin(db.UserInRole, db.User.UserID, db.UserInRole.UserInRoleID)
			.Where(db.UserInRole.RoleID, roleID);

			return query.ExecuteScalar<User, int>();
		}

		public static List<User> LoadPagedByRoleID(int roleID, int recordsPerPage, int currentPageIndex, object orderByColumn, bool orderAsc, bool activeOnly)
		{
			return new Query()
			.Select(new SqlName(db.User + ".*"))
			.From(db.User)
			.InnerJoin(db.UserInRole, db.User.UserID, db.UserInRole.UserID)
			.Where(db.UserInRole.RoleID, roleID)
			.Case(activeOnly, o => o.Where(new SqlName("IsActive"), true))
			.Case(orderByColumn != null, o => o.OrderBy(new SqlName("User." + orderByColumn), orderAsc ? false : true))
			.Skip(recordsPerPage * currentPageIndex)
			.Take(recordsPerPage)
			.ToList<User>();
		}

		/// <summary>
		/// Get user's emails only
		/// </summary>
		/// <param name="roleID"></param>
		/// <param name="newsSubscribedOnly"></param>
		/// <returns></returns>
		public List<String> GetEmails(int roleID, bool newsSubscribedOnly)
		{
			return new Query()
				.Select(db.User.Email)
				.From(db.User)
				.Case(roleID > 0, o => o.InnerJoin(db.UserInRole, db.User.UserID, db.UserInRole.UserID))
				.Case(newsSubscribedOnly, o => o.Where(db.User.Newsletter, true))
				.Case(roleID > 0, o => o.Where(db.UserInRole.RoleID, roleID))
				.ToList<User>()
				.Select(o => o.Email)
				.ToList();
		}

		#endregion
	}
}
