﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Data.SqlClient;
using Ice.Orm.Core.Entity; 
using Ice.Orm.Core.Criteria;

namespace Ice.Net.Ecommerce.BusinessObjects.Database.VISTACART
{
	public partial class WebPageCollection : List<WebPage>
	{
		#region Database methods

		public static List<WebPage> LoadCollectionByCategoryID(int categoryID, bool activeOnly)
		{
			return new Query()
			.Select(new SqlName(db.WebPage + ".*"))
			.From(db.WebPage)
			.InnerJoin(db.WebPageInCategory, db.WebPage.WebPageID, db.WebPageInCategory.WebPageID)
			.Where(db.WebPageInCategory.CategoryID, categoryID)
			.Case(activeOnly, o => o.Where(db.Message.IsActive, true))
			.ToList<WebPage>();
		}

		public static List<WebPage> LoadCollectionByCampaignID(int campaignID, bool activeOnly)
		{
			return new Query()
			.Select(new SqlName(db.WebPage + ".*"))
			.From(db.WebPage)
			.InnerJoin(db.WebPageInCampaign, db.WebPage.WebPageID, db.WebPageInCampaign.WebPageID)
			.Where(db.WebPageInCampaign.CampaignID, campaignID)
			.Case(activeOnly, o => o.Where(db.Message.IsActive, true))
			.ToList<WebPage>();
		}

		public int CountTotalProducts()
		{
			return new Query().Count(db.WebPage).ExecuteScalar<WebPage, int>();
		}

		public static int CountByCategoryID(int categoryID, bool activeOnly)
		{
			Query query = new Query()
			.Count(db.WebPage)
			.InnerJoin(db.WebPageInCategory, db.WebPage.WebPageID, db.WebPageInCategory.WebPageID)
			.Where(db.WebPageInCategory.CategoryID, categoryID)
			.Case(activeOnly, o => o.Where(db.WebPage.IsActive, true));

			return query.ExecuteScalar<WebPage, int>();
		}

		public static int CountByCampaignID(int campaignID, bool activeOnly)
		{
			Query query = new Query()
			.Count(db.WebPage)
			.InnerJoin(db.WebPageInCampaign, db.WebPage.WebPageID, db.WebPageInCampaign.WebPageID)
			.Where(db.WebPageInCampaign.CampaignID, campaignID)
			.Case(activeOnly, o => o.Where(db.WebPage.IsActive, true));

			return query.ExecuteScalar<WebPage, int>();
		}

		public static List<WebPage> LoadPaged(int recordsPerPage, int currentPageIndex, object orderByColumn, bool orderAsc, bool activeOnly)
		{
			return new Query()
			.Select(new SqlName(db.WebPage + ".*"))
			.From(db.WebPage)
			.Case(activeOnly, o => o.Where(new SqlName("IsActive"), true))
			.Case(orderByColumn != null, o => o.OrderBy(new SqlName(orderByColumn + ""), !orderAsc))
			.Skip(recordsPerPage * currentPageIndex)
			.Take(recordsPerPage)
			.ToList<WebPage>();
		}

		public static List<WebPage> LoadPagedByCategoryID(int categoryID, int recordsPerPage, int currentPageIndex, object orderByColumn, bool orderAsc, bool activeOnly)
		{
			return new Query()
			.Select(new SqlName(db.WebPage + ".*"))
			.From(db.WebPage)
			.InnerJoin(db.WebPageInCategory, db.WebPage.WebPageID, db.WebPageInCategory.WebPageID)
			.Where(db.WebPageInCategory.CategoryID, categoryID)
			.Case(activeOnly, o => o.Where(db.WebPage.IsActive, true))
			.Case(orderByColumn != null, o => o.OrderBy(new SqlName(orderByColumn + ""), !orderAsc))
			.Skip(recordsPerPage * currentPageIndex)
			.Take(recordsPerPage)
			.ToList<WebPage>();
		}

		public static List<WebPage> LoadPagedByCampaignID(int campaignID, int recordsPerPage, int currentPageIndex, object orderByColumn, bool orderAsc, bool activeOnly)
		{
			return new Query()
			.Select(new SqlName(db.WebPage + ".*"))
			.From(db.WebPage)
			.InnerJoin(db.WebPageInCampaign, db.WebPage.WebPageID, db.WebPageInCampaign.WebPageID)
			.Where(db.WebPageInCampaign.CampaignID, campaignID)
			.Case(activeOnly, o => o.Where(db.WebPage.IsActive, true))
			.Case(orderByColumn != null, o => o.OrderBy(new SqlName(orderByColumn + ""), !orderAsc))
			.Skip(recordsPerPage * currentPageIndex)
			.Take(recordsPerPage)
			.ToList<WebPage>();
		}


		#endregion
	}
}
