﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Ice.Net.Ecommerce.BusinessObjects.Gallery
{
	public class GalleryImageCollection : System.Collections.Generic.List<GalleryImage>
	{
		public GalleryImageCollection()
		{ }

		public GalleryImageCollection(GalleryNode node)
		{
			LoadImages(node);
		}

		public static string GetFileName(string path)
		{
			return path.Substring(path.LastIndexOf("\\") + 1);
		}

		private void LoadImages(GalleryNode node)
		{
			string[] files = System.IO.Directory.GetFiles(node.GalleryPhysicalPath, "*", System.IO.SearchOption.TopDirectoryOnly);

			for (int nn = 0; nn < files.Length; nn++)
			{
				if(GalleryImage.IsSupported(files[nn]) == false)
				{
					// file type not supported - not an image
					continue;
				}

				GalleryImage image = new GalleryImage();
				System.Drawing.Bitmap bmp;
				try
				{
					bmp = new System.Drawing.Bitmap(files[nn]);
				}
				catch
				{
					// can't load image or wrong format
					continue;
				}

				image.height = (int)bmp.PhysicalDimension.Height;
				image.width = (int)bmp.PhysicalDimension.Width;

				double ratio = (double)image.width / (double)image.height;
				double thumbRatio = (double)80 / (double)60;
				if (ratio > thumbRatio)
				{
					// too width
					image.thumbWidth = 80;
					image.thumbHeight = (int)((double)image.thumbWidth / ratio);
				}
				else
				{
					// to height
					image.thumbHeight = 60;
					image.thumbWidth = (int)((double)image.thumbHeight * ratio);
				}
				image.marginLeft = (80 - image.thumbWidth) / 2;
				image.marginTop = (60 - image.thumbHeight) / 2;

				System.IO.FileStream fs = null;
				try
				{
					image.lastmod = System.IO.File.GetLastWriteTime(files[nn]).ToString("yyyy-MM-dd HH:mm:ss");
					fs = System.IO.File.OpenRead(files[nn]);
					image.size = (int)fs.Length;
				}
				catch
				{
					if (fs != null)
					{
						fs.Dispose();
					}
					continue;
				}
				string fileName = GetFileName(files[nn]);
				image.url = node.HttpGalleryPath + fileName;
				image.name = fileName;
				image.file = files[nn];

				// get relative http path
				image.relativeHttpPath = image.url.Substring(BusinessObjects.Common.ApplicationUtility.ApplicationHttpRootCurrent.Length);

				this.Add(image);

				if (fs != null)
				{
					fs.Dispose();
				}
				bmp.Dispose();
			}
		}
	}
}
