﻿
using System;
using System.Net;
using System.Net.Mail;
using System.Net.Mime;
using System.Threading;
using System.IO;
using System.Collections.Generic;

namespace Ice.Net.Ecommerce.EmailUtility
{
	public class EmailUtility
	{
		public static SmtpClient CreateSmtpClient(string server)
		{
			SmtpClient smtpClient = new SmtpClient(server);
			return smtpClient;
		}

		public static SmtpClient CreateSmtpClient(string server, int port, string username, string password)
		{
			SmtpClient smtpClient = new SmtpClient(server, port);
			// Command line argument must the the SMTP host.
			smtpClient.Credentials = new NetworkCredential(username, password);
			// Set the method that is called back when the send operation ends.
			// smtpClient.SendCompleted += new SendCompletedEventHandler(SendCompletedCallback);
			// The userState can be any object that allows your callback 
			// method to identify this send operation.
			// For this example, the userToken is a string constant.
			return smtpClient;
		}

		public static MailMessage CreateMailMessage(string fromEmail, string toEmail, string fromName, string subject, string bodyText, bool isBodyHtml, List<Attachment> attachments)
		{
			// Create a mailing address that includes a UTF8 character in the display name.
			MailAddress addressFrom = new MailAddress(fromEmail, fromName, System.Text.Encoding.UTF8);
			// Set destinations for the e-mail message.
			MailAddress addressTo = new MailAddress(toEmail);
			// Specify the message content.
			MailMessage message = new MailMessage(addressFrom, addressTo);
			message.IsBodyHtml = isBodyHtml;
			message.Body = bodyText;
			message.BodyEncoding = System.Text.Encoding.UTF8;
			message.Subject = subject;
			message.SubjectEncoding = System.Text.Encoding.UTF8;
			if (attachments != null)
			{
				foreach (Attachment attachment in attachments)
				{
					message.Attachments.Add(attachment);
				}
			}
			return message;
		}

		public static Attachment CreateAttachment(byte[] data, string fileName, string contentType)
		{
			MemoryStream ms = new MemoryStream(data);
			Attachment attachment = new Attachment(ms, new ContentType(contentType));
			attachment.ContentDisposition.FileName = fileName;
			return attachment;
		}

	}
}
