﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Web;
using Ice.Net.Ecommerce.BusinessObjects.Database.VISTACART;
using System.Text.RegularExpressions;
using Ice.Net.Ecommerce.PaymentGateways.PG;
using Ice.Net.Ecommerce.BusinessObjects.Common;
using System.Collections;
using Ice.Net.Ecommerce.OrderContentForEmail;

namespace Ice.Net.Ecommerce.PaymentGateways
{

	public class PGHelper
	{

		#region Common methods

#if !TRIAL
		private static void SendEmail(Order order)
		{
			string html = EmailBuilder.GetHtml(order);
			string mailBody = WebPage.GetContent(WebPage.NameEnum.PaymentDoneEmail).Replace("__html__", html);

			Address address = new Address();
			address.LoadFromDB(order.BillingAddressID);

			EmailUtility.EmailSenderImpl.SendEmails(
				Message.GetContent(Message.NameEnum.YourOrder),
				mailBody,
				true,
				null,
				BusinessObjects.Common.VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.FromEmail,
				BusinessObjects.Common.VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.FromName,
				new String[]{BusinessObjects.Common.VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.WebContactFormToEmail,
								address.Email});

		}
#endif

		#endregion

		#region PG redirection / CC input form

#if !TRIAL

		public static PGResult GetPGRedirectionFormWithParameters(ShoppingCart cart)
		{
			if (cart.Order.RedundantTotalProductsCount == 0)
			{
				return new PGResult(Message.GetContent(Message.NameEnum.CartIsEmpty), true);
			}

			Order order = cart.Order;
			BasePG pg = Ice.Net.Ecommerce.PaymentGateways.PGLoader.GetPG(order.PaymentTypeID);

			//*********************************
			// THROW AN ERROR IF PG IS NOT LOADED
			//*********************************
			if (pg == null)
			{
				return new PGResult("Invalid Payment Type!", false);
			}

			if (pg.AIMMethod)
			{
				// AIM METHOD, don't close cart untill it's really paid
				return pg.GetFormWithParameters(null);
			}
			else
			{
				// SIM METHOD, close cart before redirecting to payment gateway
				// close order - disable any more modification of cart items
				if (cart.CloseShoppingCart() == false)
				{
					return new PGResult(Message.GetContent(Message.NameEnum.ServerInternalError), null, null);
				}

				return pg.GetFormWithParameters(order);
			}
		}
#endif

		#endregion

		#region SIM methods

#if !TRIAL
		internal static PGStatus GetCallbackStatus(HttpContext context)
		{
			foreach (BasePG pg in PGLoader.GetActivePGs())
			{
				if (pg.AIMMethod != true)
				{
					PGStatus status = pg.GetReceivedStatus(context);
					if (status != null && status.PaymentType != null)
					{
						return status;
					}
				}
			}

			// request wasn't recognized - didn't come from any known payment gateway
			return null;
		}
#endif

		/// <summary>
		/// returnes null if request not valid, else returns Order object
		/// </summary>
		public static Order ValidatePGCallback(HttpContext context)
		{
#if !TRIAL
			PGStatus pgstatus = GetCallbackStatus(HttpContext.Current);

			if (pgstatus == null || pgstatus.PaymentType == null)
			{
				return null;
			}

			Order order = null;

			if (pgstatus.IsInfoOnly)
			{
				order = new Order();
				if (order.LoadFromDB(pgstatus.OrderID))
				{
					return order;
				}
				else
				{
					return null;
				}
			}

			if (pgstatus.PaymentType.IsDiagnostic == true)
			{
				//save diagnostic to App_Data\\Log\\DateTime.Now.Ticks_gatewayname.txt
				UtilityHelper.SaveDiagnostic(pgstatus.PaymentType.PGName, context);
			}

			// get order 
			if (pgstatus.IsSignatureOK && pgstatus.OrderID > 0)
			{
				order = new Order();
				if (order.LoadFromDB(pgstatus.OrderID) && order.OrderID == pgstatus.OrderID)
				{
					if (pgstatus.PaymentType.PGName == PGNameEnum._GoogleCheckout.ToString() && pgstatus.PaymentStatus != null) // google checkout
					{
						////order.PaymentConfirmedDate = DateTime.Now;// don't set it here, has to be null for first time email sending 
						order.TransactionId = pgstatus.TransactionId;
						order.IsActive = true;
						order.OrderStatus = Order.OrderStatusEnum.Ordered;
						order.PaymentStatus = pgstatus.PaymentStatus.GetValueOrDefault();
						// order is complete, save it to DB
						order.SaveToDBInternal(false);
					}
					else if (BusinessObjects.Common.VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.AutoEnableDownloads)
					{
						order.PaymentStatus = Order.PaymentStatusEnum.Approved;

						// order was paid, remove products from stock
						order.AdjustStockQuantity(true);
					}
					else
					{
						// disable downloading of files before the payment is confirmed by payment gateway
						// shop owner has to change status to Approwed or Paid to enable user to download files
						order.PaymentStatus = Order.PaymentStatusEnum.Waiting;
					}

					if (order.PaymentConfirmedDate == null || order.PaymentConfirmedDate < new DateTime(1900, 1, 1))
					{
						order.PaymentConfirmedDate = DateTime.Now;
						order.TransactionId = pgstatus.TransactionId;
						order.IsActive = true;
						order.OrderStatus = Order.OrderStatusEnum.Ordered;

						// order is complete, save it to DB
						if (order.SaveToDBInternal(false))
						{
							SendEmail(order); // send email on first callback only
						}
					}
					else
					{
						// order was already confirmed, it may be second callback - RBA for example
					}

					return order;

				}
				else
				{
					//received data is not valid
				}
			}

			return null;
#else
			throw new Exception("*********************************\r\nDISABLED IN TRIAL VERSION!\r\n*********************************");
#endif
		}

		#endregion

		#region AIM methods

		public static PGResult ChargeAIMMethod(CreditCardInfo ccInfo, ShoppingCart cart)
		{
#if !TRIAL

			if (cart.Order.RedundantTotalProductsCount == 0)
			{
				return new PGResult(Message.GetContent(Message.NameEnum.CartIsEmpty), true);
			}
			if (ccInfo.PaymentTypeID != cart.Order.PaymentTypeID)
			{
				return new PGResult(Message.GetContent(Message.NameEnum.InvalidPayementType), null, null);
			}


			PGResult result = null;

			BasePG pg = PGLoader.GetPG(cart.Order.PaymentTypeID); // load P

			//*********************************
			// THROW AN ERROR IF PG IS NOT LOADED
			//*********************************
			if (pg == null)
			{
				return new PGResult("Invalid Payment Type!", false);
			}

			if (pg.AIMMethod)
			{
				result = pg.ChargeAIMMethodWorker(ccInfo, cart.Order);
			}

			if (result == null)
			{
				// invalid data
				return new PGResult("Invalid data!", null, null);
			}
			else if (result.Success)
			{
				Order order = cart.Order;
				// charged, close order now, update order object
				order.TransactionId = result.TransactionId;
				order.IsActive = true;

				if (order.PaymentConfirmedDate == null || order.PaymentConfirmedDate < new DateTime(1900, 1, 1) || order.OrderStatus != Order.OrderStatusEnum.Ordered)
				{
					if (BusinessObjects.Common.VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.AutoEnableDownloads)
					{
						order.PaymentStatus = Order.PaymentStatusEnum.Approved;

						// order was paid, remove products from stock
						order.AdjustStockQuantity(true);
					}
					else
					{
						// disable downloading of files before the payment is confirmed by payment gateway
						// shop owner has to change status to Approwed or Paid to enable user to download files
						order.PaymentStatus = Order.PaymentStatusEnum.Waiting;
					}

					order.PaymentConfirmedDate = DateTime.Now;
					order.OrderStatus = Order.OrderStatusEnum.Ordered;

					// Remove ordered products from stock, close shopping cart.
					if (cart.CloseShoppingCart())
					{
						SendEmail(cart.Order);
					}
				}
				else
				{
					// order was already confirmed, it may be second callback
				}

				return new PGResult(null, pg.GetFinalForm(cart.Order.OrderID), null);

			}
			else
			{
				// CC not charged
				return result;
			}
#else
			throw new Exception("*********************************\r\nDISABLED IN TRIAL VERSION!\r\n*********************************");
#endif
		}

		#endregion


	}
}
