﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Web;
using Ice.Net.Ecommerce.BusinessObjects.Database.VISTACART;
using System.Text.RegularExpressions;
using Ice.Net.Ecommerce.PaymentGateways.PG;
using Ice.Net.Ecommerce.BusinessObjects.Common;
using System.Collections;

namespace Ice.Net.Ecommerce.PaymentGateways
{

	/**************************************************************************************
	 * NEW PAYMENT GATEWAY INTEGRATION
	 * 
	 * 1) add NewGateway name to PGNameEnum
	 * 2) add PGNewGateway class to PG directory, inherit from BasePG
	 * 3) add PGNewGateway type to PGLoader.mPGstypes[]
	 * 4) set correct property PGname value in PGNewGateway class
	 * 5) open administration -> Payment Types, add new db table row with correct PGname
	 * 
	*************************************************************************************/

	public enum PGNameEnum
	{
		_Invoice,
		_PayPal,
		_2CheckOut,
		_ChronoPay,
		_AuthorizeNetAIM,
		_FreePayDkAIM,
		_RaiffeisenHR,
		_WebtehHR,
		_PayWayHR,
		////_FreePayDkSIM,
		////_FreePayDkAIM,
		_GTPay,
		_GoogleCheckout
	}

	internal class PGLoader
	{
		static Type[] mPGstypes = new Type[]{
			typeof(PGGTPay),
			typeof(PGInvoice),
			typeof(PGPayPal),
			typeof(PG2CheckOut),
			typeof(PGChronoPay),
			typeof(PGAuthorizeNetAIM),
			////typeof(PGFreePayAIM),
			////typeof(PGFreePaySIM)
			typeof(PGRaiffeisenHR),
			typeof(PGWebtehHR),
			typeof(PGPayWayHR),
			typeof(PGGoogleCheckout)
		};

		static Hashtable mPGTypeByName = null;

		public static void Initialize()
		{
			// call on application start to invoke static PGLoader() constructor
		}

		static PGLoader() // static constructor // 
		{
			// initialize mPGTypeByName
			// add new payment types to PaymentType database
			mPGTypeByName = new Hashtable();

			foreach (Type pgType in mPGstypes)
			{
				BasePG pg = (BasePG)System.Activator.CreateInstance(pgType);

				string pgName = pg.PGName;

				mPGTypeByName[pgName] = pgType;

				// if pg type setting doesn't exsit in PaymentType database, add new db row
				PaymentType pt = PaymentType.LoadByPGName(pgName);

				if(pt == null)
				{
					pt.PGName = pgName;
					pt.Name = pgName.Replace("_", "");
					pt.SaveToDB();
				}
			}

		}

		public static List<BasePG> GetActivePGs()
		{
			List<PaymentType> ptColl = GetAllPaymentTypesActiveOnly();

			List<BasePG> pgs = new List<BasePG>();

			foreach (PaymentType pt in ptColl)
			{
				BasePG pg = GetPG(pt.PGName);
				pg.PaymentType = pt; // set PaymentType since it is already loaded
				pgs.Add(pg);
			}
			return pgs;
		}

		public static BasePG GetPG(int? paymentTypeID)
		{
			PaymentType pt = new PaymentType();
			pt.LoadFromDB(paymentTypeID);
			PG.BasePG pg = GetPG(pt.PGName);
			pg.PaymentType = pt; // set PaymentType since it is already loaded
			return pg;
		}

		public static BasePG GetPG(string pgName)
		{
			if (!string.IsNullOrEmpty(pgName))
			{
				Type pgType = mPGTypeByName[pgName] as Type;
				if (pgType != null)
				{
					return (BasePG)System.Activator.CreateInstance(pgType);
				}
			}
			throw new Exception("Payment gateway not found: " + pgName);
		}

		public static List<PaymentType> GetAllPaymentTypes()
		{
			return SmartObjectLoader.LoadCollection<PaymentType>(1000, false, db.PaymentType.Ordinal, false);
		}

		public static List<PaymentType> GetAllPaymentTypesActiveOnly()
		{
			return SmartObjectLoader.LoadCollection<PaymentType>(1000, true, db.PaymentType.Ordinal, false);
		}
		


	}




}