﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Security.Cryptography;
using System.IO;

namespace Ice.Net.Ecommerce.Security.Cryptography
{

	#region SymetricCryptoHelper class

	internal abstract class SymetricCryptoHelper
	{
		private SymmetricAlgorithm mServiceProvider = null;

		protected SymetricCryptoHelper()
		{
		}

		/// <summary>
		/// Creates IV and key from password.
		/// </summary>
		/// <param name="password"></param>
		protected SymetricCryptoHelper(string password, SymmetricAlgorithm serviceProvider)
		{
			mServiceProvider = serviceProvider;
			byte[] hash = MD5.Create().ComputeHash(Encoding.UTF8.GetBytes(password));
			////byte[] hash = SHA1.Create().ComputeHash(Encoding.UTF8.GetBytes(password));
			byte[] keyIV = new Byte[mServiceProvider.IV.Length];
			byte[] key = new Byte[mServiceProvider.Key.Length];
			while (hash.Length < keyIV.Length || hash.Length < key.Length)
			{
				byte[] tmp = new byte[hash.Length * 2];
				Array.Copy(hash, 0, tmp, 0, hash.Length);
				Array.Copy(hash, 0, tmp, hash.Length, hash.Length);
				hash = tmp;
			}
			Array.Copy(hash, keyIV, keyIV.Length);
			Array.Reverse(hash);
			Array.Copy(hash, key, key.Length);
			InitializeKeys(keyIV, key);
		}

		/// <summary>
		/// Accepts base64 encoded keys.
		/// </summary>
		/// <param name="keyIV"></param>
		/// <param name="key"></param>
		protected SymetricCryptoHelper(string keyIV, string key, SymmetricAlgorithm serviceProvider)
		{
			mServiceProvider = serviceProvider;
			InitializeKeys(Convert.FromBase64String(keyIV), Convert.FromBase64String(key));
		}

		private void InitializeKeys(byte[] keyIV, byte[] key)
		{
			mServiceProvider.IV = keyIV;
			mServiceProvider.Key = key;
		}

		public byte[] Encrypt(byte[] data)
		{
			return mServiceProvider.CreateEncryptor().TransformFinalBlock(data, 0, data.Length);
		}

		public byte[] Encrypt(string plainText)
		{
			return Encrypt(Encoding.UTF8.GetBytes(plainText));
		}

		public byte[] Decrypt(byte[] chipperBytes)
		{
			try
			{
				return mServiceProvider.CreateDecryptor().TransformFinalBlock(chipperBytes, 0, chipperBytes.Length);
			}
			catch
			{
			}
			return null;
		}

		public string DecryptToString(byte[] chipperBytes)
		{
			byte[] data = Decrypt(chipperBytes);
			if (data != null)
			{
				return Encoding.UTF8.GetString(data);
			}
			else
			{
				return null;
			}
		}

	}

	#endregion

	#region DESCryptoHelper class

	internal class DESCryptoHelper : SymetricCryptoHelper
	{
		/// <summary>
		/// Creates IV and key from password.
		/// </summary>
		/// <param name="password"></param>
		public DESCryptoHelper(string password)
			: base(password, new DESCryptoServiceProvider())
		{

		}
	}

	#endregion

	#region TripleDESCryptoHelper class

	internal class TripleDESCryptoHelper : SymetricCryptoHelper
	{
		/// <summary>
		/// Creates IV and key from password.
		/// </summary>
		/// <param name="password"></param>
		public TripleDESCryptoHelper(string password)
			: base(password, new TripleDESCryptoServiceProvider())
		{

		}
	}

	#endregion


}