﻿using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Web;
using System.Web.Services;
using System.Web.Services.Protocols;
using Ice.Net.Ecommerce.BusinessObjects.Database.VISTACART;
using GCART = Ice.Net.Ecommerce.BusinessObjects.Database.VISTACART;
using System.Collections.Generic;
using Ice.Net.Ecommerce.BusinessObjects.Common;
using Ice.Net.Ecommerce.BusinessObjects.Gallery;
using Ice.Net.Ecommerce.NewsletterUtility;
using Ice.Orm.Core.Criteria;
using Ice.Orm.Core.Entity;
using Ice.Orm.Core.Schema;
using System.Text.RegularExpressions;

namespace Ice.Net.Ecommerce.Web.Admin.VISTACART.WebService
{
	using Ecommerce.CurrencyUpdater;
	using Ice.Orm.Core.Criteria;
	using Ice.Orm.Core.Entity;
	using Ice.Orm.Core.DataStore;

	/// <summary>
	/// Summary description for GService
	/// </summary>
	[WebService(Namespace = "http://iceasp.net")]
	[WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
	[ToolboxItem(false)]
	//[System.Web.Script.Services.ScriptService]
	public sealed class GService : System.Web.Services.WebService
	{
		static int mCount = 0;

		#region Authentication methods

		[IceWebMethod]
		public void Logout()
		{
			VISTACARTHelper.CurrentUser.AdminTicket.SignOut();
		}

		private void Authenticate()
		{
			Authenticate(false);
		}

		private void Authenticate(bool removeCache)
		{
			Authenticate2(removeCache);
		}

		private void Authenticate2(bool removeCache)
		{

#if TRIAL
			
			mCount++;
			if (mCount % 5 == 0)
			{
				// show trial message
				throw new Exception(@"
*********************************
*********************************
*********************************

VISTA CART TRIAL EDITION

Buy full version at www.iceasp.net

*********************************
*********************************
*********************************

");
			}

#endif

			User user = new User();
			if (VISTACARTHelper.CurrentUser.AdminTicket.IsAuthenticated
				&& user.LoadFromDB(VISTACARTHelper.CurrentUser.AdminTicket.UserID)
				&& user.IsActive
				&& user.IsInRole(Role.NameEnum.Administrators))
			{
				// ok
			}
			else if (BusinessObjects.Common.OnlineDemoSkinHelper.IsOnlineDemoAdmin && VISTACARTHelper.CurrentUser.AdminTicket.IsAuthenticated)
			{
				// ok
			}
			else
			{
				System.Threading.Thread.Sleep(3000); // slow down invalid login attempts
				throw new Exception("_not_authenticated_");
			}

			if (removeCache)
			{
				// remove items from Application Cache
				VISTACARTHelper.CleanApplicationStore();

				// remove items from HttpContex.Current.Cache
				//VISTACARTHelper.CleanGlobalCache();
			}
		}

		[IceWebMethod]
		public bool AdminLogin(string username, string password)
		{
			User user = new User();
			if (!string.IsNullOrEmpty(username)
				&& !string.IsNullOrEmpty(password)
				&& (user = Ecommerce.BusinessObjects.Database.VISTACART.User.LoadByUsername(username)) != null
				&& user.IsActive
				&& user.Password == password
				&& user.IsInRole(Role.NameEnum.Administrators))
			{
				user.LastLoginDate = DateTime.Now;
				user.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);

				// ok
				BusinessObjects.Common.VISTACARTHelper.CurrentUser.AdminTicket.SignIn(user, false);
				return true;
			}
			else if (BusinessObjects.Common.OnlineDemoSkinHelper.IsOnlineDemoAdmin && username == "admin")
			{
				// ok
				user.UserID = 1;
				user.IsActive = true;
				user.Username = "admin";
				user.Password = "admin";
				user.Email = "admin@admin.com";
				BusinessObjects.Common.VISTACARTHelper.CurrentUser.AdminTicket.SignIn(user, false);
				return true;
			}
			else
			{
				// fail
				System.Threading.Thread.Sleep(3000); // slow down invalid login attempts
				return false;
			}
		}

		#endregion

		#region Database Table editor

		//private static string mDatabaseName = "VISTACART";
		//private static string mEntityNamespace = "Ice.Net.Ecommerce.BusinessObjects.Database.VISTACART";
		private static DataStoreBase mDataStore = null;

		private static string mEntityNamespace = typeof(BusinessObjects.Database.VISTACART.Address).Namespace;

		private DataStoreBase GetDataStore()
		{
			if (mDataStore == null)
			{
				mDataStore = DataStoreCollection.GetFirstAvailable();
			}
			return mDataStore;
		}

		private SchemaTableHelper GetSchemaTableHelper(string tableName)
		{
			// get entity by table name
			EntitySetting ss = EntitySetting.Get(GetEntityType(tableName));
			return ss.SchemaTableHelper;
		}

		private Type GetEntityType(string tableName)
		{
			return Type.GetType(mEntityNamespace + "." + tableName);
		}

		public class ExtConfig
		{
			public int start = 0;
			public int limit = -1;
			public string sort;
			public string dir;
			public string tableName;
			public int? CategoryID;
			public int? RoleID;
			public int? CampaignID;

			public bool UseJoin()
			{
				return CategoryID > 0 || RoleID > 0 || CampaignID > 0;
			}

			public string InWhat()
			{
				if (CategoryID > 0)
				{
					return "Category";
				}
				else if (RoleID > 0)
				{
					return "Role";
				}
				else if (CampaignID > 0)
				{
					return "Campaign";
				}
				else
				{
					return "Unknown";
				}
			}

			public int? GetIDValue()
			{
				if (CategoryID > 0)
				{
					return CategoryID;
				}
				else if (RoleID > 0)
				{
					return RoleID;
				}
				else if (CampaignID > 0)
				{
					return CampaignID;
				}
				else
				{
					return null;
				}
			}
		}

		public class ColumnInfo
		{
			public string ColumnName;
			public bool AllowDBNull;
			public bool AutoIncrement;
			public string TypeName;
			public int MaxLength;
			public bool ReadOnly;
			public bool Unique;
			public bool PK;

			public ColumnInfo(EntityFieldSetting schemaColumn)
			{
				ColumnName = schemaColumn.ColumnName;
				AllowDBNull = schemaColumn.AllowDBNull;
				AutoIncrement = schemaColumn.IsAutoIncrement;
				TypeName = Type.GetTypeCode(schemaColumn.DataType) + "";
				MaxLength = schemaColumn.ColumnSize;
				ReadOnly = schemaColumn.IsReadOnly;
				Unique = schemaColumn.IsUnique;
				PK = schemaColumn.IsPrimaryKey;
			}

		}

		[IceWebMethod]
		public List<ColumnInfo> GetSchemaColumns(string tableName)
		{
			Authenticate();
			SchemaColumnCollection schemaColumns = GetSchemaTableHelper(tableName).SchemaColumns;
			List<ColumnInfo> listCols = new List<ColumnInfo>();
			foreach (EntityFieldSetting col in schemaColumns)
			{
				listCols.Add(new ColumnInfo(col));
			}
			return listCols;
		}

		[IceWebMethod]
		public List<string> GetTablesNames()
		{
			Authenticate();
			return GetDataStore().GetTablesNames();
		}

		[IceWebMethod]
		public SimpleDataTable GetTableDataPaged(ExtConfig extConfig)
		{
			Authenticate();

			string sql = " from " + SqlName.CreateFullSqlName(extConfig.tableName) + " ";
			
			if (extConfig.UseJoin())
			{
				sql += "inner join " + SqlName.CreateFullSqlName(extConfig.tableName + "In" + extConfig.InWhat()) + " " +
					"on " + SqlName.CreateFullSqlName(extConfig.tableName + "." + extConfig.tableName + "ID") +
					"=" + SqlName.CreateFullSqlName(extConfig.tableName + "In" + extConfig.InWhat() + "." + extConfig.tableName + "ID");

				sql += " where " + SqlName.CreateFullSqlName(extConfig.tableName + "In" + extConfig.InWhat() + "." + extConfig.InWhat() + "ID") + " = {0}";//, extConfig.GetIDValue());
			}

			string selectSql =  "select " + SqlName.CreateFullSqlName(extConfig.tableName + ".*") + sql;
			if (extConfig.sort != null && extConfig.dir != null)
			{
				selectSql += " order by " + SqlName.CreateFullSqlName(extConfig.tableName + "." + extConfig.sort) + " " + extConfig.dir;
			}

			SchemaTableHelper sch = GetSchemaTableHelper(extConfig.tableName);

			Query select = new Query().CreateQuery(selectSql, extConfig.GetIDValue()).Skip(extConfig.start).Take(extConfig.limit);

			// get data
			SimpleDataTable dt = select.ToSimpleDataSet<User>()[0];// any table, doesn't matter which, just to get db settings from // GetDataStore().GetDataTable(select, sch);
			
			// get count
			Query count= new Query().CreateQuery("select count(*) " + sql, extConfig.GetIDValue());

			dt.TotalCount = count.ExecuteScalar<User, int>();// (int)GetDataStore().ExecuteScalar(count, sch);

			return dt;
		}

		[IceWebMethod]
		public int DeleteRows(string tableName, List<Dictionary<string, object>> htList)
		{
			Authenticate();
			SchemaTableHelper sch = GetSchemaTableHelper(tableName);
			int rows = 0;
			foreach (Dictionary<string, object> ht in htList)
			{
				CaselessDictionary gg = new CaselessDictionary(ht);
				rows += GetDataStore().Delete(sch, null, gg);
			}
			return rows;
		}

		[IceWebMethod]
		public List<Dictionary<string, object>> UpdateRows(string tableName, List<Dictionary<string, object>> htList)
		{
			return SaveRows(tableName, htList, false);
		}

		[IceWebMethod]
		public List<Dictionary<string, object>> InsertRows(string tableName, List<Dictionary<string, object>> htList)
		{
			return SaveRows(tableName, htList, true);
		}

		private List<Dictionary<string, object>> SaveRows(string tableName, List<Dictionary<string, object>> htList, bool forceInsert)
		{
			Authenticate(true);
			SchemaTableHelper sch = GetSchemaTableHelper(tableName);
			List<Dictionary<string, object>> newValues = new List<Dictionary<string, object>>();
			foreach (Dictionary<string, object> ht in htList)
			{
				SimpleDataRow htOld = forceInsert ? null : sch.NewSchemaTableDataRow();
				CaselessDictionary gg = new CaselessDictionary(ht);
				GetDataStore().Save(sch, htOld, gg);
				newValues.Add(gg.ToDictionary());
			}
			return newValues;

		}

		#endregion

		#region Currency methods

		[IceWebMethod]
		public Currency GetDefaultCurrency()
		{
			Authenticate(false);
			return VISTACARTHelper.AppDefaults.Currency;
		}

		[IceWebMethod]
		public CurrencyRatesUpdaterResult UpdateCurrencyRatesFromCurrencyServer()
		{
			Authenticate(true);
			CurrencyRatesInfo ratesInfo = CurrencyRatesUpdater.GetCurrencyRatesInfo(VISTACARTHelper.AppDefaults.Currency.ISOCode, null, null);

			int updatedObjects = 0;

			if (ratesInfo != null && ratesInfo.Success && ratesInfo.Items.Count > 0)
			{
				foreach (CurrencyRate currencyRate in ratesInfo.Items)
				{
					Currency currency = Currency.LoadByISOCode(currencyRate.ISOCode) ?? new Currency();

					currency.ISOCode = currencyRate.ISOCode;
					currency.ConversionRate = currencyRate.ConversionRate;

					if (string.IsNullOrEmpty(currency.Name))
					{
						currency.Name = currencyRate.Name;
					}

					currency.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
					{
						updatedObjects++;
					}
				}

				string msg = "Items date: " + ratesInfo.Date.ToLocalTime() + ".\r\n" + ratesInfo.Items.Count + " items retrieved.\r\n" + updatedObjects + " saved to database.";
				return new CurrencyRatesUpdaterResult(true, updatedObjects, msg);
			}
			else
			{
				return new CurrencyRatesUpdaterResult(false, 0, ratesInfo.ErrorMessage);
			}
		}

		[IceWebMethod]
		public List<Currency> GetCurrencyCollection()
		{
			Authenticate(false);
			return SmartObjectLoader.LoadCollection<Currency>(1000, false, db.Currency.Ordinal, false);
		}

		[IceWebMethod]
		public List<Currency>GetCurrencyCollectionActiveOnly()
		{
			Authenticate(false);
			return SmartObjectLoader.LoadCollection<Currency>(1000, true, db.Currency.Ordinal, false);
		}

		[IceWebMethod]
		public void SaveCurrencyCollection(List<Currency> currencies)
		{
			Authenticate(true);
			foreach (Currency cur in currencies)
			{
				// when deserialized from JSON, object is newly created and values are set using set properties, if invoked SaveToDB, it would be inserted since wasn't loaded from database
				cur.SaveToDB(WriteModeEnum.SaveAllWithIDCheck); // update always
			}
		}

		[IceWebMethod]
		public void NewCurrency()
		{
			Authenticate(true);
			Currency cur = new Currency();
			cur.Ordinal = int.MaxValue;
			cur.SaveToDB(WriteModeEnum.SaveAllWithIDCheck); // insert always
		}

		[IceWebMethod]
		public void DeleteCurrency(Currency currency)
		{
			Authenticate(true);
			currency.DeleteFromDB();

		}

		#endregion

		#region WeightUnit methods

		[IceWebMethod]
		public List<WeightUnit> GetWeightUnitCollection()
		{
			Authenticate(false);
			return SmartObjectLoader.LoadCollection<WeightUnit>(1000, false, db.WeightUnit.Ordinal, false);
		}

		[IceWebMethod]
		public List<WeightUnit> GetWeightUnitCollectionActiveOnly()
		{
			Authenticate(false);
			return  SmartObjectLoader.LoadCollection<WeightUnit>(1000, true, db.WeightUnit.Ordinal, false);
		}

		[IceWebMethod]
		public void SaveWeightUnitCollection(List<WeightUnit> list)
		{
			Authenticate(true);
			foreach (WeightUnit wu in list)
			{
				wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck); // update always
			}

		}

		[IceWebMethod]
		public void NewWeightUnit()
		{
			Authenticate(true);
			WeightUnit wu = new WeightUnit();
			wu.Ordinal = int.MaxValue;
			wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck); // insert new instance

		}

		[IceWebMethod]
		public void DeleteWeightUnit(WeightUnit wu)
		{
			Authenticate(true);
			wu.DeleteFromDB();

		}

		#endregion

		#region Tax methods

		[IceWebMethod]
		public List<Tax> GetTaxCollection()
		{
			Authenticate(false);
			return SmartObjectLoader.LoadCollection< Tax>(1000, false, db.Tax.Ordinal, false);
		}

		[IceWebMethod]
		public List<Tax> GetTaxCollectionActiveOnly()
		{
			Authenticate(false);
			return SmartObjectLoader.LoadCollection<Tax>(1000, true, db.Tax.Ordinal, false);
		}

		[IceWebMethod]
		public void SaveTaxCollection(List<Tax> list)
		{
			Authenticate(true);
			foreach (Tax wu in list)
			{
				wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck); // after deserialization, update
			}

		}

		[IceWebMethod]
		public void NewTax()
		{
			Authenticate(true);
			Tax wu = new Tax();
			wu.Ordinal = int.MaxValue;
			wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck); // insert

		}

		[IceWebMethod]
		public void DeleteTax(Tax wu)
		{
			Authenticate(true);
			wu.DeleteFromDB();

		}

		#endregion

		#region ShippingType methods

		[IceWebMethod]
		public List<ShippingType> GetShippingTypeCollection()
		{
			Authenticate(false);
			return SmartObjectLoader.LoadCollection<ShippingType>(1000, false, db.ShippingType.Ordinal, false);
		}

		[IceWebMethod]
		public List<ShippingType>GetShippingTypeCollectionActiveOnly()
		{
			Authenticate(false);
			return SmartObjectLoader.LoadCollection<ShippingType>(1000, true, db.ShippingType.Ordinal, false);
		}


		[IceWebMethod]
		public void SaveShippingTypeCollection(List<ShippingType> list)
		{
			Authenticate(true);
			foreach (ShippingType wu in list)
			{
				wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
			}

		}

		[IceWebMethod]
		public void NewShippingType()
		{
			Authenticate(true);
			ShippingType wu = new ShippingType();
			wu.Ordinal = int.MaxValue;
			wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck); // insert

		}

		[IceWebMethod]
		public void DeleteShippingType(ShippingType wu)
		{
			Authenticate(true);
			wu.DeleteFromDB();

		}

		#endregion

		#region Country methods

		[IceWebMethod]
		public List<Country> GetCountryCollection()
		{
			Authenticate(false);
			return SmartObjectLoader.LoadCollection<Country>(1000, false, db.Country.Ordinal, false);
		}

		[IceWebMethod]
		public List<Country> GetCountryCollectionActiveOnly()
		{
			Authenticate(false);
			return SmartObjectLoader.LoadCollection<Country>(1000, true, db.Country.Ordinal, false);
		}

		[IceWebMethod]
		public void SaveCountryCollection(List<Country> list)
		{
			Authenticate(true);
			foreach (Country wu in list)
			{
				wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
			}

		}

		[IceWebMethod]
		public void NewCountry()
		{
			Authenticate(true);
			Country wu = new Country();
			wu.Ordinal = int.MaxValue;
			wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);

		}

		[IceWebMethod]
		public void DeleteCountry(Country wu)
		{
			Authenticate(true);
			wu.DeleteFromDB();

		}

		#endregion

		#region State methods

		[IceWebMethod]
		public List<State> GetStateCollection()
		{
			Authenticate(false);
			return SmartObjectLoader.LoadCollection<State>(1000, false, db.State.Ordinal, false);
		}

		[IceWebMethod]
		public List<State> GetStateCollectionActiveOnly()
		{
			Authenticate(false);
			return SmartObjectLoader.LoadCollection<State>(1000, true, db.State.Ordinal, false);
		}

		[IceWebMethod]
		public List<State> GetStateCollectionByCountryID(int countryID, bool activeOnly)
		{
			Authenticate(false);
			return StateCollection.LoadByCountryID(countryID, activeOnly);
		}

		[IceWebMethod]
		public void SaveStateCollection(List<State> list)
		{
			Authenticate(true);
			foreach (State wu in list)
			{
				wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
			}

		}

		[IceWebMethod]
		public void NewState()
		{
			Authenticate(true);
			State wu = new State();
			wu.Ordinal = int.MaxValue;
			wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);

		}

		[IceWebMethod]
		public void DeleteState(State wu)
		{
			Authenticate(true);
			wu.DeleteFromDB();

		}

		#endregion

		#region Role methods

		[IceWebMethod]
		public List<Role> GetRoleCollection()
		{
			Authenticate(false);
			return SmartObjectLoader.LoadCollection<Role>(1000, false, db.Role.Ordinal, false);
		}

		[IceWebMethod]
		public List<Role> GetRoleCollectionTreeNodes(string pid)
		{
			Authenticate(false);
			if (pid == "")
			{
				return SmartObjectLoader.LoadCollection<Role>(1000, false, db.Role.Ordinal, false);
			}
			else
			{
				return new RoleCollection();
			}
		}

		[IceWebMethod]
		public List<Role> GetRoleCollectionActiveOnly()
		{
			Authenticate(false);
			return SmartObjectLoader.LoadCollection<Role>(1000, true, db.Role.Ordinal, false);
		}

		[IceWebMethod]
		public void SaveRoleCollection(List<Role> list)
		{
			Authenticate(true);
			foreach (Role wu in list)
			{
				wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
			}

		}

		[IceWebMethod]
		public Role NewRole()
		{
			Authenticate(true);
			Role wu = new Role();
			wu.Ordinal = int.MaxValue;
			wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);

			return wu;
		}

		[IceWebMethod]
		public Role NewRoleWithName(string name)
		{
			Authenticate(true);
			Role wu = new Role();
			wu.Ordinal = int.MaxValue;
			wu.Name = name;
			wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);

			return wu;
		}

		[IceWebMethod]
		public bool DeleteRole(Role wu)
		{
			Authenticate(true);
			if (wu.CountUsers() > 0)
			{
				throw new Exception("Please remove users form role first.");
			}
			return wu.DeleteFromDB();
		}

		/// <summary>
		/// Activate or deactivate role. Returns true if action was successful.
		/// </summary>
		/// <param name="roleID"></param>
		/// <param name="isActive"></param>
		[IceWebMethod]
		public Role SetRoleActive(int roleID, bool isActive)
		{
			Role role = new Role();
			if (role.LoadFromDB(roleID))
			{
				role.IsActive = isActive;
				role.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
				{
					return role;
				}
			}
			return null;
		}

		[IceWebMethod]
		public Role SetRoleName(int roleID, string name)
		{
			Role role = new Role();
			if (role.LoadFromDB(roleID))
			{
				role.Name = name;
				role.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
				{
					return role;
				}
			}
			return null;
		}

		#endregion

		#region PaymentType methods

		[IceWebMethod]
		public string GetPaymentTypeControlPanel(int paymentTypeId)
		{
			PaymentGateways.PG.BasePG pg = Ice.Net.Ecommerce.PaymentGateways.PGLoader.GetPG(paymentTypeId);
			return pg.GetControlPanelHtml();
		}

		[IceWebMethod]
		public List<PaymentType> GetPaymentTypeCollection()
		{
		    Authenticate(false);
		    return PaymentGateways.PGLoader.GetAllPaymentTypes();
		}

		[IceWebMethod]
		public List<PaymentType> GetPaymentTypeCollectionActiveOnly()
		{
		    Authenticate(false);
		    return SmartObjectLoader.LoadCollection<PaymentType>(1000, true, db.PaymentType.Ordinal, false);
		}

		[IceWebMethod]
		public void SavePaymentTypeCollection(List<PaymentType> list)
		{
			Authenticate(true);
			foreach (PaymentType wu in list)
			{
				wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
			}

		}

		[IceWebMethod]
		public void NewPaymentType()
		{
			Authenticate(true);
			PaymentType wu = new PaymentType();
			wu.Ordinal = int.MaxValue;
			wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);

		}

		[IceWebMethod]
		public void DeletePaymentType(PaymentType wu)
		{
			Authenticate(true);
			wu.DeleteFromDB();

		}

		#endregion

		#region AppGlobalSettings methods

		[IceWebMethod]
		public AppGlobalSetting GetAppDefaultSetting()
		{
			Authenticate(false);
			return VISTACARTHelper.AppDefaults.AppGlobalSettingsItem;
		}

		[IceWebMethod]
		public List<AppGlobalSetting> GetAppGlobalSettingCollection()
		{
			Authenticate(false);
			return SmartObjectLoader.LoadCollection<AppGlobalSetting>(1000, false, db.AppGlobalSetting.Ordinal, false);
		}

		[IceWebMethod]
		public void SaveAppGlobalSettingCollection(List<AppGlobalSetting> list)
		{
			Authenticate(true);
			foreach (AppGlobalSetting wu in list)
			{
				wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
			}
			VISTACARTHelper.CleanApplicationStore();
			// set UserInfo cookie
			VISTACARTHelper.CurrentUser.UserInfo.Update(VISTACARTHelper.AppDefaults.Currency.CurrencyID, 0, VISTACARTHelper.AppDefaults.WeightUnit.WeightUnitID);
		}

		[IceWebMethod]
		public void NewAppGlobalSetting()
		{
			Authenticate(true);
			AppGlobalSetting wu = new AppGlobalSetting();
			wu.Ordinal = int.MaxValue;
			wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);

		}

		[IceWebMethod]
		public void DeleteAppGlobalSetting(AppGlobalSetting wu)
		{
			Authenticate(true);
			wu.DeleteFromDB();

		}

		#endregion

		#region Users methods

		[IceWebMethod]
		public PagedCollection<User> GetUserCollectionPagedSorted(int recordsPerPage, int startRecordIndex, object orderByColumn, bool orderAsc, int roleID, int dummy)
		{
			Authenticate(false);
			PagedCollection<User> pc = null;

			if (roleID > 0)
			{
				// load by role
				pc = new PagedCollection<User>();
				pc.totalRecords = UserCollection.CountByRoleID(roleID); // total records in db table by role id
				pc.collection = UserCollection.LoadPagedByRoleID(roleID, recordsPerPage, startRecordIndex/recordsPerPage, orderByColumn, orderAsc, false);
				return pc;
			}
			else
			{
				// load all
				pc = SmartObjectLoader.LoadCollectionPaged<GCART.User>(recordsPerPage, startRecordIndex / recordsPerPage, orderByColumn, orderAsc, false);
			}
			return pc;
		}

		[IceWebMethod]
		public PagedCollection<User> GetUserCollectionPaged(int recordsPerPage, int startRecordIndex)
		{
			Authenticate(false);
			PagedCollection<User> pc = SmartObjectLoader.LoadCollectionPaged<GCART.User>(recordsPerPage, startRecordIndex / recordsPerPage, null, false, false);
			return pc;
		}

		[IceWebMethod]
		public List<User> GetUserCollectionActiveOnly()
		{
			Authenticate(false);
			return SmartObjectLoader.LoadCollection<User>(1000, true, db.User.UserID, false); // order DESC
		}

		[IceWebMethod]
		public void SaveUserCollection(List<User> list)
		{
			Authenticate(true);
			foreach (User wu in list)
			{
				wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
			}

		}

		/// <summary>
		/// Add new user to db. Returns UserID.
		/// </summary>
		/// <returns></returns>
		[IceWebMethod]
		public int NewUser()
		{
			Authenticate(true);
			User wu = new User();
			////	wu.LastLoginDate = DateTime.Now;
			wu.CreationDate = DateTime.Now;
			wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);

			return wu.UserID;
		}

		[IceWebMethod]
		public void DeleteUser(User wu)
		{
			Authenticate(true);
			wu.DeleteFromDB();

		}

		[IceWebMethod]
		public void DeleteUsers(int[] userIDs)
		{
			Authenticate(true);
			foreach (int userID in userIDs)
			{
				User user = new User();
				user.UserID = userID;
				user.DeleteFromDB();
			}
		}

		[IceWebMethod]
		public void DeleteUserCollection(List<User> list)
		{
			Authenticate(true);
			foreach (User user in list)
			{
				User usr = new User();
				if (usr.LoadFromDB(user.UserID))
				{
					usr.DeleteFromDB();
				}
			}

		}

		[IceWebMethod]
		public int GetUserCount()
		{
			UserCollection col = new UserCollection();
			return col.CountTotalUsers();
		}

		#endregion

		#region Address methods

		[IceWebMethod]
		public Address GetAddress(int addressID)
		{
			Authenticate(false);
			Address address = new Address();
			if (address.LoadFromDB(addressID))
			{
				return address;
			}
			else
			{
				return new Address();
			}
		}

		[IceWebMethod]
		public Address SaveAddress(Address address)
		{
			Authenticate(true);
			address.SaveToDB(WriteModeEnum.SaveAllWithIDCheck); // update if AddressID > 0, else insert
			return address;
		}

		#endregion

		#region PagerSize methods

		[IceWebMethod]
		public List<PagerSize> GetPagerSizeCollection()
		{
			Authenticate(false);
			return SmartObjectLoader.LoadCollection<PagerSize>(1000, false, db.PagerSize.Ordinal, false);
		}

		[IceWebMethod]
		public List<PagerSize> GetPagerSizeCollectionActiveOnly()
		{
			Authenticate(false);
			return SmartObjectLoader.LoadCollection<PagerSize>(1000, true, db.PagerSize.Ordinal, false);
		}

		[IceWebMethod]
		public void SavePagerSizeCollection(List<PagerSize> list)
		{
			Authenticate(true);
			foreach (PagerSize wu in list)
			{
				wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
			}

		}

		[IceWebMethod]
		public void NewPagerSize()
		{
			Authenticate(true);
			PagerSize wu = new PagerSize();
			wu.Ordinal = int.MaxValue;
			wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);

		}

		[IceWebMethod]
		public void DeletePagerSize(PagerSize wu)
		{
			Authenticate(true);
			wu.DeleteFromDB();

		}

		#endregion

		#region UsersInRoles methods

		/// <summary>
		/// Add or remove users to roles
		/// </summary>
		/// <param name="roleIDs"></param>
		/// <param name="userIDs"></param>
		/// <param name="add">
		///	<para>true: add users to roles</para>
		///	<para>false: remove users from roles</para>
		/// </param>
		/// <returns></returns>
		[IceWebMethod]
		public int UsersInRoles(int[] roleIDs, int[] userIDs, bool add)
		{
			int count = 0;
			foreach (int roleID in roleIDs)
			{
				Role role = new Role();
				if (role.LoadFromDB(roleID))
				{
					foreach (int userID in userIDs)
					{
						GCART.User user = new User();
						if (user.LoadFromDB(userID))
						{
							count++;
							if (add)
							{
								user.AddRole(role);
							}
							else
							{
								user.RemoveRole(role);
							}
						}
					}
				}
			}
			return count;
		}

		#endregion

		#region Campaign methods

		[IceWebMethod]
		public List<Campaign> GetCampaignCollectionTreeNodes(string pid)
		{
			Authenticate(false);
			if (pid == "")
			{
				return SmartObjectLoader.LoadCollection<Campaign>(1000, false, db.Campaign.Ordinal, false);
			}
			else
			{
				return new List<Campaign>();
			}
		}

		[IceWebMethod]
		public List<Campaign> GetCampaignCollection()
		{
			Authenticate(false);
			return SmartObjectLoader.LoadCollection<Campaign>(1000, false, db.Campaign.Ordinal, false);
		}

		[IceWebMethod]
		public List<Campaign> GetCampaignCollectionActiveOnly()
		{
			Authenticate(false);
			return SmartObjectLoader.LoadCollection<Campaign>(1000, true, db.Campaign.Ordinal, false);
		}

		[IceWebMethod]
		public void SaveCampaignCollection(List<Campaign> list)
		{
			Authenticate(true);
			foreach (Campaign wu in list)
			{
				wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
			}

		}

		[IceWebMethod]
		public Campaign SaveCampaign(Campaign campaign)
		{
			Authenticate(true);
			campaign.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
			{

				return campaign;
			}
		}

		[IceWebMethod]
		public Campaign NewCampaign()
		{

			Authenticate(true);
			Campaign wu = new Campaign();
			wu.Ordinal = int.MaxValue;
			wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
			{
				return wu;
			}
		}

		[IceWebMethod]
		public Campaign NewCampaignWithName(string name)
		{
			Authenticate(true);
			Campaign wu = new Campaign();
			wu.Ordinal = int.MaxValue;
			wu.Name = name;
			wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);

			return wu;
		}

		[IceWebMethod]
		public bool DeleteCampaign(Campaign wu)
		{
			Authenticate(true);
			return wu.DeleteFromDB();
		}

		[IceWebMethod]
		public Campaign SetCampaignActive(int campaignID, bool isActive)
		{
			Campaign wu = new Campaign();
			if (wu.LoadFromDB(campaignID))
			{
				wu.IsActive = isActive;
				wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
				{
					return wu;
				}
			}
			return null;
		}

		[IceWebMethod]
		public Campaign SetCampaignName(int campaignID, string name)
		{
			Campaign wu = new Campaign();
			if (wu.LoadFromDB(campaignID))
			{
				wu.Name = name;
				wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
				{
					return wu;
				}
			}
			return null;
		}

		#endregion

		#region ProductInCampaign methods

		[IceWebMethod]
		public int ProductsInCampaigns(int[] campaignsIDs, int[] productsIDs, bool add)
		{
			// add or remove products from campaign
			int count = 0;
			foreach (int campaignID in campaignsIDs)
			{
				Campaign campaign = new Campaign();
				if (campaign.LoadFromDB(campaignID))
				{
					foreach (int productID in productsIDs)
					{
						Product product = new Product();
						if (product.LoadFromDB(productID))
						{
							if (add)
							{
								if (!product.IsInCampaign(campaign))
								{
									count++;
									product.AddCampaign(campaign);
								}
							}
							else
							{
								product.RemoveCampaign(campaign);
							}
						}
					}
				}
			}
			return count;
		}

		#endregion

		#region ProductInCategory methods

		[IceWebMethod]
		public int ProductsInCategories(int[] categoriesIDs, int[] productsIDs, bool add)
		{
			// add or remove products from categories
			int count = 0;
			foreach (int categoryID in categoriesIDs)
			{
				Category category = new Category();
				if (category.LoadFromDB(categoryID))
				{
					foreach (int productID in productsIDs)
					{
						Product product = new Product();
						if (product.LoadFromDB(productID))
						{
							
								if (add)
								{
									if (!product.IsInCategory(category))
									{
										count++;
										product.AddCategory(category);
									}
								}
								else
								{
									product.RemoveCategory(category);
								}
							}
						
					}
				}
			}
			return count;
		}

		#endregion

		#region Product methods

		[IceWebMethod]
		public PagedCollection<Product> GetProductCollectionPagedSorted(int recordsPerPage, int startRecordIndex, object orderByColumn, bool orderAsc, int categoryID, int campaignID)
		{
			Authenticate(false);
			PagedCollection<Product> pc = null;

			if (categoryID > 0)
			{
				// load by categoryID
				pc = new PagedCollection<Product>();
				
				pc.totalRecords = ProductCollection.CountByCategoryID(categoryID, false); // total records in db table by categoryID
				pc.collection = ProductCollection.LoadPagedByCategoryID(categoryID, recordsPerPage, startRecordIndex / recordsPerPage, orderByColumn, orderAsc, false);
				return pc;
			}
			else if (campaignID > 0)
			{
				// load by campaignID
				pc = new PagedCollection<Product>();
				pc.totalRecords = ProductCollection.CountByCampaignID(campaignID, false); // total records in db table by campaignID
				pc.collection = ProductCollection.LoadPagedByCampaignID(campaignID, recordsPerPage, startRecordIndex / recordsPerPage, orderByColumn, orderAsc, false);
				return pc;
			}
			else
			{
				// load all
				pc = new PagedCollection<Product>();
				pc.totalRecords = new Query().Count(db.Product).ExecuteScalar<Product, int>(); // count all records
				pc.collection = ProductCollection.LoadPaged(recordsPerPage, startRecordIndex / recordsPerPage, orderByColumn, orderAsc, false);
				return pc;
			}
		}

		[IceWebMethod]
		public void SaveProductCollection(List<Product> list)
		{
			Authenticate(true);
			foreach (Product wu in list)
			{
				wu.UpdateDate = DateTime.Now;
				wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
			}

		}

		/// <summary>
		/// Add new Product to db. Returns UserID.
		/// </summary>
		/// <returns></returns>
		[IceWebMethod]
		public int NewProduct()
		{
			Authenticate(true);
			Product wu = new Product();
			wu.Ordinal = int.MaxValue;
			wu.CreationDate = DateTime.Now;
			wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);

			return wu.ProductID;
		}

		[IceWebMethod]
		public void DeleteProduct(Product wu)
		{
			Authenticate(true);
			wu.DeleteFromDB();

		}

		[IceWebMethod]
		public void DeleteProducts(int[] productsIDs)
		{
			Authenticate(true);
			foreach (int productID in productsIDs)
			{
				Product wu = new Product();
				wu.ProductID = productID;
				wu.DeleteFromDB();
			}
		}

		#endregion

		#region ProductImage methods

		[IceWebMethod]
		public List<ProductImage> GetProductImageCollection(int productID)
		{
			Authenticate(false);
			Product product = new Product();
			if (product.LoadFromDB(productID))
			{
				return product.GetProductImageCollection();
			}
			else
			{
				return new List<ProductImage>();
			}
		}

		[IceWebMethod]
		public ProductImage NewProductImage(int productID)
		{
			Authenticate(true);
			ProductImage pimg = new ProductImage();
			pimg.ProductID = productID;
			pimg.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
			return pimg;
		}

		[IceWebMethod]
		public bool DeleteProductImage(ProductImage pimg)
		{
			Authenticate(true);
			return pimg.DeleteFromDB();
		}

		/// <summary>
		/// Returns number of images saved to database
		/// </summary>
		/// <param name="productID"></param>
		/// <param name="imageLocation"></param>
		/// <returns></returns>
		[IceWebMethod]
		public int SaveProductImageCollection(List<ProductImage> productImages, int productID)
		{
			Authenticate(true);
			int saved = 0;
			foreach (ProductImage pimg in productImages)
			{
				pimg.ProductID = productID;
				pimg.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
				{
					saved++;
				}
			}
			return saved;
		}

		#endregion

		#region Category methods

		public class CategoryShortcutCollection : List<CategoryShortcutCollection.CategoryShortcut>
		{
			public class CategoryShortcut
			{
				public string CategoryShortcutID;
				public string Name;

				public CategoryShortcut()
				{
				}

				public CategoryShortcut(string name)
				{
					this.Name = name;
					this.CategoryShortcutID = name;
				}
			}
			public CategoryShortcutCollection()
			{
				System.Collections.SortedList sl = new SortedList();

				foreach (string name in Enum.GetNames(typeof(Category.Shortcut)))
				{
					sl.Add(name, name);
				}

				foreach (DictionaryEntry de in sl)
				{
					this.Add(new CategoryShortcut((string)de.Value));
				}
			}
		}

		public class CategoryShortcutCollection2 : List<Hashtable>
		{
			public CategoryShortcutCollection2()
			{
				string[] names = Enum.GetNames(typeof(Category.Shortcut));
				List<String> listNames = new List<string>();
				foreach (string name in names)
				{
					listNames.Add(name);
				}
				listNames.Sort();

				foreach(string name in listNames)
				{
					Hashtable ht = new Hashtable();
					ht.Add("ShortcutName", name);
					ht.Add("Name", name);
					this.Add(ht);
				}
			}
		}

		[IceWebMethod]
		public CategoryShortcutCollection2 GetCategoryShortcutCollection()
		{
			return new CategoryShortcutCollection2();
		}

		[IceWebMethod]
		public CategoryShortcutCollection GetCategoryShortcutCollectionActiveOnly()
		{
			return new CategoryShortcutCollection();
		}

		[IceWebMethod]
		public List<Category> GetCategoryCollection()
		{
			Authenticate(false);
			return SmartObjectLoader.LoadCollection<Category>(1000, false, db.Category.Ordinal, false);
		}

		[IceWebMethod]
		public List<Category> GetCategoryCollectionTreeNodes(int? pid)
		{
			Authenticate(false);
			int ipid = pid > 0 ? pid.Value : -1;
			
			return CategoryCollection.LoadByParentCategoryID(ipid, false, false);
		}

		[IceWebMethod]
		public List<Category> GetChildrenCategoryCollection(int parentCategoryID)
		{
			Authenticate(false);
			return CategoryCollection.LoadByParentCategoryID(parentCategoryID, false, false);
		}

		[IceWebMethod]
		public List<Category> GetCategoryCollectionActiveOnly()
		{
			Authenticate(false);
			return SmartObjectLoader.LoadCollection<Category>(1000, true, db.Category.Ordinal, false);
		}

		[IceWebMethod]
		public void SaveCategoryCollection(List<Category> list)
		{
			Authenticate(true);
			foreach (Category wu in list)
			{
				wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
			}

		}

		[IceWebMethod]
		public Category SaveCategory(Category category)
		{
			Authenticate(true);
			category.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
			{

				return category;
			}
			return null;
		}

		[IceWebMethod]
		public void NewCategory()
		{
			NewCategoryWithName("", 0);
		}

		[IceWebMethod]
		public Category NewCategoryWithName(string name, int parentCategoryID)
		{
			Authenticate(true);
			Category wu = new Category();
			wu.Ordinal = int.MaxValue;
			wu.Name = name;
			wu.IsInMap = true;
			wu.IsInTree = true;
			wu.ParentCategoryID = parentCategoryID;
			wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);

			return wu;
		}

		[IceWebMethod]
		public bool DeleteCategory(Category wu)
		{
			Authenticate(true);
			return wu.DeleteFromDB();
		}

		[IceWebMethod]
		public Category SetCategoryActive(int categoryID, bool isActive)
		{
			Category wu = new Category();
			if (wu.LoadFromDB(categoryID))
			{
				wu.IsActive = isActive;
				wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
				{
					return wu;
				}
			}
			return null;
		}

		[IceWebMethod]
		public Category SetCategoryName(int categoryID, string name)
		{
			Category wu = new Category();
			if (wu.LoadFromDB(categoryID))
			{
				wu.Name = name;
				wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
				{
					return wu;
				}
			}
			return null;
		}

		#endregion

		#region Gallery methods


		[IceWebMethod]
		public List<GalleryNode> GetGalleryNodeCollection(GalleryNode parentNode)
		{
			Authenticate(false);
			if (parentNode == null)
			{
				// get root node
				GalleryNode root = new GalleryNode(null, null);
				List<GalleryNode> nodes = new List<GalleryNode>();
				nodes.Add(root);
				return nodes;
			}
			else
			{
				return parentNode.GetChildren();
			}
		}
		[IceWebMethod]
		public bool NewGalleryNode(GalleryNode parentNode, string newNodeName)
		{
			Authenticate(true);
			if (System.IO.Directory.Exists(parentNode.GalleryPhysicalPath + "\\" + newNodeName) == false)
			{
				// directory doesn't exist, create new
				System.IO.Directory.CreateDirectory(parentNode.GalleryPhysicalPath + "\\" + newNodeName);
			}
			return true;

		}
		[IceWebMethod]
		public bool DeleteGalleryNode(GalleryNode node)
		{
			Authenticate(true);
			System.IO.Directory.Delete(node.GalleryPhysicalPath, true);
			return true;
		}
		[IceWebMethod]
		public bool MoveGalleryNode(GalleryNode node, GalleryNode parentNode)
		{
			Authenticate(true);
			try
			{
				System.IO.Directory.Move(node.GalleryPhysicalPath, parentNode.GalleryPhysicalPath + "\\" + node.GalleryDirectory);
			}
			catch
			{
				return false; // error
			}
			return true;// ok
		}
		[IceWebMethod]
		public bool RenameGalleryNode(GalleryNode node)
		{
			Authenticate(true);
			System.IO.Directory.Move(node.GalleryPhysicalPath, node.ParentPhysicalPath + "\\" + node.GalleryDirectory);
			return true;
		}

		[IceWebMethod]
		public GalleryImageCollection GetGalleryImageCollection(GalleryNode node)
		{
			Authenticate(false);
			GalleryImageCollection galleryImages = new GalleryImageCollection(node);
			return galleryImages;
		}

		[IceWebMethod]
		public bool GalleryCopyImages(string[] imageFiles, GalleryNode targetNode)
		{
			Authenticate(false);
			foreach (string file in imageFiles)
			{
				string targetFile = targetNode.GalleryPhysicalPath + GalleryImageCollection.GetFileName(file);
				if (System.IO.File.Exists(targetFile))
				{
					throw new Exception(String.Format("File {0} already exists at destination. Operation canceled.", GalleryImageCollection.GetFileName(file)));
				}
			}

			foreach (string file in imageFiles)
			{
				System.IO.File.Copy(file, targetNode.GalleryPhysicalPath + GalleryImageCollection.GetFileName(file), false);
			}
			return true;
		}

		[IceWebMethod]
		public bool GalleryDeleteImages(string[] imageFiles)
		{
			Authenticate(true);
			foreach (string file in imageFiles)
			{
				System.IO.File.SetAttributes(file, System.IO.FileAttributes.Normal);
				System.IO.File.Delete(file);
			}
			return true;
		}

		[IceWebMethod]
		public bool GalleryRenameImage(GalleryImage image)
		{
			Authenticate(true);
			if (GalleryImage.IsSupported(image.name) == false)
			{
				throw new Exception("Extension not supported.");
			}
			string filePathOnly = image.file.Remove(image.file.LastIndexOf("\\"));
			System.IO.File.SetAttributes(image.file, System.IO.FileAttributes.Normal);
			System.IO.File.Move(image.file, filePathOnly + "\\" + image.name);
			return true;
		}

		#endregion

		#region Manufacturer methods

		[IceWebMethod]
		public List<Manufacturer> GetManufacturerCollection()
		{
			Authenticate(false);
			return SmartObjectLoader.LoadCollection<Manufacturer>(1000, false, db.Manufacturer.Ordinal, false);
		}

		[IceWebMethod]
		public List<Manufacturer> GetManufacturerCollectionActiveOnly()
		{
			Authenticate(false);
			return SmartObjectLoader.LoadCollection<Manufacturer>(1000, true, db.Manufacturer.Ordinal, false);
		}

		[IceWebMethod]
		public void SaveManufacturerCollection(List<Manufacturer> list)
		{
			Authenticate(true);
			foreach (Manufacturer wu in list)
			{
				wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
			}

		}

		[IceWebMethod]
		public void NewManufacturer()
		{
			Authenticate(true);
			Manufacturer wu = new Manufacturer();
			wu.Ordinal = int.MaxValue;
			wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);

		}

		[IceWebMethod]
		public void DeleteManufacturer(Manufacturer wu)
		{
			Authenticate(true);
			wu.DeleteFromDB();

		}

		#endregion

		#region WebPageInCampaign methods

		[IceWebMethod]
		public int WebPagesInCampaigns(int[] campaignsIDs, int[] webPagesIDs, bool add)
		{
			// add or remove webPages from campaign
			int count = 0;
			foreach (int campaignID in campaignsIDs)
			{
				Campaign campaign = new Campaign();
				if (campaign.LoadFromDB(campaignID))
				{
					foreach (int webPageID in webPagesIDs)
					{
						WebPage webPage = new WebPage();
						if (webPage.LoadFromDB(webPageID))
						{
							if (add)
							{
								if (!webPage.IsInCampaign(campaign))
								{
									count++;
									campaign.AddWebPage(webPage);
								}
							}
							else
							{
								campaign.RemoveWebPage(webPage);
							}
						}

					}
				}
			}
			return count;
		}

		#endregion

		#region WebPageInCategory methods

		[IceWebMethod]
		public int WebPagesInCategories(int[] categoriesIDs, int[] webPagesIDs, bool add)
		{
			// add or remove webPages from categories
			int count = 0;
			foreach (int categoryID in categoriesIDs)
			{
				Category category = new Category();
				if (category.LoadFromDB(categoryID))
				{
					foreach (int webPageID in webPagesIDs)
					{
						WebPage webPage = new WebPage();
						if (webPage.LoadFromDB(webPageID))
						{

							if (add)
							{
								if (!webPage.IsInCategory(category))
								{
									count++;
									category.AddWebPage(webPage);
								}
							}
							else
							{
								category.RemoveWebPage(webPage);
							}
						}

					}
				}
			}
			return count;
		}

		#endregion

		#region WebPage methods

		[IceWebMethod]
		public PagedCollection<WebPage> GetWebPageCollectionPagedSorted(int recordsPerPage, int startRecordIndex, object orderByColumn, bool orderAsc, int categoryID, int campaignID)
		{
			Authenticate(false);
			PagedCollection<WebPage> pc = null;

			if (categoryID > 0)
			{
				// load by categoryID
				pc = new PagedCollection<WebPage>();
				pc.totalRecords = WebPageCollection.CountByCategoryID(categoryID, false); // total records in db table by categoryID
				pc.collection = WebPageCollection.LoadPagedByCategoryID(categoryID, recordsPerPage, startRecordIndex / recordsPerPage, orderByColumn, orderAsc, false);
				return pc;
			}
			else if (campaignID > 0)
			{
				// load by campaignID
				pc = new PagedCollection<WebPage>();
				pc.totalRecords = WebPageCollection.CountByCampaignID(campaignID, false); // total records in db table by campaignID
				pc.collection = WebPageCollection.LoadPagedByCampaignID(campaignID, recordsPerPage, startRecordIndex / recordsPerPage, orderByColumn, orderAsc, false);
				return pc;
			}
			else
			{
				// load all
				pc = new PagedCollection<WebPage>();
				pc.totalRecords = new Query().Count(db.WebPage).ExecuteScalar<WebPage, int>(); // count all records
				pc.collection = WebPageCollection.LoadPaged(recordsPerPage, startRecordIndex / recordsPerPage, orderByColumn, orderAsc, false);
				return pc;
			}
		}

		[IceWebMethod]
		public List<WebPage> GetWebPageCollection()
		{
			Authenticate(false);
			return SmartObjectLoader.LoadCollection<WebPage>(1000, false, db.WebPage.Ordinal, false);
		}

		[IceWebMethod]
		public WebPage GetWebPageByID(int webPageID)
		{
			Authenticate(false);
			WebPage wp = new WebPage();
			wp.LoadFromDB(webPageID);
			return wp;
		}

		[IceWebMethod]
		public List<WebPage> GetWebPageCollectionActiveOnly()
		{
			Authenticate(false);
			return SmartObjectLoader.LoadCollection<WebPage>(1000, true, db.WebPage.Ordinal, false);
		}

		[IceWebMethod]
		public void SaveWebPageCollection(List<WebPage> list)
		{
			Authenticate(true);
			foreach (WebPage wu in list)
			{
				wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
			}
		}

		[IceWebMethod]
		public WebPage SaveWebPageWithLinksResolving(WebPage wp, bool convertToRelativeLinks)
		{
			if (convertToRelativeLinks)
			{
				// case non-sensitive replace
				System.Text.RegularExpressions.Regex re = new System.Text.RegularExpressions.Regex(BusinessObjects.Common.ApplicationUtility.ApplicationHttpRootCurrent, System.Text.RegularExpressions.RegexOptions.IgnoreCase);
				wp.Content = re.Replace(wp.Content, "");
			}
			wp.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
			return wp;
		}

		[IceWebMethod]
		public int NewWebPage()
		{
			Authenticate(true);
			WebPage wu = new WebPage();
			wu.Ordinal = int.MaxValue;
			wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
			{
				return wu.WebPageID;
			}
		}

		[IceWebMethod]
		public void DeleteWebPage(WebPage wu)
		{
			Authenticate(true);
			wu.DeleteFromDB();

		}

		[IceWebMethod]
		public void DeleteWebPages(int[] webPageIDs)
		{
			Authenticate(true);
			foreach (int webPageID in webPageIDs)
			{
				WebPage wu = new WebPage();
				wu.WebPageID = webPageID;
				wu.DeleteFromDB();
			}
		}


		#endregion

		#region AttributeGroup methods

		[IceWebMethod]
		public List<AttributeGroup>GetAttributeGroupCollection(int productID)
		{
			Authenticate(false);
			return AttributeGroupCollection.LoadByProductID(productID);
		}

		[IceWebMethod]
		public void SaveAttributeGroupCollection(List<AttributeGroup> list, int productID)
		{
			Authenticate(true);
			Product product = new Product();
			if (product.LoadFromDB(productID))
			{
				foreach (AttributeGroup wu in list)
				{
					wu.ProductID = productID;
					wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
				}
			}

		}

		[IceWebMethod]
		public void NewAttributeGroup(int productID)
		{
			Authenticate(true);
			AttributeGroup wu = new AttributeGroup();
			wu.ProductID = productID;
			wu.Name = "name?";
			wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);

		}

		[IceWebMethod]
		public void DeleteAttributeGroup(AttributeGroup wu)
		{
			Authenticate(true);
			wu.DeleteFromDB();

		}

		#endregion

		#region SkinDirectory methods

		public class SkinDirectoryItem
		{
			public string SkinDirectory = "";
			public string Name = "";
		}

		private string GetNameOnly(string fullPath)
		{
			string nameOnly = fullPath.Substring(fullPath.LastIndexOf("\\") + 1);
			return nameOnly;
		}

		[IceWebMethod]
		public List<SkinDirectoryItem> GetSkinDirectoryCollection()
		{
			string skinRoot = HttpContext.Current.Request.PhysicalApplicationPath + System.Configuration.ConfigurationManager.AppSettings["SkinsRoot"].Trim('/');
			List<SkinDirectoryItem> directories = new List<SkinDirectoryItem>();

			// load child direcotries
			string[] childDirectories = System.IO.Directory.GetDirectories(skinRoot);
			foreach (string directory in childDirectories)
			{
				string dirName = GetNameOnly(directory);
				SkinDirectoryItem sd = new SkinDirectoryItem();
				sd.SkinDirectory = dirName;
				sd.Name = dirName;

				directories.Add(sd);
			}

			return directories;
		}

		#endregion

		#region SSLMode methods

		public class SSLModeItem
		{
			public int SSLModeID = 0;
			public string Name = "";
		}

		[IceWebMethod]
		public List<SSLModeItem> GetSSLModeCollection()
		{
			List<SSLModeItem> sslModesList = new List<SSLModeItem>();

			foreach (int sslModeInt in Enum.GetValues(typeof(SSLModeUtility.SSLModeEnum)))
			{
				SSLModeItem sslModeItem = new SSLModeItem();
				sslModeItem.SSLModeID = sslModeInt;
				sslModeItem.Name = ((SSLModeUtility.SSLModeEnum)sslModeInt).ToString();
				sslModesList.Add(sslModeItem);
			}

			return sslModesList;
		}

		#endregion

		#region Message methods

		[IceWebMethod]
		public List<Message> GetMessageCollection()
		{
			Authenticate(false);
			return SmartObjectLoader.LoadCollection<Message>(1000, false, db.Message.Ordinal, false);
		}

		[IceWebMethod]
		public List<Message> GetMessageCollectionActiveOnly()
		{
			Authenticate(false);
			return SmartObjectLoader.LoadCollection<Message>(1000, true, db.Message.Ordinal, false);
		}

		[IceWebMethod]
		public void SaveMessageCollection(List<Message> list)
		{
			Authenticate(true);
			foreach (Message wu in list)
			{
				wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
			}

		}

		[IceWebMethod]
		public void NewMessage()
		{
			Authenticate(true);
			Message wu = new Message();
			wu.Ordinal = int.MaxValue;
			wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);

		}

		[IceWebMethod]
		public void DeleteMessage(Message wu)
		{
			Authenticate(true);
			wu.DeleteFromDB();

		}

		#endregion

		#region Order methods

		public class OrderStatuses
		{
			public class NameValue
			{
				public string Name;
				public int Value;

				public NameValue() { }

				public NameValue(string name, int value)
				{
					this.Name = name;
					this.Value = value;
				}
			}

			public List<NameValue> OrderStatus = new List<NameValue>();
			public List<NameValue> PaymentStatus = new List<NameValue>();

			public OrderStatuses()
			{
				foreach (int value in Enum.GetValues(typeof(Order.OrderStatusEnum)))
				{
					OrderStatus.Add(new NameValue(((Order.OrderStatusEnum)value).ToString(), value));
				}
				foreach (int value in Enum.GetValues(typeof(Order.PaymentStatusEnum)))
				{
					PaymentStatus.Add(new NameValue(((Order.PaymentStatusEnum)value).ToString(), value));
				}
			}
		}

		public class OrderStatusCollection : List<Hashtable>
		{
			public OrderStatusCollection()
			{
				foreach (int value in Enum.GetValues(typeof(Order.OrderStatusEnum)))
				{
					Hashtable ht = new Hashtable();
					ht.Add("OrderStatusEnumId", value);
					ht.Add("Name", ((Order.OrderStatusEnum)value).ToString());
					this.Add(ht);
				}
			}
		}

		public class PaymentStatusCollection : List<Hashtable>
		{
			public PaymentStatusCollection()
			{
				foreach (int value in Enum.GetValues(typeof(Order.PaymentStatusEnum)))
				{
					Hashtable ht = new Hashtable();
					ht.Add("PaymentStatusEnumId", value);
					ht.Add("Name", ((Order.PaymentStatusEnum)value).ToString());
					this.Add(ht);
				}
			}
		}

		[IceWebMethod]
		public OrderStatusCollection GetOrderStatusCollection()
		{
			return new OrderStatusCollection();
		}

		[IceWebMethod]
		public PaymentStatusCollection GetPaymentStatusCollection()
		{
			return new PaymentStatusCollection();
		}

		[IceWebMethod]
		public OrderStatuses GetOrderStatuses()
		{
			return new OrderStatuses();
		}

		[IceWebMethod]
		public PagedCollection< Order> GetOrderCollectionPagedSorted(int recordsPerPage, int startRecordIndex, object orderByColumn, bool orderAsc, int option1, int option2)
		{
			Authenticate(false);
			PagedCollection< Order> pc = null;

			pc = new PagedCollection<Order>();
			pc.totalRecords = new Query().Count(db.Order).ExecuteScalar<Order, int>(); // count all records
			List<Order> orders = OrderCollection.LoadPaged(recordsPerPage, startRecordIndex / recordsPerPage, orderByColumn, orderAsc);
			pc.collection = orders;
			return pc;
		}

		[IceWebMethod]
		public void SaveOrderCollection(List<Order> list)
		{
			Authenticate(true);
			foreach (Order wu in list)
			{
				wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
			}
		}

		/// <summary>
		/// Add new Order to db. Returns UserID.
		/// </summary>
		/// <returns></returns>
		[IceWebMethod]
		public int NewOrder()
		{
			Authenticate(true);
			Order wu = new Order();
			wu.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);

			return wu.OrderID;
		}

		[IceWebMethod]
		public void DeleteOrder(Order wu)
		{
			Authenticate(true);
			wu.DeleteFromDB();

		}

		[IceWebMethod]
		public int DeleteOrders(int [] orderIds)
		{
			Authenticate(true);
			int count = 0;
			foreach (int oid in orderIds)
			{
				Order order = new Order();
				if (order.LoadFromDB(oid))
				{
					if (order.DeleteFromDB())
					{
						count++;
					}
				}
			}
			return count;
		}

		[IceWebMethod]
		public string GetOrderHtml(int orderID)
		{
			Authenticate(false);
			Order order = new Order();
			if (order.LoadFromDB(orderID))
			{
				return Ecommerce.OrderContentForEmail.EmailBuilder.GetHtml(order);
			}
			else
			{
				return "Can't load order with id: " + orderID;
			}
		}

		#endregion

		#region Newsletter methods

		[IceWebMethod]
		public int NewsletterSend(int roleId, bool subscribedOnly, string subject, string bodyHtml)
		{
			// returns number of emails to send to
			// load users
			UserCollection users = new UserCollection();
			List<string> emails = users.GetEmails(roleId, subscribedOnly);
			int emailsCount = emails.Count;
			NewsletterSender.SendNewsletter(emails, subject, bodyHtml);
			return emailsCount;
		}

		[IceWebMethod]
		public NewsStatus GetNewsletterStatus()
		{
			return NewsletterSender.GetStatus();
		}

		[IceWebMethod]
		public bool NewsletterStartStop(bool startRunning)
		{
			if (startRunning)
			{
				NewsletterSender.Start();
			}
			else
			{
				NewsletterSender.Abort();
			}
			return startRunning;
		}

		[IceWebMethod]
		public NewsStatus NewsletterReset()
		{
			NewsletterSender.Reset();
			return NewsletterSender.GetStatus();
		}

		[IceWebMethod]
		public NewsLog GetNewsletterLog()
		{
			return NewsletterSender.GetLog();
		}

		#endregion


	}
}
