﻿
using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.IO;
using System.Security.Cryptography.X509Certificates;

namespace Ice.Net.Ecommerce.Web
{
	internal class GHttpResponseStatus
	{
		private string mResponseText;
		HttpStatusCode mStatusCode;
		private string mStatusDescription;
		private byte[] mResponseData;

		public byte[] ResponseData
		{
			get { return mResponseData; }
			set { mResponseData = value; }
		}

		public string ResponseText
		{
			get { return mResponseText; }
			set { mResponseText = value; }
		}
		public string StatusDescription
		{
			get { return mStatusDescription; }
			set { mStatusDescription = value; }
		}
		public HttpStatusCode StatusCode
		{
			get { return mStatusCode; }
			set { mStatusCode = value; }
		}
	}

	internal class GHttpRequest
	{
		public static X509Certificate GetCertificate(string fullCertFilePath)
		{
			if (fullCertFilePath != null && fullCertFilePath.Length > 2)
			{
				return X509Certificate.CreateFromCertFile(fullCertFilePath);
			}
			return null;
		}

		/// <summary>
		/// Request POST method, parameters...
		/// Returns respose text.
		/// </summary>
		public static GHttpResponseStatus MakePostRequest(string url, string postParameters) // parameters:  name1=value1&name2=value2
		{
			return MakePostRequest(url, postParameters, null);
		}

		/// <summary>
		/// Request POST method, parameters...
		/// Returns respose text.
		/// </summary>
		public static GHttpResponseStatus MakePostRequest(string url, string postParameters, string certificatePath) // parameters:  name1=value1&name2=value2
		{
			GHttpResponseStatus status = new GHttpResponseStatus();
#if LOCAL
			url = "http://localhost/pftest/default.aspx";
#endif

			Stream os = null;

			try
			{
				System.Net.HttpWebRequest httpWebRequest = (System.Net.HttpWebRequest)System.Net.HttpWebRequest.Create(url);
				//string ProxyString = System.Configuration.ConfigurationManager.AppSettings[GetConfigKey("proxy")];
				//webRequest.Proxy = new WebProxy (ProxyString, true);
				//Commenting out above required change to App.Config
				////httpWebRequest.ContentType = "application/json";
				////httpWebRequest.UserAgent = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022)";

				httpWebRequest.ContentType = "application/x-www-form-urlencoded";
				httpWebRequest.Method = "POST";
				byte[] bytes = Encoding.ASCII.GetBytes(postParameters + "");

				// add certificate
				X509Certificate cert = GetCertificate(certificatePath);
				if (cert != null)
				{
					httpWebRequest.ClientCertificates.Add(cert);
				}
				// accept unsigned certificates
				//ServicePointManager.CertificatePolicy = new AcceptAllCertificatePolicy();// requires Full trust level

				// send the Post
				httpWebRequest.ContentLength = bytes.Length;   //Count bytes to send
				os = httpWebRequest.GetRequestStream();
				os.Write(bytes, 0, bytes.Length);         //Send it
				os.Flush();
				os.Close();
				os = null;

				// get response
				HttpWebResponse webResponse = (HttpWebResponse)httpWebRequest.GetResponse();
				Stream respStream = webResponse.GetResponseStream();

				// we have to read bytes, than we will convert it to string with proper encoding
				MemoryStream ms = new MemoryStream();
				int val;
				while ((val = respStream.ReadByte()) != -1)
				{
					ms.WriteByte((byte)val);
				}
				respStream.Dispose();
				status.ResponseData = ms.ToArray();
				ms.Dispose();
				status.ResponseText = System.Text.Encoding.UTF8.GetString(status.ResponseData);
				status.StatusCode = webResponse.StatusCode;
				status.StatusDescription = webResponse.StatusDescription;
			}
			catch (WebException ex)
			{
				status.ResponseText = ex.Message;
			}
			finally
			{
				if (os != null)
				{
					os.Flush();
					os.Close();
				}
			}
			return status;
		}



		/// <summary>
		/// Request POST method, parameters...
		/// Returns respose text.
		/// </summary>
		public static GHttpResponseStatus MakeGetRequest(string url, string getParameters) // parameters:  name1=value1&name2=value2
		{
			return MakeGetRequest(url, getParameters, null);
		}

		/// <summary>
		/// Request POST method, parameters...
		/// Returns respose text.
		/// </summary>
		public static GHttpResponseStatus MakeGetRequest(string url, string getParameters, string certificatePath) // parameters:  name1=value1&name2=value2
		{
			GHttpResponseStatus status = new GHttpResponseStatus();
#if LOCAL
			url = "http://localhost/pftest/default.aspx";
#endif
			if (getParameters != null && getParameters.Length > 2) // min length is 3, example a=1
			{
				url = url + "?" + getParameters;
			}

			try
			{
				System.Net.HttpWebRequest httpWebRequest = (System.Net.HttpWebRequest)WebRequest.Create(url);
				//string ProxyString = System.Configuration.ConfigurationManager.AppSettings[GetConfigKey("proxy")];
				//webRequest.Proxy = new WebProxy (ProxyString, true);
				//webRequest.ContentType = "application/x-www-form-urlencoded";
				httpWebRequest.Method = "GET";

				// add certificate
				X509Certificate cert = GetCertificate(certificatePath);
				if (cert != null)
				{
					httpWebRequest.ClientCertificates.Add(cert);
				}
				// accept unsigned certificates
				//ServicePointManager.CertificatePolicy = new AcceptAllCertificatePolicy(); // Requires full trust level

				// get response
				HttpWebResponse webResponse = (HttpWebResponse)httpWebRequest.GetResponse();
				Stream respStream = webResponse.GetResponseStream();

				// we have to read bytes, than we will convert it to string with proper encoding
				MemoryStream ms = new MemoryStream();
				int val;
				while ((val = respStream.ReadByte()) != -1)
				{
					ms.WriteByte((byte)val);
				}
				respStream.Dispose();
				status.ResponseData = ms.ToArray();
				ms.Dispose();
				status.ResponseText = System.Text.Encoding.UTF8.GetString(status.ResponseData);
				status.StatusCode = webResponse.StatusCode;
				status.StatusDescription = webResponse.StatusDescription;
			}
			catch (WebException ex)
			{
				status.ResponseText = ex.Message;
			}
			return status;
		}
	}

	internal class AcceptAllCertificatePolicy : ICertificatePolicy
	{
		public AcceptAllCertificatePolicy()
		{
		}

		public bool CheckValidationResult(ServicePoint sPoint, X509Certificate cert, WebRequest wRequest, int certProb)
		{
			// Always accept
			return true;
		}
	}

}