﻿using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using Ice.Net.Ecommerce.BusinessObjects.Common;
using Ice.Net.Ecommerce.BusinessObjects.Database.VISTACART;
using System.Text;
using System.Collections.Specialized;
using System.Collections.Generic;
using Ice.Net.Ecommerce.BusinessObjects.Database;
using Ice.Orm.Core.Entity;
using Ice.Net.Ecommerce.CacheProvider;

namespace Ice.Net.Ecommerce.Web.Modules.Common
{
	public class RequestHelper
	{
		private static object mObjLock = new object();

		/// <summary>
		/// Stores RequestHelper object into Context.Items collection. Invoking updates clicks count too.
		/// </summary>
		public static RequestHelper Current
		{
			get
			{
				RequestHelper rh = RequestStore.Retrieve<RequestHelper>();
				if (rh == null)
				{
					rh = new RequestHelper();
					
					lock (mObjLock)
					{
						rh.UpdateClicksCount();
					}

					RequestStore.Save(rh);

					LicenseManager.Current.TestMe(true);
				}
				return rh;
			}
		}

		public bool ShowProductsList
		{
			get { return FrontParameters.ProductID <= 0; }
		}

		public bool ShowProductDetails
		{
			get { return FrontParameters.ProductID > 0; }
		}

		private PlaceHolder mPlaceholderControl = null;

		public PlaceHolder PlaceholderControl
		{
			get 
			{
				if (ShowProductDetails)
				{
					// don't show dynamic pages
					return new PlaceHolder();
				}

				if (mPlaceholderControl == null)
				{
					mPlaceholderControl = new PlaceHolder();
					// load web pages
					LoadWebPagesContentControls(this.CurrentCategory.CategoryID);
				}

				return mPlaceholderControl; 
			}
		}

		private RequestHelper()
		{
		}

		public void UpdateClicksCount()
		{
			FrontParameters fp = this.FrontParameters;

			if (fp.Kind == FrontParameters.KindEnum.RedirectToFile)
			{
				return;
			}

			// create Meta info
			if (fp.WebPageID > 0)
			{
				WebPage page = new WebPage();
				if (page.LoadFromDB(fp.WebPageID))
				{
					page.ClicksCount++;
					page.LastClickDate = DateTime.UtcNow;
					page.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
				}
			}
			else if (fp.ProductID > 0)
			{
				Product product = new Product();
				if (product.LoadFromDB(fp.ProductID))
				{
					product.ClicksCount++;
					product.LastClickDate = DateTime.UtcNow;
					product.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
				}
			}
			else
			{
				int categoryID = fp.CategoryID;
				if (categoryID < 1)
				{
					categoryID = CurrentCategory.CategoryID;
				}
				Category category = new Category();
				if (category.LoadFromDB(categoryID))
				{
					category.ClicksCount++;
					category.LastClickDate = DateTime.UtcNow;
					category.SaveToDB(WriteModeEnum.SaveAllWithIDCheck);
				}
			}
		}

		private ProductsPager mProductsPager = null;
		/// <summary>
		/// Load products into pager on demand and keep cached in private variable.
		/// </summary>
		public ProductsPager ProductsPager
		{
			get
			{
				if (mProductsPager == null)
				{
					mProductsPager = new ProductsPager(this.CurrentCategory.CategoryID, this.FrontParameters, false, false);
				}
				return mProductsPager;
			}
		}

		private ProductsPager mProductsPagerSpecials = null;
		/// <summary>
		/// Load products into pager on demand and keep cached in private variable.
		/// </summary>
		public ProductsPager ProductsPagerSpecials
		{
			get
			{
				if (mProductsPagerSpecials == null)
				{
					mProductsPagerSpecials = new ProductsPager(this.CurrentCategory.CategoryID, this.FrontParameters, true, false);
				}
				return mProductsPagerSpecials;
			}
		}

		private ProductsPager mProductsPagerNewProducts = null;
		/// <summary>
		/// Load products into pager on demand and keep cached in private variable.
		/// </summary>
		public ProductsPager ProductsPagerNewProducts
		{
			get
			{
				if (mProductsPagerNewProducts == null)
				{
					mProductsPagerNewProducts = new ProductsPager(this.CurrentCategory.CategoryID, this.FrontParameters, false, true);
				}
				return mProductsPagerNewProducts;
			}
		}

		private Product mProduct = null;

		public Product Product
		{
			get
			{
				if (mProduct == null)
				{
					mProduct = new Product();
					if (mProduct.LoadFromDB(this.FrontParameters.ProductID) && mProduct.IsActive)
					{
					}
					else
					{
						mProduct = new Product();
					}
				}
				return mProduct;
			}
		}

		private ProductData mProductData = null;
		
		public ProductData ProductData
		{
			get
			{
				if (mProductData == null)
				{
					mProductData = new ProductData(this.Product, this.FrontParameters);
				}
				return mProductData;
			}
		}

		private FrontParameters mFrontParameters = null;
		/// <summary>
		/// <para>Get on demand and keep cached in private variable.</para>
		/// </summary>
		public FrontParameters FrontParameters
		{
			get
			{
				if (mFrontParameters == null)
				{
					mFrontParameters = new FrontParameters(System.Web.HttpContext.Current);
				}
				return mFrontParameters;
			}
		}

		private BusinessObjects.Database.VISTACART.CategoryCollection mCategoryChain = null;
		/// <summary>
		/// Get clicked category chain keep cached in private variable, root category first.
		/// </summary>
		public BusinessObjects.Database.VISTACART.CategoryCollection CategoryChain
		{
			get
			{
				if (mCategoryChain == null)
				{
					mCategoryChain = new Ice.Net.Ecommerce.BusinessObjects.Database.VISTACART.CategoryCollection();
					mCategoryChain.LoadCategoriesChain(this.FrontParameters.CategoryID, VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.RootCategoryID, true);
				}
				return mCategoryChain;
			}
		}

		private String mNavigationMap = null;
		/// <summary>
		/// <para>Get navigation map and keep cached in local variable.</para>
		/// </summary>
		public String GetNavigationMap(String separator, bool excludeRootCategory)
		{
			if (mNavigationMap == null)
			{
				mNavigationMap = NavigationMap.GetNavigationMap(this.FrontParameters, VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.RootCategoryID, separator, excludeRootCategory);
			}
			return mNavigationMap;
		}

		private PageMetaInfo mPageMetaInfo = null;
		/// <summary>
		/// Get page meta info and keep cached in local variable.
		/// </summary>
		public PageMetaInfo PageMetaInfo
		{
			get
			{
				if (mPageMetaInfo == null)
				{
					mPageMetaInfo = new PageMetaInfo(this.FrontParameters, VISTACARTHelper.AppDefaults.AppGlobalSettingsItem.RootCategoryID);
				}
				return mPageMetaInfo;
			}
		}

		private CurrentUser mCurrentUser = null;
		/// <summary>
		/// Get current user object and keep in local variable
		/// </summary>
		public CurrentUser CurrentUser
		{
			get
			{
				if (mCurrentUser == null)
				{
					mCurrentUser = VISTACARTHelper.CurrentUser;
				}
				return mCurrentUser;
			}
		}

		private AppDefaults mAppDefaults = null;
		/// <summary>
		/// Get AppDefaults object and keep in local variable.
		/// </summary>
		public AppDefaults AppDefaults
		{
			get
			{
				if (mAppDefaults == null)
				{
					mAppDefaults = VISTACARTHelper.AppDefaults;
				}
				return mAppDefaults;
			}
		}

		public bool IsRootCategory
		{
			get { return this.CurrentCategory.CategoryID == this.AppDefaults.AppGlobalSettingsItem.RootCategoryID; }
		}

		private Category mCurrentCategory = null;

		public Category CurrentCategory
		{
			get
			{
				if (mCurrentCategory == null)
				{
					int clickedCategoryID = this.FrontParameters.CategoryID;
					if (clickedCategoryID == 0)
					{
						clickedCategoryID = this.AppDefaults.AppGlobalSettingsItem.RootCategoryID.GetValueOrDefault();
					}

					mCurrentCategory = new Category();
					if (!string.IsNullOrEmpty(this.FrontParameters.SearchExpression))
					{
					}
					else if (this.FrontParameters.Kind == FrontParameters.KindEnum.CategoryShortcut && (mCurrentCategory = Category.LoadByShortcutName(this.FrontParameters.CategoryShortcutName)) != null && mCurrentCategory.IsActive && mCurrentCategory.IsPublished)
					{
						this.FrontParameters.CategoryID = mCurrentCategory.CategoryID;
						this.FrontParameters.LinkName = mCurrentCategory.Name;
					}
					else if (mCurrentCategory.LoadFromDB(clickedCategoryID) && mCurrentCategory.IsActive && mCurrentCategory.IsPublished)
					{
						this.FrontParameters.CategoryID = mCurrentCategory.CategoryID;
						this.FrontParameters.LinkName = mCurrentCategory.Name;
					}
				}
				return mCurrentCategory;
			}
		}

		private List<WebPage>GetWebPages(int clickedCategoryID)
		{
			return WebPageCollection.LoadPagedByCategoryID(clickedCategoryID, 100, 0, db.WebPage.Ordinal, false, true);
		}

		private void LoadWebPagesContentControls(int clickedCategoryID)
		{
			List<WebPage> wpcoll = GetWebPages(clickedCategoryID);

			if (wpcoll == null || wpcoll.Count == 0)
			{
			}
			else
			{
				foreach (WebPage wp in wpcoll)
				{
					if (wp.IsActive == false)
					{
						continue;
					}
					if (!string.IsNullOrEmpty(wp.AscxModulePath)) // if it has ascx file path, use ascx
					{
						string path = wp.AscxModulePath.Replace("|SkinDirectory|", ApplicationUtility.SkinDirectory);
						Page p = new Page();
						Control cc = (Control)p.LoadControl(path);
						mPlaceholderControl.Controls.Add(cc);
					}
					else // use html
					{
						mPlaceholderControl.Controls.Add(new LiteralControl(wp.Content));
					}
				}
			}

		}
	}
}
