using System;
using System.Web;
using System.Web.Security;
using System.Security.Principal;

namespace Ice.Elmah.Security
{
	//****************************************************************
	// IMPORTANT: web config settings: <authentication mode="None"/>
	//****************************************************************

	/// <summary>
	/// Summary description for AuthenticationModule.
	/// </summary>
	public class BasicElmahAuthenticationModule : IHttpModule
	{
		protected virtual string REALM { get { return HttpContext.Current.Request.UserHostName; } }

		#region IHttpModule Members

		public void Init(HttpApplication context)
		{
			//  Add BasicModule.Init implementation
			context.AuthenticateRequest += new EventHandler(context_AuthenticateRequest);
			context.EndRequest += new EventHandler(context_EndRequest);
			context.PreSendRequestHeaders += new EventHandler(context_PreSendRequestHeaders); // Fired before buffered response headers are sent
		}

		void context_PreSendRequestHeaders(object sender, EventArgs e)
		{
			//	context_EndRequest(sender, e);
		}

		public void Dispose()
		{
			// Add BasicModule.Dispose implementation
		}

		#endregion

		/// <summary>
		/// Handles the AuthenticateRequest event of the application context.
		/// </summary>
		/// <param name="sender">The Application instance.</param>
		/// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		private void context_AuthenticateRequest(object sender, EventArgs e)
		{
			HttpApplication application = (HttpApplication)sender;
			if (application.Request.RawUrl.Contains("elmah.axd") && !application.Context.Request.IsAuthenticated)
			{
				string sAUTH = application.Request.ServerVariables["HTTP_AUTHORIZATION"];
				if (sAUTH == null) return;
				//Received Credentials, Authenticate user
				if (sAUTH.Substring(0, 5).ToUpper() == "BASIC")
				{
					string[] sCredentials;
					sCredentials = Base64Decode(sAUTH.Substring(6)).Split(':');
					GenericPrincipal UserPrincipal = Authenticate(sCredentials);
					if (UserPrincipal != null)
					{
						bool authenticated = FormsAuthentication.Authenticate(sCredentials[0], sCredentials[1]);
						application.Context.User = UserPrincipal;
					}
				}
			}
		}

		/////// <summary>
		/////// Authenticates the specified credentials. Return null if not authenticated.
		/////// </summary>
		/////// <param name="Credentials">The credentials (Username and Password).</param>
		/////// <param name="Roles">The string array containing roles.</param>
		/////// <returns></returns>
		////protected virtual GenericPrincipal Authenticate(string[] Credentials)
		////{
		////    string[] Roles = null;
		////    GenericPrincipal UserPrincipal = new GenericPrincipal(new GenericIdentity(Credentials[0]), Roles);
		////    return UserPrincipal;
		////}

		static string _username = System.Configuration.ConfigurationManager.AppSettings["ElmahUsername"];
		static string _password = System.Configuration.ConfigurationManager.AppSettings["ElmahPassword"];

		protected GenericPrincipal Authenticate(string[] Credentials)
		{
			if (string.IsNullOrEmpty(_username) || (string.IsNullOrEmpty(_password)))
			{
				throw new Exception("Elamah username & password missing in web.config!");
			}

			// Please override with your own logic of authentication
			if (Credentials.Length == 2 && Credentials[0] == _username && Credentials[1] == _password)
			{
				return new GenericPrincipal(new GenericIdentity(Credentials[0]), null);
			}
			else
			{
				// invalid data
				System.Threading.Thread.Sleep(2000);
				return null;
			}
		}

		/// <summary>
		/// Decodes Base64 encoded string.
		/// </summary>
		/// <param name="EncodedData">The encoded data.</param>
		/// <returns></returns>
		private string Base64Decode(string EncodedData)
		{
			try
			{
				System.Text.UTF8Encoding encoder = new System.Text.UTF8Encoding();
				System.Text.Decoder utf8Decode = encoder.GetDecoder();
				byte[] todecode_byte = Convert.FromBase64String(EncodedData);
				int charCount = utf8Decode.GetCharCount(todecode_byte, 0, todecode_byte.Length);
				char[] decoded_char = new char[charCount];
				utf8Decode.GetChars(todecode_byte, 0, todecode_byte.Length, decoded_char, 0);
				return new String(decoded_char);
			}
			catch (Exception e)
			{
				throw new Exception("Error in base64Decode" + e.Message);
			}
		}

		/// <summary>
		/// Handles the EndRequest event of the application context.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		private void context_EndRequest(object sender, EventArgs e)
		{
			HttpApplication application = (HttpApplication)sender;
			if (application.Request.RawUrl.Contains("elmah.axd") && application.Response.StatusCode == 401)
			{
				application.Response.ClearHeaders(); // asp.net nakuca sve moguce WWW-Authenticate protokole, ukloniti ih sve 
				application.Response.StatusCode = 401;
				application.Response.AddHeader("WWW-Authenticate", "BASIC Realm=" + REALM);
			}
		}
	}
}
