﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Ice.Orm.Core.DataStore;
using System.Reflection;
using Ice.Utility;
using System.Data;

namespace Ice.Orm.Core.Criteria
{
	public class ObjectMaterializer
	{
		private static bool IsTypeOf(Type baseType, Type inheritedType)
		{
			Type curr = inheritedType;

			while (true)
			{
				if (baseType == curr)
				{
					return true;
				}

				if (curr != null)
				{
					curr = curr.BaseType;
				}
				else
				{
					return false;
				}
			}
		}

		public static List<TResult> Translate<TResult>(DataTable dataTable, DataStoreBase store = null)
		{
			return Translate<TResult>(SimpleDataTable.Create(dataTable), store);
		}

		public static List<TResult> Translate<TResult>(SimpleDataTable gdt, DataStoreBase store)
		{
			return Translate(typeof(TResult), gdt, store).Cast<TResult>().ToList();
		}

		public static List<object> Translate(Type resultType, SimpleDataTable gdt, DataStoreBase store)
		{
			bool useSetDataRow = true; // useSetDataRow = false for objects inherited from EntityBase with custom properties whos values can't be accessed via EntityBse.GetValue<>()
			List<object> list = new List<object>();
			object res1;
			ConstructorInfo[] constructors = null;
			object[] parameters;
			object obj;
			bool isEntityBaseType = IsTypeOf(typeof(Entity.EntityBase), resultType);
			bool isAnonymousType = !isEntityBaseType && resultType.Name.Contains("AnonymousType") && (constructors = resultType.GetConstructors()).Count() > 0;

			for (int n = 0; n < gdt.Rows.Count; n++)
			{
				SimpleDataRow row = gdt.GetDataRow(n);

				if (isEntityBaseType)
				{
					Entity.EntityBase t = (Entity.EntityBase)(object)Activator.CreateInstance(resultType);
					t.SetDataRow(row);
					list.Add((object)t);

					if (store != null) // non-default datastore
					{
						t.SetDataStore(store);
					}
				}
				else if (isAnonymousType)
				{
					List<object> parametersList = new List<object>();
					// compound select is supported: .Select((o, k, t) => new {oglas = o, kor = k, o.Name, Tip = t.Name, k.Username })
					// .SelectDirect(o=>new{o.Name, o.Age})
					// there is no parameterless constructor
					foreach (ParameterInfo pi in constructors.First().GetParameters())
					{
						if (IsTypeOf(typeof(Entity.EntityBase), pi.ParameterType))
						{
							if (useSetDataRow)
							{
								// fill as EntityBase type via SetDataRow()
								Entity.EntityBase t = (Entity.EntityBase)Activator.CreateInstance(pi.ParameterType);
								t.SetDataRow(row);
								if (store != null)
								{
									t.SetDataStore(store);
								}
								parametersList.Add(t);
							}
							else
							{
								// fill properties
								object t = Activator.CreateInstance(pi.ParameterType);

								throw new NotImplementedException("to be implemented");
							}
						}
						else
						{
							IceTypeConverter.TryChangeType(row.ItemArray[row.Table.GetColumnIndex(pi.Name, true)], pi.ParameterType, out obj);
							parametersList.Add(obj);
						}
					}
					parameters = parametersList.ToArray();
					res1 = Activator.CreateInstance(resultType, parameters);
					list.Add(res1);
				}
				else if (row.ItemArray.Length == 1 && IceTypeConverter.TryChangeType(row.ItemArray[0], resultType, out obj))
				{
					// it row.ItemsArray.Lenght == 1, result may by eg. List<int>, than try to change ItemArray[0] to resultType
					// .SelectDirect(o=> o.Name)
					list.Add(obj);
				}
				else
				{
					// parameterless constructor found, it can be any custom object, but not Anonymous type

					res1 = Activator.CreateInstance(resultType);

					resultType.GetProperties().Select(o => new
					{
						Name = o.Name,
						Property = o,
						ColumnIndex = row.Table.GetColumnIndex(o.Name, false) // case non-sensitive search, don't throw exception if column not found, same as MS Translator<>
					})
					.Where(o => o.ColumnIndex >= 0) // fill properties for which there are columns found, other are ignored
					.Select(o => new
					{
						Name = o.Name,
						Property = o.Property,
						Value = IceTypeConverter.ChangeType(row.ItemArray[o.ColumnIndex], o.Property.PropertyType) // throw exception if type can't be converted
					})
					.ToList()
					.ForEach(o =>
					{
						o.Property.SetValue(res1, o.Value, null);
					});

					list.Add(res1);
				}

			}
			return list;
		}

	}
}
