﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;
using System.Linq;

namespace Ice.Orm.Core.Criteria
{
	using LINQ;
	using System.Collections.ObjectModel;

	public class QParameterCollection
	{
		public string ParameterPrefix = "@";

		private List<QParameter> items = new List<QParameter>();

		public ReadOnlyCollection<QParameter> AsReadOnly()
		{
			return items.AsReadOnly();
		}

		private string GetUniqueName()
		{
			return this.ParameterPrefix + "p" + items.Count;
		}

		/// <summary>
		/// Add parameter to the collection, creates unique Name if Name is null or empty. If the same parameter.Name already exist, create new QParameter. Returns QParameter.
		/// </summary>
		/// <param name="p"></param>
		/// <returns></returns>
		public QParameter Add(QParameter p)
		{
			if (string.IsNullOrEmpty(p.Name))
			{
				// the most common situation, adding new parameter
				p.Name = GetUniqueName();
			}
			else if (items.Contains(p))
			{
				// parameter is already added, parameter will be reused in sql text, it won't work on ODBC and OleDB
				// create new parameter clone so it will work with Odbc and Oledb
				p = new QParameter(p.Value, p.Column) { Name = GetUniqueName() };
			}
			else if (items.Where(o => o.Name == p.Name).Any())
			{
				// parameter with the same name already exists, create new unique name
				//p.Name = GetUniqueName();
				// manually created parameter with duplicated name
				throw new Exception("QParameter with the same name already exists in the collection, name: " + p.Name);
			}
			
			items.Add(p);
			return p;
		}

		public int Count { get { return items.Count; } }

		public string GetInsertStatement(string tableName)
		{
			return "INSERT INTO " + SqlName.CreateFullSqlName(tableName) + " (" + items.Select(o => SqlName.CreateFullSqlName(o.Column)).ToStringJoined(", ") + ") VALUES (" + items.Select(o => o.Name).ToStringJoined(", ") + ")";
		}

		public string GetUpdateStatement(string tableName)
		{
			return "UPDATE " + SqlName.CreateFullSqlName(tableName) + " SET " + items.Select(o => SqlName.CreateFullSqlName(o.Column) + " = " + o.Name).ToStringJoined(", ");
		}

	}


}
