﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Ice.Orm.Core.DataStore;
using Ice.Orm.Core.Entity;
using Ice.Orm.Core.Criteria.SqlBlocks;

namespace Ice.Orm.Core.Criteria
{
	[System.Diagnostics.DebuggerDisplay("{_query.QueryWithValues}")]
	public class QuerySelect<TSource, TResult> : ICompilableQuery
		where TSource : EntityBase
	{
		QueryBase _query;

		internal QuerySelect(QueryBase source)
		{
			this._query = source;
		}

		public QuerySelect<TSource, TResult> Skip(int skip)
		{
			_query.SkipCount = skip;
			return this;
		}

		public QuerySelect<TSource, TResult> Take(int count)
		{
			_query.TakeCount = count;
			return this;
		}

		/// <summary>
		/// Add DISTINCT to sql
		/// </summary>
		/// <returns></returns>
		public QuerySelect<TSource, TResult> Distinct()
		{
			_query.IsDistinct = true;
			return this;
		}

		public List<TResult> ToList()
		{
			return _query.ToList<TSource, TResult>();
		}

		public TResult FirstOrDefault()
		{
			this._query.TakeCount = 1;
			return this.ToList().FirstOrDefault();
		}

		#region ICompilableQuery Members

		IEnumerable<SqlSegment> ICompilableQuery.SelectSqlSegments()
		{
			return _query.SelectSqlSegments();
		}

		CompiledQuery ICompilableQuery.CompileQuery()
		{
			return _query.CompileQuery();
		}

		#endregion
	}



}
