﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Ice.Orm.Core.Criteria;
using System.Collections.ObjectModel;

namespace Ice.Orm.Core.Criteria.SqlBlocks
{
	internal class SqlInsertBlock : ISqlBlock
	{
		public BlockKindEnum Kind { get { return BlockKindEnum.Insert; } }

		private SqlSegmentCollection _sqlSegments = new SqlSegmentCollection();

		public void Add(SqlName tableName, params QParameter[] args)
		{
			this.TableName = tableName.ToFullSqlName();
			args.ForEach(o => _sqlSegments.Add(o));
		}

		private string TableName;
		
		public void Clear() { _sqlSegments.Clear(); this.TableName = null; }

		public bool IsEmpty { get { return string.IsNullOrEmpty(this.TableName) || !_sqlSegments.Any(); } }

		public IEnumerable<SqlSegment> SelectSqlSegments()
		{
			if (!this.IsEmpty)
			{
				yield return new SqlSegment("INSERT INTO " + this.TableName + "\r\n(");
				yield return new SqlSegment(_sqlSegments.AsEnumerable().Select(o => o.Parameter.FullSqlColumn).ToStringJoined(", "));
				yield return new SqlSegment(")\r\nVALUES (");

				// insert comma between 2 values
				bool firstAdded = false;
				foreach (SqlSegment seg in _sqlSegments.AsEnumerable())
				{
					if (firstAdded)
					{
						yield return new SqlSegment(",");
					}
					yield return seg;
					firstAdded = true;
				}
				yield return new SqlSegment(")");
			}
		}

	}
}
