﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;

namespace Ice.Orm.Core.Criteria.SqlBlocks
{
	internal class SqlOrderByBlock : ISqlBlock
	{
		private SqlSegmentCollection _sqlSegments = new SqlSegmentCollection();

		public BlockKindEnum Kind { get { return BlockKindEnum.OrderBy; } }

		public void AddText(string sqlText)
		{
			if (string.IsNullOrEmpty(sqlText))
			{
				return;
			}
			sqlText = sqlText.TrimStart();
			if (sqlText.StartsWith("order by", StringComparison.OrdinalIgnoreCase))
			{
				sqlText = sqlText.Substring(8);
			}
			_sqlSegments.Add(sqlText);
		}

		public void AddColumn(string columnName, bool descending)
		{
			_sqlSegments.Add(columnName + " " + (descending ? "DESC" : "ASC"));
		}

		public void AddColumn(SqlName columnName, bool descending)
		{
			AddColumn(columnName.ToFullSqlName(), descending);
		}

		public void AddSegments(IEnumerable<SqlSegment> segments)
		{
			segments.AsEnumerable().ForEach(o => _sqlSegments.Add(o));
		}

		public void Clear() { _sqlSegments.Clear(); }
		public bool IsEmpty { get { return !_sqlSegments.Any(); } }

		public IEnumerable<SqlSegment> SelectSqlSegments()
		{
			if (!this.IsEmpty)
			{
				yield return new SqlSegment("\r\n");
				yield return new SqlSegment("ORDER BY " + _sqlSegments.AsEnumerable().Select(o=>o.Text).ToStringJoined(", "));
			}
		}

	}
}
