﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;

namespace Ice.Orm.Core.Criteria.SqlBlocks
{
	/// <summary>
	/// Create select statement. Names must be separated with ',' which has to be added to _sqlSegments at creation time.
	/// </summary>
	internal class SqlSelectBlock : ISqlBlock
	{
		private SqlSegmentCollection _sqlSegments = new SqlSegmentCollection();

		public BlockKindEnum Kind { get { return BlockKindEnum.Select; } }
		public bool IsDistinct { get; set; }
		public int Skip { get; set; }
		public int? Take { get; set; }
		public bool ParentherizeTop { get; set; }
		public bool UseOffsetAndLimit { get; set; }

		public void AddText(string sqlText)
		{
			if (string.IsNullOrEmpty(sqlText))
			{
				return;
			}

			if (this._sqlSegments.Any())
			{
				this._sqlSegments.Add(", ");
			}
			sqlText = sqlText.TrimStart();
			if (sqlText.StartsWith("select", StringComparison.OrdinalIgnoreCase))
			{
				sqlText = sqlText.Substring(6);
			}
			this._sqlSegments.Add(sqlText);
		}

		public void AddNames(params SqlName[] names)
		{
			if (this._sqlSegments.Any())
			{
				this._sqlSegments.Add(", ");
			}
			if (names == null || names.Length == 0)
			{
				this._sqlSegments.Add("*");
			}
			else
			{
				this._sqlSegments.Add(names.Select(o => o.ToFullSqlName()).ToStringJoined(", "));
			}
		}

		public void AddSegment(SqlSegment segment)
		{
			_sqlSegments.Add(segment);
		}

		public void AddSegments(IEnumerable<SqlSegment> segments)
		{
			segments.AsEnumerable().ForEach(o => { _sqlSegments.Add(o); });
		}

		public void Clear() { _sqlSegments.Clear(); }
		public bool IsEmpty { get { return !_sqlSegments.Any(); } }

		public IEnumerable<SqlSegment> SelectSqlSegments()
		{
			if (!this.IsEmpty)
			{
				// items alreay have ',' beween, created in SelectorSqlFormater.VisitBindingList()
				string take = null;
				if (!this.UseOffsetAndLimit)
				{
					int? takeVal = this.Skip + this.Take;
					if (takeVal > 0)
					{
						if (this.ParentherizeTop)
						{
							take = "(" + takeVal + ")";
						}
						else
						{
							take = takeVal + "";
						}
						take = " TOP " + take;
					}
				}
				string distinct = (this.IsDistinct ? " DISTINCT " : null);
				yield return new SqlSegment("SELECT" + take + distinct + " " + _sqlSegments.AsEnumerable().Select(o => o.Text).ToStringJoined());
			}
		}
	}
}
