﻿
//********************************
//#define IceTRACE
//********************************


using System;
using System.Collections;
using System.Data;
using System.Data.Odbc;
using System.Text;
using System.Collections.Generic;
using System.Data.Common;
using System.Threading;

namespace Ice.Orm.Core.DataStore
{
	using Criteria;
	using Entity;
	using Utility;
	using Schema;


	public class ConnectionContextPool
	{
		private List<ConnectionContext> mConnections = null;
		private int mBussyCount = 0;
		private Dictionary<Thread, ConnectionContext> mThreadConnections = null;

		// not static because DataStoreBase creates single instance of ConnectionContextPool
		private object mPoolLock = new object();

		public ConnectionContextPool()
		{
		}

		public void Initialize(int connectionPoolSize)
		{
			if (connectionPoolSize < 1)
			{
				throw new Exception("Database connection pool size " + connectionPoolSize + ", it has to be > 0");
			}

			if (mConnections != null)
			{
				return; // already initialized
			}

			mThreadConnections = new Dictionary<Thread, ConnectionContext>();
			mConnections = new List<ConnectionContext>(connectionPoolSize);

			for (int n = 0; n < connectionPoolSize; n++)
			{
				mConnections.Add(new ConnectionContext());
			}
		}


		/// <summary>
		/// Get available free ConnectionContext object or wait for one to become free
		/// </summary>
		/// <returns></returns>
		public ConnectionContext GetConnContext()
		{
			// try to get next free object, else wait for one to become free
			lock (mPoolLock)
			{
				while (true)
				{
					// find by the same thread, the same thread may reuse context, required for running in Transaction
					if (mBussyCount > 0)
					{
						ConnectionContext context;
						if (mThreadConnections.TryGetValue(Thread.CurrentThread, out context))
						{
							context.IsBussy = true;
							return context;
						}
					}

					// find next available
					foreach (ConnectionContext context in mConnections)
					{
						if (!context.IsBussy)
						{
							context.ThreadOwner = Thread.CurrentThread;
							context.IsBussy = true;

							mThreadConnections.Add(Thread.CurrentThread, context);

							mBussyCount++;

#if IceTRACE			
						System.Diagnostics.Trace.WriteLine("ConnectionContext inc! Bussy count: " + mBussyCount);
#endif
							return context;
						}
					}

#if IceTRACE
					 System.Diagnostics.Trace.WriteLine("ConnectionContext all bussy! Bussy count: " + mBussyCount);
#endif

					// if comes here, all connections are bussy, wait for one to be released
					System.Threading.Monitor.Wait(mPoolLock);
				}
			}
		}

		public void ReleaseConnContext(ConnectionContext context)
		{
			// close connection if required
			lock (mPoolLock)
			{
				context.IsBussy = false; // decrement mEnterCount
				if (!context.IsBussy) // not bussy only if mEnterCount == 0
				{
					mThreadConnections.Remove(context.ThreadOwner);

					context.ThreadOwner = null;

					mBussyCount--;

#if IceTRACE
					System.Diagnostics.Trace.WriteLine("ConnectionContext dec! Bussy count: " + mBussyCount);
#endif

					System.Threading.Monitor.Pulse(mPoolLock);
				}
			}
		}
	}


}

