﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Web;
using System.Linq;

namespace Ice.Orm.Core.DataStore
{
	public class DataStoreCollection
	{
		private static bool FileExists(string file)
		{
			try
			{
				return System.IO.File.Exists(HttpRuntime.AppDomainAppPath + file);
			}
			catch
			{
				return false;
			}
		}

		static bool mLocalSqlDatabase = FileExists(@"..\LocalSqlDatabase.txt");

		private static bool IsLocalSqlDatabase { get { return mLocalSqlDatabase; } }

		private static Dictionary<string, DataStoreBase> mItems = new Dictionary<string, DataStoreBase>();

		public static DataStoreBase GetFirstAvailable()
		{		
			lock (mItems)
			{
				return	mItems.Where(o => !o.Key.StartsWith("SelfTestDB"))
					.Where(o => !IsLocalSqlDatabase || o.Key.EndsWith("_LOCAL"))
					.Select(o=>o.Value)
					.FirstOrDefault();
			}

			throw new Exception("No DataStore specified!");
		}

		/// <summary>
		/// Get existing from static collection, else load new from XML and add to static collection
		/// </summary>
		/// <param name="databaseName"></param>
		/// <returns></returns>
		public static DataStoreBase GetByDatabaseName(string dbName)
		{
			lock (mItems)
			{
				DataStoreBase dbStore;

				if (IsLocalSqlDatabase && !dbName.Contains("_LOCAL")) // test for RemoteDB store
				{
					dbName += "_LOCAL";
				}

				if(!mItems.TryGetValue(dbName, out dbStore))
				{
					// not found in the collection, load new from XML
					DataStoreSetting dbSetting = new DataStoreSetting(dbName);

					// load DBStore depending on dbSetting parameters
					dbStore = DataStoreSelector.GetDataStore(dbSetting);
					
					mItems.Add(dbName, dbStore);
				}
				return dbStore;
			}
		}
	}
}