﻿/********************************************************************
 *  System.Data.SqlServerCe.dll requires Full Trust
********************************************************************/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.Common;
using System.Data.Odbc;
using System.Text.RegularExpressions;
using System.Reflection;
using System.Data;
using Ice.Orm.Core.Utility;
using Ice.Orm.Core.Entity;
using Ice.Orm.Core.Criteria;
using Ice.IO;
using System.IO;

namespace Ice.Orm.Core.DataStore.MSSQLCE
{
	internal class MSSQLCEDataStore : DataStoreBase
	{
		public MSSQLCEDataStore(DataStoreSetting dbSetting)
			: base(dbSetting)
		{
		}

		static Type readerType;
		static Type connectionType;
		static Type dataAdapterType;

		/// <summary>
		/// connection which will be used to create new command only
		/// </summary>
		static IDbConnection mConnection;
		/// <summary>
		/// command which will be used to create new parameter only
		/// </summary>
		static IDbCommand mCommand;

		private static bool initDone = new Func<bool>(() =>
		{
			// System.Data.SqlServerCe.dll ne moze biti refrenciran jer zelim forsirati verziju dll-a 3.5.10, a vs ubacuje novu verziju 4.0 koja ne radi dobro

			try
			{
				string[] files = null;

				try
				{
					// web application
					files = System.IO.Directory.GetFiles(System.Web.HttpRuntime.BinDirectory, "*.dll", System.IO.SearchOption.AllDirectories);
				}
				catch
				{
					// win application
					files = System.IO.Directory.GetFiles(Path.GetFullPath(Assembly.GetExecutingAssembly().Location), "*.dll", System.IO.SearchOption.AllDirectories);
				}

				// works with SqlServerCe.dll 3.5.10 version only
				string file = files.First(o => o.EndsWith("SqlCE3510\\System.Data.SqlServerCe.dll", StringComparison.OrdinalIgnoreCase));

				Assembly a = Assembly.LoadFrom(file);

				connectionType = a.GetType("System.Data.SqlServerCe.SqlCeConnection");
				readerType = a.GetType("System.Data.SqlServerCe.SqlCeDataReader");
				dataAdapterType = a.GetType("System.Data.SqlServerCe.SqlCeDataAdapter");

				mConnection = (IDbConnection) Activator.CreateInstance(connectionType);
				mCommand = mConnection.CreateCommand();

				return true;
			}
			catch
			{
				throw new Exception("System.Data.SqlServerCe.dll can't be loaded!"); 
			}

		}).Invoke();

		protected override IDbConnection NewDbConnection()
		{
			return (IDbConnection) Activator.CreateInstance(connectionType);
		}

		protected override IDbCommand NewDbCommand()
		{
			return mConnection.CreateCommand();
		}

		protected override IDbDataAdapter NewDbDataAdapter()
		{
			return (IDbDataAdapter)Activator.CreateInstance(dataAdapterType);
		}

		/// <summary>
		/// if false, create TOP n, if true create TOP (n)
		/// </summary>
		public override bool ParentherizeTop
		{
			get { return true; }
		}

		protected override string GetSelectSQLForSchema(string dbTableName)
		{
			return "SELECT TOP (1) * FROM " + SqlName.CreateFullSqlName(dbTableName) + " WHERE 1<>1"; // must use top (1), top 1 won't work with SQL CE
		}

		protected override DataTable LoadTablesNamesFromDB()
		{
			return base.GetDataTable("SELECT * FROM INFORMATION_SCHEMA.TABLES").ToDataTable();
		}

		/// <summary>
		/// If true, use named parameters, else use ? instead
		/// </summary>
		/// <returns></returns>
		public override bool UseNamedParameters
		{
			get { return true; }
		}

		protected override IDbDataParameter CreateIDbDataParameter(string parameterName, object value, EntityFieldSetting schemaColumn)
		{
			IDbDataParameter parameter = mCommand.CreateParameter();
			parameter.ParameterName = parameterName;
			parameter.Value = value;

			FixDbParameterValueAndType(parameter, schemaColumn);

			if (schemaColumn != null)
			{
				if (schemaColumn.ProviderType != null)
				{
					PropertyInfo src = schemaColumn.ProviderType.GetType().GetProperty("SqlDbType");
					object sqldbtype = src.GetValue(schemaColumn.ProviderType, null);

					PropertyInfo pi = parameter.GetType().GetProperty("SqlDbType");
					pi.SetValue(parameter, sqldbtype, null);
				}
			}

			return parameter;
		}

		protected override string PostInsertSQL
		{
			get { return "SELECT @@IDENTITY;"; }

		}

		protected override bool SupportCompoundQuery
		{
			get { return false; }
		}


	}
}

