﻿using System;
using System.Collections;
using System.Data;
using System.Data.SqlClient;
using MySql.Data.MySqlClient;
using System.Collections.Generic;
using System.Text.RegularExpressions;

// REQUIRES HIGH OR FULL TRUST

namespace Ice.Orm.Core.DataStore.MySQL
{
	using Criteria;
	using Entity;
	using Schema;

	public class MySQLDataStore : DataStoreBase
	{
		public MySQLDataStore(DataStoreSetting dbSetting) : base(dbSetting)
		{
		}

		protected override IDbConnection NewDbConnection()
		{
			return new MySqlConnection();
		}

		protected override IDbCommand NewDbCommand()
		{
			return new MySqlCommand();
		}

		protected override IDbDataAdapter NewDbDataAdapter()
		{
			return new MySqlDataAdapter();
		}

		/// <summary>
		/// Defines the parameter preffix to use in commands for this data store.
		/// </summary>
		public override string ParameterPreffix
		{
			get { return "?"; }
		}

		/// <summary>
		/// Defines the object preffix to use in queries for this data store.
		/// </summary>
		protected override string ColumnPreffix
		{
			get { return "`"; }
		}

		/// <summary>
		/// Defines the object suffix to use in queries for this data store.
		/// </summary>
		protected override string ColumnSuffix
		{
			get { return "`"; }
		}

		/// <summary>
		/// If true, use named parameters, else use ? instead
		/// </summary>
		/// <returns></returns>
		public override bool UseNamedParameters
		{
			get { return true; }
		}

		public override bool UseOffsetAndLimit
		{
			get { return true; }
		}

		protected override IDbDataParameter CreateIDbDataParameter(string parameterName, object value, EntityFieldSetting schemaColumn)
		{
			MySqlParameter parameter = new MySqlParameter(parameterName, value);

			if (value != null && value.GetType() == typeof(bool))
			{
				parameter.Value = (bool) value ? 1 : 0; // bool is repersented as 1 or 0
			}

			FixDbParameterValueAndType(parameter, schemaColumn);

			if (schemaColumn.ProviderType != null)
			{
				parameter.MySqlDbType = (MySqlDbType)schemaColumn.ProviderType;
			}

			return parameter;
		}

		internal override string CleanUpSqlText(string sqlStatement)
		{
			return base.CleanUpSqlText(sqlStatement).Replace("[", ColumnPreffix).Replace("]", ColumnSuffix);
		}

		protected override string PostInsertSQL
		{
			get { return "SELECT LAST_INSERT_ID()"; }
		}

		protected override bool SupportCompoundQuery
		{
			get { return true; }
		}

		////protected override string FixSqlTextToSpeedUpSelect(string sql, int skip, int? take)
		////{
		////    sql = this.ReplaceTopWithLimit(sql); // if it is custom query with top(n) inside

		////    if (take != null && take >= 0 && sql.IndexOf("LIMIT ", StringComparison.OrdinalIgnoreCase) < 0)
		////    {
		////        int top = skip + take.GetValueOrDefault(); // skip can be > 0, while take == null

		////        sql = sql.Trim();

		////        // add LIMIT to the end
		////        sql = sql + "\r\n" + "LIMIT " + top;
		////    }
		////    // OFFSET can't be created here, if we create sql with limit and offset, we have to set skip=0, take=null to prevent datareader from skiping records
		////    return sql;
		////}

		protected override string GetSelectSQLForSchema(string dbTableName)
		{
			return CleanUpSqlText("SELECT * FROM " + this.ColumnPreffix +  dbTableName + this.ColumnSuffix+ " WHERE 1<>1 LIMIT 1");
		}

	}
}
