﻿using System;
using System.Collections;
using System.Data;
using System.Collections.Generic;
using System.Linq;

namespace Ice.Orm.Core.Entity
{
	using Criteria;
	using DataStore;
	using Utility;
	using Schema;
	using System.Data.SqlClient;

	public abstract class EntityBase<T> : EntityBase where T:EntityBase<T>
	{
		////public new virtual T ReloadFromDB()
		////{
		////    base.ReloadFromDB();
		////    return (T)this;
		////}

		////public new virtual T DeleteFromDB()
		////{
		////    base.DeleteFromDB();
		////    return (T)this;
		////}

		public new virtual T SaveToDB()
		{
			base.SaveToDB();
			return (T)this;
		}

		public new virtual T SaveToDB(WriteModeEnum mode)
		{
			base.SaveToDB(mode);
			return (T)this;
		}

		public new virtual T ReloadFromDB()
		{
			base.ReloadFromDB();
			return (T)this;
		}

		public virtual T CloneValues()
		{
			return base.CloneValues<T>();
		}

		/// <summary>
		/// Change data store. If dataStoreName is null, set default database.
		/// </summary>
		/// <param name="dataStoreName"></param>
		public new T SetDataStore(string dataStoreName)
		{
			base.SetDataStore(dataStoreName);
			return (T)this;
		}

		/// <summary>
		/// Change data store.
		/// </summary>
		/// <param name="dataStore"></param>
		public new T SetDataStore(DataStoreBase dataStore)
		{
			base.SetDataStore(dataStore);
			return (T)this;
		}

		public T Case(bool condition, Action<T> onTrue, Action<T> onFalse = null)
		{
			return base.Case<T>(condition, onTrue, onFalse);
		}

		/// <summary>
		/// // for MS SQL only
		/// </summary>
		/// <param name="items"></param>
		public static void BulkInsert(IEnumerable<T> items, int batchSize = 10000)
		{
			if (items.Any())
			{
				// for MS SQL only
				T firstItem = items.First();

				DataTable schemaTable = firstItem.GetSchemaTableHelper().SchemaTable.Clone();
				items.ForEach(o =>
				{
					schemaTable.Rows.Add(schemaTable.Columns.Cast<DataColumn>().Select(column=>o[column.ColumnName]).ToArray());
				});

				using (SqlBulkCopy bulkCopy = new SqlBulkCopy(firstItem.GetDataStore().DbSetting.ConnectionString))
				{
					bulkCopy.BatchSize = batchSize;
					bulkCopy.BulkCopyTimeout = 30;
					bulkCopy.DestinationTableName = firstItem.GetSchemaTableHelper().TableName;

					bulkCopy.WriteToServer(schemaTable);

					bulkCopy.Close();
				}

			}
		}
	}




}
