﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Ice.Orm.Core.Criteria;
using System.Linq.Expressions;
using System.Collections.ObjectModel;
using Ice.Orm.Core.Entity;
using Ice.Orm.Core.Schema;

namespace Ice.Orm.Core.LINQ
{
	internal class SelectorSqlFormater : SqlFormater
	{
		public SelectorSqlFormater(QueryBase q)
		{
			this.query = q;
		}

		public new Expression Visit(Expression exp)
		{
			return base.Visit(exp);
		}

		protected override Expression VisitLambda(LambdaExpression lambda)
		{
			//// to handle .Select(o=>o.Active)
			Expression body = this.Visit(lambda.Body);

			if (body != lambda.Body)
			{
				return Expression.Lambda(lambda.Type, body, lambda.Parameters);
			}

			if (body.NodeType == ExpressionType.MemberAccess || body.NodeType == ExpressionType.Parameter)
			{
				EntitySetting es;
				
				if((es = AttributedTableHelper.GetEntitySetting(body.Type)) != null)
				{
					// .Select(o=>o) // body.NodeType == ExpressionType.Parameter
					// .Select(o => o.HitKorisnik) // body.NodeType == ExpressionType.MemberAccess, where HitKorisnik is association
					this.WriteLiteral(SqlName.CreateFullSqlName(es.SchemaTableHelper.TableName) + ".*");
				}
				else if (body.Type.FullName.StartsWith("System.Collections.Generic.List") && (es = AttributedTableHelper.GetEntitySetting(body.Type.GetGenericArguments().FirstOrDefault())) != null)
				{
					throw new NotImplementedException("selecting collection is not implemented in translator: " + body);
					// this code is ok, but selecting collection is not implemented in translator when collection is association and is inside new anonymous type
					// handles .Select(o => o.HitPictureList); where HitPictureList is List<HitPicture>
					this.WriteLiteral(SqlName.CreateFullSqlName(es.SchemaTableHelper.TableName) + ".*");
				}
				else
				{
					// .Select(o => o.HitKorisnik.HitKorisnikID) // this is already written
				}
			}
			
			return lambda;
		}

		protected override NewExpression VisitNew(NewExpression nex)
		{
			string[] properties = nex.Type.GetProperties().Select(o => o.Name).ToArray();

			IEnumerable<Expression> args = this.VisitExpressionListNewObject(nex.Arguments, properties);
			if (args != nex.Arguments)
			{
				if (nex.Members != null)
					return Expression.New(nex.Constructor, args, nex.Members);
				else
					return Expression.New(nex.Constructor, args);
			}
			return nex;
		}

		protected ReadOnlyCollection<Expression> VisitExpressionListNewObject(ReadOnlyCollection<Expression> original, string[] properties)
		{
			int count = 0;

			List<Expression> list = null;
			for (int i = 0, n = original.Count; i < n; i++)
			{
				if (count++ > 0)
				{
					this.WriteLiteral(", "); // for select statement and array inline
				}

				Expression p = this.Visit(original[i]);

				string alias = null;
				bool noAlias = false;

				if (p.NodeType == ExpressionType.MemberAccess)
				{
					MemberExpression m = (MemberExpression)p;
					EntityFieldSetting efs;
					EntitySetting es;
					if ((efs = AttributedTableHelper.GetEntityFieldSetting(m.Member)) != null)
					{
						// .Select(o => new { bb = o.HitKorisnik.HitKorisnikID + 10, dd = o.HitKorisnik, cc = o }); // handles o.HitKorisnik.HitKorisnikID
						alias = efs.ColumnName;
					}
					else if ((es = AttributedTableHelper.GetEntitySetting(m.Type)) != null)
					{
						// member has EntitySetting attribute, it is association
						this.WriteLiteral(SqlName.CreateFullSqlName(es.SchemaTableHelper.TableName) + ".*");
						noAlias = true;
					}
					else if (m.Type.FullName.StartsWith("System.Collections.Generic.List") && (es = AttributedTableHelper.GetEntitySetting(m.Type.GetGenericArguments().FirstOrDefault())) != null)
					{
						throw new NotImplementedException("selecting collection is not implemented in translator: " + m);
						// this code is ok, but selecting collection is not implemented in translator when collection is association and is inside new anonymous type
						this.WriteLiteral(SqlName.CreateFullSqlName(es.SchemaTableHelper.TableName) + ".*");
						noAlias = true;
					}
					else
					{
						throw new NotSupportedException("\"" + p + "\" can't be translated into T-SQL, there is not EntityFieldSetting attribute defined");
					}
				}
				else if (p.NodeType == ExpressionType.Parameter)
				{
					noAlias = true;

					EntitySetting es;
					if ((es = AttributedTableHelper.GetEntitySetting(p.Type)) != null)
					{
						// .Select(o => new { bb = o.HitKorisnik.HitKorisnikID + 10, dd = o.HitKorisnik, cc = o }); // handles dd = o.HitKorisnik, cc = o
						this.WriteLiteral(SqlName.CreateFullSqlName(es.SchemaTableHelper.TableName) + ".*");
					}
					else
					{
						throw new NotSupportedException("\"" + p + "\" can't be translated into T-SQL, there is not EntitySetting attribute defined");
					}
				}

				if (!noAlias && alias != properties[i])
				{
					// only if it is not parameter: .Select((o, k, t) => new { Oglas = o, t.Name, k.Username }) translate into HitOglas.* without AS HitOglas
					//// have to write AS column if property name is different than argument name
					//// example: new {Name = o.FirstName}, since o.FirstName is MemberAccess, this would not be translated correctly
					//// expamle: .Select((k, o, a) =>new {Prvi = o.HitAdKindID, k.Ime, a.IsActive, FirstName = k.Name + " " + k.Firma, LastDate = o.PublishedDate})
					this.WriteLiteral(" AS " + SqlName.CreateFullSqlName(properties[i]));
				}

				if (list != null)
				{
					list.Add(p);
				}
				else if (p != original[i])
				{
					list = new List<Expression>(n);
					for (int j = 0; j < i; j++)
					{
						list.Add(original[j]);
					}
					list.Add(p);
				}
			}
			if (list != null)
			{
				return list.AsReadOnly();
			}
			return original;
		}

		protected override MemberAssignment VisitMemberAssignment(MemberAssignment assignment)
		{
			// format select: var nn1 = new Query<HitKorisnik>().Select(k => new HitKorisnik { Active = k.Active && k.HitKorisnikID > 0, Ime = k.Email + " ovo je e-mail" });

			Expression e = this.Visit(assignment.Expression);
			
			this.WriteLiteral(" AS " + SqlName.CreateFullSqlName(assignment.Member.Name));

			if (e != assignment.Expression)
			{
				return Expression.Bind(assignment.Member, e);
			}
			return assignment;
		}

		protected override IEnumerable<MemberBinding> VisitBindingList(ReadOnlyCollection<MemberBinding> original)
		{
			// format select: var nn1 = new Query<HitKorisnik>().Select(k => new HitKorisnik { Active = k.Active && k.HitKorisnikID > 0, Ime = k.Email + " ovo je e-mail" });

			int count = 0;

			List<MemberBinding> list = null;
			for (int i = 0, n = original.Count; i < n; i++)
			{
				if (count++ > 0)
				{
					this.WriteLiteral(","); // for select statement and array inline
				}

				MemberBinding b = this.VisitBinding(original[i]);
				if (list != null)
				{
					list.Add(b);
				}
				else if (b != original[i])
				{
					list = new List<MemberBinding>(n);
					for (int j = 0; j < i; j++)
					{
						list.Add(original[j]);
					}
					list.Add(b);
				}
			}
			if (list != null)
				return list;
			return original;
		}

	}
}
