﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;

namespace Ice.Orm.Core.Schema
{
	public class SchemaColumnCollection
	{
		Dictionary<string, Entity.EntityFieldSetting> mItemsCaseSensitive = new Dictionary<string, Entity.EntityFieldSetting>();
		Dictionary<string, Entity.EntityFieldSetting> mItemsLowerCaseKeys = new Dictionary<string, Entity.EntityFieldSetting>();

		/// <summary>
		/// key case non-sensitive get or set
		/// </summary>
		/// <param name="key"></param>
		/// <returns></returns>
		public Entity.EntityFieldSetting this[string key]
		{
			get
			{
				Entity.EntityFieldSetting sc;
				if (mItemsCaseSensitive.TryGetValue(key, out sc) || mItemsLowerCaseKeys.TryGetValue(key.ToLower(), out sc))
				{
					return sc;
				}
				else
				{
					return null; // when using inner joins, schema column doesnt exist for joined db tables
				}
			}
			set
			{
				mItemsCaseSensitive[key] = value;
				mItemsLowerCaseKeys[key.ToLower()] = value;
			}
		}

		/// <summary>
		/// key case non-sensitive check
		/// </summary>
		/// <param name="key"></param>
		/// <returns></returns>
		public bool Contains(string key)
		{
			return mItemsCaseSensitive.ContainsKey(key) || mItemsLowerCaseKeys.ContainsKey(key.ToLower());
		}

		public int Count
		{
			get { return mItemsCaseSensitive.Count;}
		}

		/// <summary>
		/// Columns names
		/// </summary>
		public ICollection<string> Keys
		{
			get { return mItemsCaseSensitive.Keys;}
		}

		public ICollection<Entity.EntityFieldSetting> Values
		{
			get { return mItemsCaseSensitive.Values; }
		}

		public IEnumerator<Entity.EntityFieldSetting> GetEnumerator()
		{
			return mItemsCaseSensitive.Values.GetEnumerator();
		}


	}
}
