using System;
using System.Data;
using System.Collections.Generic;
using System.Linq;
using System.Data.Odbc;
using System.Data.OleDb;

namespace Ice.Orm.Core.Utility
{
	public class SqlTypesVsNetTypes
	{
		public class SingleType
		{
			public string SQLtype;
			public Type[] DotNETtypes;
			public SqlDbType SqlDbType;
			public string SqlDataReaderSqlType;
			public DbType[] DbTypes;
			public string SqlDataReaderDbType;

			public OdbcType OdbcType { get { return TypeConverters.GetOdbcType(this.SqlDbType); } }
			public OleDbType OleDbType { get { return TypeConverters.GetOleDbType(this.SqlDbType); } }
			public TypeCode TypeCode { get { return TypeConverters.GetTypeCode(this.SqlDbType); } }
		}

		#region Types mapping

		public static readonly List<SingleType> Types = new List<SingleType>()
		.Concat(new SingleType()
		{
			SQLtype = "bigint",
			DotNETtypes = new Type[] { typeof(System.Int64) },
			SqlDbType = SqlDbType.BigInt,
			SqlDataReaderSqlType = "GetSqlInt64",
			DbTypes = new DbType[] { DbType.Int64 },
			SqlDataReaderDbType = "GetInt64"
		})
		.Concat(new SingleType()
		{
			SQLtype = "binary",
			DotNETtypes = new Type[] { typeof(System.Byte[]) },
			SqlDbType = SqlDbType.VarBinary,
			SqlDataReaderSqlType = "GetSqlBinary",
			DbTypes = new DbType[] { DbType.Binary },
			SqlDataReaderDbType = "GetBytes"
		})
		.Concat(new SingleType()
		{
			SQLtype = "bit",
			DotNETtypes = new Type[] { typeof(System.Boolean) },
			SqlDbType = SqlDbType.Bit,
			SqlDataReaderSqlType = "GetSqlBoolean",
			DbTypes = new DbType[] { DbType.Boolean },
			SqlDataReaderDbType = "GetBoolean"
		})
		.Concat(new SingleType()
		{
			SQLtype = "char",
			DotNETtypes = new Type[] { typeof(System.String), typeof(System.Char[]) },
			SqlDbType = SqlDbType.Char,
			SqlDataReaderSqlType = "GetSqlString",
			DbTypes = new DbType[] { DbType.AnsiStringFixedLength, DbType.String },
			SqlDataReaderDbType = "GetString,GetChars"
		})
		.Concat(new SingleType()
		{
			SQLtype = "date",
			DotNETtypes = new Type[] { typeof(System.DateTime) },
			SqlDbType = SqlDbType.Date,
			SqlDataReaderSqlType = "GetSqlDateTime",
			DbTypes = new DbType[] { DbType.Date },
			SqlDataReaderDbType = "GetDateTime"
		})
		.Concat(new SingleType()
		{
			SQLtype = "datetime",
			DotNETtypes = new Type[] { typeof(System.DateTime) },
			SqlDbType = SqlDbType.DateTime,
			SqlDataReaderSqlType = "GetSqlDateTime",
			DbTypes = new DbType[] { DbType.DateTime },
			SqlDataReaderDbType = "GetDateTime"
		})
		.Concat(new SingleType()
		{
			SQLtype = "datetime2",
			DotNETtypes = new Type[] { typeof(System.DateTime) },
			SqlDbType = SqlDbType.DateTime2,
			SqlDataReaderSqlType = "None",
			DbTypes = new DbType[] { DbType.DateTime2 },
			SqlDataReaderDbType = "GetDateTime"
		})
		.Concat(new SingleType()
		{
			SQLtype = "datetimeoffset",
			DotNETtypes = new Type[] { typeof(System.DateTimeOffset) },
			SqlDbType = SqlDbType.DateTimeOffset,
			SqlDataReaderSqlType = "none",
			DbTypes = new DbType[] { DbType.DateTimeOffset },
			SqlDataReaderDbType = "GetDateTimeOffset"
		})
		.Concat(new SingleType()
		{
			SQLtype = "decimal",
			DotNETtypes = new Type[] { typeof(System.Decimal) },
			SqlDbType = SqlDbType.Decimal,
			SqlDataReaderSqlType = "GetSqlDecimal",
			DbTypes = new DbType[] { DbType.Decimal },
			SqlDataReaderDbType = "GetDecimal"
		})
		.Concat(new SingleType()
		{
			SQLtype = "FILESTREAM",
			DotNETtypes = new Type[] { typeof(System.Byte[]) },
			SqlDbType = SqlDbType.VarBinary,
			SqlDataReaderSqlType = "GetSqlBytes",
			DbTypes = new DbType[] { DbType.Binary },
			SqlDataReaderDbType = "GetBytes"
		})
		.Concat(new SingleType()
		{
			SQLtype = "float",
			DotNETtypes = new Type[] { typeof(System.Double) },
			SqlDbType = SqlDbType.Float,
			SqlDataReaderSqlType = "GetSqlDouble",
			DbTypes = new DbType[] { DbType.Double },
			SqlDataReaderDbType = "GetDouble"
		})
		.Concat(new SingleType()
		{
			SQLtype = "image",
			DotNETtypes = new Type[] { typeof(System.Byte[]) },
			SqlDbType = SqlDbType.Binary,
			SqlDataReaderSqlType = "GetSqlBinary",
			DbTypes = new DbType[] { DbType.Binary },
			SqlDataReaderDbType = "GetBytes"
		})
		.Concat(new SingleType()
		{
			SQLtype = "int",
			DotNETtypes = new Type[] { typeof(System.Int32) },
			SqlDbType = SqlDbType.Int,
			SqlDataReaderSqlType = "GetSqlInt32",
			DbTypes = new DbType[] { DbType.Int32 },
			SqlDataReaderDbType = "GetInt32"
		})
		.Concat(new SingleType()
		{
			SQLtype = "money",
			DotNETtypes = new Type[] { typeof(System.Decimal) },
			SqlDbType = SqlDbType.Money,
			SqlDataReaderSqlType = "GetSqlMoney",
			DbTypes = new DbType[] { DbType.Decimal, DbType.Currency },
			SqlDataReaderDbType = "GetDecimal"
		})
		.Concat(new SingleType()
		{
			SQLtype = "nchar",
			DotNETtypes = new Type[] { typeof(System.String), typeof(System.Char[]) },
			SqlDbType = SqlDbType.NChar,
			SqlDataReaderSqlType = "GetSqlString",
			DbTypes = new DbType[] { DbType.StringFixedLength },
			SqlDataReaderDbType = "GetString,GetChars"
		})
		.Concat(new SingleType()
		{
			SQLtype = "ntext",
			DotNETtypes = new Type[] { typeof(System.String), typeof(System.Char[]) },
			SqlDbType = SqlDbType.NText,
			SqlDataReaderSqlType = "GetSqlString",
			DbTypes = new DbType[] { DbType.String },
			SqlDataReaderDbType = "GetString,GetChars"
		})
		.Concat(new SingleType()
		{
			SQLtype = "numeric",
			DotNETtypes = new Type[] { typeof(System.Decimal) },
			SqlDbType = SqlDbType.Decimal,
			SqlDataReaderSqlType = "GetSqlDecimal",
			DbTypes = new DbType[] { DbType.Decimal },
			SqlDataReaderDbType = "GetDecimal"
		})
		.Concat(new SingleType()
		{
			SQLtype = "nvarchar",
			DotNETtypes = new Type[] { typeof(System.String), typeof(System.Char[]) },
			SqlDbType = SqlDbType.NVarChar,
			SqlDataReaderSqlType = "GetSqlString",
			DbTypes = new DbType[] { DbType.String },
			SqlDataReaderDbType = "GetString,GetChars"
		})
		.Concat(new SingleType()
		{
			SQLtype = "real",
			DotNETtypes = new Type[] { typeof(System.Single) },
			SqlDbType = SqlDbType.Real,
			SqlDataReaderSqlType = "GetSqlSingle",
			DbTypes = new DbType[] { DbType.Single },
			SqlDataReaderDbType = "GetFloat"
		})
		.Concat(new SingleType()
		{
			SQLtype = "rowversion",
			DotNETtypes = new Type[] { typeof(System.Byte[]) },
			SqlDbType = SqlDbType.Timestamp,
			SqlDataReaderSqlType = "GetSqlBinary",
			DbTypes = new DbType[] { DbType.Binary },
			SqlDataReaderDbType = "GetBytes"
		})
		.Concat(new SingleType()
		{
			SQLtype = "smalldatetime",
			DotNETtypes = new Type[] { typeof(System.DateTime) },
			SqlDbType = SqlDbType.DateTime,
			SqlDataReaderSqlType = "GetSqlDateTime",
			DbTypes = new DbType[] { DbType.DateTime },
			SqlDataReaderDbType = "GetDateTime"
		})
		.Concat(new SingleType()
		{
			SQLtype = "smallint",
			DotNETtypes = new Type[] { typeof(System.Int16) },
			SqlDbType = SqlDbType.SmallInt,
			SqlDataReaderSqlType = "GetSqlInt16",
			DbTypes = new DbType[] { DbType.Int16 },
			SqlDataReaderDbType = "GetInt16"
		})
		.Concat(new SingleType()
		{
			SQLtype = "smallmoney",
			DotNETtypes = new Type[] { typeof(System.Decimal) },
			SqlDbType = SqlDbType.SmallMoney,
			SqlDataReaderSqlType = "GetSqlMoney",
			DbTypes = new DbType[] { DbType.Decimal },
			SqlDataReaderDbType = "GetDecimal"
		})
		.Concat(new SingleType()
		{
			SQLtype = "sql_variant",
			DotNETtypes = new Type[] { typeof(System.Object) },
			SqlDbType = SqlDbType.Variant,
			SqlDataReaderSqlType = "GetSqlValue",
			DbTypes = new DbType[] { DbType.Object },
			SqlDataReaderDbType = "GetValue"
		})
		.Concat(new SingleType()
		{
			SQLtype = "text",
			DotNETtypes = new Type[] { typeof(System.String), typeof(System.Char[]) },
			SqlDbType = SqlDbType.Text,
			SqlDataReaderSqlType = "GetSqlString",
			DbTypes = new DbType[] { DbType.String },
			SqlDataReaderDbType = "GetString,GetChars"
		})
		.Concat(new SingleType()
		{
			SQLtype = "time",
			DotNETtypes = new Type[] { typeof(System.TimeSpan) },
			SqlDbType = SqlDbType.Time,
			SqlDataReaderSqlType = "none",
			DbTypes = new DbType[] { DbType.Time },
			SqlDataReaderDbType = "GetDateTime"
		})
		.Concat(new SingleType()
		{
			SQLtype = "timestamp",
			DotNETtypes = new Type[] { typeof(System.Byte[]) },
			SqlDbType = SqlDbType.Timestamp,
			SqlDataReaderSqlType = "GetSqlBinary",
			DbTypes = new DbType[] { DbType.Binary },
			SqlDataReaderDbType = "GetBytes"
		})
		.Concat(new SingleType()
		{
			SQLtype = "tinyint",
			DotNETtypes = new Type[] { typeof(System.Byte) },
			SqlDbType = SqlDbType.TinyInt,
			SqlDataReaderSqlType = "GetSqlByte",
			DbTypes = new DbType[] { DbType.Byte },
			SqlDataReaderDbType = "GetByte"
		})
		.Concat(new SingleType()
		{
			SQLtype = "uniqueidentifier",
			DotNETtypes = new Type[] { typeof(System.Guid) },
			SqlDbType = SqlDbType.UniqueIdentifier,
			SqlDataReaderSqlType = "GetSqlGuid",
			DbTypes = new DbType[] { DbType.Guid },
			SqlDataReaderDbType = "GetGuid"
		})
		.Concat(new SingleType()
		{
			SQLtype = "varbinary",
			DotNETtypes = new Type[] { typeof(System.Byte[]) },
			SqlDbType = SqlDbType.VarBinary,
			SqlDataReaderSqlType = "GetSqlBinary",
			DbTypes = new DbType[] { DbType.Binary },
			SqlDataReaderDbType = "GetBytes"
		})
		.Concat(new SingleType()
		{
			SQLtype = "varchar",
			DotNETtypes = new Type[] { typeof(System.String), typeof(System.Char[]) },
			SqlDbType = SqlDbType.VarChar,
			SqlDataReaderSqlType = "GetSqlString",
			DbTypes = new DbType[] { DbType.AnsiString, DbType.String },
			SqlDataReaderDbType = "GetString,GetChars"
		})
		.Concat(new SingleType()
		{
			SQLtype = "xml",
			DotNETtypes = new Type[] {typeof(System.String) },
			SqlDbType = SqlDbType.Xml,
			SqlDataReaderSqlType = "GetSqlXml",
			DbTypes = new DbType[] { DbType.Xml },
			SqlDataReaderDbType = "none"
		})
		.ToList();

		#endregion

		public static string FindSQLType(Type dotNetType)
		{
			dotNetType = Nullable.GetUnderlyingType(dotNetType) ?? dotNetType;
			return Types.Where(o => o.DotNETtypes.Contains(dotNetType)).Select(o => o.SQLtype).FirstOrDefault();
		}

		public static string FindSQLType(DbType dbType)
		{
			return Types.Where(o => o.DbTypes.Contains(dbType)).Select(o => o.SQLtype).FirstOrDefault();
		}

	}
}