﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Ice.Converter
{
	public class IceBase64
	{
		/// <summary>
		/// Summary description for Base64Encoder.
		/// </summary>
		class Base64Encoder
		{
			static Dictionary<byte, char> _indexCharDic = IceBaseConvert.JavaScriptRFCTable64.Select((o, n) => new
			{
				Index = (byte)n,
				Chr = (char)o
			}).ToDictionary(k => k.Index, k => k.Chr);

			static char alignmentChar = IceBaseConvert.JavaScriptRFCTable64.Last(); // last char is alignment char

			public static string GetEncoded(byte[] input)
			{
				byte[] source = input;
				int blockCount = 0;
				int paddingCount = 0;

				int length = input.Length;
				if ((length % 3) == 0)
				{
					paddingCount = 0;
					blockCount = length / 3;
				}
				else
				{
					paddingCount = 3 - (length % 3);//need to add padding
					blockCount = (length + paddingCount) / 3;
				}
				int length2 = length + paddingCount;//or blockCount *3


				byte[] source2;
				source2 = new byte[length2];
				//copy data over insert padding
				for (int x = 0; x < length2; x++)
				{
					if (x < length)
					{
						source2[x] = source[x];
					}
					else
					{
						source2[x] = 0;
					}
				}

				byte b1, b2, b3;
				byte temp, temp1, temp2, temp3, temp4;
				byte[] buffer = new byte[blockCount * 4];
				char[] result = new char[blockCount * 4];
				for (int x = 0; x < blockCount; x++)
				{
					b1 = source2[x * 3];
					b2 = source2[x * 3 + 1];
					b3 = source2[x * 3 + 2];

					temp1 = (byte)((b1 & 252) >> 2);//first

					temp = (byte)((b1 & 3) << 4);
					temp2 = (byte)((b2 & 240) >> 4);
					temp2 += temp; //second

					temp = (byte)((b2 & 15) << 2);
					temp3 = (byte)((b3 & 192) >> 6);
					temp3 += temp; //third

					temp4 = (byte)(b3 & 63); //fourth

					buffer[x * 4] = temp1;
					buffer[x * 4 + 1] = temp2;
					buffer[x * 4 + 2] = temp3;
					buffer[x * 4 + 3] = temp4;

				}

				for (int x = 0; x < blockCount * 4; x++)
				{
					// 6 bits to char
					result[x] = _indexCharDic[buffer[x]];
				}


				//covert last "A"s to "=", based on paddingCount
				switch (paddingCount)
				{
					case 0: 
						break;
					case 1: 
						result[blockCount * 4 - 1] = alignmentChar; 
						break;
					case 2: 
						result[blockCount * 4 - 1] = alignmentChar;
						result[blockCount * 4 - 2] = alignmentChar;
						break;
					default: 
						break;
				}
				StringBuilder sb = new StringBuilder();
				foreach (char ch in result)
				{
					sb.Append(ch);
				}
				return sb.ToString();
			}
		}

		/// <summary>
		/// Summary description for Base64Decoder.
		/// </summary>
		class Base64Decoder
		{
			static Dictionary<byte, byte> _charIndexDic = IceBaseConvert.JavaScriptRFCTable64.Select((o, n) => new
			{
				Index = (byte)n,
				Chr = (byte)o
			}).ToDictionary(k => k.Chr, k => k.Index);

			static char alignmentChar = IceBaseConvert.JavaScriptRFCTable64.Last(); // last char is alignment char

			public static byte[] GetDecoded(byte[] input)
			{
				int length2, length3;
				int blockCount;
				int paddingCount;

				int temp = 0;
				byte[] source = input;
				int length = input.Length;

				//find how many padding are there
				for (int x = 0; x < 2; x++)
				{
					if (input[length - x - 1] == alignmentChar)
						temp++;
				}
				paddingCount = temp;
				//calculate the blockCount;
				//assuming all whitespace and carriage returns/newline were removed.
				blockCount = length / 4;
				length2 = blockCount * 3;

				byte[] buffer = new byte[length];//first conversion result
				byte[] buffer2 = new byte[length2];//decoded array with padding
				byte chr;

				for (int x = 0; x < length; x++)
				{
					// char to 6 bits
					chr = source[x];
					
					if ((char)chr == alignmentChar)
					{
						buffer[x] = 0;
					}
					else
					{
						buffer[x] = _charIndexDic[chr];// get index in table64 where chr is
					}

				}

				byte b, b1, b2, b3;
				byte temp1, temp2, temp3, temp4;

				for (int x = 0; x < blockCount; x++)
				{
					temp1 = buffer[x * 4];
					temp2 = buffer[x * 4 + 1];
					temp3 = buffer[x * 4 + 2];
					temp4 = buffer[x * 4 + 3];

					b = (byte)(temp1 << 2);
					b1 = (byte)((temp2 & 48) >> 4);
					b1 += b;

					b = (byte)((temp2 & 15) << 4);
					b2 = (byte)((temp3 & 60) >> 2);
					b2 += b;

					b = (byte)((temp3 & 3) << 6);
					b3 = temp4;
					b3 += b;

					buffer2[x * 3] = b1;
					buffer2[x * 3 + 1] = b2;
					buffer2[x * 3 + 2] = b3;
				}
				//remove paddings
				length3 = length2 - paddingCount;
				byte[] result = new byte[length3];

				for (int x = 0; x < length3; x++)
				{
					result[x] = buffer2[x];
				}

				return result;
			}
		}

		public static string ToBase64String(byte[] arr)
		{
			if(arr == null || arr.Length == 0)
			{
				return "";
			}

			return Base64Encoder.GetEncoded(arr);
		}

		public static string ToBase64String(string str)
		{
			return ToBase64String(Encoding.UTF8.GetBytes(str));
		}


		public static byte[] DecodeBase64StringToArray(string base64string)
		{
			if(string.IsNullOrEmpty(base64string))
			{
				return new byte[0];
			}

			return Base64Decoder.GetDecoded(Encoding.UTF8.GetBytes(base64string));
		}

		public static string DecodeBase64StringToString(string base64string)
		{
			byte[] data = DecodeBase64StringToArray(base64string);
			return data == null || data.Length == 0 ? null : Encoding.UTF8.GetString(data);
		}

	}
}