﻿#if !SILVERLIGHT

using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.IO;
using System.IO.Compression;

namespace Ice.IO.Compression
{
	public abstract class CompressionHelper<T> where T:Stream
	{
		public static byte[] Compress(byte[] data)
		{
			MemoryStream ms = new MemoryStream();
			T comprStream = (T)Activator.CreateInstance(typeof(T), ms, CompressionMode.Compress, true);
			comprStream.Write(data, 0, data.Length);
			comprStream.Close();
			return ms.ToArray();
		}

		public static byte[] Decompress(byte[] data)
		{
			MemoryStream ms = new MemoryStream(data);
			ms.Position = 0;
			T comprStream = (T)Activator.CreateInstance(typeof(T), ms, CompressionMode.Decompress);
			MemoryStream outputStream = new MemoryStream();
			byte[] buffer;
			int read;
			do
			{
				buffer = new byte[1024];
				read = comprStream.Read(buffer, 0, buffer.Length);
				outputStream.Write(buffer, 0, read);
			}
			while (read > 0);

			return outputStream.ToArray();
		}

	}

	public class DeflateHelper : CompressionHelper<DeflateStream>
	{

	}

	public class GZipHelper : CompressionHelper<GZipStream>
	{

	}

}

#endif
