﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

namespace Ice.JavaScript.Packer
{
	public enum CompressionLevel
	{
		/// <summary>
		/// 0. DEFAULT: Pack for JS, Minify for CSS
		/// </summary>
		Auto = 0,

		/// <summary>
		/// 1. No compression
		/// </summary>
		None = 1,

		/// <summary>
		/// 2. Minify script (JS or CSS)
		/// </summary>
		Minify = 2,

		/// <summary>
		/// 3. Shorten variables names using AjaxMin and minify, if CSS, minify only
		/// </summary>
		Shrink = 3,

		/// <summary>
		/// 4. Shrink and pack if JS, minify if CSS
		/// </summary>
		Pack = 4,

		/////// <summary>
		/////// JSPack and scramble keywords if JS, minify if CSS
		/////// </summary>
		////Scramble = 5, // doesn't work ok!!!

	}

	public class IcePacker
	{
		private static string RemoveCssComments(string jscode)
		{
			//remove <%-- --%> comments
			// jscode = Regex.Replace(jscode, @"(<%--[^\255]*?--%>)", " ");

			//remove <!-- --> comments
			// jscode = Regex.Replace(jscode, @"(<!--[^\255]*?-->)", " ");

			//remove /* */ comments
			jscode = Regex.Replace(jscode, @"\/\*[^\255]*?\*\/", " ");

			// remove // comments
			// jscode = Regex.Replace(script, @"\/\/[^\r\n]*[\r\n]+", " ");

			return jscode;
		}

		public static string Pack(string jscode, CompressionLevel level, bool isCSS)
		{
			if (level == CompressionLevel.Auto)
			{
				level = isCSS ? CompressionLevel.Minify : CompressionLevel.Pack;
			}

			if (level > CompressionLevel.None)
			{
				if (isCSS) // CSS minimize
				{
					//remove /* */ comments
					jscode = Regex.Replace(jscode, @"\/\*[^\255]*?\*\/", " ");

					// remove any space including tab, new line, space...
					jscode = Regex.Replace(jscode, "[\\s]+", " ");

					// remove multiple spaces
					jscode = Regex.Replace(jscode, "[ ]+", " ");

					// remove space near to symbols // ne, ovo rasturi sav css, mora imati prazno izmedju naziva klasa, prazno u: solid 1px red, itd.
					//jscode = Regex.Replace(jscode, @"[ ]*([\;\{\}\[\]\(\)\:\,\=\-\+\*\/\<\>]+)[ ]*", "$1");

					jscode = Regex.Replace(jscode, @"[ ]*([\;\:\{\}\,]+)[ ]*", "$1"); // testirano, ovo je ok

					jscode = jscode.TrimStart();

					//jscode = jscode.Replace("}", "}\r\n");
				}
				else // JS minimize & pack
				{
					ECMAScriptPacker packer = new ECMAScriptPacker();

					////if (level == CompressionLevel.Scramble)
					////{
					////    packer.ScrambleKeywords = JSPacker.Core.ECMAScriptPacker.ScrambleEnum.Key2;
					////    packer.Encoding = JSPacker.Core.ECMAScriptPacker.PackerEncoding.Normal;
					////    jscode = packer.Pack(jscode);
					////}
					////else 
					if (level == CompressionLevel.Pack)
					{
						packer.Encoding = ECMAScriptPacker.PackerEncoding.Normal;
						jscode = packer.Pack(jscode);
					}
					else if (level == CompressionLevel.Minify)
					{
						packer.Encoding = ECMAScriptPacker.PackerEncoding.Minify;
						jscode = packer.Pack(jscode);
					}
					else // level == CompressionLevel.None
					{
						// don't pack
						// don't minimize
					}
				}
			}

			return jscode;
		}
	}
}
