﻿#if !SILVERLIGHT

using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;

namespace Ice.Serialization.Json.Core.Converters
{
	using Serializer;
	using System.Text;

	internal class DataTableConverter : JavaScriptConverter
	{
		private bool CanSerialize(Type type)
		{
			return type != null && type == typeof(DataTable);
		}

		private bool CanConvert(object obj, Type toType)
		{
			if (toType != null && toType == typeof(DataTable))
			{
				return true;
			}

			if (obj is IDictionary<string, object>)
			{
				IDictionary<string, object> dic = (IDictionary<string, object>)obj;
				object key;
				if (dic.TryGetValue("__icetype", out key) && object.Equals(key, typeof(DataTable).FullName))
				{
					// must have __icetype key
					// toType is property type, so if deserializing to object, property type is unknown (null)
					return true;
				}
			}

			return false;
		}

		private object Convert(object obj, JavaScriptSerializer serializer)
		{
			IDictionary<string, object> dictionary = (IDictionary<string, object>)obj;

			List<string> columns = serializer.ConvertToType<List<string>>(dictionary["Columns"]);
			List<object[]> rows = serializer.ConvertToType<List<object[]>>(dictionary["Rows"]);

			DataTable dataTable = new DataTable();

			columns.ForEach(columnName => dataTable.Columns.Add(columnName));

			rows.ForEach(rowItemsArray => dataTable.Rows.Add(rowItemsArray));

			dataTable.TableName = dictionary.TryGetValueOrDefault("TableName") as string;

			int totalCount;
			if (int.TryParse(dictionary.TryGetValueOrDefault("TotalCount") + "", out totalCount))
			{
				dataTable.ExtendedProperties["TotalCount"] = totalCount;
			}

			return dataTable;
		}

		public override bool TryConvert(object obj, Type toType, JavaScriptSerializer serializer, out object result)
		{
			if (this.CanConvert(obj, toType))
			{
				result = Convert(obj, serializer);
				return true;
			}
			else
			{
				result = null;
				return false;
			}
		}

		private void Serialize(object obj, StringBuilder sb, JavaScriptSerializer serializer, JavaScriptSerializer.SerializationFormat serializationFormat)
		{
			DataTable dt = (DataTable)obj;

			Dictionary<string, object> dictionary = new Dictionary<string, object>();			
			dictionary.Add("TableName", dt.TableName);
			dictionary.Add("Columns", dt.Columns.Cast<DataColumn>().Select(o=>o.ColumnName).ToArray());
			dictionary.Add("Rows", dt.Rows.Cast<DataRow>().Select(o=>o.ItemArray).ToArray());
			int? totalCount = null;
			if (dt.ExtendedProperties.Contains("TotalCount"))
			{
				// if using data paging, TotalCount is total count of items, while dt.Rows.Count() is currently loaded number of items
				totalCount = System.Convert.ToInt32(dt.ExtendedProperties["TotalCount"]);
			}

			dictionary.Add("TotalCount", totalCount);
			
			dictionary.Add("__icetype", typeof(DataTable).FullName);

			serializer.Serialize(dictionary, sb, serializationFormat);

		}

		public override bool TrySerialize(object obj, StringBuilder sb, JavaScriptSerializer serializer, JavaScriptSerializer.SerializationFormat serializationFormat)
		{
			if (obj != null && this.CanSerialize(obj.GetType()))
			{
				this.Serialize(obj, sb, serializer, serializationFormat);
				return true;
			}
			else
			{
				return false;
			}
		}

	}

}
#endif
