﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using Ice.Threading.Abstract;

namespace Ice.Threading
{
	public class IceThreadPool : IceThreadPoolBase<IceThreadPool, IceThreadPool.PoolItem>
	{
		public class PoolItem : PoolItemBase
		{
		}

		private Action mWorkerMethod;

		public IceThreadPool(int threadsCount, Action workerMethod)
			: base(threadsCount)
		{
			this.mWorkerMethod = workerMethod;
		}

		protected override void InvokeWorkerMethod(PoolItem ss)
		{
			this.mWorkerMethod.Invoke();
		}
	}

	public class IceThreadPool<TResult> :IceThreadPoolBase<IceThreadPool<TResult>, IceThreadPool<TResult>.PoolItemWithResult>
	{
		public class PoolItemWithResult : PoolItemBase
		{
			/// <summary>
			/// Value returned from callback method.
			/// </summary>
			public TResult Result { get; set; }
		}

		private Func<TResult> mWorkerMethod;

		public IceThreadPool(int threadsCount, Func<TResult> workerMethod)
			: base(threadsCount)
		{
			this.mWorkerMethod = workerMethod;
		}

		protected override void InvokeWorkerMethod(PoolItemWithResult ss)
		{
			ss.Result = this.mWorkerMethod.Invoke();
		}

		/// <summary>
		/// Results returned from worker method.
		/// </summary>
		/// <returns></returns>
		public IEnumerable<TResult> Results()
		{
			return this.Items().Select(o=>o.Result);
		}
	}

}
