﻿using System;
using System.Threading;

namespace Ice.Threading
{
	/// <summary>
	/// Timer compatible with .NET and Silverlight
	/// </summary>
	public class IceTimer
	{
		Thread _thr;
		private double _interval = 1000;

		public double Interval
		{
			get { return _interval; }
			set
			{
				_interval = value;
				if (_thr != null)
				{
					this.Stop();
					this.Start();
				}
			}
		}

		public bool AutoReset { get; set; }

		public bool Enabled
		{
			get { return _thr != null; }
			set { if (value) { this.Start(); } else { this.Stop(); } }
		}

		public event EventHandler Elapsed;

		private void StartImpl()
		{
			_thr = new Thread(() =>
			{
				Thread.Sleep((int)this.Interval);

				_thr = null;

				if (this.AutoReset)
				{
					this.StartImpl();
				}

				if (this.Elapsed != null)
				{
					this.Elapsed.Invoke(this, new EventArgs());
				}

			});
			_thr.IsBackground = true;
			_thr.Start();
		}

		public void Start()
		{
			this.Stop(); // abort previous timer
			this.StartImpl();
		}

		public void Stop()
		{
			if (_thr != null)
			{
				_thr.Abort();
				_thr = null;
			}
		}
	}
}
