﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Ice.Utility
{
	public struct BoxValue<T>
	{
		private readonly T _value;
		private readonly bool _hasValue;

		public bool HasValue { get { return _hasValue; } }

		public T Value { get { return _value; } }

		public T GetValueOrDefault(T defaultValue)
		{
			return this.HasValue ? this.Value : defaultValue;
		}

		public T GetValueOrDefault()
		{
			return this.HasValue ? this.Value : default(T);
		}

		public BoxValue(T value)
		{
			_value = value;
			_hasValue = true;
		}

		public static implicit operator BoxValue<T>(T value)
		{
			return new BoxValue<T>(value);
		}

		public static implicit operator T(BoxValue<T> box)
		{
			return box.Value;
		}

		public override int GetHashCode()
		{
			return this.Value == null ? 0 : this.Value.GetHashCode();
		}

		public override bool Equals(object obj)
		{
			if ((obj == null) || !(obj is BoxValue<T>))
			{
				return false;
			}
			BoxValue<T> box = (BoxValue<T>)obj;
			return this.HasValue && box.HasValue && object.Equals(this.Value, box.Value);
		}

		public static bool operator ==(BoxValue<T> left, BoxValue<T> right)
		{
			return left.HasValue && right.HasValue && object.Equals(left.Value, right.Value);
		}

		public static bool operator !=(BoxValue<T> left, BoxValue<T> right)
		{
			return (!(left == right));
		}


	}

}
