﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Web;
using System.Web.Compilation;
using System.Web.UI;

namespace Ice.Web
{
	public class CompiledPageHelper
	{
		static MethodInfo mSetIntrinsics;
		static MethodInfo mFrameworkInitialize;
		static MethodInfo mPerformPreInit;
		static MethodInfo mInitRecursive;

		static CompiledPageHelper()
		{
			var methods1 = typeof(Page).GetMethods(System.Reflection.BindingFlags.NonPublic | System.Reflection.BindingFlags.Public | System.Reflection.BindingFlags.Instance);
			mSetIntrinsics = methods1.Where(o => o.Name == "SetIntrinsics" && o.GetParameters().Count() == 2).FirstOrDefault();
			mFrameworkInitialize = methods1.Where(o => o.Name == "FrameworkInitialize" && o.GetParameters().Count() == 0).FirstOrDefault();
			mPerformPreInit = methods1.Where(o => o.Name == "PerformPreInit" && o.GetParameters().Count() == 0).FirstOrDefault();
			mInitRecursive = methods1.Where(o => o.Name == "InitRecursive" && o.GetParameters().Count() == 1).FirstOrDefault();
		}

		private static void InitializeFramework(Page page)
		{
			HttpContext context = HttpContext.Current;
			mSetIntrinsics.Invoke(page, new object[] { context, false });
			mFrameworkInitialize.Invoke(page, null);
			mPerformPreInit.Invoke(page, null); // load default controls values from aspx file, any later step after this one is not required
			////mInitRecursive.Invoke(page, new object[] { null });
		}

		public static T LoadPage<T>(string virtualPath) where T : Page
		{
			T page = (T)BuildManager.CreateInstanceFromVirtualPath(virtualPath, typeof(T));
			InitializeFramework(page);
			return page;
		}

		public static T LoadPage<T>(Type pageType) where T : Page
		{
			T page = (T)Activator.CreateInstance(pageType);
			InitializeFramework(page);
			return page;
		}

		public static T LoadControl<T>(string virtualPath) where T : UserControl
		{
			Page p = new Page();
			InitializeFramework(p); // to have this.Request in page
			return (T)p.LoadControl(virtualPath); // load default controls values from ascx file
		}

		public static T LoadControl<T>(Type controlType) where T : UserControl
		{
			Page p = new Page();
			InitializeFramework(p); // to have this.Request in page
			return (T)p.LoadControl(controlType, null);
		}
	}
}
