﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web;

namespace Ice.Web
{
	public class CookieHelper
	{

		public static bool HasCookie(string cookieName)
		{
			cookieName = HttpUtility.UrlEncode(cookieName);
			return HttpContext.Current.Request.Cookies.Get(cookieName) != null;
		}

		public static void DeleteCookie(string cookieName)
		{
			cookieName = HttpUtility.UrlEncode(cookieName);
			HttpContext.Current.Request.Cookies.Remove(cookieName);
			HttpContext.Current.Response.Cookies.Remove(cookieName);
			HttpCookie c = new HttpCookie(cookieName) { Expires = DateTime.Now.AddYears(-10) };
			HttpContext.Current.Response.Cookies.Set(c);
		}

		public static void SaveCookie(string cookieName, string value, DateTime? expiration = null)
		{
			cookieName = HttpUtility.UrlEncode(cookieName);
			HttpContext.Current.Request.Cookies.Remove(cookieName);
			HttpContext.Current.Response.Cookies.Remove(cookieName);
			HttpCookie c = new HttpCookie(cookieName, HttpUtility.UrlEncode(value + ""));
			if (expiration != null)
			{
				c.Expires = c.Expires = expiration.GetValueOrDefault();
			}
			HttpContext.Current.Response.Cookies.Set(c);
			HttpContext.Current.Request.Cookies.Set(c);
		}

		/// <summary>
		/// Save session cookie
		/// </summary>
		/// <param name="cookieName"></param>
		/// <param name="value"></param>
		public static void SaveSessionCookie(string cookieName, string value)
		{
			SaveCookie(cookieName, value, null);
		}

		public static string GetCookieValue(string cookieName)
		{
			cookieName = HttpUtility.UrlEncode(cookieName);
			HttpCookie c = HttpContext.Current.Request.Cookies.Get(cookieName);
			if (c != null)
			{
				return HttpUtility.UrlDecode(c.Value);
			}
			return null;
		}

		public static T GetCookieValueToType<T>(string cookieName)
		{
			string val = GetCookieValue(cookieName);
			if (val != null)
			{
				T tmp;
				if (Ice.Utility.IceTypeConverter.TryChangeType<T>(val, out tmp))
				{
					return tmp;
				}
			}
			return default(T);
		}

		public class JSON
		{
			public static void SerializeToCookie(string cookieName, object value, DateTime? expiration = null)
			{
				SaveCookie(cookieName, Ice.Serialization.Json.IceJson.Serialize(value), expiration);
			}

			public static T DeserializeFromCookie<T>(string cookieName)
			{
				string json = GetCookieValue(cookieName);
				if(string.IsNullOrEmpty(json))
				{
					return default(T);
				}

				// let's throw exception if value can't be deserialized, exception has to be handled in user's code:
				return Ice.Serialization.Json.IceJson.Deserialize<T>(json);
			}
		}
	}
}
