﻿namespace Ice.Web.UI
{
    using System;
    using System.Collections;
    using System.ComponentModel;
    using System.Globalization;
    using System.Reflection;
	using System.Text.RegularExpressions;
	using System.Linq;

	public sealed class DataBinder
    {
        private static readonly char[] expressionPartSeparator = new char[] { '.' };
        private static readonly char[] indexExprEndChars = new char[] { ']', ')' };
        private static readonly char[] indexExprStartChars = new char[] { '[', '(' };

		/// <summary>
		/// 
		/// </summary>
		/// <param name="container">object with values, IEnumerable, or IDictionary</param>
		/// <param name="expressionParts">case sensitive, eg. Customer.Addresses.3.Street or Customer.Addresses.[3].Street</param>
		/// <returns></returns>
		public static object Eval(object container, string expression)
		{
			if (expression == null)
			{
				throw new ArgumentNullException("expression");
			}
			expression = expression.Trim();
			if (expression.Length == 0)
			{
				throw new ArgumentNullException("expression");
			}
			if (container == null)
			{
				return null;
			}

			string[] expressionParts = expression.Split(expressionPartSeparator).Select(o => o.Trim().Trim('(', '[', ']', ')')).Where(o => !string.IsNullOrEmpty(o)).ToArray();
			int index;
            object propertyValue = container;
            for (int i = 0; (i < expressionParts.Length) && (propertyValue != null); i++)
            {
                string propName = expressionParts[i];

				if (int.TryParse(propName, out index))
				{
					int cnt = -1;
					foreach (var o in ((IEnumerable)propertyValue))
					{
						cnt++;
						if (cnt == index)
						{
							propertyValue = o;
						}
					}
					throw new ArgumentNullException("DataBinder_Index_Not_Found: " + propertyValue.GetType().FullName, index + "");
				}
				else if(container is IDictionary)
				{
					propertyValue = ((IDictionary)container)[propName];
				}
				else
				{
					PropertyInfo pi = propertyValue.GetType().GetProperty(propName, BindingFlags.Instance | BindingFlags.Public | BindingFlags.NonPublic);
					if (pi == null)
					{
						throw new ArgumentNullException("DataBinder_Property_Not_Found: " + propertyValue.GetType().FullName, propName);
					}
					propertyValue = pi.GetValue(propertyValue, null);
				}
            }
            return propertyValue;
        }

    }
}

