﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

namespace Ice.Xml.Parser
{
	[DebuggerDisplay("{Text}")]
	public class Segment
	{
		static List<SegmentKind> _enumValues = Enum.GetValues(typeof(SegmentKind)).Cast<SegmentKind>().ToList();
		static List<string> _enumNames = _enumValues.Select(o => o.ToString()).ToList();
		static Dictionary<SegmentKind, string> _dicNames = _enumValues.ToDictionary(o => o, o => o.ToString());

		Match _match;

		public Segment(Match match)
		{
			_match = match;
		}

		public Segment(SegmentKind kind)
		{
			_kind = kind;
		}

		public int Level { get; set; }

		public int StartPosition { get { return _match != null ? _match.Index : -1; } }

		public Match Match { get { return _match; } }

		private SegmentKind GetSuccessfulKind()
		{
			return _enumValues
				.Select(o => new
				{
					Kind = o,
					Group = _match.Groups[_dicNames[o]]
				}).Where(o => o.Group != null && o.Group.Success)
				.Select(o=>o.Kind).FirstOrDefault();
		}

		public string GetGroupValue(SegmentKind groupName)
		{
			Group g = _match.Groups[_dicNames[groupName]];
			return g == null ? null : g.Value;
		}

		public bool IsSuccessful(SegmentKind groupName)
		{
			return _match.Groups[_dicNames[groupName]].Success;
		}

		private SegmentKind? _kind;
		/// <summary>
		/// first successful group
		/// </summary>
		public virtual SegmentKind Kind { get { return (_kind ?? (_kind = GetSuccessfulKind())).GetValueOrDefault(); } }

		public virtual string Text { get { return _match.Value; } }

	}

}
