﻿#if DEBUG && !SILVERLIGHT

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;
using System.IO;
using System.Threading;
using Ice.Utility;

namespace Ice.Xml.Parser.SelfTextEngine
{

	public class SelfTest
	{
		class TestFile
		{
			public int Number { get; set; }
			public string OriginalContent { get; set; }
			public string ExpectedContent { get; set; }
			public SourceKindEnum Kind { get; set; }
			public string FileOriginal { get; set; }
			public string FileExpected { get; set; }

			public TestFile(int number, string originalFile, string expectedFile, SourceKindEnum kind)
			{
				this.Number = number;
				this.OriginalContent = EmbeddedHelper.GetFileText(originalFile);
				this.ExpectedContent = EmbeddedHelper.GetFileText(expectedFile);
				this.Kind = kind;
				this.FileOriginal = originalFile;
				this.FileExpected = expectedFile;
			}
		}

		static List<TestFile> testList = new List<TestFile>()
		{
			new TestFile(1, "AppSetting.xml", "AppSetting.xml", SourceKindEnum.Xml),
			new TestFile(2, "AppSetting_reader.xml", "AppSetting_reader.xml", SourceKindEnum.Xml),
			new TestFile(3, "rssexample.xml", "rssexample.xml", SourceKindEnum.Xml),
			new TestFile(4, "sampleRss091.xml", "sampleRss091.xml", SourceKindEnum.Xml),
			new TestFile(5, "WebForm2.aspx", "WebForm2.aspx.expected.aspx",  SourceKindEnum.Html), // html, when parsed, output html is not the same as original, there is white space and non-selfclosing images
			new TestFile(6, "geolocation.xml", "geolocation.xml", SourceKindEnum.Xml),
			new TestFile(7, "file_json.json", "file_json.json.expected.xml", SourceKindEnum.Json), // json is parsed and saved as xml (.expected)
			new TestFile(8, "html5test.html", "html5test_expected.html",  SourceKindEnum.Html),
		};

		private static void LinqTest1()
		{
			string html = EmbeddedHelper.GetFileText("parse-test-hourly.html");
			XmlDocument doc = XmlDocument.Parse(html);

			string city = doc
				.DescendantNodes(o => o.TagName == "ul" && o.Id == "country-breadcrumbs")
				.DescendantNodes(o => o.TagName == "li" && o.Class == "last")
				.Select(o => o.InnerText)
				.FirstOrDefault();

			var dataTable = doc.DescendantNodes(o => o.TagName == "table" && o.Class == "data").First();
			var temperatureNode = dataTable.DescendantNodes(o => o.TagName == "tr" && o.Class == "temp");
			var temperatureValues = temperatureNode.DescendantNodes(o => o.TagName == "td").Select(o => o.InnerText.Replace("&#176;", "").Trim()).ToList();
			var realFeelValues = temperatureNode.NextSiblings(o => o.Class == "realfeel").Take(1).DescendantNodes(o => o.TagName == "td").Select(o => o.InnerText.Replace("&#176;", "").Trim()).ToList();
			var windSpeedValues = dataTable.DescendantNodes(o => o.TagName == "th" && o.InnerText.StartsWith("Wind")).Take(1).ParentNodes().ChildNodes(o => o.TagName == "td").Select(o => o.InnerText).ToList();
			var hourValues = dataTable.DescendantNodes(o => o.TagName == "thead").Take(1).DescendantNodes(o => o.TagName == "th").Skip(1).ChildNodes(o => o.NodeType == XmlNodeType.Text).Select(o => o.InnerText).ToList();
			var forecastTds = dataTable.DescendantNodes(o => o.TagName == "tr" && o.Class == "forecast").ChildNodes(o => o.TagName == "td").ToList();
			var forecastValues = forecastTds.Select(o => o.InnerText.Trim()).ToList();
			var isNightValues = forecastTds.Select(o => o.Class.Contains("night")).ToList();
			var imageClass = dataTable.DescendantNodes(o => o.TagName == "tr" && o.Class == "forecast").DescendantNodes(o => o.TagName == "div" && o.Class.StartsWith("icon")).Select(o => o.Class.Replace("icon", "").Trim()).ToList();
			var preivousSibblings = doc.DescendantNodes(o => o.Class == "sub-list home-garden-list").Take(1).PreviousSiblings().ToList();
			var nextSibblings = doc.DescendantNodes(o => o.Class == "sub-list home-garden-list").Take(1).NextSiblings().ToList();

			var q1 = doc.Query("//table.data//tr.temp//td:skip(5):take(10)").ToList();
			var q2 = doc.Query("//th[class=first]:first").ToList();
			var q3 = doc.Query("//th[class=first]^tr[class=realfeel]").ToList();

			if (!(preivousSibblings.Count == 9 && nextSibblings.Count == 9 && temperatureValues.Count == 8 && realFeelValues.Count == 8 && windSpeedValues.Count == 8 && hourValues.Count == 8 && forecastTds.Count == 8 && forecastValues.Count == 8 && isNightValues.Count == 8 && imageClass.Count == 8 && q1.Count == 3 && q2.Count == 1 && q3.Count == 2))
			{
				throw new Exception("XmlLinq self-test failed in LinqTest1()");
			}
		}

		/// <summary>
		/// Set to true to skip self test
		/// </summary>
		public static bool SkipSelfTest { get; set; }

		public static void RunSelfTest()
		{
			Thread.CurrentThread.CurrentCulture = System.Globalization.CultureInfo.InvariantCulture; // serialization of decimal latitude and longitue id json test

			if (!SkipSelfTest)
			{
				foreach (TestFile tt in testList)
				{
					XmlDocument doc = XmlDocument.Parse(tt.OriginalContent, tt.Kind);
					if (doc.OutterXml != tt.ExpectedContent)
					{
						try
						{
							File.AppendAllText("d:\\XmlParserError.txt", "XmlParser self-test failed in test no " + tt.Number
								+ "\r\n------------------------------------ Current ---------------------------------------------------------\r\n"
								+ doc.OutterXml
								+ "\r\n------------------------------------ Expected ---------------------------------------------------------\r\n"
								+ tt.ExpectedContent
								+ "\r\n---------------------------------------------------------------------------------------------\r\n"
								
								, Encoding.UTF8);
						}
						catch
						{

						}

						throw new Exception("XmlParser self-test failed in test no " + tt.Number);
					}
				}

				// linkq test
				LinqTest1();

			}
		}


	}

}

#endif