﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;
using System.Linq;
using System.Text.RegularExpressions;
using System.Web;

namespace System.Data
{
	public class SqlDirectHelper
	{
		private SqlConnection Connection {get;set;}
		private string ConnString { get; set; }

		public SqlDirectHelper(string connectionString)
		{
			this.ConnString = connectionString;
		}

		public SqlDirectHelper OpenConnection()
		{
			if (this.Connection == null)
			{
				this.Connection = new SqlConnection(this.ConnString);
			}

			if(Connection.State != ConnectionState.Open)
			{
				this.Connection.Open();
			}

			return this;
		}

		public void CloseConnection()
		{
			if (this.Connection != null && Connection.State != ConnectionState.Closed)
			{
				this.Connection.Close();
			}
		}

		private SqlCommand CreateCommand()
		{
			if (this.Connection == null)
			{
				this.Connection = new SqlConnection(this.ConnString);
			}

			SqlCommand cmd = this.Connection.CreateCommand();
			cmd.Connection = this.Connection;
			cmd.CommandTimeout = this.Connection.ConnectionTimeout;
			return cmd;
		}

		private SqlCommand PrepareCommand(string query, params object[] parameters)
		{
			SqlCommand cmd = CreateCommand();

			if (parameters.Any())
			{
				int uniqueNum = 0;
				cmd.CommandText = new Regex("\\{([\\d]+)\\}").Replace(query, delegate(Match m)
				{
					int index = System.Convert.ToInt32(m.Groups[1].Value);
					object par = parameters[index];
					SqlParameter sqlpar = par as SqlParameter;

					if (par == null || par == DBNull.Value || (sqlpar != null && (sqlpar.Value == null || sqlpar.Value == DBNull.Value)))
					{
						return "NULL";
					}
					else
					{
						string name = "@p" + index + "_" + (uniqueNum++);

						if (sqlpar != null)
						{
							sqlpar.ParameterName = name;
							cmd.Parameters.Add(sqlpar);
						}
						else
						{
							cmd.Parameters.AddWithValue(name, parameters[index]);
						}
						return name;
					}
				});
			}
			else
			{
				cmd.CommandText = query;
			}

			return cmd;
		}

		public int ExecuteCommandWorker(SqlCommand cmd)
		{
			try
			{
				bool closeConn = false; // will close only if connection is opened here, if connection is opened already, will not be closed at the end
				if (cmd.Connection.State != ConnectionState.Open)
				{
					closeConn = true;
					cmd.Connection.Open();
				}

				int rows = 0;
				Exception excmd = null;

				try
				{
					rows = cmd.ExecuteNonQuery();
				}
				catch (Exception ex)
				{
					excmd = ex;
				}

				if (closeConn)
				{
					cmd.Connection.Close();
				}

				if (excmd != null)
				{
					throw new Exception(cmd.Parameters.Cast<SqlParameter>().Select(o => o.ParameterName + ": " + o.SqlDbType).ToStringJoined(", "), excmd);
				}

				return rows;
			}
			catch (Exception ex)
			{
				throw new Exception(cmd.Parameters.Cast<SqlParameter>().Select(o => o.ParameterName + ": " + o.SqlDbType).ToStringJoined(", "), ex);
			}
		}

		public int ExecuteCommand(string query, params object[] parameters)
		{
			return ExecuteCommandWorker(PrepareCommand(query, parameters));
		}

		public int ExecuteCommandWithParameters(string query, params SqlParameter[] parameters)
		{
			SqlCommand cmd = CreateCommand();
			cmd.CommandText = query;
			parameters.ForEach(o => cmd.Parameters.Add(o));
			return ExecuteCommandWorker(cmd);
		}

		public DataSet ExecuteQueryToDataSet(string query, params object[] parameters)
		{
			try
			{
				SqlDataAdapter da = new SqlDataAdapter(PrepareCommand(query, parameters));
				DataSet ds = new DataSet();

				da.Fill(ds);

				return ds;
			}
			catch
			{
				throw;
			}
		}

		public static TResult Convert<TResult>(object obj)
		{
			return Ice.Utility.IceTypeConverter.ChangeType<TResult>(obj);
		}
	}

}