﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace System.Linq
{
	/// <summary>
	/// Thread safe list.
	/// </summary>
	public class IceList<TItem>
	{
		private object _objLock = new object();
		private List<TItem> _list = new List<TItem>();

		public virtual void Add(TItem item)
		{
			lock(_objLock)
			{
				_list.Add(item);
			}
		}

		public virtual void AddRange(IEnumerable<TItem> collection)
		{
			lock(_objLock)
			{
				_list.AddRange(collection);
			}
		}

		/// <summary>
		/// Clone collection to List
		/// </summary>
		/// <returns></returns>
		public List<TItem> ToList()
		{
			lock(_objLock)
			{
				return _list.ToList();
			}
		}

		public bool Any()
		{
			lock(_objLock)
			{
				return _list.Any();
			}
		}

		public int Count
		{
			get
			{
				lock (_objLock)
				{
					return _list.Count;
				}
			}
		}


	}
}
