﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Text;
using System.IO;
using System.IO.Compression;
using Ice.Web.Script.Services;

namespace Ice.Web
{
	public class IceHttpCompressionFilter
	{
		/*
		 * add handler to Global.asax:
		 * 
		 protected void Application_PostRequestHandlerExecute(object sender, EventArgs e)
		{
			Ice.Web.GHttpCompression.SetDefaultCompression(HttpContext.Current);
		}
		 
		 */
		#region ASP.NET COMPRESSION

		const string __noResponseCompression = "__noResponseCompression";
		const string __responseCompressionUsed = "__responseCompressionUsed";

		/// <summary>
		/// Add compresion filter for text, javascript content type, except for .asd files
		/// </summary>
		/// <param name="context"></param>
		/// <param name="isAjaxRequest"></param>
		/// <returns></returns>
		public static void AddCompressionFilter(HttpContext context)
		{
			if (context == null || context.Response == null || context.Response.IsRequestBeingRedirected)
			{
				return;
			}

			if (context.Response.Filter is DeflateStream || context.Response.Filter is GZipStream)
			{
				// if compression is already set on Application_BeginRequest event (Global.asax)
				return;
			}

			string contentType = context.Response.ContentType.ToLower();

			if (contentType.Contains("text") || contentType.Contains("json") || contentType.Contains("javascript"))
			{
				// allow compression
			}
			else
			{
				return;
			}

			if (HasCompressionFilter(context))
			{
				return;
			}

			if (context.Items.Contains(__noResponseCompression) || context.Items.Contains(__responseCompressionUsed))
			{
				return;
			}

			context.Items[__responseCompressionUsed] = true;

			string pathInfo = context.Request.ServerVariables["PATH_INFO"]; // samo path bez query stringa
			if (pathInfo.EndsWith(IceAjaxHandler.IceHandlerPath, StringComparison.OrdinalIgnoreCase))
			{
				// allow compression
			}
			else if (pathInfo.EndsWith(".axd", StringComparison.OrdinalIgnoreCase)) 
			{
				// webresource.axd i scriptresource.axd dodaje vec interno filter za kompresiju
				return; // don't allow
			}

			string acceptEncoding = context.Request.ServerVariables["HTTP_ACCEPT_ENCODING"];//// context.Request.Headers["Accept-Encoding"];

			if (string.IsNullOrEmpty(acceptEncoding))
			{
				return;
			}

			if (acceptEncoding == "*" || acceptEncoding.IndexOf("gzip", StringComparison.OrdinalIgnoreCase) >= 0)
			{
				// gzip
				context.Response.Filter = new GZipStream(context.Response.Filter, CompressionMode.Compress);
				context.Response.AppendHeader("Content-Encoding", "gzip");

			}
			else if (acceptEncoding.IndexOf("deflate", StringComparison.OrdinalIgnoreCase) >= 0)
			{
				// defalte
				context.Response.Filter = new DeflateStream(context.Response.Filter, CompressionMode.Compress);
				context.Response.AppendHeader("Content-Encoding", "deflate");
				
			}


			return;
		}

		public static bool HasCompressionFilter(HttpContext context)
		{
			// test if compression added by the system, not here, so there is no __responseCompressionUsed key added
			if (context == null || context.Response == null)
			{
				return false;
			}

			return context.Response.Filter is DeflateStream || context.Response.Filter is GZipStream;
		}

		/// <summary>
		/// Removes Defalte or GZip compression, restores original filter
		/// </summary>
		/// <param name="context"></param>
		public static void RemoveCompressionFilter(HttpContext context)
		{
			if (context == null || context.Response == null)
			{
				return;
			}

			context.Items[__noResponseCompression] = true;

			while(context.Response.Filter is DeflateStream)
			{
				context.Response.Filter = (context.Response.Filter as DeflateStream).BaseStream;
			}

			while(context.Response.Filter is GZipStream)
			{
				context.Response.Filter = (context.Response.Filter as GZipStream).BaseStream;
			}

		}

		#endregion

	}
}
