﻿/// <reference path="Ice.js" />
/// <reference path="Ice.Event.js" />

/*********************************************************************************
INSTRUCTIONS form Ice.AjaxHistory:
single update: 
Ice.AjaxHistory.Update("#div11", new html content)

multiple update:
Ice.AjaxHistory.BeginUpdate()
	.Update(selector, new html)
	.Update(selector, new html)
	.Update(selector, new html)
	...
	.Commit(true)

*********************************************************************************/

if (typeof (Ice.AjaxHistory) == "undefined")
{
	Ice.AjaxHistory = new (function ()
	{
		//#if VSINTELLISENSE
		var _stateObj;
		if (Ice.IsVSIntellisene)
		{
			_stateObj = {
				url: window.location.href,
				pathname: window.location.pathname,
				search: window.location.search,
				title: document.title,
				arr:
				[
					{
						selector: "",
						html: ""
					},
					{
						selector: "",
						html: ""
					}
				]
			};
		}
		function _stateObjType() { return _stateObj; }
		function _stateArrType() { return _stateObj.arr;}
		//#endif

		////function SaveAjaxState (selector, html)
		////{
		////	if (window.history.state && window.history.state.url == window.location.href)
		////	{
		////		window.history.state.url = window.location.href;
		////		window.history.state.title = document.title;
		////		window.history.state.arr.push({ selector: selector, html: html });
		////	}
		////	else if (window.history.replaceState)
		////	{
		////		EnsureEvents();

		////		window.history.replaceState({
		////			url: window.location.href,
		////			title: document.title,
		////			arr: [{ selector: selector, html: html }]
		////		}, window.location.href);
		////	}
		////};

		function ReplaceAjaxState(stateObj)
		{
			/// <summary>
			/// 
			/// </summary>
			/// <param name="stateObj" type="_stateObjType"></param>
			
			if (window.history.replaceState)
			{
				EnsureEvents();

				if (stateObj)
				{
					window.history.replaceState(stateObj, window.location.href);
				}
			}
		};

		function ReplaceAjaxStateImpl(arr)
		{
			/// <summary>
			/// 
			/// </summary>
			/// <param name="arr" type="_stateArrType"></param>
			
			ReplaceAjaxState({
				url: window.location.href,
				pathname: window.location.pathname,
				search: window.location.search,
				title: document.title,
				arr: arr
			});
		}

		function RemoveAjaxState()
		{
			if (window.history.replaceState)
			{
				window.history.replaceState({}, window.location.href);
			}
		};

		function RestoreState(stateObj)
		{
			/// <summary>
			/// 
			/// </summary>
			/// <param name="stateObj" type="_stateObjType"></param>
			
			if (stateObj && stateObj.arr && window.history.replaceState) // ako path and query isti
			{
				EnsureEvents();

				document.title = stateObj.title;

				for (var n = 0; n < stateObj.arr.length; n++)
				{
					$$ice(stateObj.arr[n].selector).SetInnerHtml(stateObj.arr[n].html);
				}
				// save current stateObj again if wil go back than forward
				//window.history.replaceState(stateObj, stateObj.url);
			}
		}

		function EnsureEvents()
		{
			if (window.localStorage)
			{
				// if localStorage is supported
				window.localStorage["__iceAjaxHistory"] = "yes";
			}

			// do not load state on reload F5, must get new content from server
			////window.onload = function ()
			////{
			////	// to reload current state on window reload with F5, this won't work if redirected to different page
			////	if ("state" in window.history)
			////	{
			////		RestoreState(window.history.state);
			////	}
			////	// else not supported (on Safari)
			////};

			// can not use attachEvent for next events, it won't work

			Ice.Event.Attach(window, "popstate", function (ev) { RestoreState(ev.state); });

			////window.onpopstate = function (ev)
			////{
			////	RestoreState(ev.state);
			////};

			////window.onhashchange = function (a, b, c, d, e)
			////{
			////	// not used right now
			////};
		}

		function ReplaceUrlHash(hash)
		{
			window.location.hash = hash;
		};

		function NewRandomHash()
		{
			// firefox will register new url only if property hash is explicitly set
			window.location.hash = "!x" + new Date().getTime() + Math.random() * 1000;
		};

		if (window.localStorage && window.localStorage["__iceAjaxHistory"] == "yes")
		{
			// if items already exists in history and window is reloaded
			// window refreshed, we have items in history, so create events
			EnsureEvents();
		}

		function UpdateManager(owner)
		{
			var __manager = this;
			var arr = [];

			__manager.Update = function (selector, html)
			{
				arr.push(
					{
						selector: selector,
						html: html
					});

				return __manager;
			};

			__manager.Commit = function (saveAjaxHistory)
			{
				/// <summary>
				/// 
				/// </summary>
				/// <param name="saveAjaxHistory">false: just update html, without saving history obj;;; true: update html and save to history obj</param>

				if (saveAjaxHistory)
				{
					// save current state
					var prevArr = [];
					for (var n = 0; n < arr.length; n++)
					{
						prevArr.push({
							selector: arr[n].selector,
							html: $$ice(arr[n].selector).SelectInnerHtml().FirstOrDefault()
						});
					}
					ReplaceAjaxStateImpl(prevArr);
				}

				if (saveAjaxHistory)
				{
					// change hash 
					// hash should be changed before ajax request, but it is not possible here
					NewRandomHash();
				}

				// update content
				for (var n = 0; n < arr.length; n++)
				{
					// update page content
					$$ice(arr[n].selector).SetInnerHtml(arr[n].html);
				}

				if (saveAjaxHistory)
				{
					// save new current state for new url (required if go back than go forward)
					var newArr = [];
					for (var n = 0; n < arr.length; n++)
					{
						newArr.push({
							selector: arr[n].selector,
							html: $$ice(arr[n].selector).SelectInnerHtml().FirstOrDefault()
						});
					}
					ReplaceAjaxStateImpl(newArr);
				}

				return owner;
			};
		}

		this.SingleUpdate = function (selector, html)
		{
			/// <summary>
			/// Update and commit using ajax history.
			/// </summary>
			/// <param name="selector"></param>
			/// <param name="html"></param>
			/// <param name="pageTitle"></param>

			return new UpdateManager(this).Update(selector, html).Commit(true);
		};

		this.BeginUpdate = function ()
		{
			/// <summary>
			/// for multiple updates. Use manager.Update(...) to update content. At the end call .Commit()
			/// </summary>

			return new UpdateManager(this);
		};


	})();

}

