﻿
Ice.DataBinder =
{
	GetValue: function (dataObj, dataProperty)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="dataObj">object with properties or array</param>
		/// <param name="dataProperty" type="String|Integer">case sensitive full path property name, eg. style.color, or index if object is array, eg. style.color.3.name, where number 3 is index in array</param>

		if (arguments.length < 2)
		{
			Ice.Diagnostic.Exception("Ice.DataBinder.GetValue() requires 2 arguments");
		}

		var arr = dataProperty.split(".");
		var val = dataObj;
		var prop;
		while (arr.length > 0 && (prop = arr.shift())) // will ignore empty prop, eg. Player.Playing. when splited, last value is "", and has to be ignored
		{
			var index = parseInt(prop, 10);
			prop = (index > 0 || index === 0) && (index + "") == prop ? index : prop;  // if it is number, use index as property
			try { val = val[prop]; } catch (e) { return null; }
		}

		return val;
	},

	SetValue: function (dataObj, dataProperty, value)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="dataObj"></param>
		/// <param name="dataProperty" type="String|Integer">case sensitive full path property name, eg. style.color.3.name, where number 3 is index in array style.color</param>
		/// <param name="value">new value</param>

		if (arguments.length < 3)
		{
			Ice.Diagnostic.Exception("Ice.DataBinder.SetValue() requires 3 arguments");
		}

		var arr = dataProperty.split(".");
		var prop;
		var obj = dataObj;
		var tmp = obj;
		while (arr.length > 0 && (prop = arr.shift())) // will ignore empty prop
		{
			var index = parseInt(prop, 10);
			prop = (index > 0 || index === 0) && (index + "") == prop ? index : prop;  // if it is number, use index as property

			if (arr.length > 0)
			{
				tmp = obj[prop];
				if (!tmp)
				{
					// not last property, but datacontext has no object, create new object {}
					// this feature is used at imagine player
					obj = obj[prop] = {};
				}
				else
				{
					obj = tmp;
				}
			}
			else
			{
				// last property, assing value
				obj[prop] = value;
			}


		}
	}
};
