﻿//******** NO EXTERNAL DEPENDENCIES ********

if (typeof (Ice) == "undefined") { Ice = {}; }

Ice.DataTable = function (data)
{
	/// <summary>Constructor. The Ice.DataTable Object that provides DataTable functionality</summary>
	/// <param name="data" optional="true" type="ArrayOfObjects|DataTable|Ice.DataTable|Null">The ArrayOfObjects or DataTable that this instance will work with.</param>

	this.Columns = [];
	this.Rows = [];
	this.TotalCount = null;
	this.TableName = null;
	this.CreateFrom(data);
};

Ice.DataTable.__class = true;

Ice.DataTable.prototype = new (function ()
{
	this.Columns; // = [];
	this.Rows; // [],
	this.TotalCount = null;
	this.TableName = null;

	//#region ToObjectsArray

	function CreateObject(columns, itemsArray)
	{
		var obj = {};
		for (var k = 0; k < columns.length; k++)
		{
			obj[columns[k]] = itemsArray[k];
		}
		return obj;
	}

	this.ToObjectsArray = function ()
	{
		/// <summary>
		/// to objects array, return array
		/// </summary>

		var arr = [];

		for (var n = 0; n < this.Rows.length; n++)
		{
			arr.push(CreateObject(this.Columns, this.Rows[n]));
		}
		return arr;
	};

	//#endregion

	//#region FromObjectsArray

	function CreateColumns(obj)
	{
		var columns = [];
		if (columns.length == 0)
		{
			for (var p in obj)
			{
				columns.push(p);
			}
		}
		return columns;
	}

	function GetItemsArray(obj, columns)
	{
		var values = [];
		for (var n = 0; n < columns.length; n++)
		{
			values.push(obj[columns[n]]);
		}
		return values;
	}

	function FromObjectsArray(objArray)
	{
		for (var n = 0; n < objArray.length; n++)
		{
			var obj = objArray[n];
			if (n == 0)
			{
				this.Columns = CreateColumns(obj);
			}
			var itemsArray = GetItemsArray(obj, this.Columns);
			this.Rows.push(itemsArray);
		}
	};

	//#endregion

	//#region FromDataTable

	function FromDataTable(dataTable)
	{
		/// <summary>
		/// create from dataTable's data
		/// </summary>
		/// <param name="dataTable"></param>
		this.Columns = dataTable.Columns.slice(0);
		this.Rows = dataTable.Rows.slice(0);
		this.TotalCount = dataTable.TotalCount;
		this.TableName = dataTable.TableName;
	};

	//#endregion

	this.Clone = function ()
	{
		/// <summary>
		/// clone this datatable, returns cloned data table
		/// </summary>
		var dt = new Ice.DataTable();
		dt.Columns = this.Columns.slice(); // create copy of array
		dt.Rows = this.Rows.slice(); // create copy of array
		dt.TotalCount = this.TotalCount;
		dt.TableName = this.TableName;
		return dt;
	};

	this.CreateFrom = function (data)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="data" optional="true" type="ArrayOfObjects|DataTable|Ice.DataTable|Null">The ArrayOfObjects or DataTable that this instance will work with.</param>

		if (data)
		{
			if (data.Columns && data.Rows)
			{
				FromDataTable.call(this, data);
			}
			else if (data.push && data.pop)
			{
				FromObjectsArray.call(this, data);
			}
			else
			{
				// invalid data
				Ice.Diagnostic.Exception("Ice.DataTable: Invalid data, can't create data table");
			}
		}
	};

})();

