﻿/// <reference path="Ice.js" />
/// <reference path="Ice.Net.js" />

Ice.RegisterNs("Ice.Diagnostic");

/***********************************************************/
// has to be set on server, ony if ElmahLog="true"
Ice.Diagnostic.ServerUrl = null;
/***********************************************************/

/// <reference path="template:SERVER_DIAGNOSTIC_URL" />

Ice.Diagnostic.Alert = function (msg)
{
	/// <summary>
	/// Show alert() message.
	/// </summary>
	/// <param name="msg"></param>

	alert(msg);
};

Ice.Diagnostic.WriteLine = function (msg)
{
	/// <summary>
	/// Write using console.log, only if IsDebugDefined
	/// </summary>
	/// <param name="msg"></param>
	if (Ice.IsDebugDefined && window.console && console.log)
	{
		console.log(msg);
	}
};

Ice.Diagnostic.ElmahLog = function (elmahTitle, elmahDescription)
{
	/// <summary>
	/// Send exeption to server.
	/// </summary>
	/// <param name="elmahTitle"></param>
	/// <param name="elmahDescription"></param>

	if (Ice.IsVSIntellisene) { return; }

	if (Ice.Diagnostic.ServerUrl)
	{
		try
		{
			var xmlhttp = Ice.Net.WebRequest.GetXMLHttpRequest(Ice.Diagnostic.ServerUrl);
			//open request
			// since setRequestHeader is not supported on IE8 & IE9 XDomainRequest, it will throw exception here, that is why we have try-catch
			xmlhttp.open("POST", Ice.Diagnostic.ServerUrl, true);
			xmlhttp.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
			xmlhttp.setRequestHeader("x-ice-ajax", "ice-exception"); // custom header can't be set on crossdomain request

			var post = ["title=" + encodeURIComponent(elmahTitle || "Ice.WriteElmah")];

			try
			{
				post.push("msg=" + encodeURIComponent(elmahDescription || ""));
				post.push("url=" + encodeURIComponent(window.location.href));
				post.push("cookie=" + encodeURIComponent(document.cookie));

				post.push("zHtml=" + encodeURIComponent(document.documentElement.innerHTML.substr(0, 500)));
				post.push("screen=" + encodeURIComponent(Ice.JSON.Serialize(screen, 3)));
				post.push("location=" + encodeURIComponent(Ice.JSON.Serialize(location, 3)));
				post.push("navigator=" + encodeURIComponent(Ice.JSON.Serialize(navigator, 3))); // max 5 recursions
				post.push("document=" + encodeURIComponent(Ice.JSON.Serialize(document, 1))); // 1 recursion!!!!!

			}
			catch (e1)
			{
				post.push("zHtml=" + encodeURIComponent("* document.documentElement.innerHTML not available*"));
			}

			//send request
			xmlhttp.send(post.join("&"));
		}
		catch (e2)
		{
			// can't be sent to server
		}
	}
};

Ice.Diagnostic.Exception = function (title, description)
{
	/// <summary>
	/// Send exception to server using WriteElmah(). If DEBUG defined, show alert(...) message, else be silent.
	/// </summary>
	/// <param name="title" type="string">title to be show</param>
	/// <param name="description" type="string|Exception">Exception object from try-catch or description</param>

	if (Ice.IsVSIntellisene) { return; }

	var arrtitle = ["Ice.JS.Exception", title];
	var arrdescr = [];

	var exception;

	if (description && description.message && description.stack)
	{
		arrdescr.push("call stack");
		arrdescr.push(description.stack);
	}
	else
	{
		if (description)
		{
			arrdescr.push(description);
		}

		////if (Ice.IsDebugDefined)
		{
			// always generate the stack
			// stack generator:
			try
			{
				Ice.Diagnostic.Exception.StackGeneratorNonExistingMethod();// non-existing method to invoke exception
			}
			catch (ex)
			{
				exception = ex;

				if (ex.message && ex.stack)
				{
					arrdescr.push(">>> stack generated >>>");
					arrdescr.push(ex.stack);
				}
				else
				{
					arrdescr.push("<<< stack not available >>>");
				}
			}
		}
	}
	
	var ftitle = arrtitle.join("\r\n");
	var fdescr = arrdescr.join("\r\n");

	if (Ice.IsDebugDefined)
	{
		Ice.Diagnostic.ElmahLog(ftitle, fdescr);
		Ice.Diagnostic.Alert(ftitle + "\r\n" + fdescr);
	}
	
	throw ("\r\n" + ftitle + "\r\n" + fdescr); // to get call stack in browser's console
};


/*#if DEBUG*/
if (Ice.IsDebugDefined)
{
	window.onerror = function (errorMsg, url, lineNumber, columnNum, exception)
	{
		/// <summary>
		/// There is to many errors from different clients. Don't log this.
		/// </summary>
		/// <param name="errorMsg"></param>
		/// <param name="url"></param>
		/// <param name="lineNumber"></param>


		// must have next test because columnNum is undefined sometimes
		// line may be 0, column must be > 0
		////if(!(lineNumber >= 0 && columnNum > 0))
		////{
		////	// source not available
		////	return;
		////}

		if (errorMsg && errorMsg.indexOf("Ice.JS.Exception") >= 0)
		{
			// generated by throw(...) in Ice.Diagnostic.Exception(...)
			return;
		}

		try
		{
			// execute code in try-catch to prevent throwing exception from here, it would create endless loop

			var title = "Ice.JS.UncaughtJavaScriptException: " + errorMsg;
			var msg = errorMsg + "\r\nUrl: " + url + "\r\nLine: " + lineNumber + "\r\nColumn: " + columnNum + "\r\nStack: " + (exception ? exception.stack : "stack not available");

			// write to elmah
			Ice.Diagnostic.ElmahLog(title, msg);

			if(Ice.IsDebugDefined)
			{
				Ice.Diagnostic.Alert(ftitle + "\r\n" + fdescr);
			}

			//// don't call .Exception(...) from here since this may by invoked by throw ex at the end of .Exception(...), it would create endless loop
			////Ice.Diagnostic.Exception("Ice.UncaughtJavaScriptException: " + errorMsg, "UncaughtJavaScriptException: " + errorMsg + " ### Url: " + url + " ### Line: " + lineNumber + " ### Column: " + columnNum);
		}
		catch(ex)
		{

		}

	};
}
/*#endif*/





