﻿/// <reference path="Ice.List.js" />

Ice.DomQuery = function (source)
{
	/// <summary>
	/// Constructor.
	/// </summary>
	/// <param name="source">null or enumerable, must implement GetEnumerator()</param>

	// call base constructor
	Ice.List.call(this, source);

	this._predicate = "new Ice.DomQuery(" + (source || "") + ")";
};

Ice.DomQuery.__class = true;

Ice.DomQuery.prototype = (function ()
{
	var instance = new Ice.List();
	instance._typeCtor = Ice.DomQuery;
	instance._typeName = "Ice.DomQuery";

	/*
	 var base = Ice.Extend({}, instance);
	// overriding methods: save base method reference, than call base method using base.method.apply(this, arguments)
	//var base = {};
	// override ToArray()
	base.ToArray = instance.ToArray;
	instance.ToArray = function ()
	{
		if (this._source.slice && this._source.push && this._source.pop)
		{
			return this._source.slice(0);
		}
		return base.ToArray.apply(this, arguments);
	};

	// override
	instance.ToList = function ()
	{
		/// <summary>
		/// override to return Ice.DomQuery
		/// </summary>
		return new this._typeCtor(this.ToArray());
	};
	*/

	var base = {
		// base ToList method in Ice.Enumerable
		ToList: instance.ToList
	};

	instance.ToList = function ()
	{
		/// <summary>
		/// override: return new Ice.DomQuery
		/// </summary>
		return new this._typeCtor(this.ToArray());
	};

	instance.ToDomQuery = function ()
	{
		/// <summary>
		/// return new Ice.DomQuery
		/// </summary>

		return new Ice.DomQuery(this.ToArray());
	};

	instance.ToJQuery = function ()
	{
		/// <summary>
		/// Export current collection to jQuery
		/// </summary>

		return $(this.ToArray());
	};

	function TrimSpaces(str)
	{
		if (!str) { return str; }

		return (str || "").replace(new RegExp("(^[\\s]+)|([\\s]+$)", "ig"), "");
	}

	var SelectorReader = function (selector)
	{
		var _currPosition = 0;

		this.Peek = function (forwardBy)
		{
			/// <summary>
			/// Peek next char.
			/// </summary>
			/// <param name="forwardBy" optional="true">default:0 (first available on stack)</param>

			return selector.charAt(_currPosition + (forwardBy || 0));
		};

		this.Pop = function ()
		{
			/// <summary>
			/// Pop first available value, than advance index + 1.
			/// </summary>
			return selector.charAt(_currPosition++);
		};

		this.PopMatch = function (regex, origin)
		{
			/// <summary>
			/// Match from (first + origin) position, advance index by (match length + origin) and return match.
			/// </summary>
			/// <param name="regex"></param>
			/// <param name="origin" optional="true">default: 0</param>
			origin = origin || 0;
			var mm = (selector.substr(_currPosition + origin) || "").match(regex);
			if (mm && mm[0])
			{
				_currPosition += mm[0].length + origin;
				return mm;
			}
			// return undefined
		};
	};

	function attrPredicateFunc(el, attrName, attrSign, attrValue)
	{
		// currentValue: current attribute value
		// attrValue: expected attribute value

		// el[name] will get property value, if it has no value, then get attribute value or "" and convert to string
		var currValue = Ice.Type.Coalesce(el[attrName], Ice.Utility.Dom.GetAttributeValue(el, attrName), "") + ""; // must be string

		var m = attrSign;

		if (!m)
		{
			// the same as =*
			// no attr sign, statement [attrName] means has attribute value, doesn't matter which value is
			return !!(currValue);
		}
		else if (m == "=" || m == "==")
		{
			return (currValue == attrValue);
		}
		else if (m == "=*")
		{
			// has any value, but not empty
			return !!(currValue);
		}
		else if (m == "~=") // E[foo~="warning"]	Matches any E element whose "foo" attribute value is a list of space-separated values, one of which is exactly equal to "warning"
		{
			return ((" " + currValue + " ").indexOf(" " + attrValue + " ") >= 0);
		}
		else if (m == "*=") // contains value
		{
			return (currValue.indexOf(attrValue) >= 0);
		}
		else if (m == "^=") // starts with
		{
			return (currValue.indexOf(attrValue) == 0);
		}
		else if (m == "$=") // ends with
		{
			var index = currValue.lastIndexOf(attrValue);
			return ((index + attrValue.length) == currValue.length);
		}
		else if (m == "!=" || m == "<>")
		{
			return (currValue != attrValue);
		}
		else //// if (currValue && attrValue) // <, >, <=, >=, % etc
		{
			var num1 = Ice.Convert.ToNumberOrDefault(currValue, "nn"); // current attribute value
			var num2 = Ice.Convert.ToNumberOrDefault(attrValue, "nn"); // expected value

			if (num1 == "nn" || num2 == "nn")
			{
				return false;
			}

			switch (m)
			{
				case "<": return num1 < num2;
				case ">": return num1 > num2;
				case ">=": return num1 >= num2;
				case "<=": return num1 <= num2;
			}

			// unknown selector
			Ice.Diagnostic.Exception("Unsupported iceSelector (" + (attrName + ", " + attrSign + ", " + attrValue) + ") in Ice.DomQuery.BuildLambdaExpression #2");
		}
	}

	function CreateAttributeQuery(source, attrName, attrSign, attrValue)
	{
		var func = (function (attrName, attrSign, attrValue)
		{
			return (function (el)
			{
				return attrPredicateFunc(el, attrName, attrSign, attrValue);
			});

		})(attrName, attrSign, attrValue);

		return source.Where(function (o) { return func(o); });
	}

	function CreateIdQuery(source, id)
	{
		return source.Where(function (o) { return o.id == id; });
	}

	function CreateClassQuery(source, className)
	{
		return source.Where(function (o) { return Ice.Utility.Dom.HasClass(o, className); });
	}

	function CreateTagQuery(source, tag)
	{
		return source.Where(function (o) { return Ice.Utility.Dom.HasTagName(o, tag); });
	}

	function CreateColonWordQuery(source, colonWord, colonValue)
	{
		var count = parseInt(colonValue, 10) || 1;

		switch (colonWord)
		{
			case "visible":
				return source.WhereVisible(true); // specific for JS query only
			case "invisible":
			case "hidden":
				return source.WhereVisible(false); // specific for JS query only
			case "first":
			case "take":
				return source.Take(count);
			case "last":
				return source.Reverse().Take(count).Reverse();
			case "skip":
				return source.Skip(count);
			case "is":
			case "not":
				// select elements from source where subquery in :is(subquery) is true or :not(subquery) is false, input to subquery is each element from source query
				// $('div//*:not(span)'), select div's descendants, except span, '*' can be removed
				// $(".firstquery...").Query(":not(a>span, .cls1>>div)") or .Query(":not(a>span), :not(.cls1>>div)"), // select all items from first query where item is not "a" with child "span" and item with class 'cls' has no 2nd child 'div'
				// $(".firstquery...").Query(":is(a>span)"), // select all items from first query where item is "a" with child "span"
				// $(".firstquery...").Query(":is(*>span)"), // * is optional: .Query(":is(>span)") select all items from first query where item has child "span"
				return source.Where(function (o) { return new source._typeCtor([o]).Query(colonValue).Any() == (colonWord == "is"); });
				////case "has":
				////case "hasnot":
				//// OBSOLETE: use :is or :not instead since "is" or "not" is much more sophisticated version than this crappy jQuery.has(...)
				////	// The expression $('div:has(p)') matches a <div> if a <p> exists anywhere among its descendants, not just as a direct child.
				////	var doesHave = colonWord == "has";
				////	return source.Where(function (o) { return new source._typeCtor([o]).DescendantNodes().Query(colonValue).Any() == doesHave; });
			case "reverse":
				return source.Reverse();
			case "ancestor":
				return source.AncestorNodes();
			case "descendant":
				return source.DescendantNodes();
			case "child":
				return source.ChildNodes();
			case "parent":
				return source.ParentNodes();
			case "next-sibling":
				return source.NextSiblings();
			case "previous-sibling":
				return source.PreviousSiblings();
			default:
				Ice.Diagnostic.Exception("Unsupported iceSelector: " + colonWord + "(" + colonValue + ")");
		}
	}

	function SelectFilteredSingle(source, iceSelector)
	{
		var origsrc = source;
		var newsrc = source;

		iceSelector = iceSelector.replace(new RegExp("['\"]", "ig"), ""); // remove ' and "
		iceSelector = TrimSpaces(iceSelector); // trim spaces from start and end of string
		iceSelector = iceSelector.replace(new RegExp("([ ]*)([\\/\\<\\>\\!\\~\\^\\$\\*\\=]+)([ ]*)", "ig"), "$2"); // remove spaces arround special chars
		iceSelector = iceSelector.replace(new RegExp("[ ]+"), " "); // replace multiple spaces with single space

		var sett, ch, mm;

		var reader = new SelectorReader(iceSelector);

		while (ch = reader.Pop())
		{
			if (ch == '#' && (mm = reader.PopMatch(new RegExp("^([\\w\\-_]+)"))))
			{
				// #id
				newsrc = CreateIdQuery(newsrc, mm[1]);
			}
			else if (ch == '.' && (mm = reader.PopMatch(new RegExp("^([\\w\\-_]+)"))))
			{
				// .class
				newsrc = CreateClassQuery(newsrc, mm[1]);
			}
			else if (ch == ' ')
			{
				// descendant
				newsrc = newsrc.DescendantNodes();
			}
			else if (ch == '/' && reader.Peek() == '/' && reader.Peek(1) != '/') // descendants, double //
			{
				// descendant
				reader.Pop(); // pop next / too
				newsrc = newsrc.DescendantNodes();
			}
			else if (ch == '*')
			{
				newsrc = newsrc.Select(function (o) { return o; }); // must create select if the only statement is "*", this way we return all, not empty collection
			}
			else if (ch == '^') // ancestors, ice specific
			{
				newsrc = newsrc.AncestorNodes();
			}
			else if (ch == '<') // parent, << will create 2 times: .ParentNodes().ParentNodes()
			{
				newsrc = newsrc.ParentNodes();
			}
			else if (ch == '>') // children, eg. >>> wil create 3 times .ChildNodes().ChildNodes().ChildNodes()
			{
				newsrc = newsrc.ChildNodes();
			}
			else if (ch == '/' && reader.Peek() != '/') // children, x-path first level children
			{
				newsrc = newsrc.ChildNodes();
			}
			else if (ch == ':' && (mm = reader.PopMatch(new RegExp("^([\\w\\-_]+)([\\(]([^\\)]*)[\\)])*"))))
			{
				// special attribute, eg. :take(10), :first, :first(5)
				newsrc = CreateColonWordQuery(newsrc, mm[1], mm[3]);
			}
			else if (ch == '[' && (mm = reader.PopMatch(new RegExp("^[ ]*([\\w\\-_]+)[ ]*([<>\\!\\=\\~\\^\\$\\*]*)[ ]*([\\w\\:\\;\\,\\.]*)[ ]*\\]"))))
			{
				// '[attribute="value"]' // '[attribute == "value"]' // // '[attribute != "value"]' // '[attribute >= value]'
				newsrc = CreateAttributeQuery(newsrc, mm[1], mm[2], mm[3]);
			}
			else if ((mm = reader.PopMatch(new RegExp("^([\\w\\-_]+)"), -1))) // origin -1 to include current ch in match
			{
				// tag
				newsrc = CreateTagQuery(newsrc, mm[1]);
			}
			else
			{
				// unknown
				Ice.Diagnostic.Exception("Usupported iceSelector (" + (iceSelector) + ")");
			}
		}

		// if newsrc == origsrc, nothing was selected by iceSelector, return an empty eneumerable
		if (newsrc == origsrc)
		{
			newsrc = new origsrc._typeCtor();
		}
		return newsrc;
	}

	instance.Query = function (iceSelector)
	{
		/// <summary>
		/// filter current items by iceSelector
		/// </summary>
		/// <param name="iceSelector" type="string">
		///		lambda or css iceSelector:
		///		<para>TAG name: html, body...</para>
		///		<para>ID: #idvalue...</para>
		///		<para>CLASS: .mydiv, a.mydiv...;</para>
		///		<para>ATTRIBUTES: [name] (contains attribute), [name=value], [name &gt; value], ==, =, &lt;, &gt;, !=, &lt;&gt;, [name*=value] (contains value), [name^=value] (starts with value), [name$=value] (ends with value)...</para>
		///		<para>TRAVERSING: * (all), space or // (descendants), / (children), ^ (ancestors), &gt;&gt;&gt; (n-th level children), &lt;&lt; (n-th level parent) :first(n) :last(n) :skip(n) :take(n) :reverse(), with or without (n), if there is no (n), default n == 1</para>
		///		<para>from x-path: :ancestor :descendant :child :parent :next-siblings :previous-siblings</para>
		/// </param>

		if (Ice.IsVSIntellisene) { return this; }

		if (!iceSelector)
		{
			// we should create new instance maybe
			return this;
		}

		var source = this;

		if ((typeof (iceSelector) == "string" && iceSelector.indexOf("=>") > 0) || typeof (iceSelector) == "function")
		{
			// already is lambda expression or function
			return source.Where(iceSelector)._setPredicateInfo("Query", arguments);
		}

		// *********************************************************
		// multiple iceSelector processor
		// split by comma, but do not split by comma inside (...), eg. :not(div, a, body)
		var iceSelectorsArr = [];
		var currPart = "";
		var digCount = 0;
		for (var n = 0; n < iceSelector.length; n++)
		{
			var ch = iceSelector.charAt(n); // IE8 and older requires charAt
			if (ch == ')')
			{
				digCount--;
				currPart += ch;
			}
			else if (ch == '(')
			{
				digCount++;
				currPart += ch;
			}
			else if (ch == ',' && digCount == 0)
			{
				// split
				iceSelectorsArr.push(currPart);
				currPart = "";
			}
			else
			{
				currPart += ch;
			}
		}

		if (currPart.length > 0)
		{
			iceSelectorsArr.push(currPart);
		}

		////Ice.Diagnostic.WriteLine(iceSelectorsArr.join(" % "));

		////var iceSelectorsArr = iceSelector.split(","); //  " body5 div.noa, form div[id='asdc'], body div#Div1.klasa, div.collapsed.expanded,  ";
		if (iceSelectorsArr.length > 1)
		{
			var newSources = null;

			for (var n = 0; n < iceSelectorsArr.length; n++)
			{
				var sel = iceSelectorsArr[n].replace(new RegExp("((^[ ,]+)|([ ,]+$))", "ig"), ""); // tim chars
				if (sel != null && sel != "")
				{
					var tmpnewsrc = SelectFilteredSingle(source, sel);
					if (newSources == null)
					{
						newSources = tmpnewsrc;
					}
					else
					{
						newSources = newSources.Concat(tmpnewsrc);
					}
				}
			}
			return newSources._setPredicateInfo("Query", arguments);
		}
		else
		{
			return SelectFilteredSingle(source, iceSelector)._setPredicateInfo("Query", arguments);
		}
		// *********************************************************
	};

	instance.ParentNodes = function (iceSelector)
	{
		/// <summary>Parent nodes only</summary>
		/// <param name="iceSelector">lambda or css iceSelector</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		return this.Where(function (o) { return o.parentNode; })
		.Select(function (o) { return o.parentNode; })
		.Query(iceSelector)
		._setPredicateInfo("ParentNodes", arguments);
	};

	function GetAncestors(node)
	{
		var items = [];
		while (node.parentNode)
		{
			items.push(node.parentNode);
			node = node.parentNode;
		}
		return items;
	}

	instance.AncestorNodes = function (iceSelector)
	{
		/// <summary>All ancestors</summary>
		/// <param name="iceSelector">lambda or css iceSelector</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		return this.SelectMany(function (o) { return GetAncestors(o); })
		.Query(iceSelector)
		._setPredicateInfo("AncestorNodes", arguments);
	};

	instance.ChildNodes = function (iceSelector)
	{
		/// <summary>Child nodes only</summary>
		/// <param name="iceSelector">lambda or css iceSelector</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		return this
		// Where() is not required here because SelectMany can handle if o.childNodes is null or undefined
		////.Where(function (o) { return o.childNodes && o.childNodes.length > 0; }) // text nodes doesn't have childNodes property
			.SelectMany(function (o) { return o.childNodes; })
			.Query(iceSelector)
			._setPredicateInfo("ChildNodes", arguments);
	};

	instance.DescendantNodes = function (iceSelector, includeTextNodes)
	{
		/// <summary>Descendant nodes</summary>
		/// <param name="iceSelector">lambda or css iceSelector</param>
		/// <param name="includeTextNodes">if true, select text nodes too, but it is much slower</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		// return children than concat children's children
		return this.ChildNodes().Concat(this.ChildNodes().SelectMany(function (curr)
		{
			if (curr.nodeType == 1 && curr.childNodes && curr.childNodes.length > 0) // nodeType 1: element node
			{
				if (!includeTextNodes && curr.getElementsByTagName)
				{
					return curr.getElementsByTagName("*"); // doesn't get text nodes, but doesn't matter, this is much faster way
				}
				else
				{
					return new instance._typeCtor([curr]).DescendantNodes();
				}
			}
			return null; // .SelectMany() can handle if null is returned
		}))
		.Query(iceSelector)
		._setPredicateInfo("DescendantNodes", arguments);

	};

	instance.NextSiblings = function (iceSelector)
	{
		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		return this.Where(function (o) { return o && o.parentNode && o.parentNode.childNodes; })
		.SelectMany(function (o)
		{
			return new instance._typeCtor(o.parentNode.childNodes).SkipWhile(function (k) { return k != o; }).Skip(1);
		})
		.Query(iceSelector)
		._setPredicateInfo("NextSiblings", arguments);
	};

	instance.PreviousSiblings = function (iceSelector)
	{
		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		return this.Where(function (o) { return o && o.parentNode && o.parentNode.childNodes; })
		.SelectMany(function (o)
		{
			return new instance._typeCtor(o.parentNode.childNodes).TakeWhile(function (k) { return k != o; });
		})
		.Reverse() // order nodes backward so first node in collection is first previous sibling
		.Query(iceSelector)
		._setPredicateInfo("PreviousSiblings", arguments);
	};

	instance.WhereTagName = function (tagName)
	{
		/// <summary>case non-sensitive tagName</summary>
		/// <param name="tagName">tag name</param>
		if (!tagName)
		{
			Ice.Diagnostic.Exception("WhereTagName requires tagName parameter");
		}

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		tagName = tagName.toLowerCase();

		return this.Where(function (o) { return o.tagName && o.tagName.toLowerCase() == tagName; })
				._setPredicateInfo("WhereTagName", arguments);
	};

	instance.WhereAttribute = function (name, predicate)
	{
		/// <summary>filter items by attribute value</summary>
		/// <param name="name" type="String">case non-sensitive</param>
		/// <param name="predicate" type="String|Function|Null">function(stringValue){...}, lambda predicate to test attribute value. </param>
		if (!name)
		{
			Ice.Diagnostic.Exception("WhereAttribute requires name parameter");
		}

		var func = Ice.Utility.Expressions.CompileLambdaExpression(predicate);

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			func(1);
			return this;
		}
		//#endif

		// o.getAttribute("name") will get attribute value
		// o.getAttributeNode("name") will get attribute node
		return this.Where(function (o) { return func(Ice.Utility.Dom.GetAttributeValue(o, name)); })
		._setPredicateInfo("WhereAttribute", arguments);
	};

	instance.WhereHasAttribute = function (name, hasIt)
	{
		/// <summary>
		/// Filter items by attribute
		/// </summary>
		/// <param name="name">attribute name</param>
		/// <param name="hasIt" type="Boolean">OPTIONAL. If not set, default is true.</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		hasIt = arguments.length < 2 || !!hasIt;

		return this.Where(function (o)
		{
			return Ice.Utility.Dom.HasAttribute(o, name) == hasIt;
		})
		._setPredicateInfo("WhereHasAttribute", arguments);
	};

	instance.WhereStyle = function (name, predicate)
	{
		/// <summary>filter items by style value</summary>
		/// <param name="name" type="String">case sensitive, valid css style name, camel case or with -</param>
		/// <param name="predicate" type="String|Function|Null">function(numericValue){return ...;}, or value=>... lambda predicate, values with px or % are parsed into numbers</param>
		if (!name)
		{
			Ice.Diagnostic.Exception("WhereAttribute requires name parameter");
		}

		var func = Ice.Utility.Expressions.CompileLambdaExpression(predicate);

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			func(1);
			return this;
		}
		//#endif

		return this.Where(function (o) { return func(Ice.Utility.Dom.GetParsedStyleValue(o, name)); })
		._setPredicateInfo("WhereStyle", arguments);
	};

	instance.WhereId = function (idvalue)
	{
		/// <summary>where attribute obj.id='id', always case-sensitive</summary>
		/// <param name="id">Case sensitive dom element id.</param>
		if (!idvalue)
		{
			Ice.Diagnostic.Exception("WhereId requires id parameter");
		}

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		return this.WhereAttribute("id", function (o) { return o == idvalue; })
			._setPredicateInfo("WhereId", arguments);
	};

	instance.WhereHasClass = function (cssClass, hasIt)
	{
		/// <summary>where contains cssClass, always case-sensitive</summary>
		/// <param name="cssClass">Case sensitive css class name.</param>
		/// <param name="hasIt" type="Boolean">OPTIONAL. If not set, default is true.</param>
		if (!cssClass)
		{
			Ice.Diagnostic.Exception("WhereHasClass requires cssClass parameter");
		}

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		hasIt = arguments.length < 2 || !!hasIt;

		return this.Where(function (item) { return Ice.Utility.Dom.HasClass(item, cssClass) == hasIt; })
			._setPredicateInfo("WhereHasClass", arguments);
	};

	instance.SelectStyle = function (name)
	{
		/// <summary>
		/// Select NumericValue or StyleValue, which one is available first. Returns new Ice.List with values.
		/// </summary>
		/// <param name="name">style property name</param>
		if (!name)
		{
			Ice.Diagnostic.Exception("StyleValue requires name parameter");
		}

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			return new Ice.List([1]);
		}
		//#endif

		return this.Select(function (item) { return Ice.Utility.Dom.GetParsedStyleValue(item, name); })
			._setPredicateInfo("SelectStyle", arguments);
	};

	instance.SelectStyleMap = function (namesArray)
	{
		/// <summary>
		/// Select NumericValue or StyleValue, which one is available first. Return new Ice.List with objects who's properties are array names: [{name1:value1, name2:value2}, {name1: value, name2: value}, ...].
		/// </summary>
		/// <param name="namesArray">array or params of style names, ["width", border-width", "opacity"]</param>
		/// <returns>collection of objects with style name-value pairs [{name1:value1, name2:value2}, {name1:value1, name2:value2}...] objects</returns>

		var arr = namesArray.push && namesArray.pop ? namesArray : arguments;

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			var dd = { Element: HTMLDivElement.prototype };
			Ice.ForEach(arr, function (o) { dd[o] = 1; });
			return new Ice.List([dd]);
		}
		//#endif

		return this.Select(function (item)
		{
			var obj = { StyleItems: [], Element: item };

			Ice.ForEach(arr, function (name)
			{
				var ss = Ice.Utility.Dom.GetStyleItem(item, name);
				obj[ss.StyleName] = Ice.Convert.ToNumberOrDefault(ss.NumericValue, ss.StyleValue);
				obj.StyleItems.push(ss);
			});

			return obj;
		})._setPredicateInfo("SelectStyleMap", arguments).ToList();
	};

	/* 	var dimArr = (function ()
	{
	var pp = ["absoluteTop", "absoluteLeft", "position", "top", "right", "bottom", "left", "width", "height", "minWidth", "minHeight", "maxWidth", "maxHeight"];

	var arr1 = ["client", "offset", "scroll"];
	var arr2 = ["Left", "Top", "Width", "Height"];
	Ice.ForEach(arr1, function (e1)
	{
	Ice.ForEach(arr2, function (e2)
	{
	pp.push(e1 + e2);
	});
	});

	var arr4 = ["padding", "margin", "border"];
	var arr5 = ["Top", "Right", "Bottom", "Left"];

	Ice.ForEach(arr4, function (e1)
	{
	Ice.ForEach(arr5, function (e2)
	{
	pp.push(e1 + e2);
	});
	});

	return pp;
	})();

	instance.SelectDimensionsMap = function ()
	{
	return this.SelectStyleMap(dimArr);
	};
	*/

	instance.ApplyStyleMap = function (styleMap)
	{
		/// <summary>
		/// Set elements style.
		/// </summary>
		/// <param name="styleMap">{marginTop: 10, "margin-left": "22%"}</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		return this.ForEach(function (item, index)
		{
			for (var prop in styleMap)
			{
				Ice.Utility.Dom.SetStyleValue(item, prop, styleMap[prop]);
			}
		})._setPredicateInfo("ApplyStyleMap", arguments);
	};

	instance.ApplyStyle = function (nameOrCssText, value)
	{
		/// <summary>
		/// Set style value. If value is eg. 20%, it will get current value first, than set 20% of current value.
		/// </summary>
		/// <param name="name">cssText or style name, case sensitive style name or camelCase name</param>
		/// <param name="value">if cssText used, value is empty, else style value</param>

		if (!nameOrCssText)
		{
			return this; // if value is "", to allow conditional eg.: doSet ? "display:none" : ""
		}

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		if (arguments.length == 1)
		{
			if (typeof (nameOrCssText) == "string")
			{
				return this.ForEach(function (item, index)
				{
					if (item && item.style)
					{
						// setting cssText in style object acts as real css: uses last value and ignores previous definitions of the same style-name
						// works with all browsers
						item.style.cssText += ";" + nameOrCssText;
					}
				})._setPredicateInfo("ApplyStyle", arguments);
			}
			else if (typeof (nameOrCssText) == "object")
			{
				return this.ApplyStyleMap(nameOrCssText)._setPredicateInfo("ApplyStyle", arguments);
			}
		}
		else
		{
			return this.ForEach(function (item, index) { Ice.Utility.Dom.SetStyleValue(item, nameOrCssText, value); })
			._setPredicateInfo("ApplyStyle", arguments);
		}
	};

	instance.FitIntoParent = function (fitWidth, fitHeight)
	{
		/// <summary>
		/// Fit each element into it's parent.
		/// </summary>
		/// <param name="fitWidth"></param>
		/// <param name="fitHeight"></param>

		if (fitWidth)
		{
			this.ForEach(function (o) { o._fitPositionH = o.style.position }).ApplyStyle("position:absolute").ForEach(function (o)
			{
				var dim = $$ice(o).SelectStyleMap(["borderLeftWidth", "borderRightWidth", "paddingLeft", "paddingRight", "marginLeft", "marginRight"]).First();
				var pdim = $$ice(o.parentNode).SelectStyleMap(["clientWidth", "paddingLeft", "paddingRight"]).First();
				var tag = (o.tagName || "").toLowerCase();

				switch (tag)
				{
					case "select":
					case "table":
						o._fitWidth = pdim.clientWidth - pdim.paddingLeft - pdim.paddingRight;
						break;
					default:
						o._fitWidth = pdim.clientWidth - pdim.paddingLeft - pdim.paddingRight /*clientWidth include padding, but doesn't include border width*/
							- dim.borderLeftWidth
							- dim.borderRightWidth
							- dim.paddingLeft
							- dim.paddingRight
							- dim.marginLeft
							- dim.marginRight;
						break;
				}

			}).ForEach(function (o)
			{
				o.style.width = o._fitWidth + "px";
				o.style.position = o._fitPositionH || "";
				delete (o._fitPositionH);
				delete (o._fitWidth);
			});
		}

		if (fitHeight)
		{
			this.ForEach(function (o) { o._fitPositionV = o.style.position }).ApplyStyle("position:absolute").ForEach(function (o)
			{
				var dim = $$ice(o).SelectStyleMap(["borderTopWidth", "borderBottomWidth", "paddingTop", "paddingBottom", "marginTop", "marginBottom"]).First();
				var pdim = $$ice(o.parentNode).SelectStyleMap(["clientHeight", "paddingTop", "paddingBottom"]).First();
				var tag = (o.tagName || "").toLowerCase();

				switch (tag)
				{
					case "select":
					case "table":
						o._fitHeight = pdim.clientHeight - pdim.paddingTop - pdim.paddingBottom;
						break;
					default:
						o._fitHeight = pdim.clientHeight - pdim.paddingTop - pdim.paddingBottom
							- dim.borderTopWidth
							- dim.borderBottomWidth
							- dim.paddingTop
							- dim.paddingBottom
							- dim.marginTop
							- dim.marginBottom;
						break;
				}

			}).ForEach(function (o)
			{
				o.style.height = o._fitHeight + "px";
				o.style.position = o._fitPositionV || "";
				delete (o._fitPositionV);
				delete (o._fitHeight);
			});
		}
	};

	instance.SelectAttribute = function (name)
	{
		/// <summary>
		/// Select attribute value and return new Ice.List with values.
		/// </summary>
		/// <param name="name">case non-sensitive</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			return new Ice.List(["attrValue"]);
		}
		//#endif

		// get value
		return this.Select(function (item)
		{
			return Ice.Utility.Dom.GetAttributeValue(item, name);
		})._setPredicateInfo("SelectAttribute", arguments)
			.ToList();
	};

	instance.SelectAttributeMap = function (namesArray)
	{
		/// <summary>
		/// Return new Ice.List with objects who's properties are names from array: [{name1:value1, name2:value2}, {name1: value, name2: value}, ...]
		/// </summary>
		/// <param name="namesArray">array or params of style names, ["width", border-width", "opacity"]</param>
		/// <returns>collection {name1:value1, name2:value2} objects</returns>

		var arr = namesArray.push && namesArray.pop ? namesArray : arguments;

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			var dd = { Element: HTMLDivElement.prototype };
			Ice.ForEach(arr, function (o) { dd[o] = 1; });
			return new Ice.List([dd]);
		}
		//#endif

		return this.Select(function (item)
		{
			var obj = { Element: item };

			Ice.ForEach(arr, function (name)
			{
				obj[name] = Ice.Utility.Dom.GetAttributeValue(item, name);
			});

			return obj;
		})._setPredicateInfo("SelectAttributeMap", arguments)
		.ToList();
	};

	instance.SetAttribute = function (name, value, add)
	{
		/// <summary>
		/// set attribute value
		/// </summary>
		/// <param name="name">name is case non-sensitive</param>
		/// <param name="value"></param>
		/// <param name="add" type="Boolean" optional="true">OPTIONAL. If not set, default is true.</param>

		if (!name)
		{
			Ice.Diagnostic.Exception("AttributeValue requires name parameter");
		}

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		add = arguments.length < 3 || !!add;

		// set value
		return this.ForEach(function (item, index)
		{
			if (add)
			{
				Ice.Utility.Dom.SetAttributeValue(item, name, value);
			}
			else
			{
				Ice.Utility.Dom.RemoveAttribute(item, name);
			}
		})._setPredicateInfo("SetAttribute", arguments);
	};

	instance.RemoveAttribute = function (name)
	{
		/// <summary>
		/// Remove attribute by name.
		/// </summary>
		/// <param name="name">Attribute name.</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		return this.SetAttribute(name, null, false)._setPredicateInfo("RemoveAttribute", arguments);
	};

	instance.SetInnerHtml = function (html)
	{
		/// <summary>
		/// convert value to string and set as innerHTML to every element in collection.
		/// </summary>
		/// <param name="html" type="String|Object|Any"></param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		if (arguments.length == 0 || html == null || typeof (html) == "undefined")
		{
			html = "";
		}

		return this.ForEach(function (el) { el.innerHTML = html + ""; })._setPredicateInfo("SetInnerHtml", arguments);
	};

	instance.SelectInnerHtml = function ()
	{
		/// <summary>
		/// Select innerHTML value from each element and returns new Ice.List with values.
		/// </summary>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			return new Ice.List(["html"]);
		}
		//#endif

		return this.Select(function (el) { return el.innerHTML; })._setPredicateInfo("SelectInnerHtml", arguments).ToList();
	};

	function GetTextValueOnly(node)
	{
		var txt = "";
		// get start tag text and node value if exists
		if (node.tagName)
		{
			txt += node.tagName;
		}

		if (node.attributes && node.attributes.length > 0)
		{
			for (var n = 0; n < node.attributes.length; n++)
			{
				var attr = node.attributes[n];
				txt += " " + attr.name + "=\"" + attr.value + "\"";
			}
		}

		if (node.nodeValue)
		{
			if (txt && txt.length > 0)
			{
				txt += " ";
			}
			txt += node.nodeValue;
		}

		return ignoreCase ? txt.toLowerCase() : txt;
	}

	instance.WhereText = function (textToSearch, ignoreCase)
	{
		if (!textToSearch)
		{
			Ice.Diagnostic.Exception("WhereText requires textToSearch parameter");
		}

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		textToSearch = ignoreCase ? textToSearch.toLowerCase() : textToSearch;

		return this.Where(function (item) { return item && GetTextValueOnly(item).indexOf(textToSearch) >= 0; })
		._setPredicateInfo("WhereText", arguments);
	};

	function CssClass(method, cssClass)
	{
		return this.ForEach(function (item, index)
		{
			Ice.Utility.Dom[method](item, cssClass);
		});
	}

	instance.AddClass = function (cssClass, add)
	{
		/// <summary>
		/// Add css class to classes collection.
		/// </summary>
		/// <param name="cssClass"></param>
		/// <param name="add" type="Boolean" optional="true">OPTIONAL. If not set, default is true.</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		add = arguments.length < 2 || !!add;

		if (add)
		{
			return CssClass.call(this, "AddClass", cssClass)
				._setPredicateInfo("AddClass", arguments);
		}
		else
		{
			return this.RemoveClass(cssClass);
		}
	};

	instance.RemoveClass = function (cssClass)
	{
		/// <summary>
		/// Remove css class from classes collection.
		/// </summary>
		/// <param name="cssClass"></param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		return CssClass.call(this, "RemoveClass", cssClass)
			._setPredicateInfo("RemoveClass", arguments);
	};

	instance.SetClass = function (cssClass)
	{
		/// <summary>
		/// Set css class value, removing previous values.
		/// </summary>
		/// <param name="cssClass"></param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		return CssClass.call(this, "SetClass", cssClass)
			._setPredicateInfo("SetClass", arguments);
	};

	instance.ToggleClass = function (cssClass)
	{
		/// <summary>
		/// If not exists, add class, else remove it.
		/// </summary>
		/// <param name="cssClass"></param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		return CssClass.call(this, "ToggleClass", cssClass)
			._setPredicateInfo("ToggleClass", arguments);
	};

	instance.SelectClassNames = function ()
	{
		/// <summary>
		/// Select class names as new Ice.List;
		/// </summary>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			return new Ice.List(["class"]);
		}
		//#endif

		return this.SelectMany(function (el)
		{
			return el.className.match(new RegExp("[\\S]+", "g")); // returns array of matches
		})
		._setPredicateInfo("SelectClassNames", arguments).ToList();
	};

	/*
	target/srcElement of this event is ev4: it’s the element the event took place on, since the user mouses over it.

	relatedTarget, fromElement, toElement

	W3C added the relatedTarget property to mouseover and mouseout events. This contains the element the mouse came from in case of mouseover, or the element it goes to in case of mouseout.

	Microsoft created two properties to contain this information:

	fromElement refers to the element the mouse comes from. This is interesting to know in case of mouseover.
	toElement refers to the element the mouse goes to. This is interesting to know in case of mouseout.
	In our first example, relatedTarget/fromElement contains a 
	*/

	function IsElDescendant(el, parent)
	{
		if (!el || !parent || el == parent)
		{
			return false;
		}
		while (el && el != parent)
		{
			try { el = el.parentNode; } catch (e) { return false; }
		}
		return el == parent;
	}

	function GetHoverCB(isHover, callbackFunc)
	{
		return (function (sender, ev)
		{
			ev = ev || window.event; // old IE doesn't send event
			// if comes from toolbar or goes to toolbar, relTarget is null
			var relTarget = (isHover ? ev.fromElement : ev.toElement) || ev.relatedTarget;
			if (sender != relTarget && !IsElDescendant(relTarget, sender))
			{
				// enter or exit
				return callbackFunc.call(sender, sender, ev); // this is element
			}
		});
	}

	function GetTriggCountCB(triggCountMax, callbackFunc, originalCallback)
	{
		var count = 0;
		return (function (sender, ev)
		{
			var ret;
			ev = ev || window.event; // old IE doesn't send event
			if (count != null && ++count <= triggCountMax)
			{
				ret = callbackFunc.call(sender, sender, ev); // this is element
			}

			if (count != null && count >= triggCountMax)
			{
				// remove event
				new Ice.DomQuery([sender]).RemoveEvent(ev.type, originalCallback);
				count = null;
			}

			return ret;
		});
	};

	function GetCBEncaps(callback)
	{
		return (function (sender, ev)
		{
			ev = ev || window.event; // old IE doesn't send event
			var res = callback.call(sender, sender, ev); // this is element
			if (res === false)
			{
				// prevent execution if handler method returns false
				if (ev.preventDefault)
				{
					ev.preventDefault();
				}
				if (ev.stopPropagation)
				{
					ev.stopPropagation();
				}
				ev.returnValue = false;
				ev.cancelBubble = true;
				return false;
			}
			return res;
		});
	}

	instance.On = function (eventFullName, callbackFunc, useCapture, triggCountMax)
	{
		/// <summary>
		/// bind event handler to current items
		/// </summary>
		/// <param name="eventFullName" type="String|Array">
		///		<para>event name or array of event names without 'on' on start (click, mouserover, mousemove,...)</para>
		///		<para>may include namespace for simple removal as [eventType].[namespace]: click.my_one, click.mynode.divone</para>
		/// </param>
		/// <param name="callbackFunc">function(sender, event){ }, if not bound to different context, inside of function this == sender</param>
		/// <param name="useCapture">if true, use capture, trigger event before all other</param>
		/// <param name="triggCountMax">number of how many times to trigger, after that events are ignored</param>

		if (!callbackFunc)
		{
			// callback can't be null, throw exception or return this
			return this;
		}

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			callbackFunc.call(HTMLDivElement.prototype, HTMLDivElement.prototype, MouseEvent.prototype);
		}
		//#endif

		if (eventFullName.push && eventFullName.pop)
		{
			var src = this; // if eventName array length == 0, return this
			for (var n = 0; n < eventFullName.length; n++)
			{
				src = this.On(eventFullName[n], callbackFunc, useCapture, triggCountMax);
			}
			return src; // return the latest src, it is only new enumerable with current items
		}

		if (eventFullName.indexOf("on") == 0)
		{
			eventFullName = eventFullName.substr(2);
		}

		// namespace
		var arr = eventFullName.split(".");
		var evName = arr[0];
		var domEventName = evName;
		var evNamespace = null;
		if (arr.length > 1)
		{
			arr.shift();
			evNamespace = arr.join(".");
		}

		if (evName == "enterKey")
		{
			return this.OnEnter(callbackFunc);
		}
		else if (evName == "escKey")
		{
			return this.OnEscKey(callbackFunc);
		}

		return this.ForEach(function (domElement, index)
		{
			if (domElement)
			{
				var cb = GetCBEncaps(callbackFunc); // implement event.preventDefault() if callback func returns false value

				if (evName == "hover")
				{
					domEventName = "mouseover";
					cb = GetHoverCB(true, cb);
				}
				else if (evName == "hout")
				{
					domEventName = "mouseout";
					cb = GetHoverCB(false, cb);
				}

				if (triggCountMax > 0)
				{
					cb = GetTriggCountCB(triggCountMax, cb, callbackFunc); // original callbackFunc is required for event detaching
				}

				// don't test for domElement.tagName if domElement is window or document

				var fff = (function (contextItem, cbFunc)
				{
					// proxy method: since inside Ice.Event.Append cb is invoked by the browser as cb.call(this, event) where this is current element
					// Ice event handlers always have 2 arguments: function(sender, event){...}
					return (function (ev)
					{
						//// contextItem === this
						cbFunc.call(contextItem, contextItem, ev);
					});
				})(domElement, cb);

				cb = fff;// Ice.BindContext(cb, domElement); // bind: domElement is this inside of callback method

				// track event in domElement.__iceEvents
				if (!domElement.__iceEvents)
				{
					domElement.__iceEvents = new Ice.List();
				}

				var o = {
					eventFullName: eventFullName,
					evName: evName,
					evNamespace: evNamespace,
					originalCB: callbackFunc,
					domEventName: domEventName,
					cb: cb,
					capture: useCapture,
					triggCountMax: triggCountMax
				};

				domElement.__iceEvents.Add(o);

				Ice.Event.Attach(domElement, o.domEventName, o.cb, o.capture);
			}
		})._setPredicateInfo("On", arguments);
	};

	instance.RemoveEvent = function (eventFullName, callbackFunc)
	{
		/// <summary>
		/// Remove event by name and/or callbackFunc if specified. If parameters are not specified, remove all ice events.
		/// </summary>
		/// <param name="eventFullName">
		///		<para>full name including namespace, or namespace only, or name only</para>
		///		<para>full event name to be removed, without on: (click, change,...)</para>
		///		<para>may include namespace for simple removal as [eventType].[namespace]: click.my_one, click.mynode.divone</para>
		/// </param>
		/// <param name="callbackFunc">optional, original callback func. if not provided, will remove all events by eventFullName</param>
		return this.ForEach(function (domElement, index)
		{
			if (domElement && domElement.__iceEvents)
			{
				// create new List to run ForEach on it, and remove items from original domElement.__iceEvents
				domElement.__iceEvents.ToList().ForEach(function (o)
				{
					if ((!eventFullName || o.evName == eventFullName || o.eventFullName == eventFullName || o.evNamespace == eventFullName) && (!callbackFunc || callbackFunc == o.originalCB))
					{
						domElement.__iceEvents.Remove(o);
						Ice.Event.Detach(domElement, o.domEventName, o.cb, o.capture);
					}
				});
			}
		})._setPredicateInfo("RemoveEvent", arguments);
	};

	instance.OnHover = function (onOver, onOut)
	{
		/// <summary>
		/// Hover event on mouse enter or mouse leave an element.
		/// </summary>
		/// <param name="onOver">function(sender, event){...}</param>
		/// <param name="onOut">function(sender, event){...}</param>

		var tt = this;
		if (onOver)
		{
			tt = tt.On("hover", onOver);
		}
		if (onOut)
		{
			tt = tt.On("hout", onOut);
		}
		return tt;
	};

	instance.OnChange = function (callbackFunc)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="callbackFunc">function(sender, event){...}, this inside is sender</param>

		return this.On("change", callbackFunc);
	};

	instance.OnClick = function (callbackFunc)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="callbackFunc">function(sender, event){...}, this inside is sender</param>
		return this.On("click", callbackFunc);
	};

	instance.OnDblClick = function (callbackFunc)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="callbackFunc">function(sender, event){...}, this inside is sender</param>
		return this.On("dblclick", callbackFunc);
	};

	instance.OnMouseUp = function (callbackFunc)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="callbackFunc">function(sender, event){...}, this inside is sender</param>
		return this.On("mouseup", callbackFunc);
	};

	instance.OnMouseDown = function (callbackFunc)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="callbackFunc">function(sender, event){...}, this inside is sender</param>
		return this.On("mousedown", callbackFunc);
	};

	instance.OnLeftMouseDown = function (callbackFunc)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="callbackFunc">function(sender, event){...}, this inside is sender</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			callbackFunc.call(HTMLDivElement.prototype, HTMLDivElement.prototype, MouseEvent.prototype);
		}
		//#endif

		function cb(sender, event)
		{
			if (Ice.Event.IsLeftMouseButton(event))
			{
				callbackFunc.apply(this, arguments);
			}
		}

		return this.On("mousedown", cb);
	};

	instance.OnRightMouseDown = function (callbackFunc)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="callbackFunc">function(sender, event){...}, this inside is sender</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			callbackFunc.call(HTMLDivElement.prototype, HTMLDivElement.prototype, MouseEvent.prototype);
		}
		//#endif

		function cb(sender, event)
		{
			if (Ice.Event.IsRightMouseButton(event))
			{
				callbackFunc.apply(this, arguments);
			}
		}

		return this.On("mousedown", cb);
	};

	instance.OnMouseMove = function (callbackFunc)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="callbackFunc">function(sender, event){...}, this inside is sender</param>

		return this.On("mousemove", callbackFunc);
	};

	instance.OnFocus = function (callbackFunc)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="callbackFunc">function(sender, event){...}, this inside is sender</param>

		return this.On("focus", callbackFunc);
	};

	instance.OnBlur = function (callbackFunc)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="callbackFunc">function(sender, event){...}, this inside is sender</param>

		return this.On("blur", callbackFunc);
	};

	instance.OnKeyDown = function (callbackFunc)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="callbackFunc">function(sender, event){...}, this inside is sender</param>

		return this.On("keydown", callbackFunc);
	};

	instance.OnKeyUp = function (callbackFunc)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="callbackFunc">function(sender, event){...}, this inside is sender</param>

		return this.On("keyup", callbackFunc);
	};

	instance.OnKeyPress = function (callbackFunc)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="callbackFunc">function(sender, event){...}, this inside is sender</param>

		return this.On("keypress", callbackFunc);
	};

	instance.OnEnter = function (callbackFunc)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="callbackFunc">function(sender, event){...}, this inside is sender</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			callbackFunc.call(HTMLDivElement.prototype, HTMLDivElement.prototype, MouseEvent.prototype);
		}
		//#endif

		return this.OnKeyDown(function (sender, ev)
		{
			if (Ice.Event.IsEnterKey(ev))
			{
				return callbackFunc.call(sender, sender, ev);
			}
		});
	};

	instance.OnEscKey = function (callbackFunc)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="callbackFunc">function(sender, event){...}, this inside is sender</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			callbackFunc.call(HTMLDivElement.prototype, HTMLDivElement.prototype, MouseEvent.prototype);
		}
		//#endif

		return this.OnKeyDown(function (sender, ev)
		{
			if (Ice.Event.IsEscKey(ev))
			{
				return callbackFunc.call(sender, sender, ev);
			}
		});
	};

	instance.CloneNodes = function ()
	{
		/// <summary>
		/// Deep clone current nodes and return cloned nodes only.
		/// </summary>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		return this.Where("o=>o.cloneNode").Select("o=>o.cloneNode(true)")
			._setPredicateInfo("CloneNodes", arguments);

	};

	instance.RemoveFromDom = function ()
	{
		/// <summary>
		/// disconnect nodes from DOM
		/// </summary>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		return this.ForEach(function (item, index)
		{
			Ice.Utility.Dom.RemoveNodeFromDom(item);
		})
		._setPredicateInfo("RemoveFromDom", arguments);
	};

	function AddOrModify(funcName, contentArgsArray)
	{
		/// <summary>
		/// If node already have parentNode, node will be cloned
		/// </summary>
		/// <param name="funcName" type="String"></param>
		/// <param name="contentArgsArray"></param>
		var items = this.ToArray();
		var elarr = Ice.DomQuery.ConvertToElementsArray(contentArgsArray);

		for (var n1 = 0; n1 < items.length; n1++)
		{
			var cloned = [];
			// elarr may be null
			for (var k = 0; elarr && k < elarr.length; k++)
			{
				// clone nodes which already has parentNode
				var el = Ice.Utility.Dom.CloneNodeIfHasParent(elarr[k]);
				cloned.push(el);
			}
			// even if elarr is null, we have to call func, eg. ReplaceChildren to remove children from dom
			Ice.Utility.Dom[funcName](items[n1], cloned);
		}

		return new this._typeCtor(items);
	}

	instance.ReplaceWith = function (content1, content2, content3, etc)
	{
		/// <summary>
		/// replace each node with new html or element. if content is null, remove node from dom. Returnes removed nodes (original collection).
		/// </summary>
		/// <param name="content"></param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		return AddOrModify.call(this, "InsertBefore", arguments).ForEach(function (o, n) { Ice.Utility.Dom.RemoveNodeFromDom(o); })
			._setPredicateInfo("ReplaceWith", arguments);
	};

	instance.ReplaceChildren = function (content1, content2, content3, etc)
	{
		/// <summary>
		/// Replace each current collection node's children with content. if content is null, remove children only. Returns current collection.
		/// </summary>
		/// <param name="content"></param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		return AddOrModify.call(this, "ReplaceChildrenWith", arguments)
		._setPredicateInfo("ReplaceChildren", arguments);
	};

	instance.AddChildren = function (content1, content2, content3, etc)
	{
		/// <summary>
		/// Add content as last child to each node in current collection. Returns current collection.
		/// </summary>
		/// <param name="content">html, single dom element, array of dom element, Ice.DomQuery elements</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		return AddOrModify.call(this, "AddChildren", arguments)
		._setPredicateInfo("AddChildren", arguments);
	};

	instance.InsertChildren = function (content1, content2, content3, etc)
	{
		/// <summary>
		/// Insert content as first child to each node in current collection. Returns current collection.
		/// </summary>
		/// <param name="content">html, dom element or array of dom element</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		return AddOrModify.call(this, "InsertChildren", new Ice.List(arguments).Reverse().ToArray()) // arguments is dom array so there is no .reverse()
		._setPredicateInfo("InsertChildren", arguments);
	};

	instance.InsertBefore = function (content1, content2, content3, etc)
	{
		/// <summary>
		/// Insert content before every node in current collection. Returns current collection.
		/// </summary>
		/// <param name="content"></param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		return AddOrModify.call(this, "InsertBefore", arguments)
		._setPredicateInfo("InsertBefore", arguments);
	};

	instance.InsertAfter = function (content1, content2, content3, etc)
	{
		/// <summary>
		/// Insert content after every node in current collection. Returns current collection.
		/// </summary>
		/// <param name="content"></param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		return AddOrModify.call(this, "InsertAfter", new Ice.List(arguments).Reverse().ToArray()) // arguments is dom array so there is no .reverse()
		._setPredicateInfo("InsertAfter", arguments);
	};

	instance.InsertAsChildrenTo = function (iceSelector, returnNewNodes)
	{
		/// <summary>
		/// <para>Insert current collection nodes as first children into nodes specified by iceSelector. Returns added nodes, including cloned ones if there were cloned or multiplied.</para>
		/// </summary>
		/// <param name="iceSelector"></param>
		/// <param name="returnNewNodes" type="Boolean">if true, return all newly created nodes, including cloned</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		var currItems = Ice.DomQuery.FromSelector(iceSelector).ToDomQuery();
		if (!returnNewNodes)
		{
			currItems.InsertChildren(this.ToArray());
			return this;
		}
		else
		{
			var children = currItems.ChildNodes().ToDomQuery();
			currItems.InsertChildren(this.ToArray());

			var childrenAfter = currItems.ChildNodes().ToDomQuery();

			return childrenAfter.Except(children, function (a, b) { return a == b; });

			////return this._newEnumerable()._setPredicateInfo("InsertAsChildrenTo", arguments);
		}
	};

	instance.AddAsChildrenTo = function (iceSelector, returnNewNodes)
	{
		/// <summary>
		/// <para>Add current collection as last children into nodes specified by iceSelector. Returns added nodes, including cloned ones if there were cloned or multiplied.</para>
		/// </summary>
		/// <param name="iceSelector"></param>
		/// <param name="returnNewNodes" type="Boolean">if true, return all newly created nodes, including cloned</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		var currItems = Ice.DomQuery.FromSelector(iceSelector).ToDomQuery();
		if (!returnNewNodes)
		{
			currItems.AddChildren(this.ToArray());
			return this;
		}
		else
		{
			var children = currItems.ChildNodes().ToDomQuery();
			currItems.AddChildren(this.ToArray());

			var childrenAfter = currItems.ChildNodes().ToDomQuery();

			// find new children:
			return childrenAfter.Except(children, function (a, b) { return a == b; });
			////return this._newEnumerable()._setPredicateInfo("AddAsChildrenTo", arguments);
		}
	};

	instance.InsertBeforeTo = function (iceSelector, returnNewNodes)
	{
		/// <summary>
		/// <para>Insert current collection before nodes selected by iceSelector. Returns added nodes, including cloned ones if there were cloned or multiplied.</para>
		/// </summary>
		/// <param name="iceSelector"></param>
		/// <param name="returnNewNodes" type="Boolean">if true, return all newly created nodes, including cloned</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		var currItems = Ice.DomQuery.FromSelector(iceSelector).ToDomQuery();
		if (!returnNewNodes)
		{
			// nodes will not be cloned, this way it is much faster
			currItems.InsertBefore(this.ToArray());
			return this;
		}
		else
		{
			var children = currItems.ParentNodes().ChildNodes().ToDomQuery();
			currItems.InsertBefore(thisArr);

			var childrenAfter = currItems.ParentNodes().ChildNodes().ToDomQuery();
			// find new children:
			return childrenAfter.Except(children, function (a, b) { return a == b; });

			////return this._newEnumerable()._setPredicateInfo("InsertBeforeTo", arguments);
		}
	};

	instance.InsertAfterTo = function (iceSelector, returnNewNodes)
	{
		/// <summary>
		/// <para>Insert current collection after nodes selected by iceSelector. Returns added nodes, including cloned ones if there were cloned or multiplied.</para>
		/// </summary>
		/// <param name="iceSelector"></param>
		/// <param name="returnNewNodes" type="Boolean">if true, return all newly created nodes, including cloned</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		var currItems = Ice.DomQuery.FromSelector(iceSelector).ToDomQuery();

		if (!returnNewNodes)
		{
			currItems.InsertAfter(this.ToArray());
			return this;
		}
		else
		{
			var children = currItems.ParentNodes().ChildNodes().ToDomQuery();
			currItems.InsertAfter(this.ToArray());

			var childrenAfter = currItems.ParentNodes().ChildNodes().ToDomQuery();
			// find new children:
			return childrenAfter.Except(children, function (a, b) { return a == b; });

			////return this._newEnumerable()._setPredicateInfo("InsertAfterTo", arguments);
		}
	};

	instance.Sleep = function (sleepMs, onExpired)
	{
		/// <summary>
		/// Sleep execution with setTimeout. Can be used for branching. When expired, execute onExpired.call(this, this). "this" is sent both as parameter and as context.
		/// </summary>
		/// <param name="sleepMs"></param>
		/// <param name="onExpired">function to execute onExpired.call(this, this)</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			onExpired.call(this, this);
			return this;
		}
		//#endif

		var aa = this;
		setTimeout(function () { onExpired.call(aa, aa); }, sleepMs);

		return this._newEnumerable()._setPredicateInfo("Sleep", arguments);
	};

	instance.ToAnimator = function (finalMap)
	{
		/// <summary>
		/// Export current elements to animator. Animate from current style settings to finalMap settings.
		/// </summary>
		/// <param name="finalMap">final values to animate to, eg. {height:40, opacity:25, marginTop: 54}</param>

		var animator = new Ice.Animator();
		this.ForEach(function (item, index)
		{
			if (finalMap)
			{
				for (var prop in finalMap)
				{
					animator.AddElement(item, prop, finalMap[prop]);
				}
			}
			else
			{
				animator.AddElement(item); // just add element to animator
			}
		});
		return animator;
	};



	instance.SetSelectable = function (isSelectable)
	{
		/// <summary>
		/// Add onselectstart handler which returns false.
		/// </summary>
		/// <param name="isSelectable" type="Boolean">OPTIONAL. If not set, default is true.</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		isSelectable = arguments.length == 0 || !!isSelectable;

		////.iceNoUserSelect {
		////	-webkit-touch-callout: none;
		////	-webkit-user-select: none;
		////	-khtml-user-select: none;
		////	-moz-user-select: none;
		////	-ms-user-select: none;
		////	user-select: none;
		////}

		// Formal syntax: none | text | all | element

		//////// use css class because if it is set directly into style, it can not be removed
		//////var val = isSelectable ? "" : "none"; //empty value for removal
		//////var pp = ["-khtml-", "-moz-", "-ms-", "-o-", ""];
		//////var arr = ["-webkit-user-select:" + val];
		//////for(var n = 0; n < pp.length; n++)
		//////{
		//////	arr.push(pp[n] + "user-select:" + val);
		//////}
		//////arr.push("");
		//////// first create string and apply it using element.style.cssText += this, if values are applyed one by one, they will be overriden by the last one and it won't work in all browsers
		//////return this.ApplyStyle(arr.join(";"));

		if (isSelectable)
		{
			return this.RemoveClass("iceNoUserSelect");
		}
		else
		{
			return this.AddClass("iceNoUserSelect"); // class is defined in ThemeBase
		}
	};

	instance.SetEnabled = function (isEnabled)
	{
		/// <summary>
		/// Set attribute disabled=disabled and disable onclick handler. LIMITATION: "disabled" has to be set on all descendants.
		/// </summary>
		/// <param name="isEnabled" type="Boolean">OPTIONAL. If not set, default is true.</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		isEnabled = arguments.length == 0 || !!isEnabled;

		return this.ForEach(function (item, index)
		{
			Ice.Utility.Dom.SetEnabled(item, isEnabled);
		})._setPredicateInfo("SetEnabled", arguments);
	};

	instance.SetVisible = function (isVisible)
	{
		/// <summary>
		/// Set style display="none", else set display="".
		/// </summary>
		/// <param name="isVisible" type="Boolean">OPTIONAL. If not set, default is true.</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		isVisible = arguments.length == 0 || !!isVisible;

		return this.ForEach(function (item, index)
		{
			if (item && item.style)
			{
				item.style.display = isVisible ? "" : "none";
			}
		})._setPredicateInfo("SetVisible", arguments);
	};

	instance.SetReadOnly = function (isReadonly)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="isReadonly">default: true</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		if (arguments.length > 0 ? isReadonly : true)
		{
			return this.SetAttribute("readonly", "readonly");
		}
		else
		{
			this.RemoveAttribute("readonly");
		}
	};

	instance.WhereVisible = function (isVisible)
	{
		/// <summary>
		/// Test if element is visible using style.display and offsetHeight and offsetWidth
		/// </summary>
		/// <param name="isVisible" type="Boolean">OPTIONAL. If not set, default is true.</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		isVisible = arguments.length == 0 || !!isVisible;

		return this.Where(function (o)
		{
			return (o && o.parentNode && o.offsetWidth > 0 && o.offsetHeight > 0 && o.style && o.style.display != "none") == isVisible;
		})._setPredicateInfo("WhereVisible", arguments);
	};

	instance.WhereEnabled = function (isEnabled)
	{
		/// <summary>
		/// Test if "disabled" attribute exists.
		/// </summary>
		/// <param name="isEnabled" type="Boolean">OPTIONAL. If not set, default is true.</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		isEnabled = arguments.length == 0 || !!isEnabled;

		return this.Where(function (o)
		{
			return !Ice.Utility.Dom.HasAttribute(o, "disabled") == isEnabled;
		})._setPredicateInfo("WhereEnabled", arguments);
	};

	instance.SetOpacity = function (percent)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="opacity">range 0 - 100 (full visibility = 100)</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		/*filter for For IE8 and earlier */
		return this.ApplyStyle("filter", "alpha(opacity=" + percent + ")")
				.ApplyStyle("opacity", percent);
	};

	instance.SetClip = function (clipValue)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="clipValue" type="object|string">already parsed into object, or string: "margins(100% 0 0 0)" or "rect:(10px 100px 50px 53px)" </param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		return this.ForEach(function (o) { Ice.Utility.Dom.SetClip(o, clipValue); });
	};

	instance.SetTransition = function (propertyName, durationMs, func, delayMs)
	{
		/// <summary>
		/// single line transition
		/// </summary>
		/// <param name="propertyName">"none" to remove transition (optional, default: all) Specifies the name of the CSS property the transition effect is for. Css default is all.</param>
		/// <param name="durationMs">(optional, default: 400ms) Specifies how many seconds or milliseconds the transition effect takes to complete. Css default is 0</param>
		/// <param name="delayMs">(optional, default: 0) Defines when the transition effect will start. css default is 0</param>
		/// <param name="func"> (optional, default: linear)
		///		<para>linear	Specifies a transition effect with the same speed from start to end (equivalent to cubic-bezier(0,0,1,1))</para>
		///		<para>(default) ease	Specifies a transition effect with a slow start, then fast, then end slowly (equivalent to cubic-bezier(0.25,0.1,0.25,1))</para>
		///		<para>ease-in	Specifies a transition effect with a slow start (equivalent to cubic-bezier(0.42,0,1,1))</para>
		///		<para>ease-out	Specifies a transition effect with a slow end (equivalent to cubic-bezier(0,0,0.58,1))</para>
		///		<para>ease-in-out	Specifies a transition effect with a slow start and end (equivalent to cubic-bezier(0.42,0,0.58,1))</para>
		///		<para>cubic-bezier(n,n,n,n)	Define your own values in the cubic-bezier function. Possible values are numeric values from 0 to 1</para>
		/// </param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		if (arguments.length < 1) { propertyName = "all"; }
		if (arguments.length < 2) { durationMs = 400; }
		if (arguments.length < 3) { func = "linear"; }
		if (arguments.length < 4) { delayMs = 0; }

		var val = propertyName == "none" ? "" : (propertyName + " " + durationMs + "ms" + " " + func + " " + delayMs + "ms");
		this.ApplyStyle("transition", val);
		this.ApplyStyle("-webkit-transition", val);
		return this;
	};

	var transforms = ["transform", "-ms-transform", "-webkit-transform"];

	instance.SetTransform = function (transformFunc)
	{
		/// <summary>
		/// <para>Remove any previous transform and apply new.</para>
		/// <para>angle must have deg: eg. 45deg.</para>
		/// <para>x,y,z values must have px units.</para>
		/// <para>scale has no units</para>
		/// </summary>
		/// <param name="transformFunc">
		///		<para>none</para>
		///		<para>matrix(n,n,n,n,n,n) 2D transform</para>
		///		<para>matrix3d(n,n,n,n,n,n,n,n,n,n,n,n,n,n,n,n) 3D transform</para>
		///		<para>translate(x,y)</para>
		///		<para>translate3d(x,y,z)</para>
		///		<para>translateX(x)</para>
		///		<para>translateY(y)</para>
		///		<para>translateZ(z)</para>
		///		<para>scale(x,y)</para>
		///		<para>scale3d(x,y,z)</para>
		///		<para>scaleX(x)</para>
		///		<para>scaleY(y)</para>
		///		<para>scaleZ(z)</para>
		///		<para>rotate(angle)</para>
		///		<para>rotate3d(x,y,z,angle)</para>
		///		<para>rotateX(angle)</para>
		///		<para>rotateY(angle)</para>
		///		<para>rotateZ(angle)</para>
		///		<para>skew(x-angle,y-angle)</para>
		///		<para>skewX(angle)</para>
		///		<para>skewY(angle)</para>
		///		<para>perspective(n)</para>
		/// </param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		if (arguments.length == 0 || transformFunc == "none")
		{
			transformFunc = "";
		}

		for (var n = 0; n < transforms.length; n++)
		{
			this.ApplyStyle(transforms[n], transformFunc);
		}
		return this;
	};

	instance.ApplyTransform = function (transformFunc)
	{
		/// <summary>
		/// <para>Get current transform and apply modification to existing transform only.</para>
		/// <para>angle must have deg: eg. 45deg.</para>
		/// <para>x,y,z values must have px units.</para>
		/// <para>scale has no units</para>
		/// </summary>
		/// <param name="transformFunc">
		///		<para>none</para>
		///		<para>matrix(n,n,n,n,n,n) 2D transform</para>
		///		<para>matrix3d(n,n,n,n,n,n,n,n,n,n,n,n,n,n,n,n) 3D transform</para>
		///		<para>translate(x,y)</para>
		///		<para>translate3d(x,y,z)</para>
		///		<para>translateX(x)</para>
		///		<para>translateY(y)</para>
		///		<para>translateZ(z)</para>
		///		<para>scale(x,y)</para>
		///		<para>scale3d(x,y,z)</para>
		///		<para>scaleX(x)</para>
		///		<para>scaleY(y)</para>
		///		<para>scaleZ(z)</para>
		///		<para>rotate(angle)</para>
		///		<para>rotate3d(x,y,z,angle)</para>
		///		<para>rotateX(angle)</para>
		///		<para>rotateY(angle)</para>
		///		<para>rotateZ(angle)</para>
		///		<para>skew(x-angle,y-angle)</para>
		///		<para>skewX(angle)</para>
		///		<para>skewY(angle)</para>
		///		<para>perspective(n)</para>
		/// </param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		//console.log(transformFunc);

		if (arguments.length == 0 || transformFunc == "none")
		{
			return this.SetTransform("none");
		}

		var fnames = transformFunc.match(new RegExp("[\\w]+", "ig"));

		return this.ForEach(function (o)
		{
			if (o && o.style)
			{
				var newval = transformFunc;
				var curr = o.style.transform || o.style.msTransform || o.style.webkitTransform;
				if (curr)
				{
					// if new val is compund, eg.: translate(100px, -100px) rotate(170deg) scale(0.2)"
					// remove previous and add new
					newval = curr.replace(new RegExp("(" + fnames.join("|") + ")" + "[\\s]*\\([^\\)]*\\)", "ig"), "") + " " + transformFunc;
				}
				////console.log("set: " + newval);

				$$ice(o).SetTransform(newval);
			}
		});
	};

	instance.SelectTransform = function ()
	{
		/// <summary>
		/// Select transform value of current elements and return new Ice.List with string values.
		/// </summary>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			return new Ice.List(["transform"]);
		}
		//#endif

		return this.Select(function (o)
		{
			if (!o || !o.style) { return null; }
			return o.style.transform || o.style.msTransform || o.style.webkitTransform;
		}).ToList();
	};

	instance.SetLinearGradient = function (degrees, color0, color1, color3, etc)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="degrees" type="Int|String">
		///		<para>none to remove gradient</para>
		///		<para>0 is from bottom to top, negative i CCV, pozitive CV</para>
		/// </param>
		/// <param name="color0" type="String">color value or color value and position in percent starting from start (0% is start, 100% is end) eg. red, blue, or red 10%, blue 20%, etc</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene) { return this; }
		//#endif

		/////* background-image: linear-gradient(0deg, color percent, color percent, etc)*/
		////background-color: #F07575; /* fallback color if gradients are not supported */
		////background-image: -webkit-linear-gradient(top, hsl(0, 80%, 70%), #bada55 50%, red 100%); /* For Chrome and Safari */
		////background-image: -moz-linear-gradient(top, hsl(0, 80%, 70%), #bada55); /* For old Fx (3.6 to 15) */
		////background-image: -ms-linear-gradient(top, hsl(0, 80%, 70%), #bada55); /* For pre-releases of IE 10*/
		////background-image: -o-linear-gradient(top, hsl(0, 80%, 70%), #bada55); /* For old Opera (11.1 to 12.0) */ 
		////background-image: linear-gradient(to bottom, hsl(0, 80%, 70%), #bada55); /* Standard syntax; must be last */

		if (arguments.length == 0 || degrees == "none")
		{
			return this.ApplyStyle("background-color", "").ApplyStyle("background-image", "");
		}

		degrees = parseFloat(degrees);
		// all browsers uses clock-wise orientataion and 0 is at bottom
		// safari uses CCW orientation and 0 is at left
		if (Ice.Features.GetBrowser().IsSafari())
		{
			degrees = (degrees - 90) * (-1);
		}

		degrees += "deg";

		var arr = [degrees];
		for (var n = 1; n < arguments.length; n++)
		{
			arr.push(arguments[n]);
		}
		var str = arr.join(", ");

		var pp = ["-webkit-", "-moz-", "-ms-", "-o-", ""];

		var ss = ["background-color:" + (color0 || "").split(" ")[0]]; // background-color if gradient is not supported
		for (var n = 0; n < pp.length; n++)
		{
			ss.push("background-image:" + pp[n] + "linear-gradient(" + str + ")");
		}
		ss.push("");

		console.log(ss.join(";"));

		// create complete string first, than assign it as style.cssText += ..., if assigned one by one, it will override each other and the last one will be set as final, so it wont' work in all browsers
		this.ApplyStyle(ss.join(";"));
		return this;
	};


	/*
	instance.SetCentered = function ()
	{
		/// <summary>
		/// Set current elements centered into their parents
		/// </summary>
		
		return this.Where("o=>o.parentNode").ForEach(function (el, index)
		{
			el.style.marginLeft = ((el.parentNode.clientWidth - el.offsetWidth) / 2) + "px";
			el.style.marginTop = ((el.parentNode.clientHeight - el.offsetHeight) / 2) + "px";
		});
	};
	*/

	return instance;
})();

//#region static methods

Ice.DomQuery.ConvertToElementsArray = function (content1, content2, content3, etc)
{
	/// <summary>
	/// All arrays or enumerables are flattened and items are converted to HtlmElements array
	/// </summary>
	/// <param name="content1" type="String|Array|Enumerable">html, array of htmls, element or array or elements</param>

	if (arguments.length > 1)
	{
		return new Ice.List(arguments).SelectMany(function (o) { return Ice.DomQuery.ConvertToElementsArray(o); }).ToArray();
	}
	else if (arguments.length == 1)
	{
		var htmlOrElement = arguments[0];

		if (!htmlOrElement)
		{
			return htmlOrElement;
		}

		if (htmlOrElement.SelectMany)
		{
			return htmlOrElement.SelectMany(function (o) { return Ice.DomQuery.ConvertToElementsArray(o); }).ToArray(); // Ice.DomQuery collection
		}
		else if (htmlOrElement.toLowerCase) // string
		{
			var div = document.createElement("div"); // container div
			div.innerHTML = htmlOrElement;
			var items = [];
			for (var n = 0; n < div.childNodes.length; n++)
			{
				var node = div.childNodes[n];
				items.push(node);
			}

			// must remove parentNode to remove container div from parentNode
			for (var n = 0; n < items.length; n++)
			{
				var node = items[n];
				if (node && node.parentNode)
				{
					node.parentNode.removeChild(node);// to remove node.parentNode value
				}
			}
			return items;
		}
		else if (htmlOrElement.childNodes) // dom node
		{
			return [htmlOrElement];
		}
		else if (htmlOrElement.push || htmlOrElement.length) // js array // dom array, without push and pop
		{
			return new Ice.List(htmlOrElement).SelectMany(function (o) { return Ice.DomQuery.ConvertToElementsArray(o); }).ToArray();
		}
	}
	return null;
};

Ice.DomQuery.AnimateElementIntoView = function (el, alignToBottom)
{
	/// <summary>
	/// Returns Animator instance. Must invoke .Start()
	/// </summary>
	/// <param name="durationMs" type="int">default 400ms, animation duration in ms</param>
	/// <param name="alignToBottom" optional="true" type="boolean"></param>
	if (arguments.length == 0)
	{
		durationMs = 400;
	}

	var elPosition = Ice.Utility.Dom.GetElementPagePosition(el);

	var scrollTop, scrollLeft;

	if (alignToBottom)
	{
		var viewport = Ice.Utility.Dom.GetViewportDimensions();
		scrollTop = elPosition.top + elPosition.height - viewport.height;
		scrollLeft = elPosition.left + elPosition.width - viewport.width;
	}
	else
	{
		scrollTop = elPosition.top;
		scrollLeft = elPosition.left;
	}

	// IE scrolls documentElement, other scrolls body
	// always scroll body and documentElement, FF and IE works with documentElement, Chrome with body

	return $$ice(document.body, document.documentElement)
		.ToAnimator({ "scrollTop": scrollTop, "scrollLeft": scrollLeft });
};

//#endregion

//#region FromSelector

Ice.DomQuery.FromSelector = function (iceSelector)
{
	/// <summary>
	/// operates on current document.all and select elements by iceSelector as document.all.Query(args)
	/// </summary>
	/// <param name="iceSelector" type="String|Element|Array|Ice.Enumerable">
	///		lambda or css iceSelector:
	///		<para>TAG name: html, body...</para>
	///		<para>ID: #idvalue...</para>
	///		<para>CLASS: .mydiv, a.mydiv...;</para>
	///		<para>ATTRIBUTES: [name] (contains attribute), [name=value], [name &gt; value], ==, =, &lt;, &gt;, !=, &lt;&gt;, [name*=value] (contains value), [name^=value] (starts with value), [name$=value] (ends with value)...</para>
	///		<para>TRAVERSING: * (all), space or // (descendants), / (children), ^ (ancestors), &gt;&gt;&gt; (n-th level children), &lt;&lt; (n-th level parent) :first(n) :last(n) :skip(n) :take(n) :reverse(), with or without (n), if there is no (n), default n == 1</para>
	///		<para>Element state: :hidden, :visible</para>
	///		<para>from x-path: :ancestor :descendant :child :parent :next-siblings :previous-siblings</para>
	/// </param>

	//#if VSINTELLISENSE
	if (Ice.IsVSIntellisene)
	{
		return new Ice.DomQuery([document.createElement("div")]);
	}
	//#endif

	// flattern arguments and arguments which are arrays

	if (arguments.length > 1)
	{
		var qq = new Ice.DomQuery();
		for (var n = 0; n < arguments.length; n++)
		{
			qq = qq.Concat(Ice.DomQuery.FromSelector(arguments[n]));
		}
		return qq;
	}

	if (!iceSelector)
	{
		return new Ice.DomQuery();
	}

	if (iceSelector == window || iceSelector == document || (iceSelector.nodeType > 0 && iceSelector.cloneNode)) // #text node has .cloneNode()
	{
		// window, document, any other dom element
		// warning: window on some portals may have window.length > 0 property, so test if it is window, html or body element first
		return new Ice.DomQuery([iceSelector]);
	}
	else if (iceSelector == "html")
	{
		return new Ice.DomQuery([document.documentElement]);
	}
	else if (iceSelector == "body")
	{
		return new Ice.DomQuery([document.body]);
	}
	else if (iceSelector._typeName == "Ice.DomQuery")
	{
		return iceSelector;
	}
	else if (iceSelector.GetEnumerator || (iceSelector._source && iceSelector._source.GetEnumerator))
	{
		return new Ice.DomQuery(iceSelector);
	}
	else if (iceSelector.childNodes) // single dom element
	{
		return new Ice.DomQuery([iceSelector]);
	}
	else if (typeof (iceSelector) == "string" && iceSelector.length > 0) // custom string iceSelector: "div .red #home"
	{
		return new Ice.DomQuery([document]).DescendantNodes().Query(iceSelector);
	}
	else if (iceSelector.push && iceSelector.pop) // array, real
	{
		var qq = new Ice.DomQuery();
		for (var n = 0; n < iceSelector.length; n++)
		{
			qq = qq.Concat(Ice.DomQuery.FromSelector(iceSelector[n]));
		}
		return qq;
	}
	else if (iceSelector && iceSelector.length > 0 && iceSelector[0] && iceSelector[0].nodeType > 0) // dom array
	{
		var qq = new Ice.DomQuery();
		for (var n = 0; n < iceSelector.length; n++)
		{
			qq = qq.Concat(Ice.DomQuery.FromSelector(iceSelector[n]));
		}
		return qq;
	}
	else if (iceSelector)
	{
		// if selector is window, document etc.
		return new Ice.DomQuery([iceSelector]);
	}
	else
	{
		Ice.Diagnostic.Exception("Unknown iceSelector in GetDomNodesEnumerable");
	}
	////// no selector, return new instance
	////return new Ice.DomQuery(); // required for VS intellisense to work
};

//#endregion

//#region FromXml

Ice.DomQuery.FromXml = (function ()
{
	function ParseXmlString(xmlDocStr)
	{
		var xmlDoc;
		if (window.DOMParser)
		{
			var parser = new DOMParser();
			xmlDoc = parser.parseFromString(xmlDocStr, "text/xml");
		}
		else
		{
			// IE :(
			if (xmlDocStr.indexOf("<?") == 0)
			{
				xmlDocStr = xmlDocStr.substr(xmlDocStr.indexOf("?>") + 2);
			}
			xmlDoc = new ActiveXObject("Microsoft.XMLDOM");
			xmlDoc.async = false;
			xmlDoc.loadXML(xmlDocStr);
		}
		return xmlDoc;
	}

	return (function (xmlString)
	{
		/// <summary>
		/// creates source from xmlString
		/// </summary>
		/// <param name="xmlString" type="string"></param>

		var xmlDocument = ParseXmlString(xmlString);
		var docEl = xmlDocument.documentElement; // remove #root node
		return new Ice.DomQuery([docEl]);
	});
})();


//#endregion

//#region FromHtml

Ice.DomQuery.FromHtml = function (htmlString)
{
	/// <summary>
	/// creates from html string
	/// </summary>
	/// <param name="htmlString"></param>

	var arr = Ice.DomQuery.ConvertToElementsArray(htmlString);
	return new Ice.DomQuery(arr);
};

//#endregion

//#region FromJsObject

Ice.DomQuery.FromJsObject = (function ()
{
	function CreateNewNode()
	{
		return {
			tagName: null,
			className: null,
			childNodes: [],
			id: null,
			parentNode: null,
			nodeName: null,
			nodeValue: null
		};
	}

	function IsEnumerable(obj)
	{
		try
		{
			for (var p in obj)
			{
				return true;
			}
		}
		catch (e)
		{
		}
		return false;
	}

	function AddValue(parent, o)
	{
		var node = CreateNewNode();
		node.nodeValue = o;
		parent.childNodes.push(node);
	}

	function DumpArray(parent, o)
	{
		var n1 = CreateNewNode();
		for (var n = 0; n < o.length; n++)
		{
			try
			{
				Dump(n1, o[n]);
			}
			catch (ex)
			{
			}
		}
		parent.childNodes.push(n1);
	}

	function DumpObject(parent, o)
	{
		try
		{
			for (var prop in o)
			{
				var v1 = o[prop];
				var node = CreateNewNode();
				node.tagName = prop;
				parent.childNodes.push(node);
				try
				{
					Dump(node, v1);
				}
				catch (ex1)
				{
					node.nodeValue = "*** ERROR_PARSING ***";
				}
			}
		}
		catch (ex)
		{
		}
	}

	function Dump(parent, o)
	{
		if (o == null)
		{
			AddValue(parent, null);
		}
		else if (typeof (o) == "function")
		{
			return;
		}
		else if (o && (typeof (o) == "string" || typeof (o) == "number" || typeof (o) == "boolean"))
		{
			AddValue(parent, o);
		}
		else if (o && o.push && o.pop && o.length) // JS or DOM array
		{
			DumpArray(parent, o);
		}
		else if (IsEnumerable(o))
		{
			DumpObject(parent, o);
		}
		else
		{
			AddValue(parent, o);
		}
	}

	return (function (jsObject)
	{
		/// <summary>
		/// parse js object
		/// </summary>
		/// <param name="jsObject"></param>

		var root = CreateNewNode();
		Dump(root, jsObject);
		return new Ice.DomQuery([root]);
	});

})();

//#endregion

//#region HtmlElement

Ice.DomQuery.HtmlElement = function (tagName)
{
	return new Ice.DomQuery([document.createElement(tagName)]);
};

//#endregion


// global variable:
var $$ice = Ice.DomQuery.FromSelector;

