﻿/// <reference path="Ice.js" />
/// <reference path="Ice.Collections.js" />

Ice.Enumerable = function (source)
{
	/// <summary>
	/// constructor
	/// </summary>
	/// <param name="source">array, string, enumerable</param>
	source = source || [];
	if (source.GetEnumerator)
	{
		this._source = source;
	}
	else if (source.length > 0 || source.length === 0)
	{
		this._source = source; // string or array or dom array

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			// msie 8 dom array doesn't allow custom properties, so, convert it into real array
			this._source._elementInstance = this._source[0];
		}
		//#endif
	}
	else
	{
		// unknown source, source must be an array or enumerable
		Ice.Diagnostic.Exception("Invalid source in new Ice.Enumerable(source). Source must be an array or enumerable, source: " + source);
	}
	this._predicate = "new Ice.Enumerable(" + (source || "") + ")";
};

Ice.Enumerable.__class = true;

Ice.Enumerable.prototype = (function ()
{
	var instance = {};

	instance._predicate = null; // info predicate

	instance._source = null; // source must have GetEnumerator method

	instance._typeCtor = Ice.Enumerable;

	instance._typeName = "Ice.Enumerable";

	instance.GetEnumerator = function ()
	{
		if (this._source.GetEnumerator)
		{
			return this._source.GetEnumerator(); // source must be enumerable
		}
		else
		{
			return Ice.Collections.CreateEnumerator(this._source);
		}
	};

	instance.GetYielder = function ()
	{
		// yielder and enumerator are the same, both have : bool MoveNext(), object Current() and bool YieldNext({YieldValue:...})
		return this.GetEnumerator();
	};

	instance._setPredicateInfo = function (methodName, methodArgs)
	{
		// has to create _predicate: in case of exception we have to generate call stack and query text info
		if (/*Ice.IsDebugDefined &&*/ methodName)
		{
			var args = [];
			// methodArgs is not real array, create real array from it:
			for (var n = 0; methodArgs && n < methodArgs.length; n++)
			{
				args.push(methodArgs[n]);
			}

			this._predicate = methodName + "(" + (args || []).join(", ") + ")";
		}

		return this;
	};

	instance._newEnumerable = function (enumeratorCtor)
	{
		var gg = new this._typeCtor(this);
		gg._typeName = this._typeName;

		if (enumeratorCtor)
		{
			// override default GetEnumerator()
			gg.GetEnumerator = function ()
			{
				return new enumeratorCtor(gg._source);
			};
		}

		if (!gg._source._elementInstance)
		{
			gg._source._elementInstance = this._source._elementInstance;
		}

		return gg;
	};

	instance.AsEnumerable = function ()
	{
		return this._newEnumerable();
	};

	instance.ToArray = function ()
	{
		/// <summary>Gets the underlieing Array object that holds the data.</summary>
		/// <returns type="Array"></returns>

		if (this._source.slice && this._source.push)
		{
			// make sure it is array, string has .slice(), but don't have push and .pop()
			// if source already is array:
			return this._source.slice(0);
		}

		var ee = this.GetEnumerator();
		var arr = new Array();
		var yldobj = new Ice.Collections.Yield();
		while (ee.YieldNext(yldobj))
		{
			arr.push(yldobj.YieldValue);
		}
		return arr;
	};

	instance.ToList = function ()
	{
		/// <summary>
		/// Export current collection to Ice.List
		/// </summary>
		
		return new Ice.List(this.ToArray());
	};

	instance.ToStringJoined = function (separator)
	{
		/// <summary>
		/// Join elements to string, separated with separator.
		/// </summary>
		/// <param name="separator" type="String" optional="true"></param>

		if (this._source.join)
		{
			return this._source.join(separator || "");
		}
		else
		{
			return this.ToArray().join(separator || "");
		}
	};

	instance.ToJson = function (useStringify)
	{
		/// <summary>export as .Net JSON string, with MS .Net formated Date(...)</summary>
		/// <param name="useStringify" type="Boolean" optional="true">if true, export JSON string, with JSON.stringify(...); else if false, export as .Net JSON string, with MS .Net formated Date(...)</param>
		if (useStringify && JSON && JSON.stringify)
		{
			return JSON.stringify(this.ToArray());
		}
		else
		{
			return Ice.JSON.Serialize(this.ToArray());
		}
	};

	instance.ToPrettyJson = function (asHtml, recursionLimit)
	{
		/// <summary>export to pretty JSON string for printing</summary>
		/// <param name="asHtml" type="Boolean">if true, format as html</param>
		/// <param name="recursionLimit" type="Number">recusion limit, default is 100</param>
		return Ice.Diagnostic.Dump(this.ToArray(), asHtml, recursionLimit);
	};

	instance.ToDataTable = function ()
	{
		var arr = this.ToArray();
		var dt = new Ice.DataTable(arr);
		return dt;
	};

	instance.ToDictionary = function (keySelector, itemSelector)
	{
		/// <summary>
		/// Create {key: value, key1: value1}... If keys are not unique, will be overwriten.
		/// </summary>
		/// <param name="keySelector" type="Function|Lambda">function(item){....}</param>
		/// <param name="itemSelector" optional="true" type="Function|Lambda">function(item){...}</param>
		
		var keyFunc = keySelector || function (o) { return o; };
		var itemFunc = itemSelector || function (o) { return o; };
		var dic = {};

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			dic[Ice.Utility.Expressions.CompileLambdaExpression(keyFunc)(this._source._elementInstance)] = Ice.Utility.Expressions.CompileLambdaExpression(itemFunc)(this._source._elementInstance);
			return dic;
		}
		//#endif

		this.ForEach(function (item, index)
		{
			dic[Ice.Utility.Expressions.CompileLambdaExpression(keyFunc)(item)] = Ice.Utility.Expressions.CompileLambdaExpression(itemFunc)(item);
		});

		return dic;
	};

	////instance.ToIceConsole = function ()
	////{
	////	/// <summary>write to Ice.Console as pretty JSON string for printing, html formated</summary>
	////	/// <param name="options">console options: {width: n1, height: n2, top:n3, left:n4, right:n5, bottom:n6}</param>
	////	Ice.Console.WriteLine(this.ToPrettyJson(true));
	////};

	instance.ToConsole = function ()
	{
		/// <summary>write to browser's console.log(...), as pretty JSON string for printing, html formated</summary>
		/// <param name="options">console options: {width: n1, height: n2, top:n3, left:n4, right:n5, bottom:n6}</param>
		if (console & console.log)
		{
			console.log(this.ToJson(false));
		}
	};

	instance.ToQueryText = function ()
	{
		/// <summary>
		/// Dump query text for debugging.
		/// </summary>

		var query = this._predicate;
		var src = this;
		while (src = src._source)
		{
			if (src._predicate)
			{
				query = src._predicate + "\r\n" + "." + query;
			}
			if (src.iceSelector)
			{
				query += "/* " + src.iceSelector + " */";
			}
		}
		return query;
	};

	instance.ForEach = function (action)
	{
		/// <summary>Cache current items. Than invoke predicate(item, index) on each element. Returns new linq enumerable with cached items.</summary>
		/// <param name="action" type="String|Function">(item, index)=>...., lambda expression or function predicate executed on each element.</param>

		if (!action)
		{
			Ice.Diagnostic.Exception("ForEach(action) requires action");
		}

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			Ice.Utility.Expressions.CompileLambdaExpression(action)(this._source._elementInstance, 1);
			return this;
		}
		//#endif

		var cb = Ice.Utility.Expressions.CompileLambdaExpression(action);

		var arr = this.ToArray();

		for (var n = 0; n < arr.length; n++)
		{
			cb(arr[n], n); // don't use if return === false because it will break execution in chrome (return is false by default)
		}

		return new this._typeCtor(arr)._setPredicateInfo("ForEach", arguments);
	};

	instance.ForEachAsync = function (action, onComplete)
	{
		/// <summary>
		/// execute ForEach in new thread
		/// </summary>
		/// <param name="action" type="String|Function">(item, index)=>...., lambda expression or function predicate executed on each element.</param>
		/// <param name="onComplete" type="Function">(this)=>... callback on complete</param>

		if (!action)
		{
			Ice.Diagnostic.Exception("ForEachAsync(action) requires action");
		}

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			Ice.Utility.Expressions.CompileLambdaExpression(action)(this._source._elementInstance, 1);
			return this;
		}
		//#endif

		// there is not thread semaphore in JS, we must create complete collection immediately
		var arr = this.ToArray();
		var cb = Ice.Utility.Expressions.CompileLambdaExpression(action);

		setTimeout(function ()
		{
			for (var n = 0; n < arr.length; n++)
			{
				if (cb(arr[n]) === false)
				{
					break;
				}
			}
			if (onComplete)
			{
				var cc = new this._typeCtor(arrlist)._setPredicateInfo("ForEachAsync", arguments);
				onComplete.call(cc, cc);
			}
		} .BindContext(this), 0);

		return new this._typeCtor(arrlist)._setPredicateInfo("ForEachAsync", arguments);
	};

	instance.Where = function (predicate)
	{
		/// <summary>Filters a sequence of values based on a predicate(item, index) predicate.</summary>
		/// <param name="predicate" type="String|Function">(item, index)=>... lambda expression or function predicate used to determine query matches.</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			Ice.Utility.Expressions.CompileLambdaExpression(predicate)(this._source._elementInstance, 1);
			return this;
		}
		//#endif

		return this._newEnumerable(function (source)
		{
			var srcenum = null;
			var pred;
			var index;

			return Ice.Collections.CreateEnumerator(
			function ()
			{
				index = -1;
				srcenum = source.GetEnumerator();
				pred = Ice.Utility.Expressions.CompileLambdaExpression(predicate);
			},
			function (yldobj)
			{
				while (srcenum.YieldNext(yldobj))
				{
					if (Ice.IsVSIntellisene || pred(yldobj.YieldValue, ++index)) // if intellisense, ignore predicate
					{
						return true;
					}
				}
				return false;
			});

		})._setPredicateInfo("Where", arguments);
	};


	instance.Select = function (selector)
	{
		/// <summary>Projects each element of a sequence into a new form.</summary>
		/// <param name="selector" type="String|Function">(item, index)=>... lambda expression or function predicate used to determine what values to select. Compatible with MS LINQ</param>

		if (!selector)
		{
			Ice.Diagnostic.Exception("Select(selector) requires selector");
		}

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			return new this._typeCtor([Ice.Utility.Expressions.CompileLambdaExpression(selector)(this._source._elementInstance, 1)]);
		}
		//#endif

		return this._newEnumerable(function (source)
		{
			var srcenum = null;
			var index;
			var sel;

			return Ice.Collections.CreateEnumerator(
			function ()
			{
				index = -1;
				srcenum = source.GetEnumerator();
				sel = Ice.Utility.Expressions.CompileLambdaExpression(selector);
			},
			function (yldobj)
			{
				while (srcenum.YieldNext(yldobj))
				{
					yldobj.YieldValue = sel(yldobj.YieldValue, ++index);
					return true;
				}
				return false;
			});

		})._setPredicateInfo("Select", arguments);
	};

	instance.Count = function (predicate)
	{
		/// <summary>Returns the number of elements in a sequence.</summary>
		/// <param name="predicate">lambda expression or function predicate used to determine what values to count.</param>
		/// <returns type="Number"></returns>

		if (predicate)
		{
			return this.Where(predicate).ToArray().length;
		}

		return this.ToArray().length;
	};

	instance.Distinct = function (keySelector)
	{
		/// <summary>Returns distinct by keySelector elements from a sequence.</summary>
		/// <param name="keySelector">function(item){...} lambda expression or function keySelector used to determine what values to select.</param>

		var keyFunc = keySelector || function (o) { return o; };

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			Ice.Utility.Expressions.CompileLambdaExpression(keyFunc)(this._source._elementInstance);
			return this;
		}
		//#endif

		return this._newEnumerable(function (source)
		{
			var srcenum = null;
			var dict = null;
			var pred;

			return Ice.Collections.CreateEnumerator(
			function ()
			{
				srcenum = source.GetEnumerator();
				// must use [], object {} can not be used because : var kk = {}; var k1 = {}; var k2 = {}; kk[k1] is the same as kk[k2]
				dict = []; // new Ice.ArrayList();
				pred = Ice.Utility.Expressions.CompileLambdaExpression(keyFunc);
			},
			function (yldobj)
			{
				while (srcenum.YieldNext(yldobj))
				{
					var key = pred(yldobj.YieldValue);
					if (dict.indexOf(key) == -1)
					{
						dict.push(key);
						return true;
					}
				}
				return false;
			});

		})._setPredicateInfo("Distinct", arguments);
	};

	instance.Skip = function (count)
	{
		/// <summary>Skip count elements from begining</summary>

		if (!(count >= 0))
		{
			Ice.Diagnostic.Exception("Skip(count) requires count parameter >= 0");
		}

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			return this;
		}
		//#endif

		return this._newEnumerable(function (source)
		{
			var srcenum = null;
			var index;

			return Ice.Collections.CreateEnumerator(
			function ()
			{
				srcenum = source.GetEnumerator();
				index = -1;
			},
			function (yldobj)
			{
				while (srcenum.YieldNext(yldobj))
				{
					if (++index >= count)
					{
						return true;
					}
				}
				return false;
			});

		})._setPredicateInfo("Skip", arguments);
	};

	instance.Take = function (count)
	{
		/// <summary>Take count elements from begining</summary>
		if (!(count >= 0))
		{
			Ice.Diagnostic.Exception("Take(count) requires count parameter >= 0");
		}

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			return this;
		}
		//#endif

		return this._newEnumerable(function (source)
		{
			var srcenum = null;
			var index;

			return Ice.Collections.CreateEnumerator(
			function ()
			{
				srcenum = source.GetEnumerator();
				index = -1;
			},
			function (yldobj)
			{
				while (srcenum.YieldNext(yldobj))
				{
					if (++index < count)
					{
						return true;
					}
					return false;
				}
				return false;
			});

		})._setPredicateInfo("Take", arguments);
	};

	instance.Any = function (predicate)
	{
		/// <summary>Determines whether any element of a sequence exists or satisfies a condition.</summary>
		/// <param name="predicate">lambda expression or function predicate used to determine if a match exists.</param>
		/// <returns type="Boolean"></returns>

		return this.Where(predicate).Take(1).Count() > 0; // if there is at least 1 element
	};

	instance.All = function (predicate)
	{
		/// <summary>Determines whether all elements of a sequence satisfy a condition.</summary>
		/// <param name="predicate">lambda expression or function predicate used to determine if a match exists.</param>
		/// <returns type="Boolean">true if every element of the source sequence passes the test in the specified predicate predicate, or if the sequence is empty; otherwise, false.</returns>

		if (!predicate)
		{
			Ice.Diagnostic.Exception("All(predicate) requires predicate");
		}
		var pred = Ice.Utility.Expressions.CompileLambdaExpression(predicate);
		// test if any eleement does not satisfy the condition
		return this.Where(function (o) { return !pred(o); }).Take(1).Count() == 0;
	};

	instance.Reverse = function ()
	{
		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			return this;
		}
		//#endif

		/// <summary>Inverts the order of the elements in a sequence.</summary>
		return this._newEnumerable(function (source)
		{
			var buffer = null;
			var index;

			return Ice.Collections.CreateEnumerator(function ()
			{
				index = -1;
				buffer = source.ToArray();
				buffer.reverse(); // in-place reverse
			},
			function (yldobj)
			{
				if (++index < buffer.length)
				{
					yldobj.YieldValue = buffer[index];
					return true;
				}

				return false;
			});

		})._setPredicateInfo("Reverse", arguments);
	};

	instance.FirstOrDefault = function (defaultValue)
	{
		/// <summary>Returns the first element of a sequence or null.</summary>
		/// <param name="predicate">The predicate used to determine which group of elements to return the first element from.</param>
		/// <returns type="Object"></returns>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			return this._source._elementInstance;
		}
		//#endif

		var el = this.Take(1).ToArray();
		return el.length == 1 ? el[0] : defaultValue;
	};

	instance.First = function ()
	{
		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			return this._source._elementInstance;
		}
		//#endif

		var el = this.Take(1).ToArray();
		if (el.length == 1)
		{
			return el[0];
		}
		else
		{
			Ice.Diagnostic.Exception("Error in .First(), sequence contains no elements: " + "\r\n" + this.ToQueryText() + "\r\n" + ".First()");
		}
	};

	instance.Single = function ()
	{
		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			return this._source._elementInstance;
		}
		//#endif

		var el = this.Take(2).ToArray();
		if (el.length == 1)
		{
			return el[0];
		}
		else if (el.length == 0)
		{
			Ice.Diagnostic.Exception("Error in .Single(), sequence contains no elements: " + "\r\n" + this.ToQueryText() + "\r\n" + ".Single()");
		}
		else
		{
			Ice.Diagnostic.Exception("Error in .Single(), sequence contains more than single element: " + "\r\n" + this.ToQueryText() + "\r\n" + ".Single()");
		}
	};

	instance.LastOrDefault = function (defaultValue)
	{
		/// <summary>Returns the last element of a sequence or null.</summary>
		/// <returns type="Object"></returns>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			return this._source._elementInstance;
		}
		//#endif

		var el = this.ToArray();
		return el.length > 0 ? el[el.length - 1] : defaultValue;
	};

	instance.Last = function ()
	{
		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			return this._source._elementInstance;
		}
		//#endif

		var el = this.ToArray();
		if (el.length > 0)
		{
			return el[el.length - 1];
		}
		else
		{
			Ice.Diagnostic.Exception("Error in .Last(), sequence contains no elements: " + "\r\n" + this.ToQueryText() + "\r\n" + ".Last()");
		}
	};

	instance.Sum = function (selector)
	{
		/// <summary>
		/// Computes the sum of the sequence.
		/// </summary>
		/// <param name="selector">lambda selector</param>
		/// <returns type="Number|Null"></returns>
		var val = null;
		this.Select(selector).ForEach(function (o, n)
		{
			if (Ice.Type.IsNumber(o))
			{
				val = (val || 0) + o;
			}
		});
		return val;
	};

	instance.Average = function (selector)
	{
		/// <summary>
		/// Computes average of the sequence: sum / count
		/// </summary>
		/// <param name="selector">lambda selector</param>
		/// <returns type="Number|Null"></returns>
		var val = null;
		var count = 0;
		this.Select(selector).ForEach(function (o, n)
		{
			count++;
			if (Ice.Type.IsNumber(o))
			{
				val = (val || 0) + o;
			}
		});
		return val / count; // possible DevideByZero exception
	};

	instance.ElementAtOrDefault = function (index)
	{
		/// <summary>Returns the element at a specified index in a sequence or null.</summary>
		/// <param name="index">The zero-based index of the element to retrieve.</param>
		/// <returns type="Object"></returns>
		return this.Skip(index).FirstOrDefault();
	};

	instance.ElementAt = function (index)
	{
		var el = this.Skip(index).Take(1).ToArray();
		if (el.length == 1)
		{
			return el[0];
		}
		else
		{
			Ice.Diagnostic.Exception("Sequence contains no elements: " + "\r\n" + this.ToQueryText() + "\r\n" + ".ElementAt()");
		}
	};

	instance.Concat = function (secondSource)
	{
		/// <summary>Concatenates two sequences. (Is actually Idendical to the Array.concat method.)</summary>
		/// <param name="soruce2">A Ice.Linq or Array object that contains the elements to concatenate.</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			return this;
		}
		//#endif

		return this._newEnumerable(function (source)
		{
			var srcenum = null;
			var secondenum = null;

			return Ice.Collections.CreateEnumerator(
			function ()
			{
				srcenum = source.GetEnumerator();
			},
			function (yldobj)
			{
				if (secondenum == null)
				{
					while (srcenum.YieldNext(yldobj))
					{
						return true;
					}
					secondenum = new source._typeCtor(secondSource).GetEnumerator(); // than fall to the next while
				}

				while (secondenum.YieldNext(yldobj))
				{
					return true;
				}

				return false;
			});

		})._setPredicateInfo("Concat", arguments);
	};

	instance.SelectMany = function (collectionSelector)
	{
		/// <summary>
		/// flatterns collections into single collection eg. collectionSelector: o=>o.childNodes
		/// </summary>
		/// <param name="collectionSelector">function(item){return item.collection}</param>

		var collSel = collectionSelector || function (o) { return o; };

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			var item = Ice.Utility.Expressions.CompileLambdaExpression(collectionSelector)(this._source._elementInstance);
			return new this._typeCtor([item[0] || (item.FirstOrDefault ? item.FirstOrDefault() : {})]); // when GroupBy is used before, item is collection
		}
		//#endif

		return this._newEnumerable(function (source)
		{
			var srcenum = null;
			var src2 = null;
			var sel;

			return Ice.Collections.CreateEnumerator(
			function ()
			{
				srcenum = source.GetEnumerator();
				sel = Ice.Utility.Expressions.CompileLambdaExpression(collSel);
			},
			function (yldobj)
			{
				while (true)
				{
					if (src2 != null && (src2.YieldNext(yldobj)))
					{
						return true;
					}

					src2 = null;

					if (srcenum.YieldNext(yldobj))
					{
						var coll = sel(yldobj.YieldValue);
						if (coll)
						{
							if (coll.GetEnumerator)
							{
								src2 = coll.GetEnumerator();
								// continue to next while
							}
							else if (coll.length > 0)
							{
								src2 = new source._typeCtor(coll).GetEnumerator(); // create enumerable from srcenum current
								// continue to next while
							}
						}
						// else // returned [] or null or false, run next while
					}
					else
					{
						return false; // no more data
					}
				}
			});

		})._setPredicateInfo("SelectMany", arguments);
	};

	instance.Case = function (condition, onTrue, onFalse)
	{
		/// <summary>Excecutes onTrue.call(this, this) if condition is true, else executes onFalse.call(this, this)</summary>

		var src;

		if (condition && onTrue)
		{
			src = onTrue.call(this, this);
		}
		else if (!condition && onFalse)
		{
			src = onFalse.call(this, this);
		}
		else
		{
			src = this._newEnumerable();
		}

		return src._setPredicateInfo("Case", arguments);
	};

	instance.Intersect = function (secondSource, predicate, excludeIntersection)
	{
		/// <summary>Produces the set intersection of two sequences where predicate(item1, item2){....} returns true. Returns items from first source if predicate returns true.</summary>
		/// <param name="secondSource">The second Ice.Linq element or array sequence to perform the Intersect on.</param>
		/// <param name="predicate">lambda expression or function (item1, item2)=>....</param>
		/// <param name="excludeIntersection" optional="true">default: false</param>

		if (!secondSource || !predicate)
		{
			Ice.Diagnostic.Exception("Intersect(secondSource, predicate) requires secondSource and predicate parameters");
		}

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			Ice.Utility.Expressions.CompileLambdaExpression(predicate)(this._source._elementInstance, secondSource.FirstOrDefault ? secondSource.FirstOrDefault : secondSource[0]);
			return this;
		}
		//#endif

		var exclude = arguments.length == 3 && excludeIntersection;

		var predicateMethod = Ice.Utility.Expressions.CompileLambdaExpression(predicate);
		var second = new this._typeCtor(secondSource);

		return this.Where(function (f) { return exclude != second.Any(function (s) { return predicateMethod(f, s); }); })
			._setPredicateInfo("Intersect", arguments);
	};

	instance.Except = function (secondSource, predicate)
	{
		/// <summary>Exclude items from first sequence where predicate(item1, item2){....} returns true. Returns non excluded items from first source.</summary>
		/// <param name="secondSource">The second Ice.Linq element or array sequence to perform the Intersect on.</param>
		/// <param name="predicate">lambda expression or function (item1, item2)=>....</param>
		
		return this.Intersect(secondSource, predicate, true);
	};

	instance.TakeWhile = function (predicate)
	{
		///<summary>Returns elements from a sequence as long as a specified condition is true</summary>
		/// <param name="predicate" type="String|Function">(item)=>... lambda expression or function predicate used to determine query matches.</param>

		if (!predicate)
		{
			Ice.Diagnostic.Exception("TakeWhile(predicate) requires predicate");
		}

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			Ice.Utility.Expressions.CompileLambdaExpression(predicate)(this._source._elementInstance);
			return this;
		}
		//#endif

		return this._newEnumerable(function (source)
		{
			var srcenum = null;
			var pred;

			return Ice.Collections.CreateEnumerator(
			function ()
			{
				srcenum = source.GetEnumerator();
				pred = Ice.Utility.Expressions.CompileLambdaExpression(predicate);
			},
			function (yldobj)
			{
				while (Ice.IsVSIntellisene || srcenum.YieldNext(yldobj) && pred(yldobj.YieldValue))
				{
					return true;
				}
				return false;
			});

		})._setPredicateInfo("TakeWhile", arguments);
	};

	instance.SkipWhile = function (predicate)
	{
		///<summary>skip elements from a sequence as long as a specified condition is true, elements after that are returned</summary>
		/// <param name="predicate" type="String|Function">(item)=>... lambda expression or function predicate used to determine query matches.</param>

		if (!predicate)
		{
			Ice.Diagnostic.Exception("SkipWhile(predicate) requires predicate");
		}

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			Ice.Utility.Expressions.CompileLambdaExpression(predicate)(this._source._elementInstance);
			return this;
		}
		//#endif

		return this._newEnumerable(function (source)
		{
			var srcenum = null;
			var takeRest = Ice.IsVSIntellisene; // if intellisense, take all
			var pred;

			return Ice.Collections.CreateEnumerator(
			function ()
			{
				srcenum = source.GetEnumerator();
				pred = Ice.Utility.Expressions.CompileLambdaExpression(predicate);
			},
			function (yldobj)
			{
				while (srcenum.YieldNext(yldobj))
				{
					if (takeRest)
					{
						return true;
					}
					else if (!pred(yldobj.YieldValue))
					{
						takeRest = true;
						return true; // return current and all the rest
					}
				}

				return false;
			});

		})._setPredicateInfo("SkipWhile", arguments);
	};

	instance.GroupBy = function (keySelector, elementSelector)
	{
		if (!keySelector)
		{
			Ice.Diagnostic.Exception("GroupBy(keySelector) requires keySelector");
		}

		var elSelector = elementSelector || function (o) { return o; };

		var keyfunc = Ice.Utility.Expressions.CompileLambdaExpression(keySelector);
		var selectorfunc = Ice.Utility.Expressions.CompileLambdaExpression(elSelector);

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			var key = Ice.Utility.Expressions.CompileLambdaExpression(keySelector)(this._source._elementInstance);
			var item = Ice.Utility.Expressions.CompileLambdaExpression(elementSelector)(this._source._elementInstance);
			// group with key and collection of items
			var group = new this._typeCtor([item]);
			group.Key = key;
			// return collection of groups
			return new this._typeCtor([group, group]);
		}
		//#endif

		var src = this.Select(keyfunc).Distinct().Select(function (key)
		{
			var src2 = this.Where(function (m) { return keyfunc(m) == key; }).Select(selectorfunc);
			src2.Key = key; // Key required for ThenBy and for selftest
			src2.KeySelector = keySelector; // for selftest
			src2.ElementSelector = elSelector;
			src2.SubGroups = null;
			return src2;
		} .BindContext(this));

		src._predicate = "GroupBy(" + keySelector + ")";

		src.IsGroupBy = true; // used in ThenBy

		// disable ThenBy after group by, it doesn't make sense
		////src.ThenBy = function (keySelector) { return ThenByImpl.call(src, keySelector); };
		////src.ToGroupsArray = src.ToArray; // both returns groups
		return src._setPredicateInfo("GroupBy", arguments);
	};

	function SimpleOrderByEnumeratorCtor(source, selectorFunc, descending)
	{
		var enum1 = null;

		return Ice.Collections.CreateEnumerator(
		function ()
		{
			var buffer = source.ToArray();

			buffer.sort(function (a, b)
			{
				var x = selectorFunc(a);
				var y = selectorFunc(b);
				return x < y ? -1 : x > y ? 1 : 0;
			});

			if (descending)
			{
				buffer.reverse();
			}

			enum1 = Ice.Collections.CreateEnumerator(buffer); // because this is enumeratorCtor, so it must return enumerator
		},
		function (yldobj)
		{
			return enum1.YieldNext(yldobj);
		});
	}

	instance.OrderBy = function (keySelector, descending)
	{
		if (!keySelector)
		{
			Ice.Diagnostic.Exception("OrderBy(keySelector) requires keySelector");
		}

		var keyfunc = Ice.Utility.Expressions.CompileLambdaExpression(keySelector);

		// simple order without grouping is much faster, so use simple order by every time if there is no ThenBy after
		var simple = this._newEnumerable(function (source)
		{
			return new SimpleOrderByEnumeratorCtor(source, function (m) { return keyfunc(m); }, descending);
		},
		"OrderBy(" + keySelector + ", " + (descending ? "desc" : "asc") + ")");

		// grouping before ordering has to be used only if ThenBy is used after OrderBy. GroupedSrc will be prepared here, but it will be used only insite of ThenBy
		var groupsSrc = this.GroupBy(keyfunc)._newEnumerable(function (source)
		{
			return new SimpleOrderByEnumeratorCtor(source, function (m) { return m.Key; }, descending);
		},
		"GroupedOrderBy(" + keySelector + ", " + (descending ? "desc" : "asc") + ")");

		var srcFlat = groupsSrc.SelectMany(function (group)
		{
			// returns flattern and ordered collection
			return new instance._typeCtor(GetLeafGroups(group)).SelectMany(function (o) { return o; });
		});

		srcFlat.ToGroupsArray = function ()
		{
			// returns main groups
			return groupsSrc.ToArray();
		};

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			srcFlat._source._elementInstance = this._source._elementInstance;
		}
		//#endif

		//simple.ToArray() returnes flattern collection of items, while ToGroupsArray() returnes groups. GroupBy.ToArray() returns groups just like ToGroupsArray()
		simple.IsOrderBy = true;
		simple.ToGroupsArray = srcFlat.ToGroupsArray; // required in selftest and ThenBy
		simple.ThenBy = function (keySelector, descending)
		{
			return ThenByImpl.call(srcFlat, keySelector, descending);
		};
		simple.ThenByDescending = function (keySelector)
		{
			return ThenByImpl.call(srcFlat, keySelector, true);
		};

		return simple._setPredicateInfo("OrderBy", arguments);
	};

	instance.OrderByDescending = function (keySelector)
	{
		/// <summary>Sorts the elements of a sequence in descending order.</summary>
		/// <param name="keySelector">lambda expression or function keySelector used to determine how to order the data.</param>
		return this.OrderBy(keySelector, true)._setPredicateInfo("OrderByDescending", arguments);
	};

	function GetLeafGroups(group, arr)
	{
		var leafArr = arr || [];

		if (group.SubGroups)
		{
			for (var n = 0; n < group.SubGroups.length; n++)
			{
				GetLeafGroups(group.SubGroups[n], leafArr);
			}
		}
		else
		{
			leafArr.push(group);
		}

		return leafArr;
	}

	function ThenByImpl(keySelector, descending)
	{
		if (!keySelector)
		{
			Ice.Diagnostic.Exception("ThenBy(keySelector) requires keySelector");
		}

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			Ice.Utility.Expressions.CompileLambdaExpression(keySelector)(this._source._elementInstance);
		}
		//#endif

		var groupsSrc = this._newEnumerable(function (source)
		{
			var enum1;

			return Ice.Collections.CreateEnumerator(
			function ()
			{
				var mainGroups = source.ToGroupsArray();

				for (var n1 = 0; n1 < mainGroups.length; n1++)
				{
					var leafGroups = GetLeafGroups(mainGroups[n1]);
					for (var n2 = 0; n2 < leafGroups.length; n2++)
					{
						var gr = leafGroups[n2]; // group is enumerable
						gr.SubGroups = gr[source.IsGroupBy ? "GroupBy" : "OrderBy"](keySelector, descending).ToGroupsArray();
					}
				}

				enum1 = Ice.Collections.CreateEnumerator(mainGroups);
			},
			function (yldobj)
			{
				return enum1.YieldNext(yldobj);
			});

		}, "ThenBy(" + keySelector + ", " + (descending ? "desc" : "asc") + ")");

		var srcFlat = groupsSrc;

		if (this.IsGroupBy) // GroupBy
		{
			srcFlat.IsGroupBy = true;
			srcFlat.ToGroupsArray = srcFlat.ToArray;
			//#if VSINTELLISENSE
			if (Ice.IsVSIntellisene)
			{
				srcFlat._source._elementInstance = this._source._elementInstance;
			}
			//#endif
			srcFlat.ThenBy = function (keySelector) { return ThenByImpl.call(srcFlat, keySelector, false); };
		}
		else // OrderBy
		{
			srcFlat = groupsSrc.SelectMany(function (gr)
			{
				return new instance._typeCtor(GetLeafGroups(gr)).SelectMany(function (o) { return o; });
			});
			//#if VSINTELLISENSE
			if (Ice.IsVSIntellisene)
			{
				srcFlat._source._elementInstance = this._source._elementInstance;
			}
			//#endif
			srcFlat.ToGroupsArray = function () { return groupsSrc.ToArray(); };
			srcFlat.ThenBy = function (keySelector, descending) { return ThenByImpl.call(srcFlat, keySelector, descending); };
			srcFlat.ThenByDescending = function (keySelector) { return ThenByImpl.call(srcFlat, keySelector, true); };
		}

		return srcFlat;
	};

	instance.Min = function (selector)
	{
		/// <summary>Returns min value or null.</summary>
		/// <returns type="Object"></returns>
		var val = null;
		this.Select(selector).ForEach(function (o, n)
		{
			if (Ice.Type.IsNumber(o))
			{
				if (val == null || o < val)
				{
					val = o;
				}
			}
		});
		return val;
	};

	instance.Max = function (selector)
	{
		/// <summary>Returns max value or null.</summary>
		/// <returns type="Object"></returns>
		var val = null;
		this.Select(selector).ForEach(function (o, n)
		{
			if (Ice.Type.IsNumber(o))
			{
				if (val == null || o > val)
				{
					val = o;
				}
			}
		});
		return val;
	};

	instance.Cycle = function (take)
	{
		/// <summary>
		///  Cycle all elements in current collection to infinity. Must be used Take(x) to limit cycling.
		/// </summary>
		/// <param name="take" type="int">OPTIONAL, if specified, takes max 'take' items</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			return this;
		}
		//#endif

		return this._newEnumerable(function (source)
		{
			var srcenum = null;
			var hasItems;
			var count;

			return Ice.Collections.CreateEnumerator(
			function ()
			{
				hasItems = false;
				count = 0;
			},
			function (yldobj)
			{
				do
				{
					if (srcenum == null)
					{
						srcenum = source.GetEnumerator();
					}

					if (srcenum.YieldNext(yldobj))
					{
						hasItems = true;
						if (!(take >= 0) || (++count <= take))
						{
							return true;
						}
						else
						{
							return false; // no more data
						}
					}
					srcenum = null; // will start from the beginning
				}
				while (hasItems);

				return false; // no data at all
			});

		})._setPredicateInfo("Cycle", arguments);
	};



	return instance;
})();


