﻿
if (typeof (Ice) == "undefined") { Ice = {}; }

Ice.Forms = new (function(){

	var __form = {};

	//#region Serialization

	function SerializeElement(element)
	{
		/// <summary>
		/// values which browser originally send on form-submit. Returns name-value object: {Element: el, Name:name1, Value: value1}
		/// </summary>
		/// <param name="element"></param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			return {
				// dom element
				Element: element,
				// string: attribute name="..."
				Name: elementName,
				// string or array: attribute value="..." or array of values in multiselect
				Value: finalValue
			};
		}
		//#endif

		if (!(element && element.tagName && element.type && element.name))
		{
			return null; // text nodes, html nodes
		}
		var elementName = element.name;
		var finalValue = null; // output
		var tagName = element.tagName.toUpperCase();
		var type = element.type.toLowerCase();

		switch (tagName)
		{
			case "INPUT":
				switch (type)
				{
					case "text":
					case "password":
					case "hidden":
						finalValue = element.value;
						break;
					case "checkbox":
						finalValue = element.checked ? element.value : null; // if not checked, value should not be sent to server at all
						break;
					case "radio":
						finalValue = element.checked ? element.value : null; // if not checked, value should not be sent to server at all
						break;
				}
				break;

			case "TEXTAREA":
				finalValue = element.value;
				break;

			case "SELECT":
				var selectedValues = [];
				var selectCount = element.options.length;
				for (var j = 0; j < selectCount; j++)
				{
					var selectChild = element.options[j];
					if (selectChild.selected == true)
					{
						selectedValues.push(selectChild.value);
					}
				}

				if (selectedValues.length == 1)
				{
					finalValue = selectedValues[0];
				}
				else
				{
					// else if selectedValues.length > 1, keep array
					finalValue = selectedValues;
				}

				break;
		}

		// if finalValue is not set, value should not be sent to server at all

		return finalValue == null ? null : {
			// dom element
			Element: element,
			// string: attribute name="..."
			Name: elementName,
			// string or array: attribute value="..." or array of values in multiselect
			Value: finalValue
		};

	};

	__form.SerializeContainer = function (containerEl)
	{
		/// <summary>
		/// Form-serialize containerEl, returns array of Name-Value objects: [{Name:name1, Value:value1}, {Name:name2, Value:value2}, ...]
		/// </summary>
		/// <param name="containerEl"></param>
		/// <param name="nameValueDataObject"></param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			return [SerializeElement()];
		}
		//#endif

		var inps = containerEl.getElementsByTagName("input");
		var textareas = containerEl.getElementsByTagName("textarea");
		var selects = containerEl.getElementsByTagName("select");

		// can't use array.concat() because this are dom arrays, not real arrays
		var all = new Ice.List(inps).Concat(textareas).Concat(selects).ToArray();
		var finalArr = [];
		for (var n = 0; n < all.length; n++)
		{
			var result = SerializeElement(all[n]); // result is null or object
			if (result)
			{
				finalArr.push(result);
			}
		}
		return finalArr;
	};

	__form.SerializeContainerToDictionay = function (containerEl)
	{
		/// <summary>
		/// Form-serialize containerEl, returns dictionary: {name1: value1, name2: value2, ...}
		/// </summary>
		/// <param name="containerEl"></param>
		var arr = __form.SerializeContainer(containerEl);

		var dic = {};
		for (var n = 0; n < arr.length; n++)
		{
			dic[arr[n].Name] = arr[n].Value;
		}
		return dic;
	};

	//#endregion

	//#region Deserialization

	function SelectValue(selectEl, value)
	{
		for (var n = 0; n < selectEl.options.length; n++)
		{
			if (selectEl.options[n].value == value)
			{
				selectEl.options[n].selected = true;
			}
		}
	}

	function DeserializeElement(element, value)
	{
		var tagName = element.tagName.toUpperCase();
		var type = element.type.toLowerCase();

		switch (tagName)
		{
			case "INPUT":
				switch (type)
				{
					case "text":
					case "password":
					case "hidden":
						element.value = value == null ? "" : value; // remove 'null'
						break;
					case "checkbox":
					case "radio":
						element.checked = element.value == value;
						break;
				}
				break;

			case "TEXTAREA":
				element.value = value == null ? "" : value; // remove 'null'
				break;

			case "SELECT":
				// deselect all first
				for (var n = 0; n < element.options.length; n++)
				{
					element.options[n].selected = false;
				}
				switch (type)
				{
					case "select":
					case "select-one":
						// set value
						SelectValue(element, value);
						break;

					case "select-multiple":
						// select specified only
						if (value != null && value.pop) // is it array?
						{
							for (var n = 0; n < value.length; n++)
							{
								// set multiple values
								SelectValue(element, value[n]);
							}
						}
						else
						{
							// set value
							SelectValue(element, value);
						}

						break;
				}
				break;

		}
	};

	function FindElement(containerEl, tagName, name)
	{
		var elements = containerEl.getElementsByTagName(tagName);
		var namedElements = [];

		if (elements.length > 0)
		{
			for (var n = 0; n < elements.length; n++)
			{
				if (elements[n].name == name)
				{
					namedElements.push(elements[n]); // multiple elements may have the same name (radio buttons group!!!)
				}
			}
		}
		return namedElements;
	}

	__form.DeserializeContainer = function (containerEl, nameValueArray)
	{
		/// <summary>
		/// Form-deserialize nameValue object into containerEl
		/// </summary>
		/// <param name="containerEl"></param>
		/// <param name="nameValueArray">array of Name-Value objects: [{Name:name1, Value:value1}, {Name:name2, Value:value2}, ...]</param>

		for (var n = 0; n < nameValueArray.length; n++)
		{
			var name = nameValueArray[n].Name;
			var value = nameValueArray[n].Value;

			var elements = FindElement(containerEl, "input", name); // returns array of elements with name==name (radio buttons group!!!)

			if (elements.length == 0)
			{
				elements = FindElement(containerEl, "textarea", name);
				if (elements.length == 0)
				{
					elements = FindElement(containerEl, "select", name);
				}
			}

			if (elements.length > 0)
			{
				for (var index = 0; index < elements.length; index++)
				{
					DeserializeElement(elements[index], value);
				}
			}
			else
			{
				////alert("Element with name='" + prop + "' not found.");
			}
		}
	};

	//#endregion

	//#region form

	////__form.Serialize = function (containerSelector, includeMSViewState)
	////{
	////	/// <summary>
	////	/// Serialize container to dictionary.
	////	/// </summary>
	////	/// <param name="containerSelector" optional="true">if not provided, default: document</param>
	////	/// <param name="includeMSViewState" optional="true"></param>
	////	var dic = __form.SerializeContainerToDictionay($$ice(containerSelector || document).First());
	////	var d2 = {};
	////	Ice.ForEachProperties(dic, function (prop, val)
	////	{
	////		if (prop.indexOf("__EVENTVALIDATION") == 0 || prop.indexOf("__VIEWSTATE") == 0)
	////		{
	////			if(includeMSViewState)
	////			{
	////				d2[prop] = val;
	////			}
	////		}
	////		else
	////		{
	////			d2[prop] = val;
	////		}
	////	});

	////	return d2;
	////};

	__form.IceSerialize = function (containerSelector)
	{
		/// <summary>
		/// Serialize values from elements into dictionary by creating properties named as ice-id attributes.
		/// </summary>
		/// <param name="containerSelector" optional="true">if not provided, default: document</param>

		var data = {};

		// include invisible elements too
		$$ice(containerSelector || document).Query("//[ice-id=*]").ForEach(function (o)
		{
			var name = o.getAttribute("ice-id");
			if (o.type == "checkbox")
			{
				data[name] = !!o.checked;
			}
			else if (o.type == "radio")
			{
				if (o.checked)
				{
					data[name] = o.value;
				}
			}
			else
			{
				data[name] = o.value;
			}
		});

		return data;
	};

	__form.IceDeserialize = function (dataObj, containerSelector)
	{
		/// <summary>
		/// Bind data from dataObj to elements with ice-id attributes named as dataObj properties.
		/// </summary>
		/// <param name="containerSelector" optional="true">if not provided, default: document</param>

		// populate controls
		Ice.ForEachProperties(dataObj, function (pname, pvalue, index)
		{
			$$ice(containerSelector || document).Query("//[ice-id=" + pname + "]").ForEach(function (o)
			{
				if (o.type == "checkbox")
				{
					o.checked = !!pvalue;
				}
				else if (o.type == "radio")
				{
					o.checked = (o.value + "") == (pvalue + "");
				}
				else if ("value" in o)
				{
					if (pvalue == null)
					{
						pvalue = "";
					}
					o.value = pvalue + "";
					if (o.selectedIndex == -1)
					{
						o.selectedIndex = 0;
					}
				}
				else if ("innerHTML" in o)
				{
					if (pvalue == null)
					{
						pvalue = "";
					}
					o.innerHTML = pvalue + "";
				}
				else
				{
					throw "Element with ice-id=" + pname + " is not valid for data bidnding";
				}
			});
		});
	};

	//#endregion

	return __form;

})();
