﻿/// <reference path="Ice.js" />
/// <reference path="Ice.js.extensions.js" />

if (typeof (Ice) == "undefined") { Ice = {}; }

if(!Ice.JSON){Ice.JSON = {};}

Ice.JSON.Deserialize = function(jsonString)
{
	///<summary>Deserialize JSON string into object.</summary>
	///<param name="jsonString" type="String">Valid JSON string</param>
	///<returns type="Object|Array|Null">JS object</returns>
	
	if (typeof (jsonString) == "string" && !!jsonString)
	{
		////var exp = jsonString.replace(new RegExp('(^|[^\\\\])\\"\\\\/Date\\((-?[0-9]+)\\)\\\\/\\"', 'g'), "$1new Date($2)"); // Unix time (miliseconds), MS Ajax compatibility
		////var exp = jsonString.replace(new RegExp('(^|[^\\\\])\\"\\\\/IceDate\\((-?[0-9]+)\\)\\\\/\\"', 'g'), "Ice.JSON._fromServerTicks($2)"); // Unix time (miliseconds), MS Ajax compatibility
		////var exp = jsonString.replace(new RegExp('(^|[^\\\\])\\"\\\\/IceDate\\((-?[0-9]+)\\)\\\\/\\"', 'g'), "new Date($2).ToLocalTime()"); // Unix time (miliseconds), MS Ajax compatibility
		var exp = jsonString.replace(new RegExp('(^|[^\\\\])new Date\\((-?[0-9]+)\\)', 'g'), "$1new Date($2).ToLocalOffset()"); // Unix time (miliseconds), MS Ajax compatibility
		return eval('(' + exp + ')');
	}
	// will return undefined
};

Ice.JSON.Serialize = (function ()
{
	var useHasOwn = !!{}.hasOwnProperty;

	function pad(n)
	{
		return n < 10 ? "0" + n : n;
	};

	function isArray(v)
	{
		return v && typeof v.pop == 'function';
	};

	function isDate(v)
	{
		return v && typeof v.getFullYear == 'function';
	};

	var m = {
		"\b": '\\b',
		"\t": '\\t',
		"\n": '\\n',
		"\f": '\\f',
		"\r": '\\r',
		'"': '\\"',
		"\\": '\\\\'
	};

	function encodeString(s)
	{
		if (new RegExp('["\\\\\x00-\x1f]').test(s))
		{
			return '"' + s.replace(new RegExp('([\\\\\x00-\x1f\\"])', 'g'), function (a, b)
			{
				var c = m[b];
				if (c)
				{
					return c;
				}
				c = b.charCodeAt();
				return "\\u00" +
                    Math.floor(c / 16).toString(16) +
                    (c % 16).toString(16);
			}) + '"';
		}
		return '"' + s + '"';
	}

	function encodeArray(o, recursionLeft)
	{
		var a = ["["], b, i, l = o.length, v;
		for (i = 0; i < l; i += 1)
		{
			v = o[i];
			switch (typeof v)
			{
				case "undefined":
				case "function":
				case "unknown":
					break;
				default:
					if (b)
					{
						a.push(',');
					}
					a.push(v === null ? "null" : encode(v, recursionLeft - 1));
					b = true;
			}
		}
		a.push("]");
		return a.join("");
	}

	function encodeDateExt(o)
	{
		return '"' + o.getFullYear() + "-" +
                pad(o.getMonth() + 1) + "-" +
                pad(o.getDate()) + "T" +
                pad(o.getHours()) + ":" +
                pad(o.getMinutes()) + ":" +
                pad(o.getSeconds()) + '"';
	}

	function encodeDate(o)
	{
		//return '"\\/Date(' + o.getTime() + ')\\/"'; // Unix time (miliseconds), MS Ajax compatibility
		//return '"\\/IceDate(' + Ice.JSON._toServerTicks(o) + ')\\/"'; // Unix time (miliseconds), MS Ajax compatibility
		//return '"\\/IceDate(' + o.ToServerOffset().getTime() + ')\\/"'; // Unix time (miliseconds), MS Ajax compatibility
		return 'new Date(' + o.ToServerOffset().getTime() + ')'; // Unix time (miliseconds), MS Ajax compatibility
	}

	function encode(objectToSerialize, recursionLeft)
	{
		///<summary>Serialize object to JSON string.</summary>
		///<param name="objectToSerialize" type="Object|String">Object to serialize</param>
		///<returns type="String|Null">JSON string</returns>

		var o = objectToSerialize;
		if (typeof o == "undefined" || o === null)
		{
			return "null";
		} else if (isArray(o))
		{
			if (recursionLeft < 1)
			{
				return "[...]";
			}
			return encodeArray(o, recursionLeft - 1);
		}
		else if(o && o._typeName == "Ice.DateTime")
		{
			return encodeDate(o._dateItem);
		}
		else if (isDate(o))
		{
			return encodeDate(o);
		} else if (typeof o == "string")
		{
			return encodeString(o);
		} else if (typeof o == "number")
		{
			return isFinite(o) ? String(o) : "null";
		} else if (typeof o == "boolean")
		{
			return String(o);
		}
		else
		{
			if (recursionLeft < 1)
			{
				return "{...}";
			}

			var a = ["{"], b, i, v;
			for (i in o)
			{
				////if(!useHasOwn || o.hasOwnProperty(i)) 
				{
					v = o[i];
					switch (typeof v)
					{
						case "undefined":
						case "function":
						case "unknown":
							break;
						default:
							if (b)
							{
								a.push(',');
							}
							a.push(encode(i, recursionLeft - 1), ":", v === null ? "null" : encode(v, recursionLeft - 1));
							b = true;
					}
				}
			}
			a.push("}");
			return a.join("");
		}
	}

	return (function (objectToSerialize, recursionLimit)
	{
		recursionLimit = recursionLimit || 100;
		////recursionLimit++;

		return encode(objectToSerialize, recursionLimit); 
	});

})();


