﻿/// <reference path="Ice.Enumerable.js" />

Ice.List = function (source)
{
	/// <summary>
	/// constructor
	/// </summary>
	/// <param name="source">array, string, enumerable</param>

	// call base constructor
	Ice.Enumerable.call(this, source);
	
	this._predicate = "new Ice.List(" + (source || "") + ")";
};

Ice.List.__class = true;

Ice.List.prototype = (function ()
{
	var instance = new Ice.Enumerable();
	var base = Ice.Extend({}, instance);

	instance._typeCtor = Ice.List;
	instance._typeName = "Ice.List";

	////	// overriding methods: save base method reference, just method, not the instance object, than call base method using base.method.apply(this, arguments)
	////var base = {};
	////	base.Count = instance.Count;
	////	base.FirstOrDefault = instance.FirstOrDefault;
	////	base.Reverse = instance.Reverse;
	////base.ToArray = instance.ToArray;
	////instance.ToArray = function ()
	////{
	////	if (this._source.slice && this.push && this.pop)
	////	{
	////		return this._source.slice(0);
	////	}
	////	return base.ToArray.apply(this, arguments);
	////};

	instance.Add = function (value)
	{
		/// <summary>
		/// Ice.List.Add(item)
		/// </summary>
		/// <param name="value"></param>
		if (!this._source.push) { this._source = this.ToArray(); }

		this._source.push(value);
		return this;
	};

	instance.AddRange = function (arr)
	{
		/// <summary>
		/// Ice.List.AddRange(array)
		/// </summary>
		/// <param name="arr"></param>
		if (arr && arr.length > 0)
		{
			if (!this._source.push) { this._source = this.ToArray(); }
			var useCharAt = arr.charAt;
			for (var n = 0; n < arr.length; n++) // must use for(...) if adding dom array which can't be added using concat(arr)
			{
				this._source.push(useCharAt ? arr.charAt(n) : arr[n]);
			}
		}
		return this;
	};

	instance.Insert = function (index, value)
	{
		/// <summary>
		/// Ice.List.Insert(index, value)
		/// </summary>
		/// <param name="index"></param>
		/// <param name="value"></param>
		if (!this._source.splice) { this._source = this.ToArray(); }

		this._source.splice(index, 0, value);
		return this;
	};

	instance.RemoveRange = function (index, count)
	{
		/// <summary>
		/// Ice.List.RemoveRange(index, count)
		/// </summary>
		/// <param name="index"></param>
		/// <param name="count"></param>
		if (!this._source.splice) { this._source = this.ToArray(); }

		this._source.splice(index, count || 1);
		return this;
	};

	instance.RemoveAt = function (index)
	{
		/// <summary>
		/// Ice.List.RemoveAt(index)
		/// </summary>
		/// <param name="index"></param>
		return this.RemoveRange(index, 1);
	};

	instance.Remove = function (value)
	{
		/// <summary>
		/// Ice.List.Remove(value). Remove first item by value.
		/// </summary>
		/// <param name="value"></param>
		return this.RemoveAt(this.IndexOf(value));
	};

	instance.IndexOf = function (value)
	{
		/// <summary>
		/// Ice.List.IndexOf(value). Find first item with value.
		/// </summary>
		/// <param name="value"></param>
		if (!(this._source.length >= 0)) { this._source = this.ToArray(); }

		if (this._source.indexOf)
		{
			// ie8 doesn't support array.indexOf()
			// if it is array or string
			return this._source.indexOf(value);
		}

		// if it is dom array
		// var useCharAt = this._source.charAt; // this is slow as hell in firefox
		var useCharAt = typeof (this._source) == "string"; // invoke at init time, it is slow as hell if array.charAt invoked in foreach every time
		for (var n = 0; n < this._source.length; n++)
		{
			if ((useCharAt ? this._source.charAt[n] : this._source[n]) == value)
			{
				return n;
			}
		}
		return -1;
	};

	instance.Contains = function (value)
	{
		/// <summary>
		/// Ice.List.Contains(value). Returns true if contains value.
		/// </summary>
		/// <param name="value"></param>
		return this.IndexOf(value) >= 0;
	};

	instance.Clear = function ()
	{
		/// <summary>
		/// Ice.List.Clear()
		/// </summary>
		this._source = [];
		return this;
	};



	return instance;

})();

