﻿//*******************************************************************
// NO EXTERNAL DEPENDANCIES
//*******************************************************************

if (typeof (Ice) == "undefined") { Ice = {}; }

Ice.MulticastDelegate = function (defaultContext)
{
	/// <summary>
	/// New MulticastDelegate constructor.
	/// </summary>

	var _items = [];
	var _onAdd = [];

	var _this = function (callback, executionsCount)
	{
		/// <summary>
		/// Add new callback function. If callback is undefined, will not be added.
		/// </summary>
		/// <param name="callback" type="Function">function(args){...}</param>
		/// <param name="executionsCount" type="Int">remove handler after being executed executionsCount times</param>

		if (callback)
		{
			// when invoked _this(callback, executionsCount);, it has 2 arguments, second one is "undefined"
			_items.push({ cb: callback, context: _defaultContext, counted: arguments.length > 1 && executionsCount > 0, execLeft: executionsCount });

			if (_onAdd && _onAdd.length > 0)
			{
				for (var n = 0; n < _onAdd.length; n++)
				{
					_onAdd[n].call(_this);
				}
			}
		}

		return _this;
	};

	var _defaultContext = defaultContext || _this;

	_this.Add = function (func, executionsCount)
	{
		/// <summary>
		/// Add callback function.
		/// </summary>
		/// <param name="func" type="Function">function(args){...}</param>
		/// <param name="executionsCount" type="Int" optional="true">If value is set, execute n times, than remove callback method.</param>
		return _this(func, executionsCount);
	};

	_this.AddOnce = function (func)
	{
		/// <summary>
		/// Add func which will be invoked once and removed.
		/// </summary>
		/// <param name="func" type="Function">function(args){...}</param>

		return _this.Add(func, 1);
	};

	_this.OnAdd = function (callback)
	{
		/// <summary>
		/// Add callback to be executed every time when delegate is added using .Add(...) or .AddOnce(...)
		/// </summary>
		/// <param name="callback" type="Function">function(this){...}</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			callback.call(_this, _this);
		}
		//#endif

		_onAdd.push(callback);

		return _this;
	};

	_this.Invoke = function (arg0, arg1, arg2, etc)
	{
		/// <summary>
		/// Invoke with arguments.
		/// </summary>
		/// <param name="arg0"></param>
		/// <param name="arg1"></param>
		/// <param name="arg2"></param>
		/// <param name="etc"></param>

		var arr = [];

		for (var n = 0; n < _items.length; n++)
		{
			var o = _items[n];

			o.cb.apply(o.context, arguments);

			if (!o.counted || --o.execLeft > 0)
			{
				arr.push(o);
			}
		}
		
		_items = arr;

		return _this;
	};

	_this.Count = function () { return _items.length; };

	_this.Clear = function () { _items = []; return _this; };

	_this.HasItems = function () { return _items.length > 0; };

	_this.Remove = function (callback)
	{
		/// <summary>
		/// Remove by callback function.
		/// </summary>
		/// <param name="callback"></param>

		var arr = [];

		for (var n = 0; n < _items.length; n++)
		{
			var item = _items[n];
			if (item.cb != callback)
			{
				arr.push(item);
			}
			// else: item has to be removed
		}
		_items = arr;

		return _this;
	};

	return _this;
};

