﻿/// <reference path="Ice.js" />
/// <reference path="Ice.DomQuery.js" />
/// <reference path="Ice.Utility.js" />
/// <reference path="Ice.String.js.extensions.js" />
/// <reference path="Ice.DataBinder.js" />

/*#region Ice.Observable*/

Ice.Observable = function ()
{
	this._dataSource = {};
	this._listeners = [];
};

Ice.Observable.Log = function (msg)
{
	if (window && window.Ice && window.Ice.Diagnostic && Ice.IsDebugDefined)
	{
		Ice.Diagnostic.WriteLine(msg);
	}
};

Ice.Observable.Log.Enabled = false;

Ice.Observable.CreateDependenciesDic = function (propertyName)
{
	if (!propertyName)
	{
		return null;
	}
	var dic = {};
	var arr = propertyName.split('.');
	while (arr.length > 0)
	{
		dic[arr.join(".")] = true;

		arr.pop(); // remove last item
	}
	return dic;
};

Ice.Observable.prototype = new (function ()
{
	var instance = {};

	instance._dataSource = null;
	instance._listeners = null;

	//	var listenerItem = {
	//		dataProperty: null,
	//		callback: null
	//	};

	instance.SetDataSource = function (dataSource)
	{
		this._dataSource = dataSource;
		return this;
	};

	instance.GetValue = function (dataProperty)
	{
		/// <summary>
		/// get dataPropery value, if it is function, evaluate function and return value returned from function 
		/// </summary>
		/// <param name="dataProperty" type="String">data property full path</param>
		var value = Ice.DataBinder.GetValue(this._dataSource, dataProperty);
		if (typeof (value) == "function")
		{
			Ice.Diagnostic.Exception("Data property " + dataProperty + " is function, 2-way binding is not supported.")
		}
		else
		{
			return value;
		}
		////return typeof (value) == "function" ? value.call(this._dataSource) : value;
	};

	instance.SetValue = function (dataProperty, value, dontNotify)
	{
		/// <summary>
		/// set dataProperty value, if dataProperty in data model is function, value won't be set, NotifyPropertyChanged is invoked after value is set.
		/// </summary>
		/// <param name="dataProperty" type="String">data property full path</param>
		/// <param name="value"></param>
		/// <param name="dontNotify" optional="true" type="Boolean">if true, don't call NotifyPropertyChanged(...)</param>

		var cc = Ice.DataBinder.GetValue(this._dataSource, dataProperty);
		if (typeof (cc) == "function")
		{
			Ice.Diagnostic.Exception("Data property " + dataProperty + " is function, data can't be assigned");
			// updating model via function may require complete view to redraw, that is why we don't allow update via function or call NotifyPropertyChanged from method after thata is changed
			//cc.call(this, dataProperty, value);
			return;
		}
		else if(cc !== value)
		{
			// update only if new value is different
			// we may update always, doesn't matter if new value is the same as old, it is safer

			Ice.DataBinder.SetValue(this._dataSource, dataProperty, value);

			if(!dontNotify)
			{
				this.NotifyPropertyChanged(dataProperty);
			}
		}
		return this;
	};

	instance.SetValueSilent = function (dataProperty, value)
	{
		/// <summary>
		/// SetValue without calling NotifyPropertyChanged
		/// </summary>
		/// <param name="dataProperty"></param>
		/// <param name="value"></param>
		return this.SetValue(dataProperty, value, true);
	};

	instance.NotifyPropertyChanged = function(prop)
	{
		/// <summary>
		///
		/// </summary>
		/// <param name="prop">if not set, notify all listeners, required for initial binding</param>
		
		// don't use LINQ, this should be faster
		var o;
		var items = this._listeners;
		if (prop)
		{
			var arr = items;
			items = [];
			for (var n = 0; n < arr.length; n++)
			{
				if((o = arr[n]) && o.dependantDic[prop])
				{
					items.push(o);
				}
			}
		}

		/*#if DEBUG*/
		if (Ice.IsDebugDefined)
		{
			//console.log("lis: " + items.length + " : " + prop);
		}
		/*#endif*/

		for (var n = 0; n < items.length; n++)
		{
			o = items[n];

			/*#if DEBUG*/

			if (Ice.Observable.Log.Enabled)
			{
				Ice.Observable.Log("NOTIFY CHANGED: " + o.dataProperty);
			}

			/*#endif*/

			// dataSource, fullPropertyPath, value
			o.callback.call(o, this._dataSource, o.dataProperty, Ice.DataBinder.GetValue(this._dataSource, o.dataProperty));
		}

		return this;
	};

	instance.AddListener = function (dataProperty, callback)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="dataProperty" type="String|Array">full data property path: data.User.Name or array or data properties</param>
		/// <param name="callback" type="Function">function(dataModel, propertyPath, newValue){...}. this is underlaying item with additional data.</param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			var o = { dependantDic: {}, dataProperty: "Ice.Data", callback: callback };
			callback.call(o, this._dataSource || {}, o.dataProperty, {});
		}
		//#endif
		
		var arr = Ice.Type.IsArray(dataProperty) ? dataProperty : [dataProperty];
		for (var n = 0; n < arr.length; n++)
		{
			this._listeners.push({ dependantDic: Ice.Observable.CreateDependenciesDic(arr[n]), dataProperty: arr[n], callback: callback });
		}
		return this;
	};

	return instance;
})();

/*#endregion*/

/*#region Ice.ObservableBinder*/

Ice.ObservableBinder = function ()
{
	Ice.Observable.call(this);
};

Ice.ObservableBinder.prototype = new (function ()
{
	/// <summary>
	/// HTMLElement holding ObservableBinder
	/// </summary>
	this.RootElement = null;

	var instance = new Ice.Observable();
	var base = Ice.Extend({}, instance);
	var _repeaterSettings =
	{
		// static, common for all instances
		repeaters: [],
		templates: []
	};

	//#region helper methods

	function GetAbsoluteDataSourcePath(el)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="el">Descendant or RootElement itself</param>

		while (el && !el.$$iceDataSourceAbsolutePath && (el = el.parentNode))
		{
		}

		var path = el && el.$$iceDataSourceAbsolutePath ? el.$$iceDataSourceAbsolutePath : null;

		// jer ce biti null pri prvom bindanju kad imamo nestane repeater tepmalteove, vanjski ce biti removed from dom pa unutrasnji template nece imati parenta do #root#-a

		return path == "#root#" ? "" : path;

	};

	function GetBinderInstance(el)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="el">Descendant or RootElement itself</param>
		/// <returns type="Ice.ObservableBinder"></returns>

		if (Ice.IsVSIntellisene) { return new Ice.ObservableBinder(); }

		while (el && !el.$$iceObservableBinder && (el = el.parentNode))
		{
		}

		return el && el.$$iceObservableBinder ? el.$$iceObservableBinder : null;

	};

	function GetRepeaterSetting(el)
	{
		var settingId = el.getAttribute("ice-repeater-setting-id");
		if (!settingId)
		{
			return null;
		}
		return settingId ? _repeaterSettings.repeaters[parseInt(settingId, 10)] : null;
	}

	function RepeaterSettings()
	{
		return {
			header: null,
			item: null,
			alternating: null,
			separator: null,
			footer: null,
			nodata: null
		};
	};

	//#endregion

	//#region binding instructions parser

	function ExtractParametersNames(str)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="str" type="String"></param>
		/// <returns type="Ice.List"></returns>
		var query;
		if (str.StartsWith("("))
		{
			query = new Ice.List(str).Skip(1).TakeWhile("o=>o != ')'")
		}
		else if(str.Contains("=>"))
		{
			query = new Ice.List(str).TakeWhile("o=>o != '='");
		}
		else if(str.EndsWith(")"))
		{
			query = new Ice.List(str).SkipWhile("o=>o != '('").Skip(1).TakeWhile("o=>o != ')'");
		}
		
		return query.ToStringJoined().split(",").AsEnumerable().Select("o=>o.Trim()").ToList();
	}

	function CompileFunction(str, listParameters)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="str" type="String"></param>
		/// <param name="listParameters" type="Ice.List"></param>
		/// <returns type="Function"></returns>
		var str = str;
		listParameters
		.Select(function (o) { return { original: o, fixed: o.replace(new RegExp("[^\\w]", "g"), "_")}; }) // replace any non-word char with _: @.state.IsActive into $$state_isActive
		.ForEach(function (o)
		{
			// eg. replace Settings.showQuality with Settings_showQuality but don't replace Settings.showQuality in model.Settings.showQuality
			// o.original may be eg. Settings.showQuality where dot should be replaced with \. for regex, but don't mess with it, leave unescaped so dot may be any char in regex, it doesn't matter, it is OK
			str = str.replace(new RegExp("(^|[^\\w\\.])(" + o.original + ")($|[^\\w\\.])", "ig"), "$1" + o.fixed + "$3");
			////str = str.ReplaceAll(o.original, o.fixed);
			////alert(o.original + ", " + o.fixed + ", " + str);
		});
		
		if(str.Contains("=>"))
		{
			return Ice.Utility.Expressions.CompileLambdaExpression(str);
		}
		else
		{
			// take function name only, without (..)
			str = new Ice.List(str).TakeWhile("o=>o != '('").ToStringJoined();
			return eval("(" + str + ")");
		}
	}

	//#if VSINTELLISENSE
	function _instructionItemType()
	{
		return {
			// current element data path
			absoluteDataPath: "",
			// string
			leftExpr: "o=>o.name",
			// string
			rightExpr: "o=>o.name",
			// daata type to convert to, usage ice-bind="value(Number):ShowTime"
			dataType: "Number|Boolean|String",
			// is listener, create from attribute ice-listen="..."
			isListener: true || false,
			// Ice.List with properties names
			domElProperties: new Ice.List(["prop1", "prop2"]),
			// Ice.List with properties names
			dataModelProperties: new Ice.List(["prop1", "prop2"]),
			// created from leftExpr
			funcLeft: function () { },
			// created from rightExpr
			funcRight: function () { }
		};
	}
	//#endif

	function ParseSingleInstruction(m, el, attrName)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="m"></param>
		/// <param name="el"></param>
		/// <param name="attrName"></param>
		/// <returns type="_instructionItemType"></returns>

		// get data source path every time, eg: ice-bind="repeater:selitems;;;style.background-color:(DataItem)=>{console, in seccond iteration, DataItem will be selitems
		var res = {};
		
		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene)
		{
			res = _instructionItemType();
		}
		//#endif

		// m is now single binding instruction
		// datatype for 2-way binding is optional, if we want value to be converted to Number: ice-bind="value(Number):ShowTime"
		// supported dataType: Number, Boolean, String

		var index;

		// examples:
		// (value)=>value ? 'one' : 'two' ::: model.LastName, use ::: for safety
		// !value : model.LastName

		if (m.charAt(0) == "!" || m.charAt(0) == "(")
		{
			// search for the last ':'
			index = m.lastIndexOf(":::");
			if(index == -1)
			{
				index = m.lastIndexOf(":");
			}
		}
		else
		{
			// search for the first ':'
			index = m.indexOf(":::");
			if(index == -1)
			{
				index = m.indexOf(":");
			}
		}

		var leftExpr = m.substr(0, index).Trim();
		var rightExpr = m.substr(index + 1).Trim();

		var dataType;

		var mm = leftExpr.match(new RegExp("([\\w\\.]+)[\\(]([\\w]+)[\\)]")); // eg. value(Number):model.Age
		if (mm && mm[1] && mm[2])
		{
			leftExpr = mm[1];
			dataType = mm[2];
		}

		var absoluteDataPath = GetAbsoluteDataSourcePath(el);

		// base settings:
		res.leftExpr = leftExpr;
		res.rightExpr = rightExpr;
		res.absoluteDataPath = absoluteDataPath;
		res.isListener = attrName == "ice-listen";
		res.dataType = dataType;
		
		if (leftExpr == "source" || leftExpr == "repeater")
		{
			// ice-bind="source:dataPropertyName"
			// ice-bind="repeater:dataPropertyName" where dataPropertyName is data source relative path

			/*
			<input type="checkbox"
			ice-bind="source:state;;; checked: (isActive)=>isActive;;;"
			ice-listen="mouseover, mouseout: (sender, ev, binder, dataProperty)=>console.log(ev.type + ': ' + dataProperty);;;mouseover,mouseout: DoSomething;;;"*/
			if (!el.$$iceDataSourceAbsolutePath)
			{
				// path has to be set now, eg. source:state, this way isActive will be bound from state.isActive
				// must be set now, if nested repeter in repeater
				el.$$iceDataSourceAbsolutePath = absoluteDataPath ? (absoluteDataPath + "." + rightExpr) : rightExpr;
			}

			// right expression must be converted to function to prevent it from updateing
			res.funcRight = function (o) { return o; };

			if (leftExpr == "repeater" && !el.getAttribute("ice-repeater-setting-id"))
			{
				// we don't have this here, so finde binder instance from element:
				_repeaterSettings.repeaters.push(new RepeaterSettings());
				el.setAttribute("ice-repeater-setting-id", (_repeaterSettings.repeaters.length - 1) + "");
				// we want repeater to be refreshed when data source changes, so if there is no other binding to repeater, make sure it will be notified from NotifyPropertyChanged
				// to achieve this, el must have $$iceBindingList:
				// so, don't return null, but let it to create binding item for "repeater"
			}
			else
			{
				// already created
				// return null;
				// don't return here, let it to create binding item for "repeater"
			}
		}
		else if (leftExpr == "template")
		{
			if (el.getAttribute("ice-bind-template"))
			{
				// already processed, this is template clone
			}
			else
			{
				// $$iceDataSourceRelativePath has to be created on template instantinating
				// ice-bind="template:item" (item, header, footer, separator, alternating)
				var repeaterSetting = GetRepeaterSetting(el.parentNode);
				if (!repeaterSetting)
				{
					Ice.Diagnostic.Exception("template:" +rightExpr + " has no parent repeater");
				}
				// on data binding, this code is executed every time if item has ice-bind="template:nesto". it is ok first time when templates are removing from dom, but later when template clone is added to repeater, we don't want this code to execute again
				// because it would remove el from dom, mark it, but let's parse other properties in ice-bind="template:item;;;other data binding instruction"
				el.setAttribute("ice-bind-template", "repeater-" + rightExpr);
				if (repeaterSetting[rightExpr])
				{
					Ice.Diagnostic.Exception("template:" + rightExpr + " defined more than once");
				}
				repeaterSetting[rightExpr]= el;
				Ice.Utility.Dom.RemoveNodeFromDom(el);
			}
			// tempalte doesn't have any binding instructions here, just return
			return null;
		}

		// both left and right expression can have:
		// -single property name, eg. innerHTML: SongTitle (2-way binding)
		// - multiple property nameses separated by comma: mousemove, mouseover: HandlerFunc, or eg. innerHTML, value: ItemIndex (2-way binding)
		// - lambda expression (one-way binding only)
		// - may start with ! which means negation: visible: !!(Settings.Active) or !!Settings.Active (one-way binding only)



		if (rightExpr.StartsWith("!"))
		{
			// binding from data model to elment only
			// visible: !Settings.showAbove
			// convert to lambda
			var mm = rightExpr.match(new RegExp("[\\!]", "ig")); // count !
			rightExpr = rightExpr.replace(new RegExp("[\\(\\! \\)]", "ig"), ""); // visible: !!(Settings.Active) or !!Settings.Active
			rightExpr = "(" + rightExpr + ")=>" + mm.join("") + rightExpr;
			////Ice.Observable.Log("non expression converted: " + rightExpr);
		}
		
		if (rightExpr.StartsWith("(") || rightExpr.EndsWith(")") || rightExpr.Contains("=>"))
		{
			// binding from data model to element only
			// lambda expression: (...)=>...
			res.dataModelProperties = ExtractParametersNames(rightExpr);
			res.funcRight = CompileFunction(rightExpr, res.dataModelProperties).BindContext(el);
		}
		else
		{
			// 2 way binding is possible
			// don't allow comma separated names because it doesn't make sense, can't set 2 values from data model to an element property
			// ice-bind="visible:Media.activeSrc, Media.activeSrc" // can't bind multiple data values, it doesn't make sense			
			res.dataModelProperties = rightExpr.split(",").AsEnumerable().Select("o=>o.Trim()").ToList();
			if(res.dataModelProperties.Count() > 1)
			{
				Ice.Diagnostic.Exception("Multiple binding expression are not supported:\r\n" + attrName + "=\"" + leftExpr + ":" + rightExpr + "\"");
			}
		}


		

		if (leftExpr.StartsWith("!"))
		{
			// bind from element to model only
			// visible: !style.display
			// convert to lambda
			var mm = leftExpr.match(new RegExp("[\\!]", "ig")); // count !
			leftExpr = leftExpr.replace(new RegExp("[\\(\\! \\)]", "ig"), ""); // visible: !!(style.display) or !!style.display
			leftExpr = "(" + leftExpr + ")=>" + mm.join("") + leftExpr;
			////Ice.Observable.Log("non expression converted: " + leftExpr);
		}

		if (leftExpr.StartsWith("(") || leftExpr.EndsWith(")") || leftExpr.Contains("=>"))
		{
			// bind from element to model only
			// lambda expression: (...)=>...
			res.domElProperties = ExtractParametersNames(leftExpr);
			res.funcLeft = CompileFunction(leftExpr, res.domElProperties).BindContext(el);
		}
		else
		{
			// 2 way binding is possible
			// leftExpr may have many names separated by comma: mouseover, mouseout, click ...
			// allow comma separated names and the same data value will be bound to numerous element properties at once
			// ice-bind="innerHTML, title : Media.Name" // assign both innerHTML and title with value of Media.Name same as el.innerHTML = el.title = Media.Name
			// ice-listen="mouseover, mousemove: HandlerFunc"
			res.domElProperties = leftExpr.split(",").AsEnumerable().Select("o=>o.Trim()").ToList();
		}


		// create absolute dataModelProperties to be able to search properties when notify change is triggered
		// data source properties are on the right side
		res.dataModelProperties = res.dataModelProperties.Select(function (o)
		{
			if (o == "DataItem")
			{
				// example: ice-bind="repeater:myItems ;;; (DataItem)=>.... where DataItem is myItems // accessing the data source itself
				return absoluteDataPath; // used when element has $$iceDataSourceRelativePath defined and we want to use that source to bind data to the same element
			}
			else if(o.charAt(0) == "@")
			{
				// any of chars /, @, $ means start from data model root:
				// @.Player.CurrentPlaylistItem
				// @Player.CurrentPlaylistItem

				if (o.charAt(1) == ".")
				{
					return o.substr(2);
				}
				// eg. /ClickHandler must point to datasource root.ClickHandler, it may be set inside of repeater tempalte as ice-listen="click:/ClickHandler"
				return o.substr(1);
			}
			else
			{
				if(o.indexOf("DataItem.") == 0)
				{
					// allow eg. DataItem.length
					o = o.substr("DataItem.".length);
				}
			}

			return absoluteDataPath ? (absoluteDataPath + "." + o) : o;

		}).ToList();

		return res;
	}

	function ParseBindingInstructions(el, attrName)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="el"></param>
		/// <param name="attrName"></param>

		// use ice-data="prop.name" to set data source for descendant elements, this way we may use relative prop.names in ice-bind or ice-listen,
		// example: if we have data source with properties: data.Car.Front.WheelsCount, to access data: ice-bind="data.Car.Front.WheelsCount", but if we use ice-data="data.Car", we access data as: ice-bind="Front.WheelsCount"
		// multiple bind statements are separated with semicolon (;;;)
		// simple 2-way binding on input, select, textarea elements and 1-way on other elements: ice-bind="checke:state.isActive; innerHTML:text"
		// global methods, passing dataproperty value: ice-bind="innerHTML:My.Test.Nesto.GetOpis(state.isActive, activeOpis); style.color:My.Test.Nesto.GetColor(isNotActive);"
			// dataproperty is method bind as 1-way, if it is value, bind as 2-way for input, select or textarea: ice-bind="innerHTML:activeOpis"
		// lambda, param state.isActive is replaced with state_isActive: ice-bind="checked:isActive; checked: (state.isActive, state.isSelected)=>state.isActive && state.isSelected ? 'red' : 'blue'; innerHTML:text;"
		// listener only, if different from 2-way binding ice-listen="state.isActive: this.checked ? true : false"

		////Ice.Observable.Log(attrName + ": " + new Ice.List(el.attributes).Where(function (o) { return o.name == attrName; }).Select("o=>o.value").ToStringJoined(", "));

		//#if VSINTELLISENSE
		if(Ice.IsVSIntellisene)
		{
			return new Ice.List([_instructionItemType()]);
		}
		//#endif

		var str = Ice.Utility.Dom.GetAttributeValue(el, attrName);
		if (!str)
		{
			return new Ice.List();
		}

		var items = new Ice.List((str || "").split(";;;"))
		.Select(function (o) { return o.Trim([' ', '\r', '\n', '\t', ';']); }) // remove (;) too
		.Where(function (o) { return !Ice.String.IsNullOrEmpty(o); })
		.Select(function (m)
		{
			return ParseSingleInstruction(m, el, attrName);
		}).Where("o=>o").ToList();

		return items;
	}

	function EnsureSettings(el)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="el" type="HTMLElement"></param>

		//#if VSINTELLISENSE
		if (Ice.IsVSIntellisene && !el.$$iceBindingList)
		{
			el.$$iceBindingList = ParseBindingInstructions(el, 'ice-bind');
			el.$$iceListeningList = ParseBindingInstructions(el, 'ice-listen');
			el.$$iceBindingDic = { "Ice.Data.Com": "" }; // dictionary of data property paths which element depends on, used for quick preselection on notify property....
		}
		//#endif

		if (!el.$$iceBindingList)
		{
			el.$$iceBindingList = ParseBindingInstructions(el, 'ice-bind');
			el.$$iceListeningList = ParseBindingInstructions(el, 'ice-listen');
			el.$$iceBindingDic = {};
			el.$$iceBindingList.ForEach(function (bindingItem)
			{
				bindingItem.dependantDic = {};
				bindingItem.dataModelProperties.ForEach(function (prop)
				{
					var dic = Ice.Observable.CreateDependenciesDic(prop); // dic may be null
					if (dic)
					{
						Ice.Extend(bindingItem.dependantDic, dic);
						Ice.Extend(el.$$iceBindingDic, dic); // element may have many bindingItems, so copy properties to element for quick preselection on notify property....
					}
				});
			});
		}
		return el;
	}

	//#endregion

	//#region binding data to dom

	function SetElementValue(el, elProperty, value)
	{
		if (elProperty == "options" && el.tagName.toLowerCase().StartsWith("select"))
		{
			var items = value;
			var html = new Ice.List(items).Select(function (o, n)
			{
				return new Ice.DomQuery.HtmlElement("option").SetInnerHtml(o.Text).ForEach(function (opt)
				{
					opt.value = o.Value;
					opt.$$iceSelected = o.Selected ? true : false; // can't set "selected" attribute before option is attached to dom
				}).First();
			}).ToArray();
			$$ice(el).ReplaceChildren(html);
			// for IE8 and less, "selected" attribute has to be set after option is attached to dom
			$$ice(el.childNodes).Where(function(o){return o.$$iceSelected;}).SetAttribute("selected", "selected");
		}
		else
		{
			Ice.DataBinder.SetValue(el, elProperty, value); // value must be string, eg. innerHTML = 1 won't work unless 1 is string
		}
	};

	function BindDataToSimpleElement(el, bindingList)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="el" type="HTMLElement"></param>
		/// <param name="bindingList" type="Ice.List"></param>

		EnsureSettings(el);
		
		var bindingList = bindingList || el.$$iceBindingList;

		if (bindingList.Any())
		{
			var binder = GetBinderInstance(el);

			if (!binder)
			{
				// this happens when repeater is updated, previous children are removed from dom, but we're still iterating items from onclick or onchange inside AppendListenersToElement()
				return;
			}

			bindingList.ForEach(function (item)
			{
				var args = item.dataModelProperties.Select(function (o)
				{
					if (o.EndsWith("ItemIndex"))
					{
						return $$ice(el).Query("*, ^*").Where("o=>o.$$iceItemIndex >= 0").Select("o=>o.$$iceItemIndex").First();
					}
					return binder.GetValue(o);

				}).ToArray();

				var value = item.funcRight ? item.funcRight.apply(el, args) : args[0];

				// multiple properties may be assigned with the same data value, eg. ice-bind="title, innerHTML:Some.Text"
				item.domElProperties.ForEach(function (elProp)
				{
					var isRepeater = false;
					var isSource = false;

					if (elProp == "visible")
					{
						// to be able to use ice-bind="visible:true/false" instead of writing (visible)=> this.style.display = visible ? "" : "none"
						elProp = "style.display";
						value = value ? "" : "none";
					}
					else if (elProp == "repeater" || elProp == "source")
					{
						// in case when repeater is in root and datasource is array
						// value is only for loging, don't set $$iceDataSourceRelativePath here, it has to be created on ParseSingleInstruction()
						value = el.$$iceDataSourceAbsolutePath = item.dataModelProperties.Single();// must be exactly 1
						isRepeater = elProp == "repeater";
						isSource = elProp == "source";
					}

					// elProp may be "", eg: ice-bind=":(Player.Playing)=>$$ic....., so don't assign if elProp is ""

					/*#if DEBUG*/

					if (Ice.Observable.Log.Enabled)
					{
						Ice.Observable.Log("BIND DATA " + "ice-bind=" + el.getAttribute("ice-bind") + " ### " + (elProp ? (" ASSIGN " + elProp + "=" + Ice.JSON.Serialize(value)) : (" EVALUATE " + item.leftExpr + ":" + item.rightExpr)));
					}
					/*#endif*/

					if (isSource)
					{
						// do nothing any more
					}
					else if (isRepeater)
					{
						BindDataToRepeater(el);
					}
					else if (elProp)
					{
						SetElementValue(el, elProp, value);
					}
				});
			});
		}
	}

	function BindDataToRepeater(repeaterEl)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="repeaterEl" type="HTMLElement"></param>

		// add templates to dom, than bind data

		// binder is always in the root element, relative
		var binder = GetBinderInstance(repeaterEl);

		repeaterEl.innerHTML = ""; // remove previous children
		var repeater = GetRepeaterSetting(repeaterEl);
		var path = GetAbsoluteDataSourcePath(repeaterEl);

		var source = binder.GetValue(path);

		var total = (source || "").length;

		/*#if DEBUG*/
		if (Ice.Observable.Log.Enabled)
		{
			Ice.Observable.Log("BIND REPEATER " + (total || 0) + " items: " + repeaterEl.getAttribute("ice-bind"));
		}
		/*#endif*/

		if (!total) // 0
		{
			if (repeater.nodata)
			{

				var nn;
				repeaterEl.appendChild(nn = repeater.nodata.cloneNode(true));
				nn.style.display = "";
				// make repeater element visible:
				if (repeaterEl.style.display == "none") { repeaterEl.style.display = ""; }
			}
			// don't return, let's data to be bound to nodata template and it's children
			////return; // no items in source
		}
		
		// source may be Ice.Enumerable type or Array or null
		new Ice.List(source).ForEach(function (item, index)
		{
			var nn;

			if (repeater.header && index == 0)
			{
				repeaterEl.appendChild(nn = repeater.header.cloneNode(true));
				nn.style.display = "";
				// make repeater element visible:
				if (repeaterEl.style.display == "none") { repeaterEl.style.display = ""; }
			}

			if (repeater.separator && index > 0)
			{
				repeaterEl.appendChild(nn = repeater.separator.cloneNode(true));
				nn.$$iceItemIndex = index;
				nn.style.display = "";
				// make repeater element visible:
				if (repeaterEl.style.display == "none") { repeaterEl.style.display = ""; }
			}

			if (repeater.alternating && index % 2 == 1)
			{
				repeaterEl.appendChild(nn = repeater.alternating.cloneNode(true));
				nn.$$iceDataSourceAbsolutePath = repeaterEl.$$iceDataSourceAbsolutePath ? (repeaterEl.$$iceDataSourceAbsolutePath + "." + index) : (index + ""); // path must be string
				////nn.$$iceDataSourceRelativePath = index + "";
				nn.$$iceItemIndex = index;
				nn.style.display = "";
				// make repeater element visible:
				if (repeaterEl.style.display == "none") { repeaterEl.style.display = ""; }
			}
			else if (repeater.item)
			{
				repeaterEl.appendChild(nn = repeater.item.cloneNode(true));
				nn.$$iceDataSourceAbsolutePath = repeaterEl.$$iceDataSourceAbsolutePath ? (repeaterEl.$$iceDataSourceAbsolutePath + "." + index) : (index + ""); // path must be string
				////nn.$$iceDataSourceRelativePath = index + ""; // must be string
				nn.$$iceItemIndex = index;
				nn.style.display = "";
				// make repeater element visible:
				if (repeaterEl.style.display == "none") { repeaterEl.style.display = ""; }
			}

			if (repeater.footer && index == (total - 1))
			{
				repeaterEl.appendChild(nn = repeater.footer.cloneNode(true));
				nn.style.display = "";
				// make repeater element visible:
				if (repeaterEl.style.display == "none") { repeaterEl.style.display = ""; }
			}
		});

		// bind data to added descendant items (templates)
		var elements = $$ice(repeaterEl).Query("//*").WhereHasAttribute("ice-bind").ToList();
		////Ice.Observable.Log(elements.Select("o=>o.getAttribute('ice-bind')").ToStringJoined(", "));

		elements
		.ForEach(function (o)
		{
			// make sure it is removed and let it be created again since tempalte index is changed on every itearation
			// this is required for IE8< since it seems to create prototype properties and they are cloned to every element clone
			o.$$iceBindingList = null;
			o.$$iceListeningList = null;
			o.$$iceBindingListStr = null;

			BindDataToSimpleElement(o);

			AppendListenersToElement(o);

		});
	}

	//#endregion

	instance.NotifyPropertyChanged = function (propertyAbsolutePath, elementsArr)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="propertyAbsolutePath">if not defined, notify all elements</param>
		/// <param name="elementsArr" optional="true">if provided, will notify elementsArr with propertyAbsolutePath only, else, notify all listeners and dom elements by propertyAbsolutePath</param>
		

		/*#if DEBUG*/
		if (Ice.Observable.Log.Enabled)
		{
			Ice.Observable.Log("NotifyPropertyChanged: " + (propertyAbsolutePath || "#all#"));
		}
		/*#endif*/

		/*#if DEBUG*/
		var ticks1 = Date.now();
		/*#endif*/

		// first notify listeners added using .AddListener(....)
		// invoke always, this feature is used for ShowShareButton in imagine player
		base.NotifyPropertyChanged.apply(this, arguments);

		var all = elementsArr;

		// if data is not bound to DOM, there is no RootElement:
		if (!this.RootElement)
		{
			// create new element, to prevent from null exception
			this.RootElement = document.createElement("div");
		}

		if (!elementsArr)
		{
			all = this.RootElement.getElementsByTagName("*");
		}

		// notify elements for property changed, descendant of binder.RootElement only and RootElement too
		// current and descendants
		////var re = propertyAbsolutePath ? this._createSafePropertyRegex(propertyAbsolutePath) : null;

		var hasProp = !!propertyAbsolutePath;
		// find elements who are listening for data model property propertyAbsolutePath
		// this has to be much faster than LINQ:
		var bindingArr = [];
		var el;
		for (var n = -1; n < all.length; n++)
		{
			el = n == -1 ? this.RootElement : all[n];

			// use $$iceBindingDic to preselect elements with propertyAbsolutePath
			if (el.$$iceBindingDic && (!hasProp || el.$$iceBindingDic[propertyAbsolutePath]))
			{
				// extract binding items:
				// take all unless propertyAbsolutePath is defined
				var bindingList = el.$$iceBindingList;

				if (hasProp)
				{
					// take binding lists with propertyAbsolutePath only
					bindingList = el.$$iceBindingList.Where(function (item) { return item.dependantDic[propertyAbsolutePath] }).ToList();
				}

				if (bindingList.Any())
				{
					bindingArr = bindingArr.concat({
						el: el,
						bindingList: bindingList
					});
				}
			}
		}

		/*#if DEBUG*/
		var ticks2 = Date.now();
		/*#endif*/

		/*#if DEBUG*/
		if (Ice.IsDebugDefined)
		{
			//console.log("dom: " + bindingArr.length + " : " + propertyAbsolutePath);
		}
		/*#endif*/

		var o;
		for (var n = 0; n < bindingArr.length; n++)
		{
			o = bindingArr[n];
			BindDataToSimpleElement(o.el, o.bindingList);
		}

		/*#if DEBUG*/
		var ticks3 = Date.now();
		/*#endif*/

		/*#if DEBUG*/
		if (Ice.IsDebugDefined)
		{
			//console.log("time: " + (ticks2-ticks1) + ", " + (ticks3-ticks2) + ", " + (ticks3-ticks1) + " : elements: " + bindingArr.length + " : " + propertyAbsolutePath);
		}
		/*#endif*/
		
		return this;
	};

	function AppendListenersToElement(element)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="element" type="HTMLElement"></param>

		EnsureSettings(element);

		var tag = element.tagName.toLowerCase();
		var isInput = (tag == "input" || tag == "select" || tag == "textarea");

		// create ice-listen events:
		var listeningList = element.$$iceListeningList.Where(function(o){return o.isListener || isInput;}).ToList();

		listeningList.ForEach(function (item)
		{
			item.domElProperties.ForEach(function (p)
			{
				// allow multiple event names separated by comma: ice-listen="change, click, focus : ChangeModelSomeFunc, DoModelSomething"
				if (!Ice.Event.IsSupported(p))
				{
					Ice.Diagnostic.Exception("Element event " + p + " is not supported");
				}

				$$ice(element).On(p, function (sender, ev)
				{
					/*#if DEBUG*/

					if (Ice.Observable.Log.Enabled)
					{
						Ice.Observable.Log("LISTENING EVENT " + "ice-listen=" + sender.getAttribute("ice-listen") + " ### EXECUTE EVENT " + ev.type);
					}

					/*#endif*/

					var binder = GetBinderInstance(element);
					if (!binder)
					{
						// if item is in template and template is not in dom, parentNode is null, so binder will be null, Chrome only, IE doesn't have this problem
						return;
					}

					if (item.funcRight)
					{
						// external function call, this is sender
						item.funcRight.call(sender, sender, ev, binder, null);
					}
					else
					{
						// dataProp is data model property full path
						// value at dataProp must be function
						item.dataModelProperties.ForEach(function (dataProp)
						{
							var cb = Ice.DataBinder.GetValue(binder._dataSource, dataProp);
							cb = Ice.DataBinder.GetValue(binder._dataSource, dataProp);
							if (typeof (cb) != "function")
							{
								Ice.Diagnostic.Exception("Data property " + dataProp + " is not function");
							}
							else
							{
								// call to function in data model, "this" is dataSource
								cb.call(binder._dataSource, sender, ev, binder, dataProp);
							}
						});
					}
				});
			});

		});


		// create listening events from ice-bind list for updating from element to data model:
		if (isInput)
		{
			// 2-way binding (from element to data model) is available for input, select and textarea elements only

			// select from BindingList where values are not assigned using funcRight
			var bindingList = element.$$iceBindingList.Where("o=>!o.funcRight").ToList();
			
			if (bindingList.Any())
			{
				element.$$iceBindToModelList = bindingList; // attach to element itself, this way we don't have problem with different context in .On() and .ForEach() functions

				// monitor dom elements for click or change or enter:
				// IE8- doesn't have change event on checkbox or radio, use click event
				$$ice(element).On(element.type == "checkbox" || element.type == "radio" ? "click.ice_observable_event" : ["enterKey.ice_observable_event", "change.ice_observable_event"], function (sender, ev)
				{
					/*#if DEBUG*/
					
					if (Ice.Observable.Log.Enabled)
					{
						Ice.Observable.Log("BINDING EVENT " + "ice-bind=" + sender.getAttribute("ice-bind") + " ### EVENT " + ev.type);
					}

					/*#endif*/

					var binder = GetBinderInstance(element);
					
					var changed = [];

					element.$$iceBindToModelList.ForEach(function (item)
					{
						var propertyAbsolutePath = item.dataModelProperties.Single(); // if ice-bind, than there must be single datasource property

						changed.push(propertyAbsolutePath);

						if (sender.tagName.toLowerCase() == "select")
						{
							new Ice.List(sender.options).ForEach(function (o, n)
							{
								// do not call NotifyPropertyChanged untill all options are set
								binder.SetValueSilent(propertyAbsolutePath + "." + n + ".Selected", o.selected ? true : false);
							});
						}
						else
						{
							var args = item.domElProperties.Select(function (o)
							{
								return sender.type == "checkbox" || sender.type == "radio" ? (sender.checked ? true : false) : sender.value;
							}).ToArray();

							var value = item.funcLeft ? item.funcLeft.apply(element, args) : args[0];

							if (item.dataType)
							{
								switch (item.dataType)
								{
									case "Number":
										value = parseFloat(value) || null;
										break;
									case "Boolean":
										value = !!(value);
										break;
									case "String":
										value = value + "";
										break;
								}
							}

							binder.SetValueSilent(propertyAbsolutePath, value);
						}
					});

					////Ice.Observable.Log("prije: " + changed.length + ", reducted: " + new Ice.List(changed).Distinct().Count());

					// trebalo bi uzeti najkraci naziv propertia jer se za update ionak gleda .StartsWith(...) pa da se ne okidaju childreni vise puta
					new Ice.List(changed).Distinct().ForEach(function (o)
					{
						// after all options are set, call NotifyPropertyChanged
						binder.NotifyPropertyChanged(o);
					});


				});
			}
		}
	}

	function EnsureRootElement(iceSelector)
	{
		if (!this.RootElement)
		{
			var root = $$ice(iceSelector || this.RootSelector || document.documentElement).ToDomQuery();
			if (root.Count() > 1)
			{
				Ice.Diagnostic.Exception("DataBind selector selects " + root.Count() + " elements");
			}

			this.RootElement = root.First();
			this.RootElement.$$iceObservableBinder = this;
			this.RootElement.$$iceDataSourceAbsolutePath = "#root#";

			// initial data bind
			// ice-bind is both bind and listen if value is retrieved by datasource property, not if value is retrieved via function
			// ice-listen is listen only

			// preprocess templates and settings
			var elements = root.Query("*,//*").Query("[ice-bind],[ice-listen]").Distinct()
			.ForEach(function (o)
			{
				EnsureSettings(o);

			}.BindContext(this));

			// append listeners
			// query nodes again, some of them were templates and were removed from dom
			var elements = root.Query("*,//*").Query("[ice-bind],[ice-listen]").Distinct() // Distinct(): if node has both attributes, it is selected twice, so use Distinct to select it only once
			.ForEach(function (o)
			{
				AppendListenersToElement(o);

			}.BindContext(this));
		}
	}

	instance.SetRootElement = function (iceSelector)
	{
		this.RootSelector = iceSelector;
		return this;
	};
	
	instance.DataBind = function ()
	{
		/// <summary>
		/// bind this ObservableBinder to elements by iceSelector. if not set it is bound to document.
		/// </summary>
		
		EnsureRootElement.call(this);

		// WARNING: bind data after listeners are bound, this way we'll not append listeners twice inside of repeater elements
		// NotifyPropertyChanged has to be invoked at the end
		this.NotifyPropertyChanged(); // invoke NotifyPropertyChanged with no arguments to trigger all binding properties

		return this;
	};

	//////#if VSINTELLISENSE
	////if (Ice.IsVSIntellisene)
	////{
	////	instance.DataBind(document);
	////}
	//////#endif
	
	return instance;
}) ();

/*#endregion*/

