﻿/// <reference path="Ice.js" />
/// <reference path="Ice.Event.js" />

if (typeof (Ice) == "undefined") { Ice = {}; }

Ice.Page = new (function ()
{
	var __page = this;

	__page.__class = "Ice.Page";

	var ReadyStateEnum = {
		Loading: 2,
		Interactive: 3,
		Complete: 4
	};

	var _items = [];

	//#if VSINTELLISENSE
	var _intlitem = {
		// function to be executed
		Func: function () { }, 
		ExecuteIfReadyState: ReadyStateEnum.Complete,
		RegisterAjaxEndReq: true,
		IsExecutedOnce: false,
		IsMsAjaxRegistered: false,
		IsIceAjaxRegistered: false
	};
	if (Ice.IsVSIntellisene){_items.push(_intlitem);	}
	function _intlitemType() { return _intlitem;}
	//#endif

	//#region Page Events
	
	function InvokeItems(arr)
	{
		for (var n = 0; n < arr.length; n++)
		{
			arr[n]();
		}
	}

	//#region IceAjaxEndRequest

	var _iceAjaxEndItems = [];

	__page._onIceAjaxEndRequest = function (func){_iceAjaxEndItems.push(func);};

	__page._onIceAjaxEndRequest.Add = __page._onIceAjaxEndRequest;

	__page._onIceAjaxEndRequest.Invoke = function (){InvokeItems(_iceAjaxEndItems);	};

	//#endregion

	//#region IceAjaxBeginRequest

	var _iceAjaxBeginItems = [];

	__page._onIceAjaxBeginRequest = function (func){_iceAjaxBeginItems.push(func);};

	__page._onIceAjaxBeginRequest.Add = __page._onIceAjaxBeginRequest;

	__page._onIceAjaxBeginRequest.Invoke = function ()	{InvokeItems(_iceAjaxBeginItems);};

	//#endregion
	
	function RegisterIceAjaxEndReq(item)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="item" type="_intlitemType"></param>
		
		if (!item.IsIceAjaxRegistered)
		{
			item.IsIceAjaxRegistered = true;
			// register with Ice ajax EndRequest
			__page._onIceAjaxEndRequest.Add(item.Func);
		}
	}

	function RegisterMsAjaxEndReq(item)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="item" type="_intlitemType"></param>
		
		if (!item.IsMsAjaxRegistered)
		{
			// register with MS Ajax EndRequest
			if (typeof (Sys) != "undefined" && Sys.WebForms && Sys.WebForms.PageRequestManager)
			{
				item.IsMsAjaxRegistered = true;
				Sys.WebForms.PageRequestManager.getInstance().add_endRequest(item.Func);
			}
		}
	}

	function ExecuteByEnumState(stateEnum)
	{
		for (var n = 0; n < _items.length; n++)
		{
			var o = _items[n];
				
			if (!o.IsExecutedOnce && o.ExecuteIfReadyState <= stateEnum)
			{
				o.IsExecutedOnce = true;
				o.Func();
			}
			
			if (o.RegisterAjaxEndReq && stateEnum >= ReadyStateEnum.Interactive)
			{
				RegisterMsAjaxEndReq(o);
				RegisterIceAjaxEndReq(o);
			}
		}
	}

	var _retry = 0;

	function ExecuteByStringState(stateString)
	{
		switch (stateString)
		{
			case "loading":
				// nothing
				break;
			case "interactive":
				// html loaded
				if (document && document.body)
				{
					setTimeout(function ()
					{
						ExecuteByEnumState(ReadyStateEnum.Interactive);
					}, 100); // IE and Safari requires additional delay on interactive to process window variables
				}
				else
				{
					// IE 9 i stariji na interactive imaju samo document, ali dom jos nije spreman, nema body tag, ali ima documentElement
					if (++_retry < 10)
					{
						// repeat
						setTimeout(function () { ExecuteByStringState(stateString); }, 200);
					}
				}
				break;
			case "complete":
				// all resources loaded, including images, the same as window.onload
				// trigger interactive and ajaxEndReq if they were added after intractive triggered
				setTimeout(function ()
				{
					ExecuteByEnumState(ReadyStateEnum.Complete);
				}, 1);
				break;
			default:
				throw "Unknown stateString: " + stateString;
		}
	}

	var _evattached;

	function AddEvent(func, readyStateKind, registerAjaxEndReq)
	{
		_items.push({ Func: func, ExecuteIfReadyState: readyStateKind, RegisterAjaxEndReq: registerAjaxEndReq });

		if (!_evattached)
		{
			_evattached = true;

			Ice.Event.Attach(document, "readystatechange", function (ev)
			{
				ExecuteByStringState(document.readyState);
			});

			Ice.Event.Attach(window, "load", function (ev)
			{
				// tested: this works as load event OK
				ExecuteByStringState("complete");
			});

			// tested document.load - not working
			// tested document.body.load - not working, there is no body yet

		}

		// if already has interactive or complete
		ExecuteByStringState(document.readyState);
	}

	__page.OnEndResponse = function (func)
	{
		/// <summary>
		/// Execute on interactive (document.readyState=='interactive') and on every ajax end. Includes 1) clasic page interactive or load, 2) every MS ajax end, 3) every Ice Ajax end.
		/// </summary>
		/// <param name="func">function(){...}</param>

		AddEvent(func, ReadyStateEnum.Interactive, true); // <<< true to register ajax end events

		return __page;
	};

	__page.OnEndResponse.Add = __page.OnEndResponse;


	__page.OnLoadComplete = function (func)
	{
		/// <summary>
		/// Execute once at window.onload event (or document.readyState=='complete'). Includes clasic page load only.
		/// </summary>
		/// <param name="func">function(){...}</param>

		AddEvent(func, ReadyStateEnum.Complete);

		return __page;
	};

	__page.OnLoadComplete.Add = __page.OnLoadComplete;



	__page.OnInteractive = function (func)
	{
		/// <summary>
		/// Execute once on document.readyState=='interactive' or page load. It is as soon as html is interactive, but all other resources may not be loaded (eg. images)
		/// </summary>
		/// <param name="func">function(){...}</param>

		AddEvent(func, ReadyStateEnum.Interactive);

		return __page;
	};

	__page.OnInteractive.Add = __page.OnInteractive;




	var _bitems = [];

	__page.OnBeforeSubmit = function (func)
	{
		/// <summary>
		/// MS ASP.NET or AJAX.NET Specific. Add func which will be executed before page MS postback submit or MS Ajax submit.
		/// </summary>
		/// <param name="func">function(){...}</param>
		_bitems.push(func);
		return __page;
	};

	__page.OnBeforeSubmit.Add = __page.OnBeforeSubmit;

	__page.OnBeforeSubmit.Invoke = function ()
	{
		InvokeItems(_bitems);
		return true; // must return true, if returns void, MS postback will not work
	};

	//#endregion

	return __page;

})();


