﻿/// <reference path="Ice.Diagnostic.js" />

if (typeof (Ice) == "undefined") { Ice = {}; }


Ice.String = new (function ()
{
	var reEscape = new RegExp("[\\-\\[\\]\\/\\{\\}\\(\\)\\*\\+\\?\\.\\\\^\\$\\|]", "g");

	function escapeRegExp(str)
	{
		return str.replace(reEscape, "[\\$&]");
	}

	function escapeArrRegExp(charsArr)
	{
		var arr = [];
		for (var n = 0; n < charsArr.length; n++)
		{
			arr.push(escapeRegExp(charsArr[n]));
		}
		return arr;
	}

	////escapeRegExp("All of these should be escaped: \ ^ $ * + ? . ( ) | { } [ ]");
	////"All of these should be escaped: \\ \^ \$ \* \+ \? \. \( \) \| \{ \} \[ \] "

	var instance = {};

	instance.IsNullOrEmpty = function (str1, str2, str3, etc)
	{
		///<summary>returns true if any of strings is null or empty or undefined</summary>
		///<param name="str1" type="String"></param>
		for (var n = 0; n < arguments.length; n++)
		{
			var tmp = arguments[n];
			if (!(typeof (tmp) == "string" && tmp.length > 0))
			{
				return true;
			}
		}
		return false;
	};

	instance.Equals = function (string1, string2, ignoreCase)
	{
		/// <summary>
		/// Compare two string.
		/// </summary>
		/// <param name="string1"></param>
		/// <param name="string2"></param>
		/// <param name="ignoreCase">If not set, default is false.</param>
		/// <returns type=""></returns>

		if (string1 === string2)
		{
			return true;
		}

		if (ignoreCase)
		{
			string1 = (string1 || "").toLowerCase();
			string2 = (string2 || "").toLowerCase();
		}

		return string1 === string2;
	};


	var trimDef = "(" + escapeArrRegExp([' ', '\r', '\n', '\t']).join("|") + ")+"; // default chars to trim

	function Trim(str, charsArray, trimStart, trimEnd)
	{
		/// <summary>
		/// Trim string.
		/// </summary>
		/// <param name="str">String to be trimmed.</param>
		/// <param name="charsArray">Chars array to trim. If not set, will use default chars.</param>
		/// <param name="trimStart">If true, trim start.</param>
		/// <param name="trimEnd">If true, trim end.</param>
		/// <returns type="">Trimmed string.</returns>

		if (!str)
		{
			return str;
		}

		var restr = trimDef;

		if (charsArray)
		{
			restr = "(" + escapeArrRegExp(charsArray).join("|") + ")+";
		}

		if (arguments.length < 4)
		{
			trimEnd = true;
		}

		if (arguments.length < 3)
		{
			trimStart = true;
		}

		if (trimEnd && trimStart)
		{
			restr = "(^" + restr + ")|(" + restr + "$)";
		}
		else if (trimEnd)
		{
			restr += "$";
		}
		else if (trimStart)
		{
			restr = "^" + restr;
		}

		return str.replace(new RegExp(restr, "g"), "");
	};

	instance.Trim = function (str, charsArray)
	{
		/// <summary>
		/// Trim string.
		/// </summary>
		/// <param name="str">String to be trimmed.</param>
		/// <param name="charsArray">Chars array to trim. If not set, will use default chars.</param>
		return Trim(str, charsArray, true, true);
	};

	instance.TrimStart = function (str, charsArray)
	{
		return Trim(str, charsArray, true, false);
	};

	instance.TrimEnd = function (str, charsArray)
	{
		return Trim(str, charsArray, false, true);
	};

	instance.EndsWith = function (str, secondStr, ignoreCase)
	{
		if (!str) { return false; }
		return new RegExp(escapeRegExp(secondStr) + "$", ignoreCase ? "ig" : "g").test(str);
	};

	instance.StartsWith = function (str, secondStr, ignoreCase)
	{
		if (!str) { return false; }
		return new RegExp("^" + escapeRegExp(secondStr), ignoreCase ? "ig" : "g").test(str);
	};

	instance.Contains = function (str, secondStr, ignoreCase)
	{
		if (!str) { return false; }
		return new RegExp(escapeRegExp(secondStr), ignoreCase ? "ig" : "g").test(str);
	};

	instance.Substring = function (str, startIndex, length)
	{
		if (!str) { return str; } // null or ""
		var maxlen = str.length - startIndex;
		if (length > maxlen)
		{
			return str.substr(startIndex); // take all, length is larger than rest of the string
		}
		else
		{
			return str.substr(startIndex, length);
		}
	};

	instance.Remove = function (str, startIndex)
	{
		if (str && str.length > startIndex)
		{
			return str.substr(0, startIndex + 1);
		}
		else
		{
			return "";
		}
	};

	instance.Repeat = function (str, count)
	{
		/// <summary>repeated str count times</summary>
		/// <param name="str" type="Number|String|Object">The str to be repeated.</param>
		/// <param name="count" type="Number">The number of times to repeat the str in the generated sequence.</param>
		var arr = [];
		for (var n = 0; n < count; n++)
		{
			arr.push(str);
		}
		return (arr.join(""));
	};

	instance.TakeFirst = function (str, count, elipsis)
	{
		if (str && str.length > count)
		{
			return str.substr(0, count - (elipsis ? elipsis.length : 0)) + (elipsis || "");
		}
		else
		{
			return str;
		}
	};

	instance.TakeLast = function (str, count, elipsis)
	{
		if (str && str.length > count)
		{
			return (elipsis || "") + str.substr(str.length - count + (elipsis ? elipsis.length : 0), count);
		}
		else
		{
			return str;
		}
	};

	instance.Split = function (str, splitCharsArr)
	{
		if (!str) { return []; }
		return str.split(new RegExp(escapeArrRegExp(splitCharsArr).join("|"))); // switch "g" nema utjecaja
	};

	instance.ReplaceAll = function (str, search, replacement, ignoreCase)
	{
		/// <summary>
		/// Replace all occurances of search text inside of str, using RegExp.
		/// </summary>
		/// <param name="str"></param>
		/// <param name="search"></param>
		/// <param name="replacement"></param>

		if (!str) { return str; }
		var re = new RegExp(escapeRegExp(search), ignoreCase ? "ig" : "g");
		return str.replace(re, replacement);
	};

	instance.FormatWith = function (str, obj, placeHolderStart, placeHolderEnd)
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="str">input string with property names or array indexes: My name is {Name}, or My name is {0}</param>
		/// <param name="obj" type="Object|Array">object with properties {Name:Tom, Age:22, ...} or array["Tom", 22"...]</param>
		/// <param name="placeHolderStart" optional="true" type="String" default="{"></param>
		/// <param name="placeHolderEnd" optional="true" type="String">default }</param>

		placeHolderStart = placeHolderStart || "{";
		placeHolderEnd = placeHolderEnd || "}";

		if (!obj || this.IsNullOrEmpty(str))
		{
			return str;
		}

		if (placeHolderStart.Length > 1)
		{
			placeHolderStart = placeHolderStart.AsEnumerable.Select("o => '\\' + o").ToStringJoined();
		}
		else
		{
			placeHolderStart = "\\" + placeHolderStart;
		}

		if (placeHolderEnd.Length > 0)
		{
			placeHolderEnd = placeHolderEnd.AsEnumerable().Select("o => '\\' + o").ToStringJoined();
		}
		else
		{
			placeHolderEnd = "\\" + placeHolderEnd;
		}

		return str.replace(new RegExp("(" + placeHolderStart + "([\\w_\\-\\.]+)" + placeHolderEnd + ")", "g"), function (all, g1, g2, index, allStr)
		{
			var val = Ice.DataBinder.GetValue(obj, g2);
			if (!val && typeof (val) == "undefined")
			{
				Ice.Diagnostic.Exception("Exception in Ice.String.FormatWith(), " + g1 + " property not found.");
			}
			return val;
		});
	};



	return instance;

})();


