﻿
Ice.TimeSpan = function ()
{
};

Ice.TimeSpan.GetTicks = function (days, hours, minutes, seconds, miliseconds)
{
	/// <summary>
	/// Returns ticks in miliseconds
	/// </summary>
	/// <param name="days"></param>
	/// <param name="hours"></param>
	/// <param name="minutes"></param>
	/// <param name="seconds"></param>
	/// <param name="miliseconds"></param>

	return 1000 * 3600 * 24 * (days || 0)
		+ 1000 * 3600 * (hours || 0)
		+ 1000 * 60 * (minutes || 0)
		+ 1000 * (seconds || 0)
		+ (miliseconds || 0);
};

Ice.TimeSpan.Create = function (days, hours, minutes, seconds, miliseconds)
{
	var tt = new Ice.TimeSpan();
	tt._ticks = Ice.TimeSpan.GetTicks.apply(this, arguments);
	return tt;
};

Ice.TimeSpan.FromMiliseconds = Ice.TimeSpan.FromTicks = function (miliseconds) { return new Ice.TimeSpan().AddMiliseconds(miliseconds); };
Ice.TimeSpan.FromSeconds = function (seconds) { return new Ice.TimeSpan().AddSeconds(seconds); };
Ice.TimeSpan.FromMinutes = function (minutes) { return new Ice.TimeSpan().AddMinutes(minutes); };
Ice.TimeSpan.FromHours = function (hours) { return new Ice.TimeSpan().AddHours(hours); };
Ice.TimeSpan.FromDays = function (days) { return new Ice.TimeSpan().AddDays(days); };

Ice.TimeSpan.prototype = new (function ()
{
	this._ticks = 0; // in miliseconds

	this.AddTicks = function (ticksMs)
	{
		var tt = new Ice.TimeSpan();
		tt._ticks = this._ticks + ticksMs;
		return tt;
	};

	this.AddDays = function (days) { return this.AddTicks(Ice.TimeSpan.GetTicks(days)); };
	this.AddHours = function (hours) { return this.AddTicks(Ice.TimeSpan.GetTicks(0, hours)); };
	this.AddMinutes = function (minutes) { return this.AddTicks(Ice.TimeSpan.GetTicks(0, 0, minutes)); };
	this.AddSeconds = function (seconds) { return this.AddTicks(Ice.TimeSpan.GetTicks(0, 0, 0, seconds)); };
	this.AddMiliseconds = function (miliseconds) { return this.AddTicks(miliseconds); };

	this.TotalMiliseconds = function () { return this._ticks; };
	this.TotalSeconds = function () { return this._ticks / 1000; };
	this.TotalMinutes = function () { return this._ticks / 1000 / 60; };
	this.TotalHours = function () { return this.TotalMinutes() / 60; };
	this.TotalDays = function () { return this.TotalHours() / 24; };

	this.ToParts = function ()
	{
		/// <summary>
		/// Returns time parts
		/// </summary>
		var ticks = this._ticks;
		var totalSeconds = Math.floor(ticks / 1000);

		return {
			// hours part, can be > 24
			Hours:   Math.floor(ticks / 1000 / 60 / 60), 
			Minutes: Math.floor(totalSeconds / 60) % 60,
			Seconds: totalSeconds % 60,
			Miliseconds: Math.floor(ticks) % 1000,
			TimeString: this.ToStringFormated()
		};
	};

	function pad (val, len)
	{
		val = String(val);
		len = len || 2;
		while (val.length < len) val = "0" + val;
		return val;
	};

	var token = new RegExp("[H]{1,2}|[h]{1,2}|[m]{1,2}|[s]{1,2}|[f]{1,3}", "g");

	function FormatTime(ticks, format)
	{
		// 1 tick == 1 ms
		var ms = pad(Math.floor(ticks) % 1000, 3);
		var totalSeconds = Math.floor(ticks / 1000);
		var hours = Math.floor(ticks / 1000 / 60 / 60); // total hours, can be > 24
		var min = Math.floor(totalSeconds / 60) % 60;
		var s = totalSeconds % 60;

		var flags = {
			H: hours,
			HH: pad(hours),
			m: min,
			mm: pad(min),
			s: s,
			ss: pad(s),
			f: String(ms).substr(0, 1),
			ff: String(ms).substr(0, 2),
			fff: String(ms).substr(0, 3)
		};

		////Ice.Diagnostic.WriteLine(Ice.Diagnostic.Dump(flags));

		return format.replace(token, function ($0)
		{
			var dd = $0 in flags ? flags[$0] : $0.slice(1, $0.length - 1);
			////Ice.Diagnostic.WriteLine($0 + ": " + dd);
			return dd;
		});
	}

	this.ToStringFormated = function (format)
	{
		return FormatTime(this._ticks, format || "HH:mm:ss.fff");
	};

	this.toString = function ()
	{
		return this.ToStringFormated();
	};

})();

