﻿/// <reference path="Ice.DomQuery.js" />

Ice.RegisterNs("Ice.Web.UI.Utility");

/* IceSlider html example:
	
		<div class="ice-scroller-container" style="position:relative; border:solid 10px silver; overflow:auto; width:200px; height:200px;">
			<div class="ice-slider-content" style="position:relative;background:yellow;width:1400px;top:0;left:0;">
				<%#DivText %>
			</div>
			<div class="ice-slider-y" style="display:none;border-radius:6px;position:absolute;right:5px;top:15px;bottom:15px;width:2px;background:red;">
				<div class="ice-marker" style="border-radius:6px;position:absolute;right:-3px;left:-3px;height:0;background:green;"></div>
			</div>
			<div class="ice-slider-x" style="display:none;border-radius:6px;position:absolute;right:15px;bottom:5px;left:15px;height:2px;background:red;">
				<div class="ice-marker" style="border-radius:6px;position:absolute;top:-3px;bottom:-3px;width:0;background:green;"></div>
			</div>
		</div>
*/
	
/* how to use it:
	Ice.Page.OnInteractive(function ()
	{
		Ice.Web.UI.Utility.IceSlider.Create(".ice-scroller-container");
	});
*/

Ice.Web.UI.Utility.IceSlider = new (function ()
{
	function IceSliderCtor(mainEl)
	{
		if (mainEl.__iceSlider)
		{
			return mainEl.__iceSlider;
		}

		mainEl.__iceSlider = this;

		var contentEl = $$ice(mainEl).Query("//.ice-slider-content").ApplyStyle("overflow", "hidden").First();

		var hSliderEl = $$ice(mainEl).Query("//.ice-slider-x").SetOpacity(0).ApplyStyle("display", "block").First();
		var hMarkerEl = $$ice(hSliderEl).Query("//.ice-marker").First();
		var hAR = mainEl.clientWidth / contentEl.offsetWidth;

		var vSliderEl = $$ice(mainEl).Query("//.ice-slider-y").SetOpacity(0).ApplyStyle("display", "block").First();
		var vMarkerEl = $$ice(vSliderEl).Query("//.ice-marker").First();
		var vAR = mainEl.clientHeight / contentEl.offsetHeight;

		$$ice(vSliderEl).SetOpacity(100).ApplyStyle("display", vAR < 1 ? "block" : "none");
		$$ice(hSliderEl).SetOpacity(100).ApplyStyle("display", hAR < 1 ? "block" : "none");

		hMarkerEl.style.width = Math.round(hSliderEl.offsetWidth * hAR) + "px";
		var maxMarkerLeft = hMarkerEl.parentNode.clientWidth - hMarkerEl.offsetWidth;

		vMarkerEl.style.height = Math.round(vSliderEl.offsetHeight * vAR) + "px";
		var maxMarkerTop = vMarkerEl.parentNode.clientHeight - vMarkerEl.offsetHeight;

		var accelerationAbort = false;
		var moveStatistics; // Array
		var markerDragging = false;
		var startCondition; // object

		var minContentTop;
		var minContentLeft;

		function CreateContentAndAdjustMarkers(top, left)
		{
			if (typeof (top) == "number")
			{
				minContentTop = mainEl.clientHeight - contentEl.offsetHeight; // nek svaki put izracuna, sigurnije je, jer se desavalo da ne izracuna dobro prvi put
				if (top > 0) { top = 0; }
				if (top < minContentTop) { top = minContentTop; }
				contentEl.style.top = top + "px";

				var markerTop = -1 * (contentEl.offsetTop / contentEl.offsetHeight * (vSliderEl.offsetHeight));
				vMarkerEl.style.top = Math.round(markerTop) + "px";
			}

			if (typeof (left) == "number")
			{
				minContentLeft = mainEl.clientWidth - contentEl.offsetWidth;
				if (left > 0) { left = 0; }
				if (left < minContentLeft) { left = minContentLeft; }
				contentEl.style.left = left + "px";

				var markerLeft = -1 * (contentEl.offsetLeft / contentEl.offsetWidth * (hSliderEl.offsetWidth));
				hMarkerEl.style.left = Math.round(markerLeft) + "px";
			}
		}

		function CreateMarkersAndAdjustContent(top, left)
		{
			if (typeof (left) == "number")
			{
				if (left < 0) { left = 0; }
				if (left > maxMarkerLeft) { left = maxMarkerLeft; }
				hMarkerEl.style.left = left + "px";
				contentEl.style.left = -(left / maxMarkerLeft) * (contentEl.offsetWidth - mainEl.clientWidth) + "px";
			}

			if (typeof (top) == "number")
			{
				if (top < 0) { top = 0; }
				if (top > maxMarkerTop) { top = maxMarkerTop; }
				vMarkerEl.style.top = top + "px";
				contentEl.style.top = -(top / maxMarkerTop) * (contentEl.offsetHeight - mainEl.clientHeight) + "px";
			}
		}

		function CreateSliderX(sender, ev)
		{
			if (hAR < 1)
			{
				ev = ev || {};
				var deltaX = 0;
				if ("deltaX" in ev)
				{
					deltaX = ev.deltaX;
				}
				else if ("wheelDeltaX" in ev)
				{
					deltaX = -(ev.wheelDeltaX);
				}

				// ev.deltaMode: 0=pixel, 1=line, 2=page
				// ev.axis: 1=horizontal, 2=vertical
				if (ev.deltaMode == 1)
				{
					deltaX *= 30;
				}

				var left = contentEl.offsetLeft - deltaX;

				CreateContentAndAdjustMarkers(null, left);
			}
		}

		function CreateSliderY(sender, ev)
		{
			if (vAR < 1)
			{
				ev = ev || {};
				var deltaY = 0;
				if ("deltaY" in ev)
				{
					deltaY = ev.deltaY;
				}
				else if ("wheelDeltaY" in ev)
				{
					deltaY = -(ev.wheelDeltaY);
				}

				// ev.deltaMode: 0=pixel, 1=line, 2=page
				// ev.axis: 1=horizontal, 2=vertical
				if (ev.deltaMode == 1)
				{
					deltaY *= 30;
				}

				var top = contentEl.offsetTop - deltaY;

				CreateContentAndAdjustMarkers(top, null);
			}
		}

		function CreateSlider(sender, ev)
		{
			CreateSliderX(sender, ev);
			CreateSliderY(sender, ev);
		}

		function CreateAnimator(sender, velocity, axisY)
		{
			accelerationAbort = false;
			var lastVal = 0;
			var animator = new Ice.Animator().AddValue(0, velocity, function (anim, currVal)
			{
				if (accelerationAbort)
				{
					anim.Abort();
				}
				else
				{
					var obj = {};
					obj[axisY ? "deltaY" : "deltaX"] = lastVal - currVal;
					if (axisY)
					{
						CreateSliderY(sender, obj);
					}
					else
					{
						CreateSliderX(sender, obj);
					}
					lastVal = currVal;
				}
			}).Easing(Ice.Animator.Easing.easeOutCirc).Duration(Math.abs(velocity)).Start();
		}

		function CalculateVelocity(sender, statArr)
		{
			var dt = new Date().getTime();
			var list = new Ice.List(statArr).Where(function (o) { return (dt - o.time) < 500; }).ToList(); // in last 500 ms

			var first = list.FirstOrDefault();
			var last = list.LastOrDefault();

			if (first && last && first.ev && last.ev && first.time && last.time)
			{
				return {
					deltaX: last.ev.pageX - first.ev.pageX,
					deltaY: last.ev.pageY - first.ev.pageX,
					velocityX: (last.ev.pageX - first.ev.pageX) / (last.time - first.time) * 1000 || 0, // pixels/second
					velocityY: (last.ev.pageY - first.ev.pageY) / (last.time - first.time) * 1000 || 0 // pixels/second
				};
			}
		}

		function HandleAcceleration(sender, statArr)
		{
			var velocityObj = CalculateVelocity(sender, statArr);
			if (!velocityObj)
			{
				return;
			}

			if (vAR < 1 && velocityObj.velocityY && Math.abs(velocityObj.deltaY || 0) > 10)
			{
				CreateAnimator(sender, velocityObj.velocityY, true);
			}

			if (hAR < 1 && velocityObj.velocityX && Math.abs(velocityObj.deltaX || 0) > 10)
			{
				CreateAnimator(sender, velocityObj.velocityX, false);
			}
		}
		
		function Initialize()
		{
			// document
			$$ice(document).On(["mouseup", "touchend"], function (sender, ev)
			{
				if (moveStatistics && !markerDragging)
				{
					HandleAcceleration(mainEl, moveStatistics);
				}
				moveStatistics = null;
				markerDragging = false;
				startCondition = null;
			});


			// hMarkerEl, vMarkerEl
			$$ice(hMarkerEl, vMarkerEl).On(["mousedown", "touchstart"], function (sender, ev)
			{
				markerDragging = sender;
			});

			$$ice(hSliderEl).On("click", function (sender, ev)
			{
				var offsetX = Ice.Event.GetOffsetToTarget(ev).offsetX;
				CreateMarkersAndAdjustContent(null, offsetX - hMarkerEl.offsetWidth / 2);
			});

			$$ice(vSliderEl).On("click", function (sender, ev)
			{
				var offsetY = Ice.Event.GetOffsetToTarget(ev).offsetY;
				CreateMarkersAndAdjustContent(offsetY - vMarkerEl.offsetHeight / 2, null);
			});

			var faderAnimator;

			// mainEl
			$$ice(mainEl).ApplyStyle("overflow", "hidden").ForEach(function (el, n)
			{
				// create initial slider and apply settings
				$$ice(hSliderEl, vSliderEl).SetOpacity(10);
				CreateSlider(el, null);

			}).On(["mouseover", "mousedown", "touchstart"], function (sender, ev)
			{
				if (faderAnimator) { faderAnimator.Abort(); }
				faderAnimator = $$ice(hSliderEl, vSliderEl).ToAnimator({ opacity: 100 }).Start();

			}).On(["mouseout", "touchend"], function (sender, ev)
			{
				if (faderAnimator) { faderAnimator.Abort(); }
				faderAnimator = $$ice(hSliderEl, vSliderEl).ToAnimator({ opacity: 10 }).Start();

			}).On(["scroll", "mousewheel", "wheel", "DOMMouseScroll"], function (sender, ev)
			{
				CreateSlider(sender, ev);
				return false; // stop propagation

			////}).On(["keydown"], function (sender, ev)
			////{
			////	// WRITE THE CODE: up, down, left, right, page up, page down

			}).On(["mousedown", "touchstart"], function (sender, ev)
			{
				if (ev.type == "touchstart")
				{
					ev = ev.touches[0];
				}
				moveStatistics = [];
				accelerationAbort = true;

				startCondition = {
					ev: ev,
					contentLeft: contentEl.offsetLeft,
					contentTop: contentEl.offsetTop,
					vMarkerTop: vMarkerEl.offsetTop,
					hMarkerLeft: hMarkerEl.offsetLeft
				};
				return false; // stop propagation

			}).On(["mousemove", "touchmove"], function (sender, ev)
			{
				if (!startCondition) { return; }

				var currEv = ev;
				if (ev.type == "touchmove")
				{
					currEv = ev.touches[0];
				}

				if (markerDragging == hMarkerEl)
				{
					var left = Math.round(startCondition.hMarkerLeft - (startCondition.ev.pageX - currEv.pageX));
					CreateMarkersAndAdjustContent(null, left);
				}
				else if (markerDragging == vMarkerEl)
				{
					var top = Math.round(startCondition.vMarkerTop - (startCondition.ev.pageY - currEv.pageY));
					CreateMarkersAndAdjustContent(top, null);
				}
				else if (!markerDragging)
				{
					// content dragging
					var top = Math.round(startCondition.contentTop - (startCondition.ev.pageY - currEv.pageY));
					var left = Math.round(startCondition.contentLeft - (startCondition.ev.pageX - currEv.pageX));

					CreateContentAndAdjustMarkers(top, left);
				}

				moveStatistics.push({ time: new Date().getTime(), ev: currEv });
				return false; // stop propagation

			});
		}

		Initialize();

	}

	this.Create = function (selector)
	{
		$$ice(selector).ForEach(function (o, n)
		{
			new IceSliderCtor(o);
		});
	};

})();