﻿//*******************************************************************
// NO EXTERNAL DEPENDENCIES
//*******************************************************************

// if code is minimized, can't have in "use strict" in eval(...). That is why "use strict" is set in IceScriptManager depending on compression
//"use strict";

if (typeof (Ice) == "undefined")
{
	// make sure Ice is not overwritten, eg. from imagine player boot js (playerunit edit page with preview, there is Ice from boot.js and full Ice engine included with page script)

	var Ice = new (function ()
	{
		var __ice = this;

		__ice.__class = "Ice";

		//*********************************************************
		__ice.IsDebugDefined = false; // has to be set on server if DEBUG defined
		//*********************************************************

		__ice.IsVSIntellisene = false;

		//#if VSINTELLISENSE
		__ice.IsVSIntellisene = typeof (intellisense) != "undefined";
		//#endif

		__ice.BindContext = function (method, context)
		{
			///<summary>Bind 'this' inside method to the context</summary>
			///<param name="method" type="Function">Method to bind</param>
			///<param name="context" type="Object">Context to bind to</param>

			if (!method || !context) { return method; }

			//#if VSINTELLISENSE
			if (Ice.IsVSIntellisene) { return method; } // return original method to keep intellisense including method argument names
			//#endif

			return function () { return method.apply(context, arguments); };
		};

		__ice.Extend = function (destinationObject, sourceObject, overwriteIfExist)
		{
			///<summary>Copy all properties from soruceObject to destinationObject</summary>
			///<param name="destinationObject" type="Object">Destination object.</param>
			///<param name="sourceObject" type="Object">Source object</param>
			///<param name="overwriteIfExist" type="Boolean">true to overwrite already existing properties in destinationObject</param>
			///<returns type="Object">Extended destinationObject</returns>
			for (var property in sourceObject)
			{
				if (overwriteIfExist || !(property in destinationObject))
				{
					destinationObject[property] = sourceObject[property];
				}
			}
			return destinationObject;
		};

		__ice.RegisterNs = function (ns1, ns2, ns3, etc)
		{
			///<summary>Creates namespaces to be used for scoping variables and classes so that they are not global</summary>
			///<param name="ns" type="String">example: ice.net.script, or "ice", "net", "script"</param>
			///<returns type="Object|Array|Null">JS object</returns>
			var current = null, i, j, parts;
			var w = window || {}; // window || {} for VS intellisense to work. It is required for strict mode: window[parts[0]] = {};
			for (i = 0; i < arguments.length; ++i)
			{
				parts = arguments[i].split(".");
				current = w[parts[0]]; // first part
				if (typeof (current) == "undefined")
				{
					w[parts[0]] = current = {};
				}
				for (j = 1; j < parts.length; ++j)
				{
					current[parts[j]] = current[parts[j]] || {};
					current = current[parts[j]];
				}
			}
		};

		__ice.ForEach = function (array, delegate, context)
		{
			///<summary></summary>
			///<param name="enumerable" type="Array|String"></param>
			///<param name="delegate" type="Function"> // delegate: function(item, index){...}</param>
			if (!array)
			{
				return false;
			}

			//#if VSINTELLISENSE
			if (Ice.IsVSIntellisene) { delegate.call(context, array[0], 1); }
			//#endif

			if (array.push || array.length >= 0 || array.charAt)
			{
				// enumerate array, dom array or string
				var useCharAt = typeof (array) == "string"; // some browsers and doctypes require charAt to read chars from string
				for (var nn = 0; nn < array.length; nn++)
				{
					delegate.call(context, useCharAt ? array.charAt(nn) : array[nn], nn); // don't use if return === false because it will break execution in chrome (return is false by default)
				}
			}
		};

		__ice.ForEachProperties = function (obj, delegate, context)
		{
			/// <summary>
			/// 
			/// </summary>
			/// <param name="obj"></param>
			/// <param name="delegate">function(propName, propValue, index){...}</param>
			/// <param name="context"></param>

			if (!obj)
			{
				return false;
			}

			//#if VSINTELLISENSE
			if (Ice.IsVSIntellisene) { delegate.call(context, "prop", {}, 1); }
			//#endif

			// enumerate object's properties
			var index = 0;
			try
			{
				for (var prop in obj)
				{
					try
					{
						delegate.call(context, prop, obj[prop], index++); // don't use if return === false because it will break execution in chrome (return is false by default)
					}
					catch (e) { }
				}
			}
			catch (e) { }
		};


		return __ice;

	})();

	/// <reference path="template:IS_DEBUG_DEFINED" />

	Ice.RegisterNs("Ice.Globalization.CultureInfo.CurrentCulture.NumberFormat");
	Ice.Globalization.CultureInfo.CurrentCulture.NumberFormat = {
		CurrencySymbol: "$",
		NumberDecimalSeparator: ".",
		NumberGroupSeparator: ",",
		PerMilleSymbol: "‰",
		PercentSymbol: "%"
	};

	/// <reference path="template:CULTURE_INFO" />

	// make Ice global:
	window.Ice = Ice;

}


