﻿using Ice.Utility;
using Ice.Web.Resources.Themes;
using Ice.Web.UI;
using Ice.Web.UI.Css;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Text.RegularExpressions;
using System.Web;
using System.Web.UI;

namespace Ice.Web.Script.Services
{
	public class FileCompiler
	{
		Dictionary<string, bool> _included;
		Dictionary<string, List<string>> _cssDicBlocks; // split css files by it's directories for relative includes inside of css to work
		List<string> _jsRows;

		Assembly[] _assemblies;
		IceScriptManager _manager;

		public string GetJavaScriptContent()
		{
			return _jsRows.ToStringJoined("\r\n");
		}

		public Dictionary<string, string> GetCssBlocks()
		{
			return _cssDicBlocks.ToDictionary(o => o.Key, o => o.Value.ToStringJoined("\r\n"));
		}

		public FileCompiler(IceScriptManager manager, Assembly[] assemblies)
		{
			_manager = manager;
			_assemblies = assemblies;
			_included = new Dictionary<string, bool>();
			_cssDicBlocks = new Dictionary<string, List<string>>();
			_jsRows = new List<string>();
			_cssDicBlocks.Add("~/", new List<string>());
		}

		private bool IsIncluded(string content)
		{
			return _included.ContainsKey(content);
		}

		private bool AllowOnce(string content)
		{
			if (IsIncluded(content))
			{
				return false;
			}
			else
			{
				_included.Add(content, true);
				return true;
			}
		}

		private void IncludeJsOnce(string content)
		{
			if (AllowOnce(content))
			{
				_jsRows.Add(content);
			}
		}

		private void IncludeCssOnce(string content)
		{
			if (AllowOnce(content))
			{
				_cssDicBlocks.First().Value.Add(content);
			}
		}

		private void AddComment(List<string> rowsList, string comment)
		{
			// comment which work in CSS and JS
			rowsList.Add("/*** FileCompiler: " + comment + " ***/\r\n");
		}

		private void StartRegion(List<string> rowsList, string name)
		{
			// comment which work in CSS and JS
			rowsList.Add("/*#region FileCompiler: " + name + "*/\r\n");
		}

		private void EndRegion(List<string> rowsList, string name)
		{
			// comment which work in CSS and JS
			rowsList.Add("\r\n/*#endregion FileCompiler: " + name + "*/\r\n");
		}

		private void RegisterJSCodeTemplate(string templateName)
		{
			if (AllowOnce("register_code_template_" + templateName))
			{
				StartRegion(_jsRows, "CODE TEMPLATE " + templateName);

				switch(templateName)
				{
					case "IS_DEBUG_DEFINED":
						if (IceWebSettings.IsDebugDefined)
						{
							IncludeJsOnce("Ice.IsDebugDefined=true;\r\n");
						}
						break;

					case "CULTURE_INFO":
						IncludeJsOnce("Ice.Globalization.CultureInfo.CurrentCulture.NumberFormat=" + Ice.Serialization.Json.IceJson.Serialize(System.Globalization.CultureInfo.CurrentCulture.NumberFormat) + ";\r\n");
						break;

					case "NET_GLOBAL_TIMEOUT":
						IncludeJsOnce("Ice.Net.WebRequest.Globals.timeoutMs=" + IceWebSettings.IceWebAjaxTimeoutMS + ";\r\n");
						break;

					case "SERVER_DIAGNOSTIC_URL":
						if (IceWebSettings.WriteIceWebElmahLog)
						{
							string url = (IceScriptManager.GetServerHttpRoot() + IceScriptManager.CreateElmahUrl()).Replace("/", "/\" + \"").Replace(".", ".\" + \"").Replace("&", "&\" + \""); // zakamuflirati url da google ne indeksira
							IncludeJsOnce("Ice.Diagnostic.ServerUrl=\"" + url + "\";\r\n");
						}
						break;

					case "DATE_TIME_LOCALIZATION":
						{
							var names = new
	{
		Days = System.Globalization.CultureInfo.CurrentCulture.DateTimeFormat.AbbreviatedDayNames.Select(ss => "\"" + ss + "\"").ToStringJoined(", "),
		DaysLong = System.Globalization.CultureInfo.CurrentCulture.DateTimeFormat.DayNames.Select(ss => "\"" + ss + "\"").ToStringJoined(", "),
		Months = System.Globalization.CultureInfo.CurrentCulture.DateTimeFormat.AbbreviatedMonthNames.Select(ss => "\"" + ss + "\"").ToStringJoined(", "),
		MonthsLong = System.Globalization.CultureInfo.CurrentCulture.DateTimeFormat.MonthNames.Select(ss => "\"" + ss + "\"").ToStringJoined(", "),
		FirstDayOfWeek = (int)System.Globalization.CultureInfo.CurrentCulture.DateTimeFormat.FirstDayOfWeek
	};

							string js = @"Ice.DateTime.dateFormat.i18n = {
		dayNames: [{Days}],
		dayNamesLong: [{DaysLong}],
		monthNames: [{Months}],
		monthNamesLong:[{MonthsLong}],
		firstDayOfWeek: {FirstDayOfWeek}
	};
".FormatWith(names);

							IncludeJsOnce(js);
						}
						break;

					default:
						throw new NotImplementedException(templateName + " is not defined as code template");
				}

				EndRegion(_jsRows, "CODE TEMPLATE " + templateName);
			}
		}

		public void RegisterEngine()
		{
			if (AllowOnce("register_ice_engine_"))
			{
				StartRegion(_jsRows, "ICE JS ENGINE");

				if (IceWebSettings.IsDebugDefined)
				{
					// can not use strict when code is packed or auto which will be packed anyway (tested!)
					_jsRows.Add("\"use strict\";\r\n");
				}

				// always register theme it it is engine
				StartRegion(_cssDicBlocks.First().Value, "ICE THEME " + _manager.Theme);
				IncludeCssOnce(ThemeHelper.GetTheme(_manager.Theme).ThemeCss);
				EndRegion(_cssDicBlocks.First().Value, "ICE THEME " + _manager.Theme);

				RegisterFile(".Ice.Engine.All.js");

				// add types after Ice. ns is registered because it uses Ice.RegisterNs()
				// register main types
				StartRegion(_jsRows, "REGISTER PRIMITIVE TYPES");

				_jsRows.Add("Ice.RegisterNs('System.String');");
				_jsRows.Add("System.String = function (value) { return new String(value || '');};");
				_jsRows.Add("");
				_jsRows.Add("Ice.RegisterNs('System.Int32');");
				_jsRows.Add("System.Int32 = function (value) { return new Number(value || 0);};");
				_jsRows.Add("");
				_jsRows.Add("Ice.RegisterNs('System.Boolean');");
				_jsRows.Add("System.Boolean = function(value){ return new Boolean(value || false);};");
				_jsRows.Add("");

				EndRegion(_jsRows, "REGISTER PRIMITIVE TYPES");

				this.RegisterType(typeof(DayOfWeek));

				if (IceWebSettings.IsDebugDefined)
				{
#if !NOSELFTEST
					RegisterFile(".Engine.Ice.DomQuery-SELFTEST.js");
					RegisterFile(".Engine.Ice.Utility-SELFTEST.js");
#endif
				}

				EndRegion(_jsRows, "ICE ENGINE");

			}
		}
		private string GetJQueryFileName()
		{
			switch (_manager.JQuery)
			{
				case jQueryVersion.jQuery_1_7_1:
					return ".jQuery.v1_7_1.jquery-1.7.1.js";
				case jQueryVersion.jQuery_1_10_2:
					return ".jQuery.v1_10_2.jquery-1.10.2.js";
				case jQueryVersion.jQuery_2_1_0:
					return ".jQuery.v2_1_0.jquery-2.1.0.js";
				default:
					return null;
			}
		}

		public void RegisterJQuery()
		{
			if (AllowOnce("register_jquery"))
			{
				string jqueryFile = GetJQueryFileName();
				if (jqueryFile != null)
				{
					StartRegion(_jsRows, "" + _manager.JQuery);

					// include jQuery $ redefinition protection
					_jsRows.Add(@"if (!(window && window.$ && ($.fn || $.css || $.filter || $.ajax || $.each)))
{
	// include jquery only if not already included, safety check to prevent $ redefinition

");
					// add jquery
					_jsRows.Add(Ice.Utility.EmbeddedHelper.GetFileText(jqueryFile, Assembly.GetExecutingAssembly()));

					_jsRows.Add(@"
}");
					EndRegion(_jsRows, "" + _manager.JQuery);
				}
			}
		}

		public void RegisterType(Type objType)
		{
			if (AllowOnce("register_type_" + objType.FullName))
			{
				RegisterEngine();

				object instance = null;
				bool isEnum = false;

				if (typeof(Enum).IsAssignableFrom(objType))
				{
					isEnum = true;
					instance = Enum.GetValues(objType).Cast<Enum>().ToDictionary(o => o.ToString(), o => (int)(object)o);
				}
				else if (objType == typeof(string))
				{
					instance = "";
				}
				else if (objType == typeof(byte[]))
				{
					instance = new byte[] { 1 };
				}
				else if (objType.FullName == "System.Void")
				{
					instance = null;
				}
				else
				{
					try
					{
						instance = Activator.CreateInstance(objType);
					}
					catch(Exception ex)
					{
						throw new Exception(ex.Message + " " + objType.FullName);
					}
				}

				StartRegion(_jsRows, "REGISTER TYPE " + objType.FullName);
				string fullName = Ice.Utility.IceTools.GetJavaScriptTypeName(objType); // eg. {Imagine.Web.Engine.Data.tblLogType+KindEnum}
				_jsRows.Add("Ice.RegisterNs('" + fullName + "');");
				
				if (isEnum)
				{
					_jsRows.Add(fullName + " = " + Ice.Serialization.Json.IceJson.Serialize(instance, IceWebSettings.IsDebugDefined) + ";");
				}
				else if(instance is string || instance is byte || instance is byte[] || objType.IsValueType)
				{
					object val = Ice.Serialization.Json.IceJson.Serialize(instance, IceWebSettings.IsDebugDefined);
					_jsRows.Add(fullName + " = function(value){ return value || " + val + ";};");
				}
				else
				{
					_jsRows.Add(fullName + " = function(){ return " + Ice.Serialization.Json.IceJson.Serialize(instance, IceWebSettings.IsDebugDefined) + ";};");
				}

				EndRegion(_jsRows, "REGISTER TYPE " + objType.FullName);
			}
		}

		public void RegisterWebService(string fileName, string appRelativeVirtualPath)
		{
			fileName = GetAppRelativeFilePath(fileName, appRelativeVirtualPath);

			if (AllowOnce("register_service_path_ " + fileName))
			{
				RegisterEngine();

				if (!File.Exists(System.Web.Hosting.HostingEnvironment.MapPath(fileName)))
				{
					throw new FileNotFoundException(fileName);
				}

				StartRegion(_jsRows, "WEB SERVICE FILE " + fileName);
				Type compiledType = System.Web.Compilation.BuildManager.GetCompiledType(fileName);

				_jsRows.Add(WebServiceHelper.GetWebServiceJS(new FileLocation() { FileCompiler = this, CompiledType = compiledType }));
				_jsRows.Add("\r\n");
				EndRegion(_jsRows, "WEB SERVICE FILE " + fileName);
			}
		}

		public void RegisterWebService(Type compiledType)
		{
			if (AllowOnce("register_service_type_ " + compiledType.FullName))
			{
				StartRegion(_jsRows, "WEB SERVICE TYPE " + compiledType.FullName);
				_jsRows.Add(WebServiceHelper.GetWebServiceJS(new FileLocation() { FileCompiler = this, CompiledType = compiledType }));
				_jsRows.Add("\r\n");
				EndRegion(_jsRows, "WEB SERVICE TYPE " + compiledType.FullName);
			}
		}

		private string GetAppRelativeFilePath(string fileName, string appRelativeVirtualPath)
		{
			appRelativeVirtualPath = VirtualPathUtility.GetDirectory(appRelativeVirtualPath ?? "~/");

			if (fileName.StartsWith("/"))
			{
				fileName = "~" + fileName;
			}

			// load physical file, may start with ~/, / or nothing

			if (fileName.StartsWith("~/"))
			{
				// "~/StockChart.js"
				return fileName;
			}
			else if (fileName.StartsWith("../"))
			{
				// "../StockChart.js"
				return appRelativeVirtualPath + fileName;
			}
			else if (Regex.IsMatch(fileName, "[\\w\\d_]+"))
			{
				// "Stockchart.js"
				return appRelativeVirtualPath + fileName;
			}

			throw new NotSupportedException(fileName);
		}

		static Regex _reReference = new Regex("<reference[\\s]+path=\"([^\"]*)\"[\\s]*/>");

		EmbeddedHelper.EmbeddedFileInfo[] _embeddedFiles = null;

		private void RegisterContent(string fileName, string appRelativeVirtualPath)
		{
			if(!AllowOnce("register_file_content_" + fileName + appRelativeVirtualPath))
			{
				// already included
				return;
			}

			string embedded = Ice.Utility.EmbeddedHelper.GetEmbeddedFilePath(fileName);

			if(_embeddedFiles == null)
			{
				_embeddedFiles = Ice.Utility.EmbeddedHelper.GetFilesInfo(this._assemblies);
			}

			var embeddedFileInfo = _embeddedFiles.Where(o => o.Name.EndsWith(embedded, StringComparison.OrdinalIgnoreCase)).SingleOrDefault();

			// IMPORTANT:
			// add embeddedFileInfo.Name as key since the same file may be included from diferent locations and different name length, eg via RegisterScriptFile() and via automated inclusion
			// eg. semafor.aspx.js and .admin.semafor.aspx.js is the same file, so use full length embedded file name as key to prevent double inclusion
			if(embeddedFileInfo != null && !AllowOnce("register_embedded_file_name_" +  embeddedFileInfo.Name))
			{
				// file is already included
				return;
			}

			// read embedded file first
			string txt = embeddedFileInfo != null ? embeddedFileInfo.FileText : null;

			appRelativeVirtualPath = appRelativeVirtualPath ?? "~/";

			string cssDirectory = null;

			// if not found, read file on disk
			if (string.IsNullOrEmpty(txt))
			{
				string path = GetAppRelativeFilePath(fileName, appRelativeVirtualPath);
				if (!AllowOnce("register_physical_file_" + path))
				{
					// already included
					return;
				}
				cssDirectory = VirtualPathUtility.GetDirectory(path);
				string absPath = path == null ? null : System.Web.Hosting.HostingEnvironment.MapPath(path);

				if (absPath != null && File.Exists(absPath))
				{
					txt = IceTools.RemoveUtfHeader(File.ReadAllText(absPath, Encoding.UTF8));
				}
			}

			if(string.IsNullOrEmpty(txt))
			{
				throw new FileNotFoundException(fileName + " is empty or not found");
			}

			bool isCss = fileName.EndsWith(".css");

			List<string> rowsList = _jsRows;
			if(isCss)
			{
				// split css files by it's directories for relative includes inside of css to work
				if(cssDirectory == null)
				{
					// embedded css
					rowsList = _cssDicBlocks.First().Value;
				}
				else if(_cssDicBlocks.ContainsKey(cssDirectory))
				{
					rowsList = _cssDicBlocks[cssDirectory];
				}
				else
				{
					_cssDicBlocks.Add(cssDirectory, (rowsList = new List<string>()));
				}
			}

			if (fileName.EndsWith(".ice.css"))
			{
				// preprocess css
				IceStyleSheet sheet = new IceStyleSheet();
				sheet.AddCss(txt);
				txt = sheet.ToCssString();
			}

			StartRegion(rowsList, "CONTENT FROM FILE " + fileName);

			if (this._manager.SkipReferences)
			{
				rowsList.Add(txt);
			}
			else
			{
				foreach (string row in txt.Split('\n').Select(o => o.Trim('\r')))
				{
					// read rows and search for includes with <reference path=""/>
					Match m = _reReference.Match(row);
					if (m.Success)
					{
						string str = m.Groups[1].Value;
						if (str.StartsWith("template:"))
						{
							RegisterJSCodeTemplate(str.Substring("template:".Length));
						}
						else
						{
							RegisterFile(m.Groups[1].Value, cssDirectory ?? appRelativeVirtualPath);
						}
					}
					else
					{
						rowsList.Add(row);
					}
				}
			}

			EndRegion(rowsList, "CONTENT FROM FILE " + fileName);
		}

		public void RegisterContent(string content, bool isCss, string blockName)
		{
			if (AllowOnce(content))
			{
				List<string> rows = isCss ? _cssDicBlocks.First().Value : _jsRows;

				if(blockName != null)
				{
					StartRegion(rows, "CONTENT BLOCK " + blockName);
				}

				rows.Add(content);

				if (blockName != null)
				{
					EndRegion(rows, "CONTENT BLOCK " + blockName);
				}
			}
		}

		public void RegisterFile(string fileName, string appRelativeVirtualPath = null)
		{
			if (fileName.Contains(".compiled.") || fileName.Contains(".intellisense."))
			{
				return;
			}

			if (_manager.Engine)
			{
				RegisterEngine();
			}

			string extension = (Path.GetExtension(fileName) ?? "").ToLower();

			switch (extension)
			{
				case ".asmx":
				case ".aspx":
				case ".ascx":
				case ".master":
					RegisterWebService(fileName, appRelativeVirtualPath);
					break;
				case ".js":
				case ".css":
					RegisterContent(fileName, appRelativeVirtualPath);
					break;
				default:
					throw new NotSupportedException(fileName);
			}

		}

	}
}
