﻿using System;
using System.Web;

namespace Ice.Web.Script.Services
{

	class WsjsHandlerFactory : IHttpHandlerFactory
	{
		public virtual IHttpHandler GetHandler(HttpContext context, string requestType, string url, string pathTranslated)
		{
			if (context == null)
			{
				throw new ArgumentNullException("context is null");
			}


			// pazi: u json-u moze biti ICE-SC, zato prvo gledati ima li u qstr icewm
			// pazi: google bot mora moci ucitati css ili js code, ali ne dozvoliti requeste na web servis ili exception report

			string ua = context.Request.UserAgent ?? "";
			bool isBot = ua.Contains("bot") || ua.Contains("crawl");
			string path = context.Request.Path;

			if (!isBot && context.Request.Headers["x-ice-ajax"] == "ice-exception" && path.Contains(IceAjaxHandler.IceElmahRequest, true))
			{
				// test for x-ice-ajax header to avoid request from spiders
				// POST, must contain header x-ice-ajax
				// contains ICE-EX
				return new IceElmahErrorHandler();
				
			}
			else if (!isBot && context.Request.Headers["x-ice-ajax"] == "ice-ajax" && !string.IsNullOrEmpty(context.Request.QueryString["icewm"]) && path.Contains(IceAjaxHandler.IceTypeServiceRequest, true))
			{
				// test for x-ice-ajax header to avoid request from spiders
				// accepts cross-domain requests, crossdomain request can't add custom headers
				// WebMethod request
				if (context.Request.QueryString["icess"] == "1")
				{
					return new WsjsTypeHandlerWithSession();
				}
				else
				{
					return new WsjsTypeHandler();
				}
			}
			else if (path.Contains(IceAjaxHandler.IceScriptResourceRequest, true) && context.Request.Url.Query.Length > 3)
			{
				// allow both bot and non-bot
				// warning: google bot must be able to get css or js code
				// contains ICE-SC in url
				return new WsjsClientProxyHandler();
			}
			else if (isBot)
			{
				// eg. google bot
				return new BotHandler();
			}

			////string dump = Ice.Web.RequestDumper.GetRequestDump(context);
			////Ice.Elmah.ErrorLog.WriteLog(new Exception(dump));

			return new NotFoundHandler();

			// requires full trust security:
			//return new System.Web.Services.Protocols.WebServiceHandlerFactory().GetHandler(context, requestType, url, pathTranslated);
		}

		public virtual void ReleaseHandler(IHttpHandler handler)
		{
		}
	}

	class BotHandler : IHttpHandler
	{
		public void ProcessRequest(HttpContext context)
		{
			context.Response.Cache.SetCacheability(HttpCacheability.NoCache);
			context.Response.Cache.SetExpires(DateTime.Now.AddDays(-100));

			context.Response.Write("[hello]");
		}

		public bool IsReusable { get { return true; } }
	}

	class NotFoundHandler : IHttpHandler
	{
		public void ProcessRequest(HttpContext context)
		{
			context.Response.Cache.SetCacheability(HttpCacheability.NoCache);
			context.Response.Cache.SetExpires(DateTime.Now.AddDays(-100));

			context.Response.StatusCode = (int)System.Net.HttpStatusCode.NotFound; // 404 not found
		}

		public bool IsReusable { get { return true; } }
	}


}
