﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Ice.Web.Extensions;
using System.Text.RegularExpressions;
using Ice.Web.Script.Services;
using System.Web.UI.WebControls;
using System.Web.UI;
using Ice.JavaScript.Packer;
using System.Linq.Expressions;

namespace Ice.Web.UI.Css
{
	public class IceStyleSheet : IIceCssItem
	{
		/* extension .ice.css
		 * define variables with 2 underscores, variables will be replaced in css:
		__transitionDuration = 400ms;
		__menuBlueDark = #0097CF;
		__menuBlueMiddle = #07A1D9;
		__menuBlueLight = #00ADEE;
		__menuGrayDark = #B0B0B0;
		 
		 * inheritance, use __apply(className or selector) inside other class body, it will add content from className or selector:
		 * .className{font-size:10px;}
		 * .class1{color:red;}
		 * input {
		 *			__apply(.className, .class1, class3);
		 *			margin:10px;
		 *			padding:10px;
		 *	}
		 * 
		 */

		// ice variables: __name = ...;
		static Regex _iceVariables = new Regex("(?<name>((__)|([\\$][\\$]))[\\w]+)[\\s]*\\=[\\s]*(?<value>[^\\r\\n\\;]*);");
		
		// ice functions: __apply(...);
		static Regex _iceFuncs = new Regex("(?<func>((__)|([\\$][\\$]))[\\w]+)[\\s]*[\\(][\\s]*(?<value>[^\\(\\)]*)[\\s]*[\\)][\\s]*;");
		
		private static List<IceCssClass> ParseCssCode(string cssCode)
		{
			// pack it to remove all comments
			string minimizedCss = IcePacker.Pack(cssCode, CompressionLevel.Minify, true);

			// extract template variables and replace in whole text
			var matches = _iceVariables.Matches(cssCode).Cast<Match>().ForEach(o=>
			{
				minimizedCss = minimizedCss.Replace(o.Groups["name"].Value, o.Groups["value"].Value);
			});

			List<IceCssClass> list = new List<IceCssClass>();

			// parse style values
			new Regex("([^{};\\(\\}]*){([^}]*)}").Matches(minimizedCss).Cast<Match>().ForEach(m =>
			{
				string names = m.Groups[1].Value;
				string values = m.Groups[2].Value;
				var matches1 = _iceFuncs.Matches(values).Cast<Match>().ToList();
				// extract __apply(...) and replace with classes styles which aready found
				matches1.ForEach(fn=>{
					//string funcName = fn.Groups["func"].Value;
					List<string> clsnames = fn.Groups["value"].Value.Split(',').Select(o => o.Trim()).Where(o => !string.IsNullOrEmpty(o)).ToList();
					// search already existing classes and make replacement
					var found = list.Where(o=>clsnames.Any() && o.ClassNames.Intersect(clsnames, a=>a, b=>b).Any()).ToList();
					if(!found.Any())
					{
						throw new Exception("No previous definition for any of class names: " + clsnames.ToStringJoined(", "));
					}
					else
					{
						found.ForEach(k=>
						{
							values = values.Replace(fn.Value, found.Select(h=>h.GetClassStyleString()).ToStringJoined());
						});
					}
				});

				// if one class has multiple names: .myone, div.tree, a:hover { ... }
				list.Add(new IceCssClass(names.Split(',').Select(o => o.Trim()).Where(o => !string.IsNullOrEmpty(o)).ToList(),
					new IceStyleCollection(ControlExtensions.ParseStyleValues(values)))
					);
			});

			return list;
		}

		private List<IIceCssItem> _items = new List<IIceCssItem>();

		public IEnumerable<IIceCssItem> AsEnumerable()
		{
			return _items.AsEnumerable();
		}

		public IceStyleSheet AddCss(string cssCode)
		{
			List<IceCssClass> parsedCss = ParseCssCode(cssCode);
			parsedCss.ForEach(o => _items.Add(o));
			return this;
		}

		public IceStyleSheet Add(params IIceCssItem[] items)
		{
			items.ForEach(o => _items.Add(o));
			return this;
		}

		public string ToCssString()
		{
			return _items.Select(o => o.ToCssString()).Where(o => !string.IsNullOrEmpty(o)).ToStringJoined("\r\n\r\n");
		}

		public static IceStyleSheet Parse(string cssCode)
		{
			return new IceStyleSheet().AddCss(cssCode);
		}

		/// <summary>
		/// All items, including items inside MediaRules.
		/// </summary>
		/// <returns></returns>
		public IEnumerable<IIceCssItem> Items()
		{
			foreach (IIceCssItem o in _items)
			{
				if (o is IceMediaRule)
				{
					foreach (IIceCssItem c in ((IceMediaRule)o).Items())
					{
						yield return c;
					}
				}
				else
				{
					yield return o;
				}
			}
		}

		public IceStyleSheet Clone()
		{
			return new IceStyleSheet() { _items = this._items.Select(o => o.Clone()).ToList() };
		}


		IIceCssItem IIceCssItem.Clone()
		{
			return this.Clone();
		}

		/// <summary>
		/// May return multiple items
		/// </summary>
		/// <param name="className"></param>
		/// <returns></returns>
		public IEnumerable<IceCssClass> FindByClassName( string className)
		{
			return IceCssClass.SplitClassNames(className)
				.SelectMany(cls => this.Items().OfType<IceCssClass>().Where(o => o.ClassNames.Contains(cls)));
		}

		public IceStyleSheet NewCssClass(string className, string baseClass, Action<IceCssClass> c) 
		{
			IceCssClass cls = new IceCssClass(className);
			this.FindByClassName(baseClass).ForEach(o => cls.ApplyClass(o));
			this.Add(cls);
			if (c != null)
			{
				c(cls);
			}
			return this;
		}

		public IceStyleSheet NewCssClass(string className, Action<IceCssClass> c) 
		{
			IceCssClass cls = new IceCssClass(className);
			this.Add(cls);
			c(cls);
			return this;
		}

		public IceStyleSheet NewPlainText(string text) 
		{
			IcePlainCssText p = new IcePlainCssText(text);
			this.Add(p);
			return this;
		}

		public IceStyleSheet NewMediaRule(string mediaDesc, Action<IceMediaRule> m) 
		{
			IceMediaRule rule = new IceMediaRule(mediaDesc);
			this.Add(rule);
			m(rule);
			return this;
		}

		public IceStyleSheet NewMediaScreenRule(Expression<Func<IceMediaRule.MediaScreenSettings, bool>> predicate, Action<IceMediaScreenRule> m) 
		{
			IceMediaScreenRule rule = new IceMediaScreenRule(predicate);
			this.Add(rule);
			m(rule);
			return this;
		}
	}
}
