﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.UI;
using System.IO;
using System.Web.UI.WebControls;
using Ice.Web.Extensions;
using Ice.Web.Resources;

namespace Ice.Web.UI.WebControls
{
	[ParseChildren(false)]
	[PersistChildren(true)]
	public class IceButton : System.Web.UI.WebControls.Button
	{
		protected override string TagName
		{
			get { return "button"; }
		}

		protected override HtmlTextWriterTag TagKey
		{
			get { return HtmlTextWriterTag.Button; }
		}

		protected override void RenderContents(HtmlTextWriter writer)
		{
			base.RenderChildren(writer);
		}

		/// <summary>
		/// Confirmation text.
		/// </summary>
		public string Confirm{get;set;}

		public IconEnum Icon { get { return (IconEnum)(this.ViewState["_ic"] as int?).GetValueOrDefault(); } set { this.ViewState["_ic"] = (int)value; } }
		public IceLabel.IconPositionEnum IconPosition { get { return (IceLabel.IconPositionEnum)(this.ViewState["_ip"] as int?).GetValueOrDefault(); } set { this.ViewState["_ip"] = (int)value; } }
		public string HoverStyle { get { return this.ViewState["_hs"] as string; } set { this.ViewState["_hs"] = value; } }
		public string HoverCssClass { get { return this.ViewState["_hc"] as string; } set { this.ViewState["_hc"] = value; } }

		public override void RenderControl(HtmlTextWriter writer)
		{
			if (!this.Visible)
			{
				return;
			}

			if (string.IsNullOrEmpty(this.Style["cursor"]))
			{
				this.Style["cursor"] = "pointer";
			}

			if (!string.IsNullOrEmpty(this.Text))
			{
				this.Controls.Add(new LiteralControl(this.Text));
			}

			this.AddClass("iceButton");

			if (this.Icon != IconEnum.None)
			{
				var children = this.Controls.Cast<Control>().ToArray();
				this.Controls.Clear();
				this.AddChildren(new Panel().Set(o => o.Attributes["align"] = "center").ApplyStyle("white-space:nowrap")
					.AddChildren(new IceLabel() { Icon = this.Icon, IconPosition = this.IconPosition }
						.AddChildren(children)));
			}


			if (!string.IsNullOrEmpty(this.Confirm))
			{
				this.OnClientClick = "if(!confirm('" + this.Confirm + "')){return false;} " + this.OnClientClick;
			}

			if (!string.IsNullOrEmpty(this.HoverStyle))
			{
				this.AppendJsEvent(JavascriptEventEnum.MouseOver, "this.style.cssText+='" + this.HoverStyle + "';");
				this.AppendJsEvent(JavascriptEventEnum.MouseOut, "this.style.cssText+='" + this.Style.Value + "';");
			}

			if (!string.IsNullOrEmpty(this.HoverCssClass))
			{
				this.AppendJsEvent(JavascriptEventEnum.MouseOver, "this.className='" + this.HoverCssClass + "';");
				this.AppendJsEvent(JavascriptEventEnum.MouseOut, "this.className='" + this.CssClass + "';");
			}

			base.RenderControl(writer);
		}
	

	}
}
