﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.UI;
using Ice.Web.Script.Services;
using System.Web.UI.WebControls;
using Ice.Web.Extensions;
using System.Text.RegularExpressions;
using System.Web.UI.HtmlControls;
using System.ComponentModel;
using System.Drawing;
using Ice.Colorspace;

namespace Ice.Web.UI.WebControls
{
	[ParseChildren(true)]
	[PersistChildren(true)]
	public partial class IceHorizontalSplit : IcePanel
	{
		public class HSContentStyle
		{
			public Unit MinHeight { get; set; }
			public Unit MaxHeight { get; set; }
			/// <summary>
			/// initial height
			/// </summary>
			public Unit Height { get; set; }
			public string CssClass { get; set; }
			public Unit BorderWidth { get; set; }

			[DefaultValue(typeof(Color), "")]
			[TypeConverter(typeof(WebColorConverter))]
			public virtual Color BorderColor { get; set; }

			public string Style { get; set; }
		}

		public class HorSplitStyle
		{
			/// <summary>
			/// split bar height
			/// </summary>
			public Unit Height { get; set; }
			/// <summary>
			/// if true, panels are not resizable
			/// </summary>
			public bool Static { get; set; }

			[DefaultValue(typeof(Color), "")]
			[TypeConverter(typeof(WebColorConverter))]
			public virtual Color HoverColor { get; set; }
		}

		[PersistenceMode(PersistenceMode.InnerProperty)]
		public ITemplate TopContent { get; set; }

		[PersistenceMode(PersistenceMode.InnerProperty)]
		public ITemplate BottomContent { get; set; }

		[PersistenceMode(PersistenceMode.InnerProperty)]
		public ITemplate SplitContent { get; set; }

		[PersistenceMode(PersistenceMode.InnerProperty)]
		public HorSplitStyle SplitStyle { get; set; }

		[PersistenceMode(PersistenceMode.InnerProperty)]
		public HSContentStyle TopStyle { get; set; }

		[PersistenceMode(PersistenceMode.InnerProperty)]
		public HSContentStyle BottomStyle { get; set; }

		public Unit FrameWidth { get; set; }

		[DefaultValue(typeof(Color), "")]
		[TypeConverter(typeof(WebColorConverter))]
		public virtual Color FrameColor { get; set; }

		Panel topContainer = new Panel().ApplyStyle("position:absolute;top:0;left:0;bottom:0;right:0;overflow:hidden;;");
		Panel splitBarContainer = new Panel().ApplyStyle("position:absolute;top:0;left:0;bottom:0;right:0;auto;background-color:#D9E5F3; overflow:hidden;");
		Panel bottomContainer = new Panel().ApplyStyle("position:absolute;top:0;left:0;bottom:0;right:0;overflow:hidden;");

		Panel topInnerContainer = new Panel().ApplyStyle("position:absolute;top:0;left:0;bottom:0;right:0;overflow:auto;");
		Panel bottomInnerContainer = new Panel().ApplyStyle("position:absolute;top:0;left:0;bottom:0;right:0;overflow:auto;");

		protected override void OnInit(EventArgs e)
		{
			this.AddChildren
			(
				topContainer.AddChildren(topInnerContainer.AddChildren(this.TopContent)),
				splitBarContainer.AddChildren(this.SplitContent),
				bottomContainer.AddChildren(bottomInnerContainer.AddChildren(this.BottomContent))
			);

			if (this.Page != null && !this.Page.IsInAsyncPostBack())
			{
				// script has to be loaded always, even if panel is not currently visible. It may be set visible later in UpdatePanel
				IceScriptManager.GetCurrent(this).RegisterScriptFile(this, ".IceHorizontalSplit.IceHorizontalSplit.js");
			}

			base.OnInit(e);
		}

		private void SetupControls()
		{
			IceScriptManager manager = IceScriptManager.GetCurrent(this.Page);

			HorSplitStyle splitStyle = this.SplitStyle ?? new HorSplitStyle();
			HSContentStyle topStyle = this.TopStyle ?? new HSContentStyle();
			HSContentStyle bottomStyle = this.BottomStyle ?? new HSContentStyle();

			topInnerContainer.BorderColor = topStyle.BorderColor.IsEmpty ? System.Drawing.ColorTranslator.FromHtml(manager.CurrentTheme.FrameBorderColor) : topStyle.BorderColor;
			topInnerContainer.BorderWidth = topStyle.BorderWidth;
			topInnerContainer.ApplyStyle(topStyle.Style);
			topInnerContainer.AddClass(topStyle.CssClass);

			bottomInnerContainer.BorderColor = bottomStyle.BorderColor.IsEmpty ? System.Drawing.ColorTranslator.FromHtml(manager.CurrentTheme.FrameBorderColor) : bottomStyle.BorderColor;
			bottomInnerContainer.BorderWidth = bottomStyle.BorderWidth;
			bottomInnerContainer.ApplyStyle(bottomStyle.Style);
			bottomInnerContainer.AddClass(bottomStyle.CssClass);

			string frameColor = this.FrameColor.IsEmpty ? manager.CurrentTheme.FrameColor : this.FrameColor.ToHtml();
			//set from theme
			splitBarContainer.Style["background-color"] = frameColor;
			splitBarContainer.Height = splitStyle.Height;

			this.ApplyStyle("position:absolute;top:0;left:0;bottom:0;right:0;");
			this.ApplyStyle("border:solid " + (this.FrameWidth.IsEmpty ? "0" : this.FrameWidth.ToString()) + " " + (frameColor));

			if (topStyle.Height.IsEmpty)
			{
				if (bottomStyle.Height.IsEmpty)
				{
					throw new Exception("ContainerHeight has to be defined for TopPanel or BottomPanel");
				}
				else
				{
					// bottom height is defined
					splitBarContainer.AddClass("toBottom");
					splitBarContainer.Style["bottom"] = bottomStyle.Height.ToString();
					splitBarContainer.Style["top"] = "auto";

					bottomContainer.Style["height"] = bottomStyle.Height.ToString();
					bottomContainer.Style["top"] = "auto";
					topContainer.Style["bottom"] = (splitStyle.Height.Value + bottomStyle.Height.Value) + "px";
				}
			}
			else
			{
				if (!bottomStyle.Height.IsEmpty)
				{
					throw new Exception("ContainerHeight can't be defined for both panels");
				}
				else
				{
					// top height is defined
					splitBarContainer.AddClass("toTop");
					splitBarContainer.Style["top"] = topStyle.Height.ToString();
					splitBarContainer.Style["bottom"] = "auto";

					topContainer.Style["height"] = topStyle.Height.ToString();
					topContainer.Style["bottom"] = "auto";
					bottomContainer.Style["top"] = (splitStyle.Height.Value + topStyle.Height.Value) + "px";
				}
			}

			if (!splitStyle.Static)
			{
				this.Page.ExecuteJavaScript(string.Format("Ice.Web.UI.WebControls.IceHorizontalSplit.Setup('{0}', '{1}', '{2}', {3}, {4}, {5}, {6})",
					splitBarContainer.ClientID,
					frameColor,
					splitStyle.HoverColor.IsEmpty ? manager.CurrentTheme.SplitBarHoverColor : splitStyle.HoverColor.ToHtml(),
					(int)topStyle.MinHeight.Value,
					(int)(topStyle.MaxHeight.IsEmpty ? int.MaxValue : topStyle.MaxHeight.Value),
					(int)bottomStyle.MinHeight.Value,
					(int)(bottomStyle.MaxHeight.IsEmpty ? int.MaxValue : bottomStyle.MaxHeight.Value)
					));
			}
		}

		protected override void OnPreRender(EventArgs e)
		{

			SetupControls();

			base.OnPreRender(e);

		}

	}
}