﻿using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Globalization;
using System.Security.Permissions;
using System.Web;
using System.Web.UI;
using System.Web.Util;
using Ice.Web.UI.WebControls;
using System.Linq;
using Ice.Web.Extensions;
using System.Collections.Generic;
using System.Text.RegularExpressions;
using Ice.Web.UI;
using System.Web.UI.WebControls;
using System.Reflection;

namespace Ice.Web.UI.WebControls
{
	[ParseChildren(true)]
	public class IceTreeView : IcePanel
	{
		public class IceTreeNode
		{
			public string Name { get; set; }
			public string NodeID { get; set; }
			public string ParentNodeID { get; set; }
			public int? Ordinal { get; set; }
			public bool? IsLeaf { get; set; }
			public bool? ShowIcon { get; set; }
			public bool? ShowCheckbox { get; set; }
			public bool? IsChecked { get; set; }
			public bool? HidePlusMinus { get; set; }
			public bool? NodeDraggable { get; set; }
			public bool? ShouldExpand { get; set; }
			public bool? ShowMenu { get; set; }
			public bool? NodeSelectable { get; set; }
			public bool? HasParent { get; set; }
			public string ChkTitle { get; set; }
			public string ChkValue { get; set; }
			////public string ChkLabelText { get; set; }
			////public string NameTitle { get; set; }
			public List<IceTreeNode> Children { get; set; }
			public string IconUrl { get; set; }
			public bool? NodeAcceptsDrop { get; set; }
			public bool? SeparatorAcceptsDrop { get; set; }
			public bool? CopyDroppedNode { get; set; }
			public bool? MayDropToNode { get; set; }
			public bool? MayDropToSeparator { get; set; }
		}

		public delegate List<IceTreeNode> GetChildrenDelegate(string parentID);
		public delegate bool SaveNodesDelegate(List<IceTreeNode> nodes);
		public delegate IceTreeNode NewNodeDelegate(IceTreeNode node);

		public event GetChildrenDelegate GetChildren;
		public event SaveNodesDelegate SaveNodes;
		public event SaveNodesDelegate DeleteNodes;
		public event NewNodeDelegate NewNode;

		/// <summary>
		/// Javascript function name to execute
		/// </summary>
		public string OnChildrenLoadedHandler { get; set; }

		/////// <summary>
		/////// JavaScript handler, example: function(node){....}
		/////// </summary>
		////public string NodeClickHandler { get; set; }

		private void AddJsHandler(MethodInfo mi, string localMethodName, List<string> eventHandlers)
		{
			string jsMethod = "{ClientID}_tree_instance." + localMethodName;
			// if web service method is not defined or doesn't have WebMethod attribute, use default tree method which popups alert warning that method is not defined
			eventHandlers.Add(jsMethod + " = " + mi.DeclaringType.FullName + "." + mi.Name + " || " + jsMethod + ";");
		}

		private void RegisterEvents(IceScriptManager manager, List<string> eventHanlders)
		{
			// page with methods has to be registered using IceScriptManager

			if (GetChildren != null)
			{
				IceScriptManager.EnsureIceWebMethodAttribute(GetChildren.Method);
				manager.RegisterPageScriptMethods(GetChildren.Method.DeclaringType);
				AddJsHandler(GetChildren.Method, "GetChildren", eventHanlders);
			}

			if (SaveNodes != null)
			{
				IceScriptManager.EnsureIceWebMethodAttribute(SaveNodes.Method);
				manager.RegisterPageScriptMethods(SaveNodes.Method.DeclaringType);
				AddJsHandler(SaveNodes.Method, "SaveNodes", eventHanlders);
			}

			if (NewNode != null)
			{
				IceScriptManager.EnsureIceWebMethodAttribute(NewNode.Method);
				manager.RegisterPageScriptMethods(NewNode.Method.DeclaringType);
				AddJsHandler(NewNode.Method, "NewNode", eventHanlders);
			}

			if (DeleteNodes != null)
			{
				IceScriptManager.EnsureIceWebMethodAttribute(DeleteNodes.Method);
				manager.RegisterPageScriptMethods(DeleteNodes.Method.DeclaringType);
				AddJsHandler(DeleteNodes.Method, "DeleteNodes", eventHanlders);
			}
		}

		protected override void OnInit(EventArgs e)
		{
			base.OnInit(e);
		}

		protected override void OnPreRender(EventArgs e)
		{
			base.OnPreRender(e);

			if(!this.Visible)
			{
				return;
			}

			IceScriptManager manager = IceScriptManager.GetCurrent(this.Page);
			string css = Ice.Utility.EmbeddedHelper.GetFileText(".IceTreeView.treeviewres.treeview.css");
			string fixedCss = new Regex("url\\(([^\\)]+)\\)").Replace(css, (m) =>
			{
				return "url(" + IceScriptManager.CreateUrl(".IceTreeView.treeviewres." + m.Groups[1].Value.Replace("/", ".")) + ")";
			});

			manager.RegisterScriptFile(this, ".utility.IceDragger.js");
			manager.RegisterScriptFile(this, ".utility.IceMenu.js");
			manager.RegisterCssCode(fixedCss, "IceTreeView.treeviewres.treeview.css");
			manager.RegisterScriptFile(this, ".IceTreeView.treeviewres.IceTreeContextMenu.js");
			manager.RegisterScriptFile(this, ".IceTreeView.treeviewres.IceTreeNode.js");
			manager.RegisterScriptFile(this, ".IceTreeView.treeviewres.IceTreeView.js");

			List<string> eventHanlders = new List<string>();
			RegisterEvents(manager, eventHanlders);

			// don't create new treeview on ajax posback
			string js = (@"if(typeof({ClientID}_tree_instance)=='undefined'){
var {ClientID}_tree_instance = new Ice.Web.UI.WebControls.IceTreeView('{ClientID}');
" + eventHanlders.ToStringJoined("\r\n") + @"
{ClientID}_tree_instance.OnChildrenLoaded.Add({OnChildrenLoadedHandler});}
{ClientID}_tree_instance.Expand();
").FormatWith(
				new
				{
					ClientID = this.ClientID,
					OnChildrenLoadedHandler = this.OnChildrenLoadedHandler ?? "function(){}"
				}); 

			this.Page.ExecuteJavaScript(js);


		}


	}
}
