﻿
Ice.Web.UI.WebControls.Dragger = new (function ()
{
	var _instance = new (function ()
	{
		this.classDraggable = "draggable";
		this.classDraggCatch = "draggCatch";
		this.classDropTarget = "dropTarget";
		this.draggableElement = null;
		this.dropTarget = null;
		this.mStartX = 0;
		this.mStartY = 0;
		this.draggZIndex = 0; // global

		// status element
		this.draggStatusEl = document.createElement("div");
		this.draggStatusEl.className = "draggstatus delete";
		this.draggStatusEl.style.display = "none";
		document.documentElement.appendChild(this.draggStatusEl);

		function StartDragging(element, event)
		{
			StopDragging.call(this);

			if (element && Ice.Utility.Dom.HasClass(element, this.classDraggable))
			{
				this.draggableElement = element;
				this.mStartX = event.clientX;
				this.mStartY = event.clientY;
				this.draggStatusEl.style.zIndex = element.style.zIndex = ++this.draggZIndex;
				document.body.style.cursor = "pointer";
			}
		};

		function StopDragging()
		{
			this.dropTarget = null;
			this.draggStatusEl.style.display = "none";
			document.body.style.cursor = "";
			if (this.draggableElement)
			{
				this.draggableElement.style.position = "";  // remove fixed
				this.draggableElement.style.left = "";
				this.draggableElement.style.top = "";
				Ice.Utility.Dom.RemoveClass(this.draggableElement, "dragging");
			}
			this.draggableElement = null;
		};

		function onMouseMoveHandler(event)
		{
			if (this.draggableElement)
			{
				if ((Math.abs(this.mStartX - event.clientX) > 5 || Math.abs(this.mStartY - event.clientY) > 5)) // minimum 5 pixel is required to dragg before object is moved
				{
					this.draggStatusEl.style.display = "block";
					this.draggableElement.style.position = this.draggStatusEl.style.position = "fixed";
					this.draggStatusEl.style.left = this.draggableElement.style.left = event.clientX + "px";
					this.draggStatusEl.style.top = this.draggableElement.style.top = event.clientY + "px";
					Ice.Utility.Dom.AddClass(this.draggableElement, "dragging");
				}
			}
		};

		function onMouseUpHandler(event)
		{
			if (this.draggableElement)
			{
				if (this.dropTarget)
				{
					this.dropTarget.OnDrop.call(this.dropTarget, this.dropTarget, this.draggableElement, event);
				}
			}

			StopDragging.call(this);
		}

		function ShowStatusDropValid(isValid)
		{
			if (isValid)
			{
				Ice.Utility.Dom.RemoveClass(_instance.draggStatusEl, "delete");
				Ice.Utility.Dom.AddClass(_instance.draggStatusEl, "accept");
			}
			else
			{
				Ice.Utility.Dom.RemoveClass(_instance.draggStatusEl, "accept");
				Ice.Utility.Dom.AddClass(_instance.draggStatusEl, "delete");
			}
		};

		var _mouseEventInitialized = false;

		this.Initialize = function (element)
		{
			if (!_mouseEventInitialized)
			{
				_mouseEventInitialized = true;
				Ice.Event.Attach(document, "mousemove", function (event) { onMouseMoveHandler.call(this, event || window.event); }.BindContext(_instance));
				Ice.Event.Attach(document, "mouseup", function (event) { onMouseUpHandler.call(this, event || window.event); }.BindContext(_instance));
			}

			$$ice(element).Query("//.draggCatch").Where("o=>!o.__iceDraggable").ForEach("o=>o.__iceDraggable=true").OnLeftMouseDown(function (sender, ev)
			{
				StartDragging.call(_instance, $$ice(sender).Query(".draggable, ^.draggable").First(), ev); // search for draggable class in current element and it's ancestors
			});

			$$ice(element).Query("//.dropTarget").Where("o=>!o.__iceDropTarget").ForEach("o=>o.__iceDropTarget=true").OnHover(function (sender, ev)
			{
				if (_instance.draggableElement)
				{
					// test if this is valid drop target for draggableElement
					var isValid = $$ice(sender).SelectClassNames().Intersect($$ice(_instance.draggableElement).SelectClassNames(), "(a, b)=>a==b").Any();
					ShowStatusDropValid(isValid);
					if (isValid)
					{
						_instance.dropTarget = sender;
						sender.OnDragHover.call(sender, sender, true);
					}
				}

			}, function (sender, ev)
			{
				sender.OnDragHover.call(sender, sender, false);
				_instance.dropTarget = null;
				ShowStatusDropValid(false);
			});
		};
	})();

	this.Initialize = _instance.Initialize;

})();

