﻿using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Globalization;
using System.Security.Permissions;
using System.Web;
using System.Web.UI;
using System.Web.Util;
using Ice.Web.UI.WebControls;
using System.Linq;
using System.Collections.Generic;
using System.Text.RegularExpressions;
using Ice.Web.UI;
using System.Web.UI.WebControls;
using Ice.Web.Extensions;

namespace Ice.Web.UI.WebControls
{
	[ParseChildren(false)]
	[PersistChildren(true)]
    public class IceWebControl : Control, IAttributeAccessor
    {
        private System.Web.UI.AttributeCollection attrColl;
        private StateBag attrState;
		private HtmlTextWriterTag tagKey;
		private string tagName;

		public bool Enabled
		{
			get { return (this.ViewState["_disabled"] as bool?).GetValueOrDefault() == false; }
			set { this.ViewState["_disabled"] = !value; }
		}

		public bool ReadOnly
		{
			get { return (this.ViewState["_readonly"] as bool?).GetValueOrDefault(); }
			set { this.ViewState["_readonly"] = !value; }
		}

		public IceWebControl() : this("span")
		{
		
		}

        public IceWebControl(string tag)
        {
            this.tagKey = HtmlTextWriterTag.Unknown;
            this.tagName = tag;
        }

		public IceWebControl(HtmlTextWriterTag tag)
        {
            this.tagKey = tag;
        }

		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden), System.ComponentModel.Browsable(false)]
		protected virtual HtmlTextWriterTag TagKey
		{
			get
			{
				return this.tagKey;
			}
			set
			{
				this.tagKey = value;
			}
		}

		//[System.ComponentModel.Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public virtual string TagName
		{
			get
			{
				if ((this.tagName == null) && (this.TagKey != HtmlTextWriterTag.Unknown))
				{
					this.tagName = Enum.Format(typeof(HtmlTextWriterTag), this.TagKey, "G").ToLower(CultureInfo.InvariantCulture);
				}
				return this.tagName;
			}
			set
			{
				value = (value ?? "").Trim();
				if (string.IsNullOrEmpty(value))
				{
					throw new InvalidOperationException("TagName can not be empty");
				}
				// ice: allow TagName to be set in the code
				this.tagKey = HtmlTextWriterTag.Unknown;
				this.tagName = value;
			}
		}

		protected override object SaveViewState()
		{
			if (this.attrState != null)
			{
				List<string> list = new List<string>();
				var en = this.attrState.GetEnumerator();
				while (en.MoveNext())
				{
					list.Add(HttpUtility.UrlEncode(en.Key + "") + "=" + HttpUtility.UrlEncode(((StateItem)en.Value).Value + ""));
				}
				this.ViewState["_attr"] = list.ToStringJoined("&");
			}
			return base.SaveViewState();
		}

		protected override void LoadViewState(object savedState)
		{
			System.Collections.ArrayList list = savedState as System.Collections.ArrayList;
			if (list != null)
			{
				string attributes = list.Cast<object>().SkipWhile(o => !(o is IndexedString && ((IndexedString)o).Value == "_attr")).Skip(1).Cast<string>().FirstOrDefault();
				if (!string.IsNullOrEmpty(attributes))
				{
					var items = new Regex("([^&=]+)=([^&=]*)").Matches(attributes).Cast<Match>().Select(o => new
					{
						Key = HttpUtility.UrlDecode(o.Groups[1].Value),
						Value = HttpUtility.UrlDecode(o.Groups[2].Value)
					}).ToList();

					items.ForEach(o => this.Attributes[o.Key] = o.Value);
				}
			}
			
			base.LoadViewState(savedState);
		}

        protected virtual void AddAttributesToRender(HtmlTextWriter writer)
        {
            writer.AddAttribute(HtmlTextWriterAttribute.Id, this.ClientID);

			if (!this.Enabled)
			{
				writer.AddAttribute(HtmlTextWriterAttribute.Disabled, "disabled");
			}

			if (this.ReadOnly)
			{
				writer.AddAttribute(HtmlTextWriterAttribute.ReadOnly, "readonly");
			}

            if (this.attrState != null)
            {
                System.Web.UI.AttributeCollection attributes = this.Attributes;
                IEnumerator enumerator = attributes.Keys.GetEnumerator();
                while (enumerator.MoveNext())
                {
                    string current = (string) enumerator.Current;
                    writer.AddAttribute(current, attributes[current]);
                }
            }
        }

		public override void RenderControl(HtmlTextWriter writer)
		{
			if (!this.Visible)
			{
				return;
			}

			base.RenderControl(writer);
		}

        protected override void Render(HtmlTextWriter writer)
        {
            this.RenderBeginTag(writer);
            this.RenderContents(writer);
            this.RenderEndTag(writer);
        }

        public virtual void RenderBeginTag(HtmlTextWriter writer)
        {
            this.AddAttributesToRender(writer);
            HtmlTextWriterTag tagKey = this.TagKey;
            if (tagKey != HtmlTextWriterTag.Unknown)
            {
                writer.RenderBeginTag(tagKey);
            }
            else
            {
                writer.RenderBeginTag(this.TagName);
            }
        }

        protected virtual void RenderContents(HtmlTextWriter writer)
        {
            base.Render(writer);
        }

        public virtual void RenderEndTag(HtmlTextWriter writer)
        {
            writer.RenderEndTag();
        }

		string IAttributeAccessor.GetAttribute(string name)
		{
			if (char.IsUpper(name[0]))
			{
				throw new Exception(name + " not found in type " + this.GetType().Name);
			}
			return this.Attributes[name];
		}

		void IAttributeAccessor.SetAttribute(string name, string value)
		{
			if (char.IsUpper(name[0]))
			{
				throw new Exception(name + " not found in type " + this.GetType().Name);
			}
			this.Attributes[name] = value;
		}


        [System.ComponentModel.Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public System.Web.UI.AttributeCollection Attributes
        {
			get
            {
                if (this.attrColl == null)
                {
					this.attrState = new StateBag(true);
					
					this.attrColl = new System.Web.UI.AttributeCollection(this.attrState);
                }
                return this.attrColl;
            }
        }

		[System.ComponentModel.Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public CssStyleCollection Style
		{
			get
			{
				return this.Attributes.CssStyle;
			}
		}

		public string CssClass
		{
			get
			{
				return this.Attributes["class"] ?? "";
			}
			set
			{
				this.Attributes["class"] = value;
			}
		}

    }



}

