﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.UI;
using System.Web.UI.WebControls;
using Ice.Web.Script.Services;
using Ice.Web.Extensions;
using Ice.Web.Resources;
using Ice.Extensions;
using System.Web.UI.HtmlControls;
using Ice.Colorspace;

namespace Ice.Web.UI.WebControls
{
	[ParseChildren(true)]
	public class IceWindow : IcePanel
	{
		public enum WinBorderEnum
		{
			Aero,
			None,
			Tool
		}

		public enum ButtonsPositionEnum
		{
			[StringValue("center")]
			Center,
			[StringValue("right")]
			Right,
			[StringValue("left")]
			Left
		}

		public ButtonsPositionEnum ButtonsPosition { get; set; }

		[PersistenceMode(PersistenceMode.InnerProperty)]
		public ITemplate TitleContent { get; set; }

		[PersistenceMode(PersistenceMode.InnerProperty)]
		public ITemplate WindowContent { get; set; }

		[PersistenceMode(PersistenceMode.InnerProperty)]
		public ITemplate ButtonsContent { get; set; }

		public System.Drawing.Color BackgroundColor { get; set; }

		public Unit Width { get; set; }

		public Unit Height { get; set; }

		public bool Minimizable { get; set; }
		public bool Maximizable { get; set; }
		public bool Closable { get; set; }
		public bool Resizable { get; set; }
		public bool Modal { get; set; }
		public bool Show { get; set; }
		public WinBorderEnum WinBorderMode { get; set; }

		Panel mask;
		Panel winMain;
		Panel winTitle;
		Panel winContent;
		Panel winButtons;
		Panel buttonsContainer;
		Panel iceWinPanel1;
		Panel panel4;

		private Control GetTopButtonContainerAero()
		{
			return
			// Aero top right buttons container
			new Panel() { Visible = this.Minimizable || this.Maximizable || this.Closable }.AddClass("iceWinTopButtonsContainer")
			.ApplyStyle("position:absolute; right:5px; top:0px; border:solid 1px silver; border-top:none; border-radius:0px 0 4px 4px;")
			.AddChildren
			(
				// top right buttons
				new Panel().ApplyStyle("position:relative; border:solid 1px black; border-left:0; border-top:none;border-radius:0px 0 4px 4px; overflow:hidden;cursor:pointer;")
				.AddChildren
				(
					new IceImage() { Icon = IconEnum.WindowMinimize, HoverIcon = IconEnum.WindowMinimizeHover}
							.ApplyStyle("float:left;border-left:solid 1px black;")
							.Set(o => o.Attributes["onclick"] = "return false;")
							.Set(o => o.Visible = this.Minimizable),

					new IceImage() { CssClass = "iceWinBtnResize", Icon = IconEnum.WindowResize, HoverIcon = IconEnum.WindowResizeHover}
							.ApplyStyle("float:left; border-left:solid 1px black;")
							.Set(o => o.Attributes["onclick"] = "Ice.Web.UI.WebControls.IceWindowManager.FindOwnerWindow(this).Resize(this); return false;") // if used GImageButton, requires return false or it will do posback
							.Set(o => o.Visible = this.Maximizable),

					new IceImage() { Icon = IconEnum.WindowMaximized, HoverIcon = IconEnum.WindowMaximizedHover}
							.ApplyStyle("float:left; display:none; border-left:solid 1px black;")
							.Set(o => o.Attributes["onclick"] = "Ice.Web.UI.WebControls.IceWindowManager.FindOwnerWindow(this).Resize(this); return false;")
							.Set(o => o.Visible = this.Maximizable),

					new IceImage() { Icon = IconEnum.WindowClose, HoverIcon = IconEnum.WindowCloseHover}
							.ApplyStyle("float:left;border-left:solid 1px black;")
							.Set(o => o.Attributes["onclick"] = "Ice.Web.UI.WebControls.IceWindowManager.FindOwnerWindow(this).Close(this); return false;")
							.Set(o => o.Visible = this.Closable),

					new Panel().ApplyStyle("clear:both;")
				)
			);
		}

		private Control GetTopButtonsContainerTool()
		{
			return
				// Tool top right buttons container
				new Panel() { Visible = this.Minimizable || this.Maximizable || this.Closable }.AddClass("iceWinTopButtonsContainer")
				.ApplyStyle("position:absolute; right:5px; top:2px;")
				.AddChildren
				(
					// top right buttons
					new Panel().ApplyStyle("position:relative;overflow:hidden;cursor:pointer;")
					.AddChildren
					(
						new IceImage() { Icon = IconEnum.WindowToolClose, HoverIcon = IconEnum.WindowToolCloseHover}
				.ApplyStyle("float:left;")
				.Set(o => o.Attributes["onclick"] = "Ice.Web.UI.WebControls.IceWindowManager.FindOwnerWindow(this).Close(this); return false;")
				.Set(o => o.Visible = this.Closable),

				new Panel().ApplyStyle("clear:both;")
	)
);
		}

		private Control GetResizeControls()
		{
			return
			new PlaceHolder() { Visible = this.Resizable }
			.AddChildren
			(
				new Panel() { CssClass = "winTop" }.ApplyStyle("position:absolute;left:10px;top:-2px;right:120px;height:5px;cursor:n-resize;"),
				new Panel() { CssClass = "winLeft" }.ApplyStyle("position:absolute;left:-2px;top:10px;bottom:10px;width:5px;cursor:e-resize;"),
				new Panel() { CssClass = "winBottom" }.ApplyStyle("position:absolute;left:10px;bottom:-2px;right:10px;height:5px;cursor:n-resize;"),
				new Panel() { CssClass = "winRight" }.ApplyStyle("position:absolute;right:-2px;top:10px;bottom:10px;width:5px;cursor:e-resize;"),

				new Panel() { CssClass = "winTop winLeft" }.ApplyStyle("position:absolute;left:-2px;top:-2px;height:12px;width:12px;cursor:nw-resize;"),
				new Panel() { CssClass = "winTop winRight" }.ApplyStyle("position:absolute;right:-2px;top:-2px;height:12px;width:7px;cursor:ne-resize;"),
				new Panel() { CssClass = "winBottom winLeft" }.ApplyStyle("position:absolute;left:-2px;bottom:-2px;width:12px;height:12px;cursor:ne-resize;"),
				new Panel() { CssClass = "winBottom winRight" }.ApplyStyle("position:absolute;right:-2px;bottom:-2px;width:12px; height:12px;cursor:nw-resize;")
			);
		}

		string buttonsContainerHeight = "30px";

		private Control GetButtonsContainer()
		{
			return buttonsContainer = new Panel() { Visible = this.ButtonsContent != null }.ApplyStyle("position:absolute; bottom:0px; left:0; right:0;overflow:hidden;background-color:#F0F0F0;")
				.ApplyStyle("height:" + buttonsContainerHeight)
			.AddChildren
			(
				new HtmlTable().ApplyStyle("height:100%;").Set(o => o.Attributes["cellpadding"] = "0").Set(o => o.Attributes["cellspacing"] = "0")
				.Set(o => o.Attributes["align"] = this.ButtonsPosition == ButtonsPositionEnum.Left ? "left" : this.ButtonsPosition == ButtonsPositionEnum.Right ? "right" : "center")
				.AddChildren(new HtmlTableRow().AddChildren(new HtmlTableCell()
				.AddChildren(
					winButtons = new Panel().ApplyStyle("margin:0px 20px 0 20px;")
					.AddChildren(this.ButtonsContent)
				)))
			);
		}

		private Control GetWindowContent()
		{
			return winContent = new Panel() { CssClass = "iceWinContentContainer" }.ApplyStyle("position:absolute; left:0; top:0; right:0; padding:0px;overflow:auto;")
				.ApplyStyle("bottom:" + buttonsContainerHeight)
			.AddChildren(this.WindowContent);
		}

		private void BuildControlWithFrame()
		{
			this.AddClass("iceWinContainer").ApplyStyle("position:absolute;").AddChildren
			(
				mask = new Panel().AddClass("iceWinMask").ApplyStyle("display:none;position:fixed; left:0; right:0; top:0; bottom:0; background-color:black; opacity:0.2; filter: alpha(opacity=20);"),
				winMain = new Panel().AddClass("iceWinMain").ApplyStyle("display:none; position:fixed; width:500px; height:300px; background-color:#1A6F96; border:solid 1px black; box-shadow:2px 2px 6px 4px gray; border-radius:6px;")
				.AddChildren
				(
					iceWinPanel1 = new Panel().AddClass("iceWinPanel1").ApplyStyle("position:absolute; left:0; top:0; right:0; bottom:0; border:solid 1px silver; border-radius:5px; overflow:hidden;")
					.AddChildren
					(
						this.WinBorderMode == WinBorderEnum.Aero ? GetTopButtonContainerAero() : this.WinBorderMode == WinBorderEnum.Tool ? GetTopButtonsContainerTool() : null,

						winTitle = new Panel(){Visible = this.WinBorderMode != WinBorderEnum.None }
						.ApplyStyle("color:White; font-weight:bold; margin:5px 0 0 0; position:absolute; top:0px; left:5px; right:120px; height:18px; overflow:hidden;")
						.Set(o => { if (this.Maximizable) { o.Attributes["ondblclick"] = "Ice.Web.UI.WebControls.IceWindowManager.FindOwnerWindow(this).Resize(this);"; } })
						.AddChildren(this.TitleContent),

						panel4 = new Panel().ApplyStyle("position:absolute; top:25px; bottom:5px; left:5px; right:5px; background:white; border:solid 1px silver").AddChildren
						(
							new Panel().ApplyStyle("position:absolute; top:0; left:0; bottom:0; right:0; border:solid 1px black").AddChildren
							(
								GetWindowContent(),

								GetButtonsContainer()
							)
						)
					),

					GetResizeControls()
				)
			);
		}

		private void BuildControlWithNoFrame()
		{
			this.AddClass("iceWinContainer").ApplyStyle("position:absolute;").AddChildren
			(
				mask = new Panel().AddClass("iceWinMask").ApplyStyle("display:none;position:fixed; left:0; right:0; top:0; bottom:0; background-color:black; opacity:0.2; filter: alpha(opacity=20);"),
				winMain = new Panel().AddClass("iceWinMain").ApplyStyle("display:none; position:fixed; width:500px; height:300px; background-color:none; box-shadow:2px 2px 6px 4px gray;")
				.AddChildren
				(
					winTitle = new Panel(){Visible = false}, // just to prevent null exception

					GetWindowContent(),

					GetButtonsContainer(),

					GetResizeControls()
				)
			);
		}

		protected override void OnInit(EventArgs e)
		{
			base.OnInit(e);

			if (this.WinBorderMode == WinBorderEnum.None)
			{
				BuildControlWithNoFrame();
			}
			else
			{
				BuildControlWithFrame();
			}

			if (this.Page != null && !this.Page.IsInAsyncPostBack())
			{
				// script has to be loaded always, even if panel is not currently visible. It may be set visible later in UpdatePanel
				IceScriptManager.GetCurrent(this.Page).RegisterScriptFile(this, ".IceWindow.IceWindow.js");
			}
		}

		protected override void OnPreRender(EventArgs e)
		{
			base.OnPreRender(e);

			IceScriptManager manager = IceScriptManager.GetCurrent(this.Page);
	
			// modify according to theme:
			winMain.Style["background-color"] = manager.CurrentTheme.WindowFrameColor;
			buttonsContainer.Style["background-color"] = manager.CurrentTheme.FrameColor;
			buttonsContainer.Style["text-align"] = this.ButtonsPosition.GetStringValue();
			if (this.ButtonsContent == null)
			{
				winContent.ApplyStyle("bottom:0;");
			}
			if (this.WinBorderMode == WinBorderEnum.Tool || this.WinBorderMode == WinBorderEnum.None)
			{
				winMain.ApplyStyle("border-radius:none;");
			}
			if (this.WinBorderMode == WinBorderEnum.Tool)
			{
				iceWinPanel1.ApplyStyle("border-radius:none;");
				panel4.ApplyStyle("bottom:2px;left:2px;right:2px;top:21px;");
				winTitle.ApplyStyle("margin-top:2px;right:40px;");
			}
			if (this.WinBorderMode == WinBorderEnum.None)
			{
				winMain.ApplyStyle("background-color:white;");
			}

			this.Page.ExecuteJavaScript(string.Format("Ice.Web.UI.WebControls.IceWindowManager.Setup('{0}','{1}','{2}','{3}', {4}, {5});", 
				this.ClientID, winMain.ClientID, this.winTitle.ClientID, mask.ClientID, this.Modal ? "true" : "false", this.Show ? "true" : "false"));

			if (this.WindowContent == null)
			{
				throw new Exception("WindowContent is not defined");
			}

			if (!this.BackgroundColor.IsEmpty)
			{
				// if set, override all settings
				winMain.Style["background-color"] = this.BackgroundColor.ToHtml();
			}

			if (!this.Width.IsEmpty)
			{
				winMain.Style["width"] = this.Width + "";
			}

			if (!this.Height.IsEmpty)
			{
				winMain.Style["height"] = this.Height + "";
			}
		}

	}
}
