﻿/// <reference path="ice-fix-ext-before.js" />
/// <reference path="Ext_21/source/core/Ext.js" />
/// <reference path="Ext_21/source/adapter/ext-base.js" />
/// <reference path="Ext_21/ext-all-debug.js" />
/// <reference path="ice-fix-ext-after.js" />

/***********************************************************
	Ext.Ajax.request
	accepts url as web service method
***********************************************************/

Ext.Ajax.request_base = Ext.Ajax.request;
Ext.Ajax.request = function(o){
    var url = o.url || this.url;
    if(o.callback && typeof url != 'function'){
		return Ext.Ajax.request_base(o);
    }
	var serviceMethod = url;
	// url is service method
    if(this.fireEvent("beforerequest", this, o) !== false){
        var p = o.params;
        if(typeof p == "function"){
            p = p.call(o.scope||window, o);
        }
        if(typeof p == "object"){
			// ok
        }
        if(this.extraParams){
          Ext.apply(p, this.extraParams);
        }
		        
        return serviceMethod(p, function(response){ // call web service method
			var cb = o.callback || o.success;
			cb.createDelegate(o.scope)(o, response != null, response);
        });
        
    }else{
        cb.createDelegate(o.scope)(o, false, response);
        return null;
    }
};

/***********************************************************
	Ext.data.GTreeLoader 
	connects to web service method and receive JS object
***********************************************************/

Ext.tree.GTreeLoader = Ext.extend(Ext.tree.TreeLoader, {
	
	url: null, // web service method to be called
//	createNodeBase: Ext.tree.TreeLoader.prototype.createNode,
//	createNode: Ext.tree.TreeLoader.prototype.createNode, // this method may be overriden
	
	loadResponse: function(o, success, response){
		var a = o.argument;
        delete this.activeRequest;
        if(!success){
            this.fireEvent("loadexception", this, a.node, response);
            if(typeof a.callback == "function"){a.callback(this, a.node);}
            return;
        }else{
			try{
	            var o = response;
	            var node = a.node; // parent node
				node.beginUpdate();
				for(var i = 0, len = o.length; i < len; i++)
				{
					var n = this.createNode(o[i]);
					if(n)
					{
						node.appendChild(n);
					}
				}
				node.endUpdate();
		        this.fireEvent("load", this, a.node, response);
			}
			catch(e){
				this.fireEvent("loadexception", this, a.node, response, e);
				return;
			}
			
	        if(typeof a.callback == "function"){a.callback(this, a.node);}
		}
	},
	
    requestData : function(node, callback){
		if(this.fireEvent("beforeload", this, node, callback) !== false){
			this.transId = Ext.Ajax.request({
				callback: this.loadResponse,
				url: this.dataUrl||this.url,
				scope: this,
				argument: {callback: callback, node: node},
				params: node.nodeId // custom node id here
			});
		}else{
			if(typeof callback == "function"){
				callback();
			}
		}
	}
    
});

/***********************************************************
	Ext.data.GDataTableReader
	supports paging properties totalRecords and root
	o = {
		Columns: ["col1", "col2", "col3",...],
		Rows: [[row], [row], [row]]
		TotalCount:32
	};
***********************************************************/
Ext.data.GDataTableReader = Ext.extend(Ext.data.JsonReader, {
   
    read : function(dt){
		// response must be data table with properties Columns, Rows, TotalCount
        return this.readRecords(dt);
    },
    
   getMetaData : function(o){
		// o is dataTable with properties: Rows, Columns and TotalCount
		return {
			fields: o.Columns,
			totalProperty: "TotalCount",
			root: "Rows"
		}
   },
   
   readRecords : function(o){
        
		this.meta = o.metaData ? o.metaData : this.getMetaData(o); // be carefull: if property metaData exist, removes sortInfo, must use meta property instead
		
		var f = [];
		var fields = this.meta.fields;
		for(var n = 0; n < fields.length; n++)
		{
			var field = fields[n];
			if(field == null || field.charAt)
			{
				f.push({name:field, mapping:n}); // data table uses mapping by index!!! JsonReader uses mapping by property name
			}
		}
		delete this.ef;
		this.meta.fields = f;
		this.recordType = Ext.data.Record.create(this.meta.fields);
		return Ext.data.GDataTableReader.superclass.readRecords.call(this, o);
    }
});

/***********************************************************
	Ext.grid.GCheckBox
***********************************************************/
Ext.grid.GCheckBox = function(config){
    Ext.apply(this, config);
    if(!this.id){
        this.id = Ext.id();
    }
    this.renderer = this.renderer.createDelegate(this);
};

Ext.grid.GCheckBox.prototype ={
	
	checked: true, // value to set in data store if checked
	unchecked: false, // value to set in data store if not checked
	
    init : function(grid){
        this.grid = grid;
        this.grid.on('render', function(){
            var view = this.grid.getView();
            view.mainBody.on('mousedown', this.onMouseDown, this);
        }, this);
    },

    onMouseDown : function(e, t){
        if(t.className && t.className.indexOf('x-grid3-cc-'+this.id) != -1){
            e.stopEvent();
            var index = this.grid.getView().findRowIndex(t);
            var record = this.grid.store.getAt(index);
            var currentValue = record.data[this.dataIndex];
            var newValue = currentValue == this.checked ? this.unchecked : this.checked;
            record.set(this.dataIndex, newValue);
        }
    },

    renderer : function(v, p, record){
		var isChecked = v == this.checked;
        p.css += ' x-grid3-check-col-td'; 
        return '<div class="x-grid3-check-col'+(isChecked?'-on':'')+' x-grid3-cc-'+this.id+'">&#160;</div>';
    }
};

/***********************************************************
	// DateTimeField

***********************************************************/

Ext.form.DateTimeField = Ext.extend(Ext.form.DateField, {
	
	allowBlank: true,
	format: "Y-m-d H:i:s",
	
    getValue : function(){
		var hasd = 432;
		var value = Ext.form.DateField.superclass.getValue.call(this);
		var dateTime = this.parseDate(value) || "";
		return Ext.form.DateTimeField.isDateTimeNULL(dateTime) ? "" : dateTime;
    },
    
    setValue : function(date){
		var dateTime = this.parseDate(date);
		//var formated = this.isDateTimeNULL(dateTime) ? "" : this.formatDate(dateTime);
		var formated = Ext.form.DateTimeField.ToISOFormat(dateTime);
        Ext.form.DateField.superclass.setValue.call(this, formated);
    },

	menuListeners : {
        select: function(m, d){this.setValue(d);},
        show : function(){this.onFocus();},
        hide : function(){
            this.focus.defer(10, this);
            var ml = this.menuListeners;
            this.menu.un("select", ml.select,  this);
            this.menu.un("show", ml.show,  this);
            this.menu.un("hide", ml.hide,  this);
        }
    }
});

Ext.form.DateTimeField.dateTimeMinValue = new Date(1900, 01, 01).getTime(); // if date is less than this, it is consdered to be undefined or NULL

Ext.form.DateTimeField.isDateTimeNULL = function(value)
{
	return (value == null || typeof(value.getTime) != "function" || value.getTime() < Ext.form.DateTimeField.dateTimeMinValue);
};

Ext.form.DateTimeField.ToISOFormat = function(value) // ISO DATE TIME FORMAT
{
	if(Ext.form.DateTimeField.isDateTimeNULL(value))
	{
		return "";
		//return " - not set -"; 
	}
	else
	{
		return Ext.util.Format.date(value, "Y-m-d H:i:s");
	}
};


//***************************************************
// BOOL EDITOR FOR EDITABLE GRID


Ext.grid.BoolEditor = function(value)
{
	var __this = this;
	
	Ext.grid.BoolEditor.superclass.constructor.apply(this, arguments);
	if(value == true)
	{
		this.value = this.checked;
		this.setValue(this.checked);
	}
	else
	{
		this.value = this.unchecked;
		this.setValue(this.unchecked);
	}
	this.getValue = function(){return (__this.value == true || __this.value == __this.checked) ? true : false;};
};

Ext.grid.BoolEditor = Ext.extend(Ext.grid.BoolEditor, Ext.form.ComboBox, {
		checked: "true",
		unchecked: "false",
	   typeAhead: true,
	   triggerAction: 'all',
	   lazyRender:true,
	//forceSelection:true,
	   editable:false,
	   mode:"local",
	   listClass: 'x-combo-list-small',
	   displayField:"yesNo",
	   store : new Ext.data.Store({
			reader: new Ext.data.ArrayReader({},
				[
					{name:"boolValue", type:"bool"},
					{name:"yesNo", type:"string"}
				]),
			data: [[true, "true"], [false, "false"]]
			})
	});

Ext.grid.BoolEditor.Render = function(value){
	return value ? "true" : "false";
};

//***************************************************
