﻿/// <reference path="../language/en.js" />
/// <reference path="../../../Ext/adapter/ext/ext-base-ice.js" />
/// <reference path="../../../Ext/ext-all-ice.js" />


QoDesk.NewsletterComposer = Ext.extend(Ext.app.Module,
{
	name: CartLanguage.Newsletter,
	windowId: "NewsletterComposerWin",
	ajaxNs: "NewsletterComposer",
	moduleType: 'demo',
	moduleId: 'NewsletterComposerEditor',
	winTitle: "Newsletter Composer",
	iconCls: "newsletter-icon-small",
	shortcutIconCls: "newsletter-icon-large",

	init: function ()
	{
		this.launcher = {
			handler: this.createWindow,
			iconCls: this.iconCls || 'icon-grid',
			scope: this,
			shortcutIconCls: this.shortcutIconCls || 'demo-grid-shortcut',
			text: this.name,
			tooltip: "<b>" + this.name + "</b>"
		}
	},

	createWindow: function ()
	{
		var __this = this;
		var desktop = this.app.getDesktop();
		var win = desktop.getWindow(this.windowId);
		var ajaxNs = this.ajaxNs;

		if (!win)
		{

			var RoleSelector = Ext.extend(Ext.form.ComboBox, {
				typeAhead: true,
				triggerAction: 'all',
				fieldLabel: CartLanguage.SendToRole,
				anchor: "95%",
				name: 'Role',
				lazyRender: true,
				editable: false,
				mode: "local",
				listClass: 'x-combo-list-small',
				displayField: "Name",
				store: new Ext.data.Store({
					reader: new Ext.data.JsonReader(
						{ root: "" },
						[
							{ name: "Name", type: "string" },
							{ name: "RoleID", type: "int" }
						]),
					data: []
				})
			});

			var roleSel = new RoleSelector();
			roleSel.setValue(CartLanguage.Loading);
			roleSel.RoleID = -1;
			roleSel.on("select", function (a, b, c, d)
			{
				roleSel.RoleID = b.data.RoleID;
			});

			// load roles
			GService.GetRoleCollection(function (roles)
			{
				roleSel.hasFocus = false; // disable auto expanding
				roleSel.store.removeAll();
				var m = Ice.Ext.GetMetaData(roles, roleSel.store);
				if (m.collection.length > 0)
				{
					var cloned = Ice.Clone(m.collection[0]);
					cloned.Name = " - " + CartLanguage.AnyRole + " - ";
					cloned.RoleID = 0;
					m.collection.unshift(cloned);

					var cloned = Ice.Clone(m.collection[0]);
					cloned.Name = " --- " + CartLanguage.SelectRole + " --- ";
					cloned.RoleID = -1;
					m.collection.unshift(cloned);

				}
				roleSel.store.loadData(m);
				roleSel.selectedIndex = 0;
				roleSel.setValue(roleSel.store.getAt(roleSel.selectedIndex).data.Name);
				roleSel.RoleID = -1;
			});


			var boolSubscribedOnly = new Ext.grid.BoolEditor(true);
			boolSubscribedOnly.fieldLabel = CartLanguage.SubscribedOnly;
			boolSubscribedOnly.anchor = "70%";


			var subject = new Ext.form.TextField({ fieldLabel: CartLanguage.EmailSubject, anchor: "95%" });

			//var htmlEditor = new Ext.form.HtmlEditor({fieldLabel:"Body", anchor:"100%"});

			var htmlEditor = {
				xtype: 'box'
						, anchor: "100%"
				//,width:420
				//,cls:"x-form-item"
				//,height:100
						, autoEl:
						{
							tag: 'div',
							style: 'margin:0px 0 0px 0;',
							html: "<iframe id='newsEditorIframe' src='../Editor.aspx?c=news' style='width:100%; height:100px;' frameborder='0'></iframe>"
						}
			};

			var statusField =
			{
				Queue: new Ext.form.TextField({ fieldLabel: CartLanguage.Queue, width: 50, value: "" }),
				Sent: new Ext.form.TextField({ fieldLabel: CartLanguage.Sent, width: 50, value: "" }),
				Failed: new Ext.form.TextField({ fieldLabel: CartLanguage.Failed, width: 50, value: "" }),
				Sending: new Ext.form.TextField({ fieldLabel: CartLanguage.FieldSending, width: 50, value: "" })
			};

			var buttonStartStop = new Ext.Button({ iconCls: "icon_stop", minWidth: 70, disabled: true, text: "---", width: 100, handler: function ()
			{
				var startRunning = buttonStartStop.getText() == "---" ? true : false;
				GService.NewsletterStartStop(startRunning, function (isRunning)
				{
					buttonStartStop.SetStatus(isRunning);
				});
			}
			});

			buttonStartStop.SetStatus = function (isRunning)
			{
				try
				{
					if (isRunning)
					{
						buttonStartStop.setText(CartLanguage.Stop);
						buttonStartStop.setDisabled(false);
					}
					else
					{
						buttonStartStop.setText("---");
						buttonStartStop.setDisabled(true);
					}
				}
				catch (e)
				{
				}
			};

			var formPanel = new Ext.FormPanel({
				labelAlign: 'top',
				bodyStyle: 'padding:5px',
				width: 600,
				border: false,
				items:
				[
					{
						layout: "column",
						border: false,
						style: "padding:10px",
						items:
						[
							{
								columnWidth: .13,
								layout: "column",
								border: false,
								items: [new Ext.form.Label({ text: CartLanguage.Queue, style: "padding:2px" }), statusField.Queue]
							},
							{
								columnWidth: .13,
								layout: "column",
								border: false,
								items: [new Ext.form.Label({ text: CartLanguage.Sent, style: "padding:2px" }), statusField.Sent]
							},
							{
								columnWidth: .13,
								layout: "column",
								border: false,
								items: [new Ext.form.Label({ text: CartLanguage.Failed, style: "padding:2px" }), statusField.Failed]
							},
							{
								columnWidth: .13,
								layout: "column",
								border: false,
								items: [new Ext.form.Label({ text: CartLanguage.FieldSending, style: "padding:2px" }), statusField.Sending]
							},
							{
								columnWidth: .05,
								layout: "column",
								border: false,
								style: "width:30px; border:solid 1px #F9F9F9;"
								//items:[]
							},
							{
								columnWidth: .095,
								layout: "form",
								border: false,
								items: [new Ext.Button({ minWidth: 70, text: CartLanguage.Reset, iconCls: "icon_cross", handler: function () { GService.NewsletterReset(function (newsStatus) { SetStatuses(newsStatus); }); } })]
							}
						]
					},
					{
						border: true,
						height: 4
					},
					{
						layout: 'column',
						border: false,
						style: "padding:5px;",
						items: [{
							columnWidth: .5,
							layout: 'form',
							border: false,
							items:
							[
								subject
							]
						},
						{
							columnWidth: .25,
							layout: 'form',
							border: false,
							items:
							[
								roleSel
							]
						},
						{
							columnWidth: .25,
							layout: 'form',
							border: false,
							items:
							[
								boolSubscribedOnly
							]
						}]
					},

					htmlEditor
				],
				buttons:
				[
					{
						text: CartLanguage.Send,
						iconCls: "icon_email_go",
						handler: function ()
						{
							var subscribedOnly = boolSubscribedOnly.getValue();
							var roleId = roleSel.RoleID;
							if (roleId < 0)
							{
								alert(CartLanguage.SelectRole);
								return; // roles are still loading
							}

							var subj = subject.getValue();
							var ifr = document.getElementById("newsEditorIframe");
							var html = ifr.contentWindow.CuteEditor.GetHtml();

							function SaveAllConfiramation(a, b, c, d)
							{
								if (a == "yes")
								{
									//Ext.MessageBox.ShowWaitMessage(true, CartLanguage.Sending);
									GService.NewsletterSend(roleId, subscribedOnly, subj, html, function (emailsCount)
									{
										//	Ext.MessageBox.ShowWaitMessage(false);
										if (emailsCount > 0)
										{
											Ext.Msg.show({
												title: CartLanguage.Success,
												msg: CartLanguage.Sending + " " + emailsCount + " " + CartLanguage.Emails,
												//	fn: function(){win.close();}, // leave window opened!!!
												icon: Ext.Msg.INFO,
												buttons: Ext.Msg.OK
											});
										}
										else
										{
											Ext.Msg.alert(CartLanguage.Failed, CartLanguage.NoEmailsByCriteria);
										}
									});
								}
							}

							Ext.Msg.show({
								title: CartLanguage.AreYouSure,
								msg: CartLanguage.LargeNumberOfEmails,
								buttons: Ext.Msg.YESNO,
								fn: SaveAllConfiramation,
								icon: Ext.MessageBox.QUESTION
							});
						}
					},
					buttonStartStop,
				
					new Ext.Button({ minWidth: 70, text: CartLanguage.ViewLog, iconCls: "icon_script", handler: function ()
						{
							var win = window.open();
							GService.GetNewsletterLog(function (newsLog)
							{
								win.document.open();
								win.document.write("<html><body style='font-size:11px'><div>" + CartLanguage.Sent + " (" + newsLog.SentEmails.length + "):</div><div style='color:black;'>");
								for (var nn = 0; nn < newsLog.SentEmails.length; nn++)
								{
									win.document.write(newsLog.SentEmails[nn] + "<br/>");
								}
								win.document.write("</div><br/><br/>");

								win.document.write("<div>" + CartLanguage.Failed + " (" + newsLog.FailedEmails.length + "):</div><div style='color:red;'>");
								for (var nn = 0; nn < newsLog.FailedEmails.length; nn++)
								{
									win.document.write(newsLog.FailedEmails[nn] + "<br/>");
								}
								win.document.write("</div></body></html>");

								win.document.close();
								win.focus();
							});
						}
					}),
					{
						text: 'Close',
						iconCls: "icon_cancel",
						handler: function ()
						{
							win.close();
						}
					}
				]
			});

			win = new Ext.Window({
				renderTo: document.body,
				id: __this.windowId,
				title: __this.winTitle,
				width: 850,
				height: document.body.offsetHeight > 1000 ? 1000 - 40 : document.body.offsetHeight - 40,
				iconCls: __this.iconCls || 'icon-grid',
				bodyStyle: "background-color:#F9F9F9;",
				shim: false,
				frame: true,
				modal: true,
				animCollapse: false,
				maximizable: true,
				constrainHeader: true,
				layout: 'fit',
				taskbuttonTooltip: "<b>" + __this.winTitle + "</b>",
				items: formPanel
			});

			function SetStatuses(newsStatus)
			{
				if (newsStatus)
				{
					statusField.Queue.setValue(newsStatus.QueueCount);
					statusField.Sent.setValue(newsStatus.SentCount);
					statusField.Failed.setValue(newsStatus.FailedCount);
					statusField.Sending.setValue(newsStatus.SendingCount);

					buttonStartStop.SetStatus(newsStatus.SendingCount > 0);
				}
			}

			function PullStatus()
			{
				GService.GetNewsletterStatus(function (newsStatus)
				{
					SetStatuses(newsStatus);
					if (win.isVisible())
					{
						setTimeout(PullStatus, 2000);
					}
				});
			}

			function ResizeEditor()
			{
				var width = win.getInnerWidth();
				var height = win.getInnerHeight();
				//htmlEditor.setHeight(height - 80);
				var ifr = document.getElementById("newsEditorIframe");
				ifr.style.height = (height - 150) + "px";

				// resize CuteEditor
				if (ifr.contentWindow && ifr.contentWindow.CuteEditor)
				{
					ifr.contentWindow.CuteEditor.DoResize = true;
				}
			}

			win.on("resize", ResizeEditor);
			win.on("show", ResizeEditor);
			win.on("show", PullStatus);

			win.show();
		}
		else
		{
			win.show();
		}
	}

});

