﻿/// <reference path="../../language/en.js" />
//////////////////////////////////////////////////////////////////////////
// GALLERY MODULE

QoDesk.ImageGallery = function()
{

	var __this = this;
	
	this.OnImageDoubleClick = null;

	QoDesk.ImageGallery.superclass.constructor.call(this, {
		name: CartLanguage.Gallery,
		windowId: "ImageGalleryWin",
		ajaxNs: "Gallery",
		moduleType: 'demo',
		moduleId: 'ImageGalleryEditor',
		autoload: true,

		iconCls: "gallery-icon-small",
		shortcutIconCls: "gallery-icon-large" ,

		init : function(){
			this.launcher = {
				handler : this.createWindow,
				iconCls: this.iconCls || 'icon-grid',
				scope: this,
				shortcutIconCls: this.shortcutIconCls || 'demo-grid-shortcut',
				text: this.name,
				tooltip: "<b>" + this.name + "</b>"
			}
		},

		createWindow : function(){
		
			var desktop = this.app.getDesktop();
			var win = desktop.getWindow(this.windowId);
	        
			if(!win)
			{
//				var winWidth = desktop.getWinWidth() / 1.4;
//				var winHeight = desktop.getWinHeight() / 1.1;
				
				var treeTabs = new Ext.TabPanel({
					resizeTabs:true,
					tabWidth:100,
					enableTabScroll:true,
					region:"west",
					activeTab:0,
					width:250,
					split:true
				});
				
				__this.GalleryTreeTab = new Ice.Ext.ImageGalleryTree();
				__this.GalleryTreeTab.Owner = __this;
				__this.GalleryTreeTab.title = CartLanguage.Gallery;
				__this.GalleryTreeTab.iconCls = "gallery-icon-small";
				__this.GalleryTreeTab.LoadTree = __this.GalleryTreeTab.LoadGalleries;
				treeTabs.add(__this.GalleryTreeTab).show();
				
			    var galleryViewTabs = new Ext.TabPanel({
					resizeTabs:true, // turn on tab resizing
					autoWidth:false,
					minTabWidth: 150,
					tabWidth:150,
					enableTabScroll:true,
					region: "center",
					plugins: new Ext.ux.TabCloseMenu()
				});
				
				// default tab must exist at render, or won't show anything
				var defTab = new Ext.Panel();
				defTab.title = "---";
				defTab.defTab = true;
				galleryViewTabs.add(defTab).show();
				galleryViewTabs.setActiveTab(defTab);
				
				function RemoveDefaultTab()
				{
					for(var n = 0; n < galleryViewTabs.items.items.length; n++)
					{
						var tab = galleryViewTabs.items.items[n];
						if(tab.defTab)
						{
							galleryViewTabs.remove(tab);
						}
					}
				}
				
				galleryViewTabs.on("tabchange", function(galleryViewTabs, tab){
					if( ! tab.defTab)
					{
						tab.ReloadAll();
					}
				});
				
				function FindTab(galleryNode)
				{
					for(var n = 0; n < galleryViewTabs.items.items.length; n++)
					{
						var tab = galleryViewTabs.items.items[n];
						
						if(galleryNode == null && tab.GalleryNode == null)
						{
							// root gallery
							return tab;
						}
						if(galleryNode != null && tab.GalleryNode != null && tab.GalleryNode.GalleryPhysicalPath == galleryNode.GalleryPhysicalPath)
						{
							return tab;
						}
					}
					return null; //not found
				}
				
				__this.OpenTab = function(galleryNode, tabTitle, closable)
				{
					// check if tab is already created
					var tab = FindTab(galleryNode);
					if(!tab)
					{
						// create users grid
						tab = new QoDesk.ImageGalleryView(galleryNode);
						tab.Owner = __this;
						tab.title = tabTitle;
						tab.GalleryNode = galleryNode;
						tab.closable = closable;
						tab.iconCls = "gallery-icon-small";
						galleryViewTabs.add(tab).show();
						
					}
					galleryViewTabs.setActiveTab(tab);
					
					RemoveDefaultTab();
				};
						
				var win = desktop.createWindow({
					id: this.windowId,
					title: this.name,
					width:950,
					height: document.body.offsetHeight > 800 ? 800 - 40 : document.body.offsetHeight - 40,
					iconCls: this.iconCls || 'icon-grid',
					shim:false,
					animCollapse:false,
					constrainHeader:true,
					layout: 'border',
					items: [treeTabs, galleryViewTabs], 
					taskbuttonTooltip: "<b>" + this.name + "</b>"
				});
	            
	            __this.win = win;
	            
				function onWinClose()
				{
				}
	            
				win.on("beforeclose", onWinClose);
			
				function OnWinActive()
				{
					setTimeout(__this.GalleryTreeTab.LoadRootNode, 10);
					
					// event is triggered only the first time, remove it
					win.un("activate", OnWinActive);
				};
				win.on("activate", OnWinActive);
			}

			win.show(); 
		}
	});	

};

QoDesk.ImageGallery = Ext.extend(QoDesk.ImageGallery, Ext.app.Module);
