﻿/// <reference path="../../language/en.js" />
//*****************************************************
// GALLERY MODULE

QoDesk.ImageGalleryView = function(galleryNode){

	var __this = this;
	this.GalleryNode = galleryNode;
	this.title = "---";
	
	this.shortName = function(name){
		if(name.length > 15){
			return name.substr(0, 12) + '...';
		}
		return name;
	};
	var view = new Ext.DataView({
       itemSelector: 'div.thumb-wrap',
       style:'overflow:auto',
       multiSelect: true,
       plugins: new Ext.DataView.DragSelector({dragSafe:true}),
        store: new Ext.data.JsonStore({
            fields:[
                'name', "size", 'url', "file", "height", "width", "thumbHeight", "thumbWidth", "marginTop", "marginLeft", "relativeHttpPath",
                {name: 'shortName', mapping: 'name', convert: __this.shortName}
            ]
        }),
        tpl: new Ext.XTemplate(
            '<tpl for=".">',
            '<div class="thumb-wrap" id="{name}">',
				'<div class="thumb">',
					'<div class="thumb-back">',
						'<img src="{url}" style="height:{thumbHeight}px;width:{thumbWidth}px;margin-top:{marginTop}px; margin-left:{marginLeft}px;" class="thumb-img">',
					'</div>',
				'</div>',
				'<div align="center">{shortName}</div>',
				'<div align="center">{width} x {height}</div>',
				//'<div align="center">{size} bytes</div>',
            '</div>',
            '</tpl>'
        )
    });

	function OnUploadFileDone()
	{
		//Ext.MessageBox.ShowWaitMessage(false);
		__this.ReloadAll();
	}
	
	function OnUploadError(msg)
	{
		//Ext.MessageBox.ShowWaitMessage(false);
		Ext.Msg.alert(CartLanguage.Error, msg);
	}
	
	var browseButton = new Ice.BrowseButton({
		text:CartLanguage.Upload,
		scope:__this,
		iconCls:'demo-grid-add tabs',
		handler: function(bb){
			var inp = bb.detachInputFile();
			inp.addClass('x-hidden');
			var fileName = inp.getValue();
			//Ext.MessageBox.ShowWaitMessage(true, CartLanguage.Uploading);
			new Ice.FileUpload().Upload(inp.dom, __this.GalleryNode.GalleryPhysicalPath, OnUploadFileDone, OnUploadError);
		}
	});
	
	QoDesk.ImageGalleryView.superclass.constructor.call(this, {
		cls: "images-view",
        margins: '5 5 5 0',
        layout:'fit',
        tbar: [
			browseButton,
			'-',
			{
				text:CartLanguage.Delete,
				tooltip:CartLanguage.DeleteSelected,
				iconCls:'demo-grid-delete',
				handler:function(){DeleteImages();}
			},
			"-",
			{
				text: CartLanguage.Rename,
				tooltip:CartLanguage.Rename,
				iconCls:"demo-grid-edit",
				handler:function(){RenameImage();}
			},
			"-",
			{
				text: CartLanguage.Reload,
				tooltip:CartLanguage.Reload,
				iconCls:"demo-grid-refresh",
				handler:function(){__this.ReloadAll();}
			}
		],
        items: view
    });
    
    function RenameImage()
    {	
		var nodeData = view.getSelectedRecords();
		if(nodeData.length == 0)
		{
			return;
		}
		
		var selectedImage = nodeData[0].data;
						
		function DoRename(result, name)
		{	
			if(result == "ok" && name != selectedImage.name)
			{
				selectedImage.name = name;
				//Ext.MessageBox.ShowWaitMessage(true, CartLanguage.Renaming);
				GService.GalleryRenameImage(selectedImage, __this.ReloadAll);
			}
		}
		Ext.MessageBox.prompt(CartLanguage.Name + "?", null, DoRename, __this, false, selectedImage.name);
	}
	
	function DeleteImages()
	{
		var nodeData = view.getSelectedRecords();
		if(nodeData.length == 0)
		{
			return;
		}
		function DoDelete(result)
		{
			if(result == "yes")
			{
				var imageFiles =[];
				for(var nn = 0; nn < nodeData.length; nn++)
				{
					imageFiles.push(nodeData[nn].data.file);
				}
				
				//Ext.MessageBox.ShowWaitMessage(true, CartLanguage.Deleting);
				GService.GalleryDeleteImages(imageFiles, __this.ReloadAll);
			}
		}
		Ext.MessageBox.confirm(CartLanguage.AreYouSure, CartLanguage.CantBeUndone, DoDelete);
	}
	
    var menu = new Ext.menu.Menu({
		width:140,
		items: [
		{
			text: CartLanguage.Rename,
			handler:function()
			{
				menu.hide();
				RenameImage();
			}
		},
		"-",
		{
			text:CartLanguage.Delete,
			handler: function()
			{
				menu.hide();
				DeleteImages();
			}
		}]
	});
	
	 var menu2 = new Ext.menu.Menu({
		width:140,
		items: [
		{
			text: CartLanguage.Delete,
			handler:function()
			{
				menu.hide();
				DeleteImages();
			}
		}]
	});
	
	function OnDone()
	{
		//Ext.MessageBox.ShowWaitMessage(false);
	}
	
    view.on("render", function(a,b,c,d){
		var dragZone = new ImageDragZone(view, {containerScroll:true, ddGroup: 'ImageGalleryDD'});
		
		dragZone.getTreeNode = function(dragData, targetNode, action, e)
		{
			var nodeData = view.getRecords(dragData.nodes);
			var imageFiles =[];
			for(var nn = 0; nn < nodeData.length; nn++)
			{
				imageFiles.push(nodeData[nn].data.file);
			}
			
			//Ext.MessageBox.ShowWaitMessage(true, CartLanguage.Copying);
			GService.GalleryCopyImages(imageFiles, targetNode.GalleryNode, OnDone);
		};
		
    }, view);
    
   view.on("contextmenu", function(view, button, el, e){
			
			var xy = e.xy;
			e.stopEvent();
			xy[0] = xy[0] - 10;
			xy[1] = xy[1] - 10;

			function ShowMenu()
			{
    			var nodeData = view.getSelectedRecords();
    			if(nodeData.length == 1)
    			{
    				menu.showAt(xy);
    			}
    			else if(nodeData.length > 1)
    			{
    				menu2.showAt(xy);
    			}
    		}
    		setTimeout(ShowMenu, 100);
    		return false;
		}, view);

	view.on("dblclick", function(view, button, el, e){
		var selections = view.getSelectedRecords();
		if(selections.length > 0)
		{
			var imageData = selections[0].data;
			if(__this.Owner.OnImageDoubleClick != null)
			{
				__this.Owner.OnImageDoubleClick.handler(imageData, __this.Owner.OnImageDoubleClick.options);
				if(__this.Owner.OnImageDoubleClick.minimize)
				{
					__this.Owner.win.minimize();
				}
				__this.Owner.OnImageDoubleClick = null;
			}
		}
		
	}, view);
		
    function OnDataReady(images)
    {
		view.store.loadData(images);
		//Ext.MessageBox.ShowWaitMessage(false);
    }
    
	this.ReloadAll = function()
	{
		//Ext.MessageBox.ShowWaitMessage(true, CartLanguage.Loading);
		GService.GetGalleryImageCollection(__this.GalleryNode, OnDataReady);
	};
	
	///this.ReloadAll();
	
};

QoDesk.ImageGalleryView = Ext.extend(QoDesk.ImageGalleryView, Ext.Panel);

/**
 * Create a DragZone instance for our JsonView
 */
ImageDragZone = function(view, config){
    this.view = view;
    ImageDragZone.superclass.constructor.call(this, view.getEl(), config);
};
Ext.extend(ImageDragZone, Ext.dd.DragZone, {
    // We don't want to register our image elements, so let's 
    // override the default registry lookup to fetch the image 
    // from the event instead
    getDragData : function(e){
        var target = e.getTarget('.thumb-wrap');
        if(target){
            var view = this.view;
            if(!view.isSelected(target)){
                view.onClick(e);
            }
            var selNodes = view.getSelectedNodes();
            var dragData = {
                nodes: selNodes
            };
            if(selNodes.length == 1){
                dragData.ddel = target.firstChild.firstChild; // the img element
                dragData.single = true;
            }else{
                var div = document.createElement('div'); // create the multi element drag "ghost"
                div.className = 'multi-proxy';
                for(var i = 0, len = selNodes.length; i < len; i++){
                    div.appendChild(selNodes[i].firstChild.firstChild.cloneNode(true));
                    if((i+1) % 3 == 0){
                        div.appendChild(document.createElement('br'));
                    }
                }
                dragData.ddel = div;
                dragData.multi = true;
            }
            return dragData;
        }
        return false;
    },

    // this method is called by the TreeDropZone after a node drop
    // to get the new tree node (there are also other way, but this is easiest)
    getTreeNode : function(){
        var treeNodes = [];
        var nodeData = this.view.getRecords(this.dragData.nodes);
        for(var i = 0, len = nodeData.length; i < len; i++){
            var data = nodeData[i].data;
            treeNodes.push(new Ext.tree.TreeNode({
                text: data.name,
                icon: data.url,
                data: data,
                leaf:true,
                cls: 'image-node'
            }));
        }
        return treeNodes;
    },
    
    // the default action is to "highlight" after a bad drop
    // but since an image can't be highlighted, let's frame it 
    afterRepair:function(){
        for(var i = 0, len = this.dragData.nodes.length; i < len; i++){
            Ext.fly(this.dragData.nodes[i]).frame('#8db2e3', 1);
        }
        this.dragging = false;    
    },
    
    // override the default repairXY with one offset for the margins and padding
    getRepairXY : function(e){
        if(!this.dragData.multi){
            var xy = Ext.Element.fly(this.dragData.ddel).getXY();
            xy[0]+=3;xy[1]+=3;
            return xy;
        }
        return false;
    }
});
