﻿/// <reference path="../../ice-fix-ext-before.js" />
/// <reference path="../../Ext_21/source/core/Ext.js" />
/// <reference path="../../Ext_21/source/adapter/ext-base.js" />
/// <reference path="../../Ext_21/ext-all-debug.js" />
/// <reference path="../../ice-fix-ext-after.js" />
/// <reference path="DbAdminNamespace.js" />
/// <reference path="GridWithPaging.js" />
/// <reference path="TabCloseMenu.js" />
/// <reference path="CategoriesTree.js" />

QoDesk.GetProductsRightPane = function(nodeData){

	nodeData = nodeData || {Name: "all"};
	
	var isCategory = nodeData.CategoryID > 0;
	var isCampaign = nodeData.CampaignID > 0;
	
	var title = nodeData.Name;
	var categoryId = nodeData.CategoryID;
	var campaignId = nodeData.CampaignID;
	
	var baseParams = {};
	if(isCategory)
	{
		baseParams.CategoryID = categoryId;
	}
	else if(isCampaign)
	{
		baseParams.CampaignID = campaignId;
	}
	
	var base2 = Ext.apply({}, baseParams);
	
	function AddOrRemoveProducts(tableName, dataArray, add)
	{
		//  remove from category or campaign
		var method = null;
		var id = null;
		if(isCategory)
		{
			id = nodeData.CategoryID;
			method = GService.ProductsInCategories;
		}
		else if(isCampaign)
		{
			id = nodeData.CampaignID;
			method = GService.ProductsInCampaigns;
		}
		if(method)
		{
			var wp = [];
			for(var nn = 0; nn < dataArray.length; nn++)
			{
				wp.push(dataArray[nn].ProductID);
			}
			method([id], wp, add, function(){productsGrid.Reload();});
		}
		else
		{
			productsGrid.Reload();
		}
	}
	
	function AddOrRemoveWebPages(tableName, dataArray, add)
	{
		//  remove from category or campaign
		var method = null;
		var id = null;
		if(isCategory)
		{
			id = nodeData.CategoryID;
			method = GService.WebPagesInCategories;
		}
		else if(isCampaign)
		{
			id = nodeData.CampaignID;
			method = GService.WebPagesInCampaigns;
		}
		if(method)
		{
			var wp = [];
			for(var nn = 0; nn < dataArray.length; nn++)
			{
				wp.push(dataArray[nn].WebPageID);
			}
			method([id], wp, add, function(){webPagesGrid.Reload();});
		}
		else
		{
			webPagesGrid.Reload();
		}
	}
	
	var productsGrid = Ice.DbAdmin.GetGridWithPaging({
		tableName:"Product", 
		baseParams: baseParams,
		disableInlineEditing:true,
		hideSchemaInfo:true,
		multiline:true,
		sortInfo: {field:'Ordinal', direction:'DESC'},
		menuItemsBefore:
		[
			{
				text: "Designer",
				iconCls:"icon_overlays",
				handler: function(sender, e){
					var record = null;
					var selections = productsGrid.getSelectionModel().getSelections(); // use selected data as template for new record
					if(selections && selections.length > 0)
					{
						record = selections[0];
						var prodEditor = QoDesk.getModule("ProductEditEditor");
						prodEditor.createWindow(record, function(){productsGrid.Reload();});
					}
				}
			},
			"-"
		],
		columns:
		{
			"ShortDescription":
			{
				hidden:true,
				enableFormEditing:true
			},
			"LongDescription":
			{
				hidden:true,
				enableFormEditing:true
			}
		},
		methodRemove: function(tableName, dataArray, onDone){
			AddOrRemoveProducts(tableName, dataArray, false);
		},
		showRemove: isCategory || isCampaign,
		methodDelete:function(tableName, dataArr, onDone){
			var pids = [];
			for(var nn = 0; nn < dataArr.length; nn++)
			{
				pids.push(dataArr[nn].ProductID);
			}
			GService.DeleteProducts(pids, onDone);
		},
		onSaved: function(dataArray){
			AddOrRemoveProducts(null, dataArray, true);
		}
	});
	
	var webPagesGrid = Ice.DbAdmin.GetGridWithPaging({
		tableName:"WebPage", 
		baseParams: base2,
		disableInlineEditing:true,
		hideSchemaInfo:true,
		sortInfo: {field:'Ordinal', direction:'DESC'},
		multiline:true,
		menuItemsBefore:
		[
			{
				text: "Designer",
				iconCls:"icon_overlays",
				handler: function(sender, e){
					var record = null;
					var selections = webPagesGrid.getSelectionModel().getSelections(); // use selected data as template for new record
					if(selections && selections.length > 0)
					{
						record = selections[0];
						new QoDesk.WebPageEdit(record, null, true);
					}
				}
			},
			"-"
		],
		methodRemove: function(tableName, dataArray, onDone){
			AddOrRemoveWebPages(tableName, dataArray, false);
		},
		showRemove: isCategory || isCampaign,
		methodDelete:function(tableName, dataArr, onDone){
			var pids = [];
			for(var nn = 0; nn < dataArr.length; nn++)
			{
				pids.push(dataArr[nn].WebPageID);
			}
			GService.DeleteWebPages(pids, onDone);
		},
		onSaved: function(dataArray){
			AddOrRemoveWebPages(null, dataArray, true);
		},
		columns:
		{
			"Content":
			{
				hidden:true,
				enableFormEditing:true
			}
		}
	});
	
	var tab = new Ext.Panel({
		title: title,
		layout:"border",
		border:false,
		closable: categoryId > 0 || campaignId > 0,
		iconCls:"icon_application_view_columns",
		items:
		[
			{
				title:"Products",
				region:"center",
				layout:"fit",
				border:true,
				items: productsGrid
			},
			{
				title:"WebPages",
				region:"south",
				height: 250,
				layout:"fit",
				split:true,
				border:true,
				collapsible:true,
				items: webPagesGrid
			}
		]
	});
	
	tab.CategoryID = categoryId || "";
	tab.CampaignID = campaignId || "";
	
	var count = 0;
	
	tab.on("show", function(){
		if(count ++ > 0) // don't reload on first time show
		{
			productsGrid.Reload();
			webPagesGrid.Reload();
		}
	});
	
	return tab;
};




